/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *--------------------------------------------------------------------
 *
 * TCaApp.cpp
 * This class is responsible for observing all media sessions and creating
 * CA core objects for each media player instance, to enable decrypion of all
 * played media. The class also implements all the required functionality to
 * act as a KreaTV application.
 *
 * Copyright (c) 2010 Motorola, Inc. All Rights Reserved.
 * Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
 *
 *--------------------------------------------------------------------
 */

#include "TCaApp.h"
#include "TCaCore.h"

#include "interface/ToiObjectNames.h"
#include "interface/TToiNameServiceCaller.h"

#include <cstdlib>

TCaApp::TCaApp()
  throw()
  : Tracker(TTracker::GetProcessTracker()),
    ApplicationId(-1),
    NameServiceCaller(NULL),
    ApplicationServiceCaller(NULL),
    MediaServiceCaller(NULL),
    ApplicationDispatcher(NULL),
    MediaObserverDispatcher(NULL)
{
  try {
    // Connect to Name Service
    char* nameServiceAddress = ::getenv("NAMESERVICE_ADDRESS");
    if (nameServiceAddress == NULL) {
     throw TException("NAMESERVICE_ADDRESS not set.");
    }
    NameServiceCaller = new TToiNameServiceCaller(&IpcClient,
                                                 nameServiceAddress);
    // Get application ID.
    char* id = ::getenv("APPLICATION_ID");
    if (id == NULL) {
     throw TException("APPLICATION_ID not set.");
    }
    char* endPtr;
    ApplicationId = ::strtol(id, &endPtr, 10);
    if (id[0] == '\0' || endPtr[0] != '\0') {
     throw TException("APPLICATION_ID string has illegal format");
    }

    // Connect to Application Service.
    Tracker->Note("Connecting to Application Service");
    std::string address;
    NameServiceCaller->LookupObject(TOI_APPLICATION_SERVICE_NAME, address);
    ApplicationServiceCaller =
      new TToiApplicationServiceCaller(&IpcClient, address);
    ApplicationAddress = IpcServer.CreateObject();
    ApplicationDispatcher =
      new TToiApplicationDispatcher(&IpcServer, ApplicationAddress, this);

    // Connect to Media Service.
    Tracker->Note("Connecting to Media Service");
    MediaObserverAddress = IpcServer.CreateObject();
    MediaObserverDispatcher =
        new TToiMediaObserverDispatcher(&IpcServer,
                                        MediaObserverAddress,
                                        this);

    NameServiceCaller->LookupObject(MEDIA_SERVICE_NAME, MediaServiceAddress);
    MediaServiceCaller =
        new TToiMediaServiceCaller(&IpcClient, MediaServiceAddress);
    MediaServiceCaller->RegisterObserver(ApplicationId, MediaObserverAddress);
  }
  catch (TException& e) {
    Tracker->Fatal("exception: %s", e.GetText());
    ::abort();
  }
}

TCaApp::~TCaApp()
{
  for (TCaCoreMap::iterator i = CaCores.begin(); i != CaCores.end(); ++i) {
    delete (*i).second;
  }
  CaCores.clear();

  delete MediaServiceCaller;
  delete MediaObserverDispatcher;
  delete ApplicationServiceCaller;
  delete NameServiceCaller;
  delete ApplicationDispatcher;
}

void TCaApp::Run() throw()
{
  // Tell Application Service that this application has started.
  ApplicationServiceCaller->ReportStarted(ApplicationId, ApplicationAddress);

  // Enter the event loop, causing the entire program to be event driven.
  IpcServer.Run();
}

void TCaApp::Ping() throw ()
{
  ApplicationServiceCaller->ReportPing(ApplicationId);
}

void TCaApp::ChangeState(const IToiApplicationService::
                               TToiApplicationState& /*state*/) throw ()
{
  // Empty
}

void TCaApp::LoadUri(const std::string& /*uri*/,
                          const std::string& /*mimeType*/) throw ()
{
  // Empty
}

void TCaApp::ExecuteCommand(const std::string& /*command*/) throw ()
{
  // Empty
}

//------------------------------------------------------------------------
// IMediaObserver Functions.

// This event notifies the observer that new information about a stream is
// available.
void TCaApp::
OnSessionStreamInfoChanged(const IToiMediaService::
                           TToiSessionInfo& /* sessionInfo */,
                           const IToiMediaSession::
                           TToiStreamInfo& /* streamInfo */) throw()
{
  Tracker->Note("Receive OnSessionStreamInfoChanged");
}

void TCaApp::
OnSessionStateChanged(const IToiMediaService::
                      TToiSessionInfo& sessionInfo) throw()
{
  Tracker->Note("Receive OnSessionStateChanged");
  // Only interested in decoding players (not recorders).
  if (((sessionInfo.Type)&(IToiMediaSession::TYPE_PLAYER))!=
          IToiMediaSession::TYPE_PLAYER) {
    return;
  }

  //Create an instance of TCaCore for decryption of a stream in a new player.
  if (sessionInfo.State == IToiMediaSession::STATE_RUNNING) {
    TCaCoreMap::iterator i = CaCores.find(sessionInfo.Id);
    assert(i == CaCores.end());
    TCaCore* CaCore = new TCaCore(IpcServer,
                                  MediaServiceCaller,
                                  ApplicationId,
                                  sessionInfo.Id,
                                  sessionInfo.Type,
                                  sessionInfo.Url);
    CaCores[sessionInfo.Id] = CaCore;
  }

  // Delete the TCaCore instance associated with the closed media player.
  if (sessionInfo.State == IToiMediaSession::STATE_CLOSED) {
    TCaCoreMap::iterator i = CaCores.find(sessionInfo.Id);
    assert(i != CaCores.end());
    TCaCore* CaCore = (*i).second;
    delete CaCore;
    CaCores.erase(i);
  }
}
