/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *-----------------------------------------------------------------------------
 *
 * TCaCore.cpp
 * This class is responsible for enabling decryption for a media player
 * session. It sets the cipher and keys needed in the session.
 *
 * Copyright (c) 2010 Motorola, Inc. All Rights Reserved.
 * Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
 *
 *-----------------------------------------------------------------------------
 */

#include "TCaCore.h"
#include <cstdlib>
#include <unistd.h>
#include <sys/types.h>

#include "interface/IToiMediaPlayer.h"

TCaCore::TCaCore(TIpcServer& ipcServer,
                 TToiMediaServiceCaller* mediaService,
                 int32_t applicationId,
                 long sessionId,
                 long sessionType,
                 std::string sessionUrl)
    throw()
 :  IpcServer(ipcServer),
    MediaServiceCaller(mediaService),
    MediaSessionCaller(NULL),
    ApplicationId(applicationId),
    Tracker(TTracker::GetProcessTracker()),
    SessionId(sessionId),
    SessionType(sessionType),
    SessionUrl(sessionUrl),
    UpdateKeyTimerId(-1)
{
  Tracker->Note("A instance of TCaCore is created.");

  try {
    std::string sessionAddress;
    MediaServiceCaller->OpenSession(ApplicationId, SessionId,
                                    sessionAddress);
    MediaSessionCaller = new TToiMediaSessionCaller(&IpcClient,
                                 sessionAddress);
  }
  catch (TException &e) {
    Tracker->Error("exception: %s", e.GetText());
  }

  // Here just use a timer for demo the keys change each 5 seconds.
  UpdateKeyTimerId =
    IpcServer.GetEventLoop()->AddTimer(5000, this);
}

TCaCore::~TCaCore()
{
  if (MediaSessionCaller != NULL) {
    try  {
      MediaSessionCaller->ReleaseInstance();
    }
    catch (TException &e) {
      Tracker->Error("exception: %s", e.GetText());
    }
    delete MediaSessionCaller;
    MediaSessionCaller = NULL;
  }

  if (!EsPids.empty()) {
    EsPids.clear();
  }

  if (UpdateKeyTimerId != -1) {
    IpcServer.GetEventLoop()->RemoveTimer(UpdateKeyTimerId);
  }
}

bool TCaCore::DoSetDecryptKey() throw ()
{
  Tracker->Note("Set Cipher and Key of Decryption.");
  if (MediaSessionCaller == NULL) {
    return false;
  }

  try {
    // The block and residue modes are just examples.
    MediaSessionCaller->
      SetDecryptionCipher(EsPids,
                          IToiMediaSession::CIPHER_AES_128,
                          IToiMediaSession::CIPHER_BLOCK_MODE_ECB,
                          IToiMediaSession::CIPHER_RESIDUE_MODE_BACK);

    MediaSessionCaller->
      SetDecryptionKeys(EsPids,
                        false,
                        EvenControlWord, NULL,
                        OddControlWord, NULL,
                        32);
  }
  catch (TToiSessionViolationException& e) {
    Tracker->Error("CaCore: Could not set decryption keys; No session.");
    return false;
  }
  catch (TException &e) {
    Tracker->Error("exception: %s", e.GetText());
    return false;
  }
  return true;
}

//------------------------------------------------------------------------
// IIOTimeoutHandler

void TCaCore::HandleTimeout(int timerId) throw ()
{
  if (timerId == UpdateKeyTimerId) {
    // Get the cipher mode and control word from CA server.
    // For demo, set fake value.

    // Assume the pid 101 and 105 need decryption.
    EsPids.push_back(101);
    EsPids.push_back(105);

    // Set OddControlWord.
    uint8_t oddControlWord[sizeof(OddControlWord)] =
                          {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                           0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                           0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17,
                           0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f};
    memcpy(OddControlWord, oddControlWord, sizeof(OddControlWord));

    // Set EvenControlWord.
    uint8_t evenControlWord[sizeof(EvenControlWord)] =
                          {0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27,
                           0x28, 0x29, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
                           0x30, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17,
                           0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f};
    memcpy(EvenControlWord, evenControlWord, sizeof(EvenControlWord));

    // Enable the Decryption.
    DoSetDecryptKey();
  }
  else {
    Tracker->Error("[%s:%s@%d] => Unknown timer id: %d",
                    __FILE__, __FUNCTION__, __LINE__, timerId);
  }
}
