/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *--------------------------------------------------------------------
 *
 * TRecorderApp.cpp
 * This class contains all of the functionality necessary for registering
 * the application with the KreaTV TV Application Platform as well as
 * initializing the event observers, booking recording and realizing all
 * the callback functions when starting and stopping recording.
 *
 * Copyright (c) 2010 Motorola. All Rights Reserved.
 * Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
 *
 * This program is confidential and proprietary to Motorola Mobility, Inc and
 * may not be copied, reproduced, disclosed to others, published or used, in
 * whole or in part, without the expressed prior written permission of Motorola
 * Mobility, Inc.
 *
 *--------------------------------------------------------------------
 */

#include "TRecorderApp.h"

#include "debug/TTracker.h"
#include "interface/ToiObjectNames.h"
#include "interface/TToiNameServiceCaller.h"
#include "interface/IToiTypes.h"

//record url: 224.5.5.1:10000
#define RECORD_URL "224.5.5.1:10000"

//record start time: now
#define RECORD_STARTTIME ::time(NULL)

//record duration: 10 seconds
#define RECORD_DURATION 10

TRecorderApp::TRecorderApp()
  throw (TException) :
  Tracker(TTracker::GetProcessTracker()),
  ApplicationId(-1),
  NameServiceCaller(NULL),
  ApplicationServiceCaller(NULL),
  MediaServiceCaller(NULL),
  SchedulerServiceCaller(NULL),
  AssetManagerServiceCaller(NULL),
  ApplicationDispatcher(NULL),
  InformationObserverDispatcher(NULL),
  SchedulerObserverDispatcher(NULL)
{
}

TRecorderApp::~TRecorderApp() throw ()
{
  TRecorder* Recorder = NULL;
  for (TRecorderMap::iterator i = RecorderMap.begin();
       i != RecorderMap.end(); ++i) {
    Recorder = (TRecorder*)((*i).second);
    if (Recorder != NULL) {
      Recorder = (*i).second;
      Recorder->Close();
      Recorder->ReleaseInstance();
      delete Recorder;
    }
  }
  RecorderMap.clear();

  if (!SchedulerObserverAddress.empty()) {
    SchedulerServiceCaller->ReleaseObserver(SchedulerObserverAddress);
  }
  if (InformationObserverAddress.empty()) {
    InformationServiceCaller->ReleaseObserver(InformationObserverAddress);
  }

  delete SchedulerObserverDispatcher;
  delete InformationObserverDispatcher;
  delete ApplicationDispatcher;

  delete AssetManagerServiceCaller;
  delete SchedulerServiceCaller;
  delete MediaServiceCaller;
  delete InformationServiceCaller;
  delete ApplicationServiceCaller;
  delete NameServiceCaller;
}

bool TRecorderApp::Connect() throw (TException)

{
  std::string address;
  // Connect to Name Service
  char* nameServiceAddress = ::getenv("NAMESERVICE_ADDRESS");
  if (nameServiceAddress == NULL) {
    throw TException("NAMESERVICE_ADDRESS not set.");
  }

  NameServiceCaller = new TToiNameServiceCaller(&IpcClient,
                                                nameServiceAddress);

  // Connect to Application Service
  NameServiceCaller->LookupObject(TOI_APPLICATION_SERVICE_NAME, address);
  ApplicationServiceCaller =
    new TToiApplicationServiceCaller(&IpcClient, address);

  // Create an unique object address that can later be used when adding a
  // dispatcher.
  ApplicationAddress = IpcServer.CreateObject();
  ApplicationDispatcher =
    new TToiApplicationDispatcher(&IpcServer, ApplicationAddress, this);

  // Get application ID
  char* id = ::getenv("APPLICATION_ID");
  if (id == NULL) {
    throw TException("APPLICATION_ID not set.");
  }
  char* endPtr;
  ApplicationId = ::strtol(id, &endPtr, 10);
  if (id[0] == '\0' || endPtr[0] != '\0') {
    throw TException("APPLICATION_ID string has illegal format");
  }

  // Connect to Information Service
  NameServiceCaller->LookupObject(TOI_INFORMATION_SERVICE_NAME, address);
  InformationServiceCaller =
    new TToiInformationServiceCaller(&IpcClient, address);
  InformationObserverAddress = IpcServer.CreateObject();
  InformationObserverDispatcher =
    new TToiInformationObserverDispatcher(&IpcServer,
                                          InformationObserverAddress, this);
  InformationServiceCaller->RegisterObserver(ApplicationId,
                                             InformationObserverAddress);

  // Connect to Scheduler Service
  try {
    NameServiceCaller->LookupObject(TOI_SCHEDULER_SERVICE_NAME, address);
  }
  catch (TToiInvalidArgumentException&) {
    Tracker->Error("Scheduler Service is not installed");
    return false;
  }
  SchedulerServiceCaller =
    new TToiSchedulerServiceCaller(&IpcClient, address);

  SchedulerObserverAddress = IpcServer.CreateObject();
  SchedulerObserverDispatcher =
    new TToiSchedulerObserverDispatcher(&IpcServer,
                                        SchedulerObserverAddress, this);
  SchedulerServiceCaller->RegisterObserver(ApplicationId,
                                           SchedulerObserverAddress);

  // Connect to Media Service
  NameServiceCaller->LookupObject(TOI_MEDIA_SERVICE_NAME, address);
  MediaServiceCaller = new TToiMediaServiceCaller(&IpcClient, address);

  // Connect to Asset Manager Service
  try {
    NameServiceCaller->LookupObject(TOI_ASSET_MANAGER_SERVICE_NAME, address);
  }
  catch (TToiInvalidArgumentException&) {
    Tracker->Error("Asset Manager Service is not installed");
    return false;
  }
  AssetManagerServiceCaller =
    new TToiAssetManagerServiceCaller(&IpcClient, address);
  return true;
}

void TRecorderApp::Run() throw (TException)
{
  // Connect to various services.
  if (!Connect()) {
    return;
  }

  // Add subscriptiona for Information Object
  InformationServiceCaller->SubscribeObject(InformationObserverAddress,
                                            "var.time.valid", true);

  // Filter which categories the application is notified about.
  SchedulerServiceCaller->SetCategorySubscription(SchedulerObserverAddress,
                                                  "*");

  // Tell Application Service that this application has started.
  ApplicationServiceCaller->ReportStarted(ApplicationId, ApplicationAddress);

  // Create an unique object address that enables clients in other processes
  // to connect to the object.
  IpcServer.CreateObject();

  // Enter the event loop, causing the entire program to be event driven.
  IpcServer.Run();
}

void TRecorderApp::
OnObjectsChanged(const IToiInformationService::TToiObjectNameSequence&
                 objectNames)
  throw ()
{
  // Go through object name list.
  for (IToiInformationService::TToiObjectNameSequence::const_iterator i =
         objectNames.begin(); i != objectNames.end(); ++i) {
    if (*i == "var.time.valid") {
      std::string timeValidString;
      InformationServiceCaller->GetObject("var.time.valid", timeValidString);
      if (timeValidString == "TRUE") {
        // Starts booking when time is valid.
        Tracker->Note("=======Time valid. Start booking record=======");

        // Requests scheduling of a new booking.
        IToiSchedulerService::TToiBookingId bookingId;
        SchedulerServiceCaller->Schedule("dvr",
                                         "record_hd_from_ip",
                                         RECORD_STARTTIME,
                                         RECORD_DURATION,
                                         bookingId);
      }
    }
  }
}

void TRecorderApp::OnObjectsRemoved(const IToiInformationService::
                                    TToiObjectNameSequence& /*objectNames*/)
  throw ()
{
  // Ignore
}

void TRecorderApp::Ping() throw ()
{
  ApplicationServiceCaller->ReportPing(ApplicationId);
}

void TRecorderApp::ChangeState(const IToiApplicationService::
                               TToiApplicationState& /*state*/) throw ()
{
  // Ignore
}

void TRecorderApp::LoadUri(const std::string& /*uri*/,
                          const std::string& /*mimeType*/) throw ()
{
  // Ignore
}

void TRecorderApp::ExecuteCommand(const std::string& /*command*/) throw ()
{
  // Ignore
}


void TRecorderApp::OnScheduledStart(const IToiSchedulerService::
                                    TToiBooking& booking) throw (TIpcException)
{
  std::string state;
  IToiAssetManagerService::TToiAssetId assetId;
  IToiSchedulerService::TToiParameterNameSequence parameterNames;

  Tracker->Note("OnScheduledStart:booking.Category=%s",
      booking.Category.c_str());

  if (booking.Category == "dvr") {
    // Get all user defined parameters names of a booking.
    SchedulerServiceCaller->GetParameterNames(booking.Id, parameterNames);

    for (IToiSchedulerService::TToiParameterNameSequence::iterator i =
           parameterNames.begin(); i != parameterNames.end(); i++) {

      if (*i == "state") {
        // Get user defined parameter associated with a booking.
        SchedulerServiceCaller->GetParameter(booking.Id, "state", state);
      }
      else if (*i == "asset") {
        // Get user defined parameter associated with a booking.
        SchedulerServiceCaller->GetParameter(booking.Id, "asset", assetId);
      }
    }

    if (state == "incomplete") {
      if (!assetId.empty()) {
        // Removes an asset.
        AssetManagerServiceCaller->RemoveAsset(0, assetId);
      }
      try {
        // Remove a booking.
        SchedulerServiceCaller->Remove(booking.Id);
      }
      catch (TToiInvalidArgumentException& e) {
        Tracker->Error("[%s:%s@%d]=> cannot remove: booking(%d) for %s",
                         __FILE__, __FUNCTION__, __LINE__,
                         booking.Id, e.what());
      }
      catch (TIpcException& e) {
        Tracker->Error("[%s:%s@%d]=> cannot remove: booking(%d) for %s",
                         __FILE__, __FUNCTION__, __LINE__,
                         booking.Id, e.what());
      }
      return;
    }

    // Creates an asset. The identifier of the asset can be used with a media
    // recorder to save a media stream to disk.
    AssetManagerServiceCaller->CreateAsset(IToiAssetManagerService::ASSET_PVR,
                                           assetId);

    // Add user defined parameters to a booking.
    SchedulerServiceCaller->SetParameter(booking.Id, "asset", assetId);
    SchedulerServiceCaller->SetParameter(booking.Id, "state", "incomplete");

    std::string mediaRecorderAddress;

    // This function creates a new media recorder and associates it with the
    // calling application.
    MediaServiceCaller->CreateRecorderInstance(ApplicationId,
                                               mediaRecorderAddress);

    TRecorder* Recorder = new TRecorder(&IpcClient, mediaRecorderAddress);
    RecorderMap[booking.Id] = Recorder;

    // Try to establish a connection to the media source.
    Recorder->Open(RECORD_URL, assetId);

    Tracker->Note("=======Start recording...booking.Id=%d=======",booking.Id);

    // Starts recording of the stream.
    Recorder->Record();
  }
}

void TRecorderApp::OnScheduledStop(const IToiSchedulerService::
                                   TToiBooking& booking) throw (TIpcException)
{
  TRecorderMap::iterator i = RecorderMap.find(booking.Id);
  if (i == RecorderMap.end()) {
    return;
  }

  TRecorder* Recorder = (*i).second;

  Tracker->Note("=======Stop recording=======");

  // Stops any recording and disconnects from the media source.
  Recorder->Close();

  // Releases the resources used by this instance.
  Recorder->ReleaseInstance();

  // Remove the booking record.
  RecorderMap.erase(i);
  delete Recorder;
  try {
    // Remove a booking.
    SchedulerServiceCaller->Remove(booking.Id);
  }
  catch (TToiInvalidArgumentException& e) {
    Tracker->Error("[%s:%s@%d]=> cannot remove: booking(%d) for %s",
                     __FILE__, __FUNCTION__, __LINE__, booking.Id, e.what());
  }
  catch (TIpcException& e) {
    Tracker->Error("[%s:%s@%d]=> cannot remove: booking(%d) for %s",
                     __FILE__, __FUNCTION__, __LINE__, booking.Id, e.what());
  }
}

void TRecorderApp::
OnScheduledTrigger(const IToiSchedulerService::TToiBooking& /*booking*/,
                   const IToiSchedulerService::TToiDelta& /*delta*/)
  throw (TIpcException)
{
  // Ignore
}

void TRecorderApp::
OnBookingsAdded(const IToiSchedulerService::TToiBookingSequence& /*booking*/)
  throw (TIpcException)
{
  // Ignore
}

void TRecorderApp::
OnBookingsRemoved(const IToiSchedulerService::TToiBookingSequence&
                  bookingList)
  throw (TIpcException)
{
  // Responds to the booking removal event.

  const IToiSchedulerService::TToiBooking& booking = bookingList.front();

  IToiAssetManagerService::TToiAssetId assetId;
  TRecorderMap::iterator i = RecorderMap.find(booking.Id);
  if (i == RecorderMap.end()) {
    return;
  }

  TRecorder* Recorder = (*i).second;
  assetId = Recorder->GetAssetId();
  Recorder->Close();
  Recorder->ReleaseInstance();
  RecorderMap.erase(i);
  delete Recorder;
  // Removes an asset.
  AssetManagerServiceCaller->RemoveAsset(0,assetId);
}

void TRecorderApp::
OnBookingsChanged(const IToiSchedulerService::TToiBookingSequence&
                  /*bookingList*/)
  throw (TIpcException)
{
  // Ignore
}

void TRecorderApp::OnSeriesChanged(
    const IToiSchedulerService::TToiSeries& /*booking*/)
  throw (TIpcException)
{
  // Ignore
}




