/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * TRecorderApp.h
 * This file contains the declaration of the TRecorderApp class. The public
 * interface to this class includes a constructor and destructor pair, and a
 * Run() function which does the service connection and object subscription.
 * It also contains several callback functions driven by multiple types of
 * events, e.g. information object changed event and scheduler start and
 * stop events, etc.
 *
 * Copyright (c) 2010 Motorola. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef TRECORDERAPP_H
#define TRECORDERAPP_H

#include <map>

#include "debug/TTracker.h"
#include "ipc/TIpcServer.h"
#include "ipc/TIpcClient.h"

#include "interface/TToiNameServiceCaller.h"
#include "interface/TToiApplicationDispatcher.h"
#include "interface/TToiApplicationServiceCaller.h"

#include "interface/TToiInformationObserverDispatcher.h"
#include "interface/TToiInformationServiceCaller.h"

#include "interface/TToiSchedulerObserverDispatcher.h"
#include "interface/TToiSchedulerServiceCaller.h"
#include "interface/TToiMediaServiceCaller.h"
#include "interface/TToiAssetManagerServiceCaller.h"

#include "TRecorder.h"

class TRecorderApp : public IToiApplication,
                     public IToiInformationObserver,
                     public IToiSchedulerObserver
{
private:
  // A handler to print out logs
  TTracker* Tracker;

  // Application unique ID
  int ApplicationId;

  // IPC variables
  TIpcServer IpcServer;
  TIpcClient IpcClient;

  ///////////////////// Handlers of services
  // A handler to access the Name Service, which is used to locate other
  // platform services.
  TToiNameServiceCaller* NameServiceCaller;

  // A handler to access the Application Service, which is responsible for
  // maintaining the lifecycles of every application registered in the system.
  TToiApplicationServiceCaller* ApplicationServiceCaller;

  // A handler to access the Information Service, which is responsible for
  // collecting and storing information objects.
  TToiInformationServiceCaller* InformationServiceCaller;

  // A handler to access the Media Service. Here it is used to create media
  // recorder.
  TToiMediaServiceCaller* MediaServiceCaller;

  // A handler to access the Scheduler Service, which handles scheduling
  // and conflict management of resources.
  TToiSchedulerServiceCaller* SchedulerServiceCaller;

  // A handler to access to locally stored media assets and metadata
  // associated with these assets.
  TToiAssetManagerServiceCaller* AssetManagerServiceCaller;

  ///////////////////// Addresses for IPC communication
  std::string ApplicationAddress;
  std::string InformationObserverAddress;
  std::string SchedulerObserverAddress;

  ///////////////////// Dispatchers for IPC communication
  TToiApplicationDispatcher* ApplicationDispatcher;
  TToiInformationObserverDispatcher* InformationObserverDispatcher;
  TToiSchedulerObserverDispatcher* SchedulerObserverDispatcher;

  // Structure to store booking tasks: <booking ID, recorder hander>
  typedef std::map<long, TRecorder*> TRecorderMap;
  TRecorderMap RecorderMap;

  // Connect to various services
  bool Connect() throw (TException);

public:
  TRecorderApp() throw (TException);
  virtual ~TRecorderApp() throw ();

  // Launch application
  void Run() throw (TException);


  ///////////////////// IToiInformationObserver
  // Applications that wants to be updated when information objects are
  // added, removed or change values shall implement this interface.

  // This method is called when some subscribed objects have changed value.
  virtual void OnObjectsChanged(const IToiInformationService::
                                TToiObjectNameSequence& objectNames) throw ();
  // This method is called when some subscribed objects no longer exist in
  // the Information Service.
  virtual void OnObjectsRemoved(const IToiInformationService::
                                TToiObjectNameSequence& objectNames) throw ();


  ///////////////////// IToiApplication
  // The IToiApplication interface is used by the Application Service to
  // control the applications and to inform them about state changes. All
  // applications running on the KreaTV TV Application Platform must implement
  // this interface.

  // Check that the application is responding to events. The application should
  // acknowledge this event by calling IToiApplicationService::ReportPing. If
  // it doesn't acknowledge within a time specified in the application
  // properties the Application Service will consider the application as failed
  // and stop it.
  virtual void Ping() throw ();

  // Informs an application that its state has changed.
  virtual void ChangeState(const IToiApplicationService::
                           TToiApplicationState& state)
    throw ();

  // Used to tell the application to load an URI.
  virtual void LoadUri(const std::string& uri,
                       const std::string& mimeType) throw ();

  // This function is called to make the application execute a command.  The
  // application itself defines what commands it supports and how they are
  // interpreted.
  virtual void ExecuteCommand(const std::string& command) throw ();


  ///////////////////// IToiSchedulerObserver interface
  // Interface to receive notfications regarding status of scheduling requests
  // and bookings.

  // This function will be called when the start timestamp of a booking is
  // reached. If an application starts observing the booking after start
  // timestamp has passed, it will receive no such callback.
  virtual void OnScheduledStart(
      const IToiSchedulerService::TToiBooking& booking)
    throw (TIpcException);

  // This function will be called when the stop timestamp of a booking is
  // reached.
  virtual void OnScheduledStop(
      const IToiSchedulerService::TToiBooking& booking)
    throw (TIpcException);

  // This function will be called when the delta timestamp before or after the
  // start of a booking is reached.
  virtual void OnScheduledTrigger(
      const IToiSchedulerService::TToiBooking& booking,
      const IToiSchedulerService::TToiDelta& delta)
    throw (TIpcException);

  // This function will be called when one or several bookings are created.
  virtual void OnBookingsAdded(
      const IToiSchedulerService::TToiBookingSequence& booking)
    throw (TIpcException);

  // This function will be called when one or several bookings are removed.
  virtual void OnBookingsRemoved(
      const IToiSchedulerService::TToiBookingSequence& booking)
    throw (TIpcException);

  // This function will be called when one or several bookings are changed.
  virtual void OnBookingsChanged(
      const IToiSchedulerService::TToiBookingSequence& booking)
    throw (TIpcException);

  // This function will be called when a series is changed.
  virtual void OnSeriesChanged(
      const IToiSchedulerService::TToiSeries& booking)
    throw (TIpcException);

};

#endif
