package BootImage::Config;

use strict;
use warnings FATAL => 'all';

use KreaTV;

use POSIX qw(strftime);
use Sys::Hostname;

=head1 NAME

BootImage::Config - A class for parsing boot image configuration file

=head1 SYNOPSIS

    use BootImage::Config;
    my $config = BootImage::Config->new($architecture);
    $config->parse_file($file);

=head1 DESCRIPTION

=head2 Methods

=head3 new

Instanties a new object.

=cut

sub new {
    my ($class, $arch) = @_;
    my $self = bless({}, $class);

    $self->{architecture} = $arch;

    $self->{info} = {};
    $self->{sources} = [];
    $self->{iips} = {};
    $self->{iip_order} = [];

    # Setup default info
    $self->{info}{date} = strftime("%Y%m%d", localtime());
    $self->{info}{time} = strftime("%H%M%S", localtime());
    $self->{info}{host} = hostname();
    $self->{info}{user} = getlogin() || getpwuid($<);
    $self->{info}{branch} = "";
    $self->{info}{version} = "";

    return $self;
}

# Checks if the given string has an architecture list and if it does checks if
# it matches our architecture.
sub _matches_architecture {
    my ($self, $string, $prefix) = @_;

    if ($string =~ /^(.+)\((.*)\)$/) {
        my @targets = split(/,/, $self->_substitute_env_variables($2, $prefix));
        if (grep(/^\s*$self->{architecture}\s*$/i, @targets)) {
            return $1;
        }
        return "";
    }

    return $string;
}

# Replaces %VAR% with the value of the environment variable VAR.
sub _substitute_env_variables {
    my ($self, $string, $prefix) = @_;
    return "" unless $string;

    # (.*?) gives the minimal match
    while ($string =~ /%(.*?)%/) {
        my $value = $ENV{$1} or
            KreaTV::error("${prefix}environment variable '$1' not set");
        $string =~ s/%$1%/$value/g;
        pos($string) = 0;
    }

    return $string;
}

=head3 has_iip

    $config->has_iip("kreatv-test-iip");

Check if a single IIP is in the config.

=cut

sub has_iip {
    my ($self, $name) = @_;
    return defined($self->{iips}{$name}) ? 1 : 0;
}

=head3 add_iip

    $config->add_iip("kreatv-test-iip:params");

Adds a single IIP.

=cut

sub add_iip {
    my ($self, $line, $prefix) = @_;
    $prefix ||= "";
    $line = $self->_substitute_env_variables($line, $prefix);

    # Handle "append" lines (i.e. iip::params)
    if ($line =~ m/^([^:]+?)\s*::\s*(.+)?$/) {
        my $name = $self->_matches_architecture($1, $prefix);
        return 1 unless $name;

        my $params = $2 || "";
        if ($self->{iips}{$name}) {
            $self->{iips}{$name} .= ",$params" if $params;
        }
        else {
            # Only add to iip_order if it doesn't exist, not if it just doesn't
            # have any params set.
            if (!exists($self->{iips}{$name})) {
                push(@{$self->{iip_order}}, $name);
            }
            $self->{iips}{$name} = $params;
        }
    }
    # Handle normal lines, with and without parameters
    elsif ($line =~ m/^([^:]+?)(\s*:\s*(.+)?)?$/) {
        my $name = $self->_matches_architecture($1, $prefix);
        return 1 unless $name;

        my $params = $3 || "";
        if (exists($self->{iips}{$name})) {
            KreaTV::error("${prefix}duplicate entry for IIP '$name'");
        }
        $self->{iips}{$name} = $params;
        push(@{$self->{iip_order}}, $name);
    }
    else {
        return 0;
    }

    return 1;
}

=head3 remove_iip

    $config->remove_iip("kreatv-test-iip");

Removes a single IIP.

=cut

sub remove_iip {
    my ($self, $iip, $prefix) = @_;
    $prefix ||= "";
    $iip = $self->_substitute_env_variables($iip, $prefix);

    if ($iip =~ m/^([^:]+?)(\s*:.*)?$/) {
        my $name = $self->_matches_architecture($1, $prefix);
        if ($name) {
            delete($self->{iips}{$name});
            # Filter out $name from iip_order array
            @{$self->{iip_order}} = grep(!/^$name$/, @{$self->iip_order});
        }
    }
}

=head3 parse_file

    $config->parse_file($config);

Parses the configuration file and adds all IIPs found.

=cut

sub parse_file {
    my ($self, $config) = @_;

    local *CONFIG;
    open(CONFIG, $config) or
        KreaTV::error("could not open configuration file '$config': $!");

    my $longline = "";
    my $lineno = 0;

    while (<CONFIG>) {
        my $line = $_;
        $lineno += 1;

        # Remove space at start and end of line (including CR)
        $line =~ s/^\s+//s;
        $line =~ s/\s+$//s;

        # Skip empty lines
        next if $line eq "";

        my $error_prefix = "$config:$lineno ";

        # Handle comments
        if ($line =~ m/^#/) {
            # Comments can contain info (format: $name:value$)
            if ($line =~ m/\$(.+?):(.+?)\$/) {
                my $name = $self->_substitute_env_variables($1, $error_prefix);
                my $value = $self->_substitute_env_variables($2, $error_prefix);
                $self->info($name, $value);
            }
            next;
        }

        # Handle continuation lines (i.e. lines ending with \)
        if ($line =~ m/\\$/) {
            $line =~ s/\s*\\$//s;
            $longline .= $line;
            next;
        }

        $longline .= $line;
        $line = $longline;
        $longline = "";

        # Handle @include lines
        if ($line =~ m/^\@(include\S*)\s+("?)([^"]*)\2$/) {
            next unless $self->_matches_architecture($1, $error_prefix);

            my $include = $self->_substitute_env_variables($3, $error_prefix);
            if (-f $include) {
                $self->parse_file($include);
            }
            else {
                # Try relative current config file
                my $path = $config;
                if ($path =~ s!^(.*)/[^/]+$!$1! && -f "$path/$include") {
                    $self->parse_file("$path/$include");
                }
                else {
                    KreaTV::error("${error_prefix}could not find include " .
                                  "file '$include'");
                }
            }
        }
        # Handle @search lines
        elsif ($line =~ m/^@(search\S*)\s+("?)([^"]*)\2$/) {
            next unless $self->_matches_architecture($1, $error_prefix);

            my $source = $self->_substitute_env_variables($3, $error_prefix);
            push(@{$self->{sources}}, $source);
        }
        # Any other line starting with @ is an error
        elsif ($line =~ m/^@/) {
            KreaTV::error("${error_prefix}malformed directive '$line'");
        }
        # Handle lines starting with '-'
        elsif ($line =~ m/^-(.+)$/) {
            $self->remove_iip($1, $error_prefix);
        }
        # Handle all the rest
        elsif (!$self->add_iip($line, $error_prefix)) {
            KreaTV::error("${error_prefix}invalid configuration");
        }
    }

    if ($longline ne "") {
        KreaTV::error("$config last line continues past end of file");
    }

    close(CONFIG);
}

=head3 info

    my $info = $config->info("key");
    $config->info("key", "value");
    my $all_info = $config->info();

Gets or sets a specific info, or gets all info as a hash reference.

=cut

sub info {
    my $self = shift;
    if (@_) {
        my $key = shift;
        if (@_) { $self->{info}->{$key} = shift; }
        return $self->{info}->{$key};
    }
    return $self->{info};
}

=head3 sources

    my $sources = $config->sources();
    foreach my $dir (@$sources) {
        ...
    }

Gets all sources given in the config file as an array reference.

=cut

sub sources {
    my $self = shift;
    return $self->{sources};
}

=head3 iips

    my $iips = $config->iips();
    while (my ($iip, $params) = each(%$iips)) {
        ...
    }

Gets all IIPs and parameters as a hash reference.

=cut

sub iips {
    my $self = shift;
    return $self->{iips};
}

=head3 iip_order

    my $iips = $config->iips();
    my $order = $config->iip_order();
    foreach my $name (@$order) {
        my $params = $iips->{$name};
        ...
    }

Gets IIP names in the order they were given in the config.

=cut

sub iip_order {
    my $self = shift;
    return $self->{iip_order};
}

=head3 config

Returns the processed config as a string.

=cut

sub config {
    my $self = shift;

    my $str = "# Info\n";
    my $info = $self->info();
    foreach my $key (sort(keys(%$info))) {
        $str .= "# \$$key:" . $info->{$key} . "\$\n";
    }

    $str .= "\n# Sources\n";
    $str .= "\@search \"$_\"\n" foreach @{$self->sources()};

    $str .= "\n# IIPs\n";
    my $iips = $self->iips();
    foreach my $iip (sort(keys(%$iips))) {
        my $params = $iips->{$iip};
        if ($params) {
            $str .= "$iip:$params\n";
        }
        else {
            $str .= "$iip\n";
        }
    }

    return $str;
}

1;
