package IIP;

use strict;
use warnings FATAL => 'all';
use Tie::File;

use KreaTV;

sub _add_platform_service;
sub _validate_arguments;
sub _check_parameter_has_value;

### Public functions ###

# Adds a platform service at the end of the file.
sub add_platform_service_last {
    my $rootdisk = shift;
    my %args = @_;

    my $do_add = sub {
        my ($file, $xml) = @_;
        my $replace = qr/(<\/SysmanConfig>)/m;
        KreaTV::replace_in_file($file, $replace, sub { "$xml\n$1" });
    };

    _add_platform_service($rootdisk, \%args, $do_add, 1);
}

# Adds a platform service that has a placeholder.
sub add_platform_service_placeholder {
    my $rootdisk = shift;
    my %args = @_;

    my $do_add = sub {
        my ($file, $xml) = @_;
        my $replace = qr/^.*<!-- Placeholder.*name="$args{name}".*-->.*$/m;
        KreaTV::replace_in_file($file, $replace, $xml);
    };

    _add_platform_service($rootdisk, \%args, $do_add, 0);
}

# Adds a platform service before an existing service.
sub add_platform_service_before {
    my $rootdisk = shift;
    my $before = shift;
    my %args = @_;

    my $do_add = sub {
        my ($file, $xml) = @_;
        my $replace = qr/(  <Program.*name="$before")/m;
        KreaTV::replace_in_file($file, $replace, sub { "$xml\n$1" });
    };

    _add_platform_service($rootdisk, \%args, $do_add, 1);
}

# Copy everything in current directory to rootdisk.
sub copy_data_to_rootdisk {
    my $rootdisk = shift;
    KreaTV::system("cp -a . \"$rootdisk\"");
}

# Parses parameters given to an IIP and returns a hash with key/value.
#
# $params may be string with comma-separated parameters or a reference to an
# array with parameters.
#
# $optional together with $required contains all valid parameter names. If
# $optional is undefined, all parameters are valid. $required is an array with
# the name of all required parameters. If undefined no parameters are required.
#
# $multivalued is an array with parameter names where multiple values are
# allowed. The return value will have an array with values for all parameters
# listed here.
sub parse_parameters {
    my ($params, $optional, $required, $multivalued) = @_;
    $multivalued ||= [];
    my @parameters;

    if (ref($params) eq "ARRAY") {
        @parameters = @$params;
    }
    else {
        @parameters = split(/,/, $params);
    }

    my $multi = join("|", @$multivalued);
    my $multire = qr/^($multi)$/;

    my $res = {};
    foreach my $param (@parameters) {
        my ($key, $value) = split(/=/, $param, 2);
        if ($key =~ $multire) {
            $res->{$key} ||= [];
            push(@{$res->{$key}}, $value);
        }
        else {
            if (exists($res->{$key})) {
                KreaTV::error("duplicate parameter '$key'");
            }
            $res->{$key} = $value;
        }
    }

    _validate_arguments($res, $optional, $required);

    return $res;
}

# Returns the integer value for the parameter $name. $params is a hash
# reference returned from parse_parameters. The optional $options is a hash
# reference hash with the following possible entries:
#
# - default: default value to return if the parameter is undefined or ""
# - max: maximum value that the integer may have (inclusive)
# - min: minimum value that the integer may have (inclusive)
#
# If no default value is given, KreaTV::error is called.
sub get_int_parameter {
    my ($params, $name, $options) = @_;
    if (exists($options->{default}) && (!defined($params->{$name})
                                        || $params->{$name} eq "")) {
        return $options->{default};
    }
    _check_parameter_has_value($params, $name);

    if ($params->{$name} !~ /[0-9]+/) {
        KreaTV::error("value for parameter '$name' is not a number");
    }
    my $value = int($params->{$name});
    if (defined($options->{min}) && $value < $options->{min}) {
        KreaTV::error("value for parameter '$name' is too small ($value < $options->{min})");
    }
    if (defined($options->{max}) && $value > $options->{max}) {
        KreaTV::error("value for parameter '$name' is too large ($value > $options->{max})");
    }
    return $value;
}

# Returns a boolean value for the parameter $name. true, yes and 1 are
# considered true and all other values are considered false. $params is a hash
# reference returned from parse_parameters. The optional $options is a hash
# reference hash with the following possible entries:
#
# - default: default value to return if the parameter is undefined or ""
#
# If no default value is given, KreaTV::error is called.
sub get_bool_parameter {
    my ($params, $name, $options) = @_;
    if (exists($options->{default}) && (!defined($params->{$name})
                                        || $params->{$name} eq "")) {
        return $options->{default};
    }
    _check_parameter_has_value($params, $name);
    return (lc($params->{$name}) =~ /^true|yes|1$/ ? 1 : 0);
}

# Adds the information object to the information service with the given value.
# IIP must also have the dependency: finalize:kreatv-finalize-is-default
sub add_infobroker_object {
    my ($rootdisk, $object, $value) = @_;
    KreaTV::write_file_content("$rootdisk/tmp/.rootdisk/is-default/$object",
                               $value);
}

# Checks if a conflicting IIP is installed and aborts if that is the case.
sub check_conflicting_iips {
    my ($rootdisk, @conflicting_iips) = @_;
    my $iips = KreaTV::read_file_content("$rootdisk/tmp/.rootdisk/full_config");
    foreach my $iip (@conflicting_iips) {
        if ($iips =~ /^$iip/m) {
            KreaTV::error("found conflicting IIP '$iip'");
        }
    }
}

### Internal functions ###

# Internal help function for the different add_platform_service_* variants.
# Adds platform service to $rootdisk/etc/processlist.xml.
sub _add_platform_service {
    my ($rootdisk, $args, $do_add, $should_remove) = @_;

    my $optional = [ "starttime", "endtime", "dynamic" ];
    my $required = [ "name", "run" ];
    _validate_arguments($args, $optional, $required, "add_platform_service: ");

    # File to update.
    my $processlist = "$rootdisk/etc/processlist.xml";

    # Remove any existing lines.
    if ($should_remove) {
        my $remove = qr/\s*<Program.*name="$args->{name}".*\/>/;
        KreaTV::replace_in_file($processlist, $remove, '');
    }

    # Create xml string.
    my $xml = "  <Program";
    for my $key (sort keys %$args) {
        $xml .= " $key=\"$args->{$key}\"";
    }
    $xml .= "/>";

    # Add xml string to file.
    $do_add->($processlist, $xml);

    print("Added platform service '$args->{name}'\n");
}

sub _validate_arguments {
    my $args = shift;
    my $optional = shift;
    my $required = shift || [];
    my $prefix = shift || '';

    # If optional isn't set we allow all arguments.
    if ($optional) {
        my $list = join("|", @$required, @$optional);
        my $regex = qr/^($list)$/;

        # Check that all arguments are valid.
        foreach (keys %$args) {
            unless($_ =~ $regex) {
                KreaTV::error("${prefix}unknown argument '$_'");
            }
        }
    }

    # Check that all required arguments are there.
    foreach (@$required) {
        if (!exists($args->{$_})) {
            KreaTV::error("${prefix}missing required argument '$_'");
        }
    }
}

# Updates the KreaTV configuration file with the given parameters
sub update_kreatv_config {
  my $rootdisk = shift;
  my $parameters = shift;

  my @lines;
  KreaTV::system("mkdir -p $rootdisk/etc/");
  my $config = "$rootdisk/etc/kreatv.conf";
  tie(@lines, 'Tie::File', $config) or die "Could not open $config\n";

  # Update existing entries
  for(@lines) {
    if (my $key = /^\s*(\w+)\s*=\s*\w+$/) {
      if (exists($parameters->{$key})) {
      	$_ = "$key=$parameters->{$key}\n";
	delete($parameters->{$key});
      }
    }
  }

  # Add new entries
  while (my ($key, $value) = each(%$parameters)) {
    push(@lines, "$key=$value\n");
  }

  untie(@lines);
}

sub _check_parameter_has_value {
    my ($params, $name) = @_;
    if (!defined($params->{$name}) || $params->{$name} eq "") {
        KreaTV::error("no value given for parameter '$name'");
    }
}

1;
