/*
 *--------------------------------------------------------------------
 *
 * TBasicSourceElement.cpp
 *
 * Base class for basic source elements. This can be used for simple
 * source elements, i.e. file / unicast / multicast etc.
 * 
 * Copyright (c) 2005 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#include "TBasicSourceElement.h"

#include "media/streamer/IPump.h"
#include "media/streamer/IOutputPad.h"
#include "media/streamer/TMetadata.h"
#include "media/streamer/ParameterNames.h"

#include <cstdlib>
#include <cstring>
#include <cerrno>


TBasicSourceElement::TBasicSourceElement() throw ()
  : RegisteredDescriptor(-1),
    FileDescriptor(-1),
    DataFlowFlag(false),
    Buffer(NULL),
    BufferSize(0),
    ReadCount(0),
    Pump(NULL),
    OutputPad(NULL),
    EventEngine(NULL),
    SourceCallback(NULL),
    Blackboard(NULL)
{
  // Empty
}

TBasicSourceElement::~TBasicSourceElement() throw ()
{
  // Make sure to call Cleanup in derived classes destructors!
  UnregisterDescriptor();
}


// ----------------------------------------------------------------------------
// Protected functions

inline void TBasicSourceElement::RegisterDescriptor(int descriptor) throw ()
{
  if (RegisteredDescriptor == -1) {
    EventEngine->AddFileDescriptor(descriptor, IEvent::EVENT_READ);
    RegisteredDescriptor = descriptor;
  }
  else {
    // This code is designed for only one descriptor to be registered
    // at a time.
    assert(descriptor == RegisteredDescriptor);
  }
}

inline void TBasicSourceElement::UnregisterDescriptor() throw ()
{
  if (RegisteredDescriptor != -1) {
    EventEngine->RemoveFileDescriptor(RegisteredDescriptor);
    RegisteredDescriptor = -1;
  }
}

// Returns false when there wasn't data available to read
// (non-blocking I/O).
bool TBasicSourceElement::DoProcess() throw ()
{
  int rc;
  while (true) {
    rc = Read(Buffer + ReadCount, BufferSize - ReadCount);
    // Would block or EOS?
    if (rc == -1 || rc == 0) {
      break;
    }
    ReadCount += rc;
    if (IsBufferFilled()) {
      break;
    }
  }

  if (rc == -1) {
    // EAGAIN indicates "would block".
    if (errno != EAGAIN) {
      Debug->Error("Read returned errno = %d", errno);
      ::abort();
    }
    return false;
  }
  else {
    // Enough data collected to commit the buffer.
    CommitDataBuffer();
    UnregisterDescriptor();
    if (rc == 0) {
      // End of stream reached. Go to STATE_PAUSED!
      SourceCallback->OnStateChanged(IStreamerTypes::STATE_PAUSED);
      DataFlowFlag = false;
    }
    return true;
  }
}

void TBasicSourceElement::DoFlush() throw ()
{
  // Push any collected data on the pad!
  if (Buffer != NULL) {
    CommitDataBuffer();
  }
  UnregisterDescriptor();
}

void TBasicSourceElement::CommitDataBuffer() throw ()
{
  // Commit whatever we have collected and release the rest.
  if (ReadCount > 0) {
    // Observe! Unframed data (single frame with full focus).
    OutputPad->CommitSegment(Buffer, 1, ReadCount, 0, ReadCount);
  }
  if (ReadCount < BufferSize) {
    Pump->ReleaseSegment(Buffer + ReadCount, 1, BufferSize - ReadCount);
  }
  Buffer = NULL;
  BufferSize = 0;
  ReadCount = 0;
}

void TBasicSourceElement::CommitConnectionType() throw ()
{
  TMetadata* metadata = Pump->CreateMetadata();
  metadata->Set(PARAM_CONNECTION_TYPE, GetConnectionType());
  OutputPad->CommitMetadata(metadata);
}

void TBasicSourceElement::Failure() throw ()
{
  // Go to STATE_FAILED if some unsupported function is called.
  SourceCallback->OnStateChanged(IStreamerTypes::STATE_FAILED);
  SourceCallback->OnCommandResponse(false);
  DataFlowFlag = false;
  Cleanup();
}


// ----------------------------------------------------------------------------
// IElement

void TBasicSourceElement::Destroy() throw ()
{
  delete this;
}

void TBasicSourceElement::Initialize(IPump* pump,
                                     const char* /*dataFormat*/) throw ()
{
  Pump = pump;
  // Source elements should always open a single pad with "Raw"
  // format.
  OutputPad = Pump->OpenPad("Raw");
  assert(OutputPad != NULL);

  EventEngine = static_cast<IEvent*>(pump->QueryInterface("IEvent"));
  Debug = static_cast<IDebug*>(Pump->QueryInterface("IDebug"));
  SourceCallback = (static_cast<ISourceCallback*>
                    (pump->QueryInterface("ISourceCallback")));
  Blackboard = static_cast<IBlackboard*>(Pump->QueryInterface("IBlackboard"));
}

void* TBasicSourceElement::QueryInterface(const char* interfaceName) throw ()
{
  // Return interfaces that are supported.
  if (strcmp(interfaceName, "IRoot") == 0) {
    return static_cast<IRoot*>(this);
  }
  if (strcmp(interfaceName, "ISource") == 0) {
    return static_cast<ISource*>(this);
  }
  if (strcmp(interfaceName, "IEventCallback") == 0) {
    return static_cast<IEventCallback*>(this);
  }
  return NULL;
}


// ----------------------------------------------------------------------------
// IEventCallback

void TBasicSourceElement::OnEvent(int /*fd*/, uint32_t /*eventMask */) throw ()
{
  // Now we are able to read data on the file descriptor without blocking!
  DoProcess();
}

void TBasicSourceElement::OnTimeout(int /*timerId*/) throw ()
{
  // Should never happen!
  assert(false);
}


// ----------------------------------------------------------------------------
// IRoot

// Streamer core is alloting the element new buffer space.
void TBasicSourceElement::Process(uint8_t* buffer, uint32_t size) throw ()
{
  Buffer = buffer;
  BufferSize = size;

  // If data is in "flowing mode" we should process immediately.
  if (DataFlowFlag) {
    if (!DoProcess()) {
      // If there weren't enough data available we register the file
      // descriptor in the event loop to read more later on.
      RegisterDescriptor(FileDescriptor);
    }
  }
}

void TBasicSourceElement::Flush() throw ()
{
  // Flush means that any collected data should be committed and that
  // the allotted buffer space should be returned.
  DoFlush();
}


// ----------------------------------------------------------------------------
// ISource

void TBasicSourceElement::Open(const char* uri) throw()
{
  CommitConnectionType();

  if (!Connect(uri)) {
    SourceCallback->OnStateChanged(IStreamerTypes::STATE_FAILED);
    SourceCallback->OnCommandResponse(false);
  }
  else {
    SourceCallback->OnStateChanged(IStreamerTypes::STATE_PAUSED);
    SourceCallback->OnCommandResponse(true);
  }
}

void TBasicSourceElement::Close() throw()
{
  SourceCallback->OnStateChanged(IStreamerTypes::STATE_CLOSED);
  SourceCallback->OnCommandResponse(true);
  DataFlowFlag = false;
  Cleanup();
}


void TBasicSourceElement::Play(int32_t pace) throw ()
{
  if (pace == TBasicSourceElement::PACE_PLAY) {
    DataFlowFlag = true;
    // Handle possibility that we received buffer space through
    // Process() before Play() was called.
    if (Buffer != NULL) {
      RegisterDescriptor(FileDescriptor);
    }
    
    SourceCallback->OnStateChanged(IStreamerTypes::STATE_PLAYING);
    SourceCallback->OnCommandResponse(true);
  }
  else {
    Failure();    
  }
}

void TBasicSourceElement::PlayFromPosition(uint32_t /*position*/,
                                           int32_t /*pace*/) throw ()
{
  // PlayFromPosition() is unsupported.
  Failure();
}
