/*
 *--------------------------------------------------------------------
 *
 * TMulticastSourceElement.cpp
 *
 * A Streamer Source Element reading from a multicast socket.
 *
 * Copyright (c) 2005 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#include "TMulticastSourceElement.h"

#include <unistd.h>
#include <fcntl.h>
#include <cerrno>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <cassert>
#include <cstring>
#include <cstdio>


// ----------------------------------------------------------------------------
// Constructor and destructor

TMulticastSourceElement::TMulticastSourceElement() throw ()
  : JoinedFlag(false)
{
  // Empty.
}

TMulticastSourceElement::~TMulticastSourceElement() throw ()
{
  Cleanup();
}


// ----------------------------------------------------------------------------
// Private functions

void TMulticastSourceElement::OpenSocket() throw ()
{
  assert(FileDescriptor == -1);
  FileDescriptor = ::socket(PF_INET, SOCK_DGRAM, 0);
}

bool TMulticastSourceElement::CloseSocket() throw ()
{
  if (FileDescriptor == -1) {
    return true;
  }
  
  int result;
  while (true) {
    result = ::close(FileDescriptor);
    if (!(result == -1 && errno == EINTR)) {
      break;
    }
  }
  if (result == 0) {
    FileDescriptor = -1;
    return true;
  }
  return false;
}


bool TMulticastSourceElement::
JoinChannel(const char* address, uint16_t port) throw ()
{
  if (JoinedFlag) {
    return false;
  }

  sockaddr_in addr;
  ::memset(&addr, 0, sizeof(addr));
  addr.sin_family = AF_INET;
  addr.sin_port = htons(port);
  if (::inet_aton(address, &addr.sin_addr) == 0) {
    return false;
  }
  
  // Local address chosen by kernel (INADDR_ANY)
  MulticastRequest.imr_interface.s_addr = htonl(INADDR_ANY);
  MulticastRequest.imr_multiaddr = addr.sin_addr;
  
  // Set REUSEADDR to allow other clients to listen to same channel.
  // Must be done before bind.
  int reuse = 1;
  if (::setsockopt(FileDescriptor, SOL_SOCKET, SO_REUSEADDR,
                   &reuse, sizeof(int)) == -1) {
    return false;
  }
  
  if (::bind(FileDescriptor, reinterpret_cast<sockaddr*>(&addr),
             sizeof(addr)) == -1) {
    return false;
  }
  
  if (::setsockopt(FileDescriptor, IPPROTO_IP, IP_ADD_MEMBERSHIP,
                   &MulticastRequest, sizeof(MulticastRequest)) == -1) {
    return false;
  }
  JoinedFlag = true;
  return true;
}


bool TMulticastSourceElement::LeaveChannel() throw ()
{
  if (JoinedFlag) {
    JoinedFlag = false;
    return (::setsockopt(FileDescriptor, IPPROTO_IP, IP_DROP_MEMBERSHIP,
                         (char*)&MulticastRequest,
                         sizeof(MulticastRequest)) == 0);
  }
  return false;
}


bool TMulticastSourceElement::Connect(const std::string& uri) throw ()
{
  int schemeLength = 0;
  if (uri.find("igmp://") == 0) {
    schemeLength = 7;
  }
  uint32_t pos = uri.find(":", schemeLength);
  if (pos == std::string::npos) {
    return false;
  }

  std::string ipAddress = uri.substr(schemeLength, pos - schemeLength);
  uint32_t port;
  ::sscanf(uri.c_str() + pos + 1, "%u", &port);

  Debug->Debug("Connecting to multicast source %s:%u",
               ipAddress.c_str(), port);

  OpenSocket();

  // Set receive buffer size of socket.
  uint32_t size = SOCKET_BUFFER_SIZE;
  ::setsockopt(FileDescriptor, SOL_SOCKET, SO_RCVBUF, &size, sizeof(uint32_t));

  // Join the channel.
  bool result = JoinChannel(ipAddress.c_str(), port);
  if (result) {
    // Set non-blocking I/O!!!
    int flags = ::fcntl(FileDescriptor, F_GETFL);
    if (flags != -1) {
      result = (::fcntl(FileDescriptor, F_SETFL, flags | O_NONBLOCK) == 0);
    }
    else {
      result = false;
    }
  }
  return result;
}


void TMulticastSourceElement::Cleanup() throw ()
{
  if (FileDescriptor >= 0) {
    Debug->Debug("Disconnecting multicast source");
    LeaveChannel();
    UnregisterDescriptor();
    CloseSocket();
  }
}

int TMulticastSourceElement::Read(uint8_t* buffer, uint32_t size) throw ()
{
  while (true) {
    int result = ::recvfrom(FileDescriptor, buffer, size,
                            MSG_NOSIGNAL, NULL, NULL);
    if (!(result == -1 && errno == EINTR)) {
      return result;
    }
  }
}

bool TMulticastSourceElement::IsBufferFilled() throw ()
{
  // We always read at least PROCESS_BUFFER_SIZE - MAX_UDP_SIZE before
  // committing any data. 
  return (ReadCount >= (PROCESS_BUFFER_SIZE - MAX_UDP_SIZE));
}

const char* TMulticastSourceElement::GetConnectionType() throw ()
{
  return "Multicast";
}


// ----------------------------------------------------------------------------
// IRoot

uint32_t TMulticastSourceElement::GetBufferRequirement() throw ()
{
  return PROCESS_BUFFER_SIZE;
}


// ----------------------------------------------------------------------------
// ISourceElement

bool TMulticastSourceElement::IsFeatureSupported(const char* feature) throw ()
{
  // The data arrives in the same pace as it should be consumed. We
  // indicate this to the Streamer core with the "InputClocked"
  // feature.
  if (::strcmp(feature, "InputClocked") == 0) {
    return true;
  }
  return false;
}

void TMulticastSourceElement::Play(int32_t pace) throw ()
{
  if (pace == TBasicSourceElement::PACE_PLAY) {
    // Pop all datagrams from the socket input buffer to avoid playing
    // something that has been queued up a long time ago and then jump
    // to more current data if bufer has been overflowed.
    while (true) {
      uint8_t buffer;
      int result = Read(&buffer, 1);
      if (result != 1) {
        break;
      }
    }
    TBasicSourceElement::Play(pace);
  }
  else {
    Failure();
  }
}
