/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *--------------------------------------------------------------------
 *
 * TExampleObject.cpp
 *
 * This file contains implementation for NPObject wrapper and
 * for its class interface
 *
 * Copyright (c) 2010 Motorola Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#include <string>
#include "ExampleInternal.h"
#include "TExamplePlugin.h"
#include "TExampleObject.h"

extern NPClass RuntimePluginClass;

// small utility function
std::string ToString(NPIdentifier pointer) {
  NPUTF8 *text = NPN_UTF8FromIdentifier(pointer);
  std::string name(text);
  NPN_MemFree(text);
  return name;
}

TExampleObject* GetTExampleObjectByNPObject(NPObject *obj)
{
  if (NULL == obj) {
    TRACE("zero pointer");
    return NULL;
  }
  else if (obj->_class != &RuntimePluginClass) {
    TRACE("object represents wrong class %p", obj->_class);
    return NULL;
  }
  TExampleObject* p = static_cast<TExampleObject*>(obj);
  return p;
}

void NPDeallocate(NPObject *obj)
{
  TExampleObject *o = GetTExampleObjectByNPObject( obj );
  if (NULL == o) {
    TRACE("obj=%p zero pointer",obj);
    return;
  }
  if (o->isValid_) {
    TRACE("obj=%p is valid on deallocation",obj);
  }
  delete o;
  TRACE("obj=%p deleted",obj);
}

void NPInvalidate(NPObject *obj)
{
  TExampleObject* o = GetTExampleObjectByNPObject(obj);
  if (NULL == o) {
    TRACE("obj=%p zero pointer",obj);
    return;
  }
  if (!o->isValid_) {
    TRACE("obj=%p already invalid",obj);
    return;
  }
  o->isValid_=false;
  TRACE("obj=%p invalidated",obj);
}

bool NPHasMethod(NPObject *obj, NPIdentifier name)
{
  std::string text(ToString(name));
  TExampleObject* o = GetTExampleObjectByNPObject(obj);
  if (NULL == o) {
    TRACE("obj=%p,identifier=%s false (zero pointer)",obj,text.c_str());
    return false;
  }
  if (!o->isValid_) {
    TRACE("obj=%p,identifier=%s false (not valid)",obj,text.c_str());
    return false;
  }
  if (o->owner_->HasMethod(text)) {
    TRACE("obj=%p,identifier=%s true (our object)",obj,text.c_str());
    return true;
  }
  TRACE("obj=%p,identifier=%s false (nobody)",obj,text.c_str());
  return false;
}

bool NPInvoke(NPObject *obj, NPIdentifier name, const NPVariant * args,
              uint32_t argCount, NPVariant* result)
{
  std::string text(ToString(name));
  TRACE("obj=%p,identifier=%s entering",obj,text.c_str());
  TExampleObject* o = GetTExampleObjectByNPObject( obj );
  if (NULL == o) {
    TRACE("obj=%p,identifier=%s false (zero pointer)",obj,text.c_str());
    return false;
  }
  if (!o->isValid_) {
    TRACE("obj=%p,identifier=%s false (not valid)",obj,text.c_str());
    return false;
  }
  bool r = o->owner_->Invoke(text,args,argCount,result);
  TRACE("obj=%p,identifier=%s %d (our object)",
        obj,text.c_str(),(int)r);
  return r;
}

bool NPInvokeDefault(NPObject *obj, const NPVariant *args, uint32_t argCount,
                     NPVariant *result)
{
  TExampleObject* o = GetTExampleObjectByNPObject( obj );
  TRACE("obj=%p entering",obj);
  if (NULL == o) {
    TRACE("obj=%p false (zero pointer)",obj);
    return false;
  }
  if (!o->isValid_) {
    TRACE("obj=%p false (not valid)",obj);
    return false;
  }
  bool r = o->owner_->InvokeDefault(args,argCount,result);
  TRACE("obj=%p, %d (our object)",obj,(int)r);
  return r;
}

bool NPHasProperty(NPObject *obj, NPIdentifier name)
{
  std::string text(ToString(name));
  TRACE("obj=%p,identifier=%s",obj,text.c_str());
  TExampleObject* o = GetTExampleObjectByNPObject( obj );
  if (NULL == o) {
    TRACE("obj=%p,identifier=%s false (zero pointer)",obj,text.c_str());
    return false;
  }
  if (!o->isValid_) {
    TRACE("obj=%p,identifier=%s false (not valid)",obj,text.c_str());
    return false;
  }
  if (NULL != o->owner_ && o->owner_->HasProperty(text)) {
    TRACE("obj=%p,identifier=%s true (our object)",obj,text.c_str());
    return true;
  }
  TRACE("obj=%p,identifier=%s false (nobody)",obj,text.c_str());
  return false;
}

bool NPGetProperty(NPObject *obj, NPIdentifier name, NPVariant *result)
{
  std::string text(ToString(name));
  TExampleObject *o = GetTExampleObjectByNPObject(obj);
  if (NULL == o) {
    TRACE("obj=%p,identifier=%s false (zero pointer)",obj,text.c_str());
    return false;
  }
  if (!o->isValid_) {
    TRACE("obj=%p,identifier=%s false (not valid)",obj,text.c_str());
    return false;
  }
  if (NULL != o->owner_ && o->owner_->HasProperty(text)) {
    bool r = o->owner_->GetProperty(text,result);
    TRACE("obj=%p,identifier=%s %d (our object)",
          obj,text.c_str(),(int)r);
    return r;
  }
  TRACE("obj=%p,identifier=%s false (nobody)",obj,text.c_str());
  return false;
}

bool NPSetProperty(NPObject *obj, NPIdentifier name,
                   const NPVariant* value)
{
  std::string text(ToString(name));
  TExampleObject *o = GetTExampleObjectByNPObject(obj);
  if (NULL == o) {
    TRACE("obj=%p,identifier=%s false (zero pointer)",obj,text.c_str());
    return false;
  }
  if (!o->isValid_) {
    TRACE("obj=%p,identifier=%s false (not valid)",obj,text.c_str());
    return false;
  }
  if (o->owner_->SetProperty(text,value)) {
    TRACE("obj=%p,identifier=%s true (our object)",obj,text.c_str());
    return true;
  }
  TRACE("obj=%p,identifier=%s false (nobody)",obj,text.c_str());
  return false;
}

bool NPRemoveProperty(NPObject *obj, NPIdentifier name)
{
  std::string text(ToString(name));
  TExampleObject *o = GetTExampleObjectByNPObject( obj );
  if (NULL == o) {
    TRACE("obj=%p,identifier=%s false (zero pointer)",obj,text.c_str());
    return false;
  }
  if (!o->isValid_) {
    TRACE("obj=%p,identifier=%s false (not valid)",obj,text.c_str());
    return false;
  }
  bool r = o->owner_->RemoveProperty(text);
  TRACE("obj=%p,identifier=%s %d (our object)",
        obj,text.c_str(),(int)r);
  return r;
}

NPClass RuntimePluginClass = {
  NP_CLASS_STRUCT_VERSION,
  NULL,
  NPDeallocate,
  NPInvalidate,
  NPHasMethod,
  NPInvoke,
  NPInvokeDefault,
  NPHasProperty,
  NPGetProperty,
  NPSetProperty,
  NPRemoveProperty,
  NULL, // NPEnumeration -- do not provide enumeration
  NULL //NPConstruct -- nobody can construct
};

TExampleObject::TExampleObject(TExamplePlugin* owner)
  : owner_(owner),isValid_(true)
{
  assert( NULL != owner && "owner class pointer cannot be zero" );
  _class = &RuntimePluginClass;
  // 1 - because plugin will receive this object referenced
  referenceCount = 1;
  TRACE("created");
}

TExampleObject::~TExampleObject()
{
  isValid_ = false;
  TRACE("DESTROYED");
}

