/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *--------------------------------------------------------------------
 *
 * TExamplePlugin.cpp
 *
 * This file contains implementation for example runtime plugin
 *
 * Copyright (c) 2010 Motorola Inc. All Rights Reserved.
 * Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
 *
 * This program is confidential and proprietary to Motorola Mobility, Inc and
 * may not be copied, reproduced, disclosed to others, published or used, in
 * whole or in part, without the expressed prior written permission of Motorola
 * Mobility, Inc.
 *
 *--------------------------------------------------------------------
 */

#include <unistd.h>
#include <sys/time.h>
#include <cstring>
#include <cassert>
#include <cstdio>
#include <cmath>
#include <memory>
#include <set>
#include <boost/lexical_cast.hpp>
#include <boost/bind.hpp>
#include <boost/utility/value_init.hpp>
#include <interface/TNameServiceCaller.h>
#include <interface/ToiObjectNames.h>
#include "TExamplePlugin.h"
#include "TExampleObject.h"

std::string VariantToString(const NPVariant& variant)
{
  std::string result;
  switch (variant.type) {
  case NPVariantType_Void:
    result = "void";
    break;
  case NPVariantType_Null:
    result = "null";
    break;
  case NPVariantType_Bool:
    result = std::string("bool ")
             +std::string(variant.value.boolValue?"true":"false");
    break;
  case NPVariantType_Int32:
    result = std::string("int ")
             +boost::lexical_cast<std::string>(variant.value.intValue);
    break;
  case NPVariantType_Double:
    result = std::string("double ")
             +boost::lexical_cast<std::string>(variant.value.doubleValue);
    break;
  case NPVariantType_String:
    result = std::string("string ")
             +std::string(variant.value.stringValue.UTF8Characters,
                          variant.value.stringValue.UTF8Length).c_str();
    break;
  case NPVariantType_Object:
    result = std::string("object ")
             +boost::lexical_cast<std::string>(variant.value.objectValue);
    break;
  default:
    result = "unknown";
    break;
  }
  return result;
}

void DumpArguments(int count,const NPVariant* array __attribute__ ((unused)))
{
  for (int i=0;i<count;++i) {
    TRACE("\t%d - %s",i+1,VariantToString(array[i]).c_str());
  }
}

TExamplePlugin::TExamplePlugin(NPP instance)
  : parent_(instance), window_(NULL),
    //    renderer_(instance),
  methodCaller_(new TDeferredMethodCaller(eventLoop_)),
  ipcEventHandler_(&eventLoop_),
  observerName_(ipcEventHandler_.CreateObject()),
  informationObserver_(&ipcEventHandler_,observerName_,this),
  exampleEventHandle_(-1),lastPageUpdate_(0),lastFlashUpdate_(0),
  transparency_(100.0),
  animate(true)
{
  TRACE("created, npp=%p",instance);

  // instantiating IS environment
  std::string nameServiceAddress(getenv("NAMESERVICE_ADDRESS"));
  TNameServiceCaller ns(&ipcClient_, nameServiceAddress);

  // registering names
  std::string infobrokerAddress;
  ns.LookupObject(TOI_INFORMATION_SERVICE_NAME, infobrokerAddress);

  informationCaller_.reset(new
                           TToiInformationServiceCaller(&ipcClient_,
                                                        infobrokerAddress));

  // Must initialise this before getting NPNVPluginElementNPObject, as it'
  // call back into our GetValue method and require a valid plugin.
  parent_->pdata = this;

  // Say that we're a windowless plugin.
  NPN_SetValue(parent_, NPPVpluginWindowBool, false);

  TRACE("main thread is %8x",static_cast<unsigned int>(pthread_self()));

  // setting ourself as event handler
  // 1st step - get document
  TRACE("parent is %p, intsance is %p",parent_,instance);

  // timer values
  timeval now;
  gettimeofday(&now,NULL);
  baseTime_ = now.tv_sec;

#ifdef PLAIN_MODE
  // getting underlying object
  NPObject* substrate = NULL;
  NPN_GetValue(parent_, NPNVPluginElementNPObject, &substrate);
  assert(NULL != substrate && "browser returned zero as underlying object");

  TSharedVariant styleVariant =
    ObjectGetProperty(parent_, substrate,
                      NPN_GetStringIdentifier("style"));
  NPN_ReleaseObject(substrate);
  assert(styleVariant && "'style' returnd as zero");
  assert(NPVARIANT_IS_OBJECT(*styleVariant) && "'style' is not an object");
  style_.reset(NPVARIANT_TO_OBJECT(*styleVariant));
#endif

  TExampleObject *d = new TExampleObject(this);
  TNPObject(static_cast<NPObject*>(d),false).swap(defaultObject_);

  // starting event loop for correct event handling
  eventLoop_.AddTimer(100, this);
  pthread_create(&thread, NULL, ThreadEntry, this);
  StartExample();
}

TExamplePlugin::~TExamplePlugin()
{
  StopExample();
  eventLoop_.Stop();
  pthread_join(thread, NULL);
  if (defaultObject_
      && NULL != defaultObject_->_class
      && NULL != defaultObject_->_class->invalidate) {
    defaultObject_->_class->invalidate(defaultObject_.get());
  }
  TRACE("destroy completed");
}

NPError TExamplePlugin::SetWindow(NPWindow* window)
{
  TRACE();
  window_ = window;
  renderer_.SetWindow(window);
  if (NULL!=window) {
    TRACE("window: win=%p %d,%d %dx%d\n",
           static_cast<void*>(window),
           static_cast<int>(window->x),
           static_cast<int>(window->y),
           static_cast<int>(window->width),
           static_cast<int>(window->height));
  }
  return NPERR_NO_ERROR;
}

int16_t TExamplePlugin::HandleEvent(void* event)
{
  TRACE("");
  ekioh::EKPluginEvent* ekEvent = static_cast<ekioh::EKPluginEvent*>(event);
  if (ekioh::EKPluginEvent::Paint == ekEvent->type) {
    TRACE("Paint event received.");
    renderer_.TestPaint(*ekEvent);
    return true;
  }
  return false;
}

NPError TExamplePlugin::GetValue(NPPVariable variable, void *ret_value)
{
  switch (variable) {
  // plugin name
  case NPPVpluginNameString:
    *((char const **)ret_value) = "TExamplePlugin";
    TRACE("returning name");
    break;
  // plugin description
  case NPPVpluginDescriptionString:
    *((char const **)ret_value) = "Motorola Example plugin";
    TRACE("returning descr");
    break;
    /* Get the NPObject for scripting the plugin. */
  case NPPVpluginScriptableNPObject:
    ++(defaultObject_->referenceCount);
    *((NPObject **)ret_value) = defaultObject_.get();
    TRACE("returning object, counter is %d", defaultObject_->referenceCount);
    break;
  default:
    TRACE("event type %d is not supported",variable);
    return NPERR_GENERIC_ERROR;
  }
  return NPERR_NO_ERROR;
}

NPError
TExamplePlugin::SetValue(NPNVariable variable __attribute__ ((unused)),
                         void *value __attribute__ ((unused)))
{
  TRACE("variable=%d,&value=%p",variable,value);
  return NPERR_GENERIC_ERROR;
}

bool TExamplePlugin::InvokeDefault(const NPVariant * /*args*/,
                                   uint32_t /*argCount*/,
                                   NPVariant *result)
{
  TRACE("default method");
  VOID_TO_NPVARIANT(*result);
  return false;
}

const char* const Methods[] = {"inputDigit", "getIndexPage", "flofRedKey",
  "flofGreenKey", "flofYellowKey", "flofCyanKey", "getNextPage",
  "getPreviousPage", "hideSubpages", "showSubpages", "getPreviousSubpage",
  "getNextSubpage", "emptyPage", "handleEvent", "redraw"};

typedef std::set<std::string> TStringSet;

const TStringSet MethodSet(&(Methods[0]),
                           &(Methods[sizeof(Methods)/(sizeof(char *))]));

bool TExamplePlugin::HasMethod(const std::string& name)
{
  TRACE("name=%s", name.c_str());
  if (MethodSet.find(name) != MethodSet.end()) {
    return true;
  }
  return false;
}


bool TExamplePlugin::Invoke(const std::string& name,
                            const NPVariant *args,
                            uint32_t argCount,
                            NPVariant *result)
{
  TRACE("name=%s", name.c_str());

  if (MethodSet.find(name) == MethodSet.end()) {
    TRACE("false due to method not found");
    return false;
  }

  if (NULL != result) {
    VOID_TO_NPVARIANT(*result);
  }

  if (name == "inputDigit") {
    if (argCount != 1) {
       PrepareError("'inputDigit' reqires exactly 1 parameter");
       return false;
    }
    const NPVariant& argument = *args;
    int digit = -1;
    // taking last word from string representation
    std::string value = VariantToString(argument);
    size_t lastSpace = value.rfind(' ');
    if (lastSpace != std::string::npos ) {
      digit = boost::lexical_cast<int>(value.substr(lastSpace+1));
    }
    if (digit < 0 || digit > 9) {
      PrepareError("'inputDigit' accepts only difit between 0 and 9");
      return false;
    }
    TRACE("digit %d",digit);
    return true;
  }
  else {
    if (argCount != 0) {
       return false;
    }
    if ( name == "getIndexPage" ) {
      TRACE("index");
    }
    else if (name == "flofRedKey") {
      animate = !animate;
      TRACE("red");
      renderer_.SetColor(0x3FFF0000);
      renderer_.ForceRedraw(parent_);
    }
    else if ( name == "flofGreenKey" ) {
      TRACE("green");
      renderer_.SetColor(0x3F00FF00);
      renderer_.ForceRedraw(parent_);
    }
    else if ( name == "flofYellowKey" ) {
      TRACE("yellow");
      renderer_.SetColor(0x4F88800);
      renderer_.ForceRedraw(parent_);
    }
    else if ( name == "flofCyanKey") {
      TRACE("cyan");
      renderer_.SetColor(0x8F0000FF);
      renderer_.ForceRedraw(parent_);
    }
    else if ( name == "getNextPage") {
      TRACE("next");
    }
    else if ( name == "getPreviousPage" ) {
      TRACE("previous");
    }
    else if ( name == "hideSubpages" ) {
        TRACE("hide subpages");
    }
    else if ( name == "showSubpages") {
      TRACE("show subpages");
    }
    else if ( name == "getPreviousSubpage") {
      TRACE("prev subpage");
    }
    else if ( name == "getNextSubpage") {
      TRACE("next subpage");
    }
    else if ( name == "emptyPage") {
      TRACE("empty page");
    }
    else if ( name == "handleEvent") {
      TRACE("page event");
    }
    else if ( name == "redraw") {
        TRACE("redraw");
    }
    else {
      TRACE("unhandled method %s",name.c_str());
#ifdef FORCE_DEBUG
      assert(0 && "method must be handled but it isn't");
#endif
      return false;
    }
    return true;
  }
  TRACE("unhandled method2 %s",name.c_str());
#ifdef FORCE_DEBUG
  assert(0 && "method2 must be handled but it isn't");
#endif
  return false;
}

const char* const Properties[] = {"transparent"};

const TStringSet PropertySet(&(Properties[0]),
                             &(Properties[sizeof(Properties)
                                          /(sizeof(char *))]));

bool TExamplePlugin::HasProperty(const std::string& name)
{
  static int throwCounter = 7;
  try {
    --throwCounter;
    if (throwCounter<=0) {
      throw "every seven";
    }
  }
  catch (...) {
    TRACE("throwed");
    throwCounter = 7;
  }
  if (PropertySet.find(name) != PropertySet.end()) {
    TRACE("name=%s present", name.c_str());
    return true;
  }
  TRACE("name=%s NOT present", name.c_str());
  return false;
}

bool TExamplePlugin::GetProperty(const std::string& name,
                                 NPVariant *result)
{
  if (PropertySet.find(name) == PropertySet.end()) {
    TRACE("name=%s no such property",name.c_str());
    return false;
  }

  if (name == "transparent") {
    DOUBLE_TO_NPVARIANT(transparency_, *result);
    TRACE("return 'transparent' property as %.1lf", transparency_);
    return true;
  }
  TRACE("unhandled property %s",name.c_str());
#ifdef FORCE_DEBUG
  assert(0 && "property must be handled but is isn't");
#endif
  return false;
}

bool TExamplePlugin::SetProperty(const std::string& name,
                                 const NPVariant *value)
{
  if (PropertySet.find(name) == PropertySet.end()) {
    TRACE("name=%s no such property",name.c_str());
    return false;
  }
  TRACE("name=%s", name.c_str());
  if (name == "transparent") {
    double newTransparency = 100.0;
    if (NULL == value) {
      return false;
    }
    else if (NPVARIANT_IS_BOOLEAN(*value)) {
      TRACE("transparency is boolean");
      newTransparency = NPVARIANT_TO_BOOLEAN(*value)?100.0:0.0;
    }
    else if (NPVARIANT_IS_INT32(*value)) {
      TRACE("transparency is int");
      newTransparency = static_cast<double>(NPVARIANT_TO_INT32(*value));
    }
    else if (NPVARIANT_IS_DOUBLE(*value)) {
      TRACE("transparency is double");
      newTransparency = NPVARIANT_TO_DOUBLE(*value);
    }
    else if (NPVARIANT_IS_STRING(*value)) {
      TRACE("transparency is string");
      const NPString &v = NPVARIANT_TO_STRING(*value);
      std::string w(v.UTF8Characters,v.UTF8Length);
      newTransparency = boost::lexical_cast<double>(w);
    }
    else {
      // unsupported type
      PrepareError("'transparency' is of unsupported type");
      return false;
    }
    if (transparency_ != newTransparency) {
      TRACE("transparency is updated to %.1lf", newTransparency);
      //renderer_.SetTransparency(newTransparency);
      transparency_ = newTransparency;
    }
    else {
      TRACE("transparency is already %.1lf", transparency_);
    }
    return true;
  }
  TRACE("unhandled property %s",name.c_str());
#ifdef FORCE_DEBUG
  assert(0 && "property must be handled but is isn't");
#endif
  return false;
}

bool TExamplePlugin::RemoveProperty(const std::string& /*name*/)
{
  return false;
}

void TExamplePlugin::HandleEvent(int fd __attribute__ ((unused)),
                                 uint32_t event __attribute__ ((unused)))
throw ()
{
  TRACE("called.");
}

void TExamplePlugin::HandleTimeout(int) throw ()
{
  if (animate) {
    NPN_PluginThreadAsyncCall(parent_, Animate, this);
  }
  IdleAction();
}

void* TExamplePlugin::ThreadEntry(void* data)
{
  TRACE("(%8x) started",static_cast<unsigned int>(pthread_self()));
  TExamplePlugin* self = reinterpret_cast<TExamplePlugin*>(data);
  self->eventLoop_.Run();
  TRACE("finished");
  return 0;
}

void TExamplePlugin::StartExample()
{
  if ( exampleEventHandle_ != -1 ) {
    TRACE("example is running. stopping it first");
    StopExample();
  }

  TRACE("main thread is %8x",static_cast<unsigned int>(pthread_self()));
}

void TExamplePlugin::StopExample()
{
  if ( exampleEventHandle_ == -1 ) {
    TRACE("example is not running. Do nothing");
    return;
  }
  eventLoop_.RemoveDescriptor(exampleEventHandle_);
  exampleEventHandle_ = -1;
}

void TExamplePlugin::IdleAction()
{
  /* empty */
}

void TExamplePlugin::MakePageUpdate()
{
  /* empty */
}

void TExamplePlugin::
OnObjectsChanged(const
                 IToiInformationService::TToiObjectNameSequence&
                 /*objectNames*/) throw (TIpcException)
{
  /* empty */
}

void TExamplePlugin::PrepareError(const std::string &message)
{
  TRACE("EXCEPTION %s", message.c_str());
  NPN_SetException( defaultObject_.get(), message.c_str() );
}

void TExamplePlugin::Animate(void* data)
{
  TExamplePlugin* self = reinterpret_cast<TExamplePlugin*>(data);
  static uint32_t count = 0;
  self->renderer_.SetColor(count += 63 & 0xff);
  self->renderer_.ForceRedraw(self->parent_);
}
