/*
 * The EKLibrary library is proprietary and copyright Ekioh Ltd.
 * This EKLibrary file is licensed as follows:
 *
 * Copyright (c) 2008-2010 Ekioh Ltd.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#ifndef EK_EKPLUGINCLIENT_H
#define EK_EKPLUGINCLIENT_H

// Rely on including file having already brought in NPP and
// NPNetscapeFuncs so that this file can be used by both SVG engine
// and the browser. The latter uses file names that differ from those
// in the Gecko SDK (npfunctions.h vs. npupp.h).
//
// If you include SVGPluginClient.h or PluginView.h then these should
// do this for you.

namespace ekioh {

class EKPluginInstance;

/** Application-specific plugin instance.
 *
 * Each plugin instance in a document has an associated application-specific
 * object (a subclass of EKPluginClient) and a application-independent (but
 * platform-specific) object (a subclass of EKPluginInstance).
 */
class EKPluginClient
{
public:
    EKPluginClient();
    virtual ~EKPluginClient();

    /** Get EKPluginClient from Netscape plugin handle.
     *
     * @param[in] instance plugin handle
     *
     * @return EKPluginClient
     */
    static EKPluginClient* pluginClientForInstance(NPP instance)
    {
        if (instance == 0 || instance->ndata == 0)
            return 0;

        return static_cast<EKPluginClient*> (instance->ndata);
    }

    /** Set application-independent plugin instance.
     *
     * @param[in] instance application-independent plugin instance
     */
    void setInstance(EKPluginInstance* instance) { m_instance = instance; }

    /** Get application-independent plugin instance.
     *
     * @return application-independent plugin instance.
     */
    EKPluginInstance* getInstance() const { return m_instance; }

    /** Get browser-side NPAPI function table.
     *
     * @return browser-side NPAPI function table
     */
    virtual NPNetscapeFuncs* getBrowserFuncs() = 0;

protected:
    /** Get variable value from browser.
     *
     * @param[in] instance plugin instance
     * @param[in] variable variable to retrieve
     * @param[out] value variable value
     *
     * @return error code
     *
     * This is the global NPN_GetValue function that plugin clients should use
     * when initialising the NPNetscapeFuncs function table.
     */
    static NPError getValue(NPP instance, NPNVariable variable, void* value);
    static NPError setValue(NPP instance, NPPVariable variable, void* value);

    /** Get variable value from browser.
     *
     * @param[in] npp plugin instance
     * @param[in] variable variable to retrieve
     * @param[out] value variable value
     * @param[out] err error
     *
     * @return true if handled, otherwise false
     *
     * This is a client-specific version of NPN_GetValue. The global call
     * first calls this first to allow application-specific handling of
     * variable values to occur first.
     *
     * If the request isn't handled by this application-specific handler
     * then a platform-specific handler is called.
     */
    virtual bool getValue(NPP npp, NPNVariable variable,
                          void* value, NPError& err) = 0;
    virtual NPError setValue(NPPVariable variable,
                             void* value);

    static void invalidateRect(NPP instance, NPRect *rect);
    virtual void invalidateRect(NPRect* rect) { }

    static void forceRedraw(NPP instance);
    virtual void forceRedraw() { }

protected:
    EKPluginInstance* m_instance; ///< platform-specific plugin instance
};

} // namespace

#endif
