/*
 * The EKLibrary library is proprietary and copyright Ekioh Ltd.
 * This EKLibrary file is licensed as follows:
 *
 * Copyright (c) 2008-2010 Ekioh Ltd.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#ifndef EK_EKPLUGININSTANCE_H
#define EK_EKPLUGININSTANCE_H

#include "EKTypes.h"

#include "npapi.h"

struct NPObject;

namespace ekioh {

class EKFetcher;
class EKPlugin;
class EKPluginClient;
struct EKPluginEvent;
class EKString;
class EKUrl;
class PluginStream;
template <typename T> class EKRect;

/** Application-independent plugin instance.
 */
class EKPluginInstance
{
public:
    /** Constructor.
     *
     * @param[in] client application-specific plugin interface
     * @param[in] plugin plugin to create instance of
     */
    EKPluginInstance(EKPluginClient& client, EKPlugin& plugin);
    virtual ~EKPluginInstance();

    /** Create a new plugin instance.
     *
     * Platforms can override this to perform platform-specific actions
     * involved with instantiating a plugin, and call down to the default
     * implementation to do the basic instatiation. For example, X11 based
     * platforms may need to query whether the plugin needs the XEmbed
     * protocol.
     *
     * @param[in] mimeType mime type for plugin
     * @param[in] argc number of arguments
     * @param[in] argn array of argument names
     * @param[in] argv array of argument values
     *
     * @return true for success, false for failure
     */
    virtual bool nppNew(const EKString& mimeType,
                        int16_t argc, char* argn[], char* argv[]);

    /** Get plugin.
     *
     * @return EKPlugin from which this instance was created.
     */
    EKPlugin& getPlugin() { return m_plugin; }

    /** Get plugin client.
     *
     * @return EKPluginClient for this instance.
     */
    EKPluginClient& getPluginClient() { return m_pluginClient; }

    /** Get NPP handle.
     *
     * @return NPP handle for this instance.
     */
    NPP getNPP() { return &m_npp; }

    /** Get plugin function table.
     *
     * @return plugin function table for this instance.
     */
    const NPPluginFuncs& pluginFuncs() const { return m_pluginFuncs; }

    /** Load a data stream into this plugin instance.
     *
     * @param[in] url url of stream
     * @param[in] mimeType mime type of stream
     */
    void loadStream(const EKUrl& url, const EKString& mimeType);

    /** Stop loading data stream.
     */
    void doneStream(PluginStream& pluginStream);

    /** Set whether plugin is windowed or windowless.
     *
     * @param[in] isWindowed true if windowed, false if windowless
     */
    void setWindowed(bool isWindowed);

    /** Set window.
     */
    bool setWindow(const EKRect<int32_t>& rect);

    /** Query whether plugin is windowed or windowless.
     */
    bool isWindowed() const;

    /** Show plugin.
     *
     * Platform-specific method to show plugin
     */
    virtual void show() { }

    /** Hide plugin.
     *
     * Platform-specific method to hide plugin
     */
    virtual void hide() { }

    void handleEvent(const EKPluginEvent&);

protected:
    /** Fill in platform-specific window info / create platform window.
     *
     * Called just before the plugin setwindow function is called. Platforms
     * can fill in platform-specific parts of the NPWindow structure at this
     * point, and create their platform window for the plugin if appropriate.
     *
     * @param[in] window NPWindow structure
     *
     * @return true on success, false on failure
     */
    virtual bool setWindow(NPWindow& window);

protected:
    NPP_t m_npp; //!< plugin instance NPP object
    const NPPluginFuncs& m_pluginFuncs; //!< plugin function table
    EKPlugin& m_plugin; //!< plugin of which this is an instance
    EKPluginClient& m_pluginClient; //!< application-specific plugin handler

private:
    PluginStream* m_stream;
    NPWindow m_window;

    bool m_instantiated : 1;
    bool m_windowless : 1;
};

} // namespace

#endif
