# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiApplicationObserver import ToiApplicationObserver


class ToiApplicationService(KreaTvIpcCaller):

  # Enum TToiApplicationState
  STATE_REGISTERED = 0
  STATE_INSTALLING = 1
  STATE_INSTALLED = 2
  STATE_STARTING = 3
  STATE_INVISIBLE = 4
  STATE_VISIBLE = 5
  STATE_ACTIVE = 6
  STATE_STOPPED = 7

  def __init__(self, ipcproxy):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.lookup("ApplicationService")

  def RegisterApplication(self, propertyFilename):
    """This method can be called to register new applications.
    
    This is used to dynamically register new applications at
    runtime.
    
    The applications registered at runtime will automatically be
    considered to be dynamically downloadable uninstallable
    applications. In consequence they have to be downloaded and
    installed before they are started.
    
    \param propertyFilename The name of the property file of the
    application.
    
    \exception TToiInvalidArgumentException Raised when the
    supplied propertyFilename or property file is invalid.
    """
    __request = BinIpcParameterStack()
    # Write propertyFilename
    __request.push_string(propertyFilename)
    self.call(__request, "IToiApplicationService", "RegisterApplication")

  def ReportStarted(self, applicationId, address):
    """This method must be called by the application when it is
    initialized and has become fully operational. If the application
    does not do so within the time configured in the application
    properties, it will be terminated and possibly restarted.
    
    \param applicationId  The identifier of the application.
    \param address        The address where the Application
    Service can find an \ref IToiApplication interface.
    
    \exception TToiInvalidArgumentException
    Raised if the specified application identifier
    or the address is invalid.
    \see IToiApplication
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiApplicationService", "ReportStarted")

  def ReportPing(self, applicationId):
    """This method must be called by applications that receive a Ping
    from the Application Service. If the application does not do so
    within the time configured in the application properties, it
    will be terminated and possibly restarted.
    
    \param applicationId  The identifier of the application.
    
    \exception TToiInvalidArgumentException
    Raised if the specified application
    identifier is invalid.
    
    \see IToiApplication
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    self.call(__request, "IToiApplicationService", "ReportPing")

  def Activate(self, applicationId):
    """This method changes the state of the given application to
    \ref STATE_ACTIVE. The currently active application will become visible
    or invisible and if there is an application that is visible it might
    become invisible.
    
    \param applicationId  The identifier of the application to make active.
    
    \exception TToiInvalidArgumentException
    Raised if the specified application
    identifier is invalid.
    
    \see TToiApplicationState
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    self.call(__request, "IToiApplicationService", "Activate")

  def ActivateWithUri(self, applicationId, uri, mimeType):
    """This method does the same thing as \ref Activate() but it also
    tells the newly activated application to load a specific URI.
    
    \param applicationId  The identifier of the application to make active.
    \param uri            The URI that the application shall load.
    \param mimeType       The MIME type of the uri if known, otherwise empty.
    
    \exception TToiInvalidArgumentException
    Raised if the specified application
    identifier is invalid.
    
    \see IToiApplicationService::Activate()
    \see IToiApplication::LoadUri()
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write uri
    __request.push_string(uri)
    # Write mimeType
    __request.push_string(mimeType)
    self.call(__request, "IToiApplicationService", "ActivateWithUri")

  def ActivateWithCommand(self, applicationId, command):
    """This method does the same thing as \ref Activate() but it also tells
    the newly activated application to execute a command.
    
    \param applicationId  The identifier of the application to make active.
    \param command        The command the application shall execute.
    
    \exception TToiInvalidArgumentException
    Raised if the specified application
    identifier is invalid.
    
    \see IToiApplicationService::Activate()
    \see IToiApplication::ExecuteCommand()
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write command
    __request.push_string(command)
    self.call(__request, "IToiApplicationService", "ActivateWithCommand")

  def Deactivate(self, applicationId):
    """This method changes the state of the given application to
    \ref STATE_INVISIBLE IF it can be determined which application should
    become active. If it cannot be determined which application should become
    active, \ref Deactivate() does not change state of any application.
    
    Use this method to make popup applications disappear.
    
    \param applicationId  The identifier of the application to deactivate.
    
    \exception TToiInvalidArgumentException
    Raised if the specified application
    identifier is invalid.
    
    \see TToiApplicationState
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    self.call(__request, "IToiApplicationService", "Deactivate")

  def Kill(self, applicationId):
    """Kill an application. This method has no effect if the application
    is not currently running. If the application is running it will be
    restarted automatically if the restart property is true.
    This method is useful in forcing an application to a known initial
    state.
    
    \param applicationId  The identifier of the application to kill.
    
    \exception TToiInvalidArgumentException
    Raised if the specified application
    identifier is invalid.
    
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    self.call(__request, "IToiApplicationService", "Kill")

  def GetOwnApplicationId(self):
    """Returns the calling application's identifier. This function is
    mainly intended for Javascript since the identifier is also
    available in the APPLICATION_ID environment variable.
    
    \retval applicationId
    The identifier of the calling application.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiApplicationService", "GetOwnApplicationId")
    __result = []
    # Read applicationId
    __result.append(__response.pop_int32())
    return __result[0]

  def GetApplicationIds(self):
    """Returns the identifiers for all registered applications.
    
    \retval applicationsIds
    The identifiers of all registered applications.
    
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiApplicationService", "GetApplicationIds")
    __result = []
    # Read applicationIds
    applicationIds = []
    for k in range(__response.pop_int32()):
      applicationIds.append(__response.pop_int32())
    __result.append(applicationIds)
    return __result[0]

  def GetBoolProperty(self, applicationId, property):
    """Returns the value of a boolean property for an application.
    
    \param applicationId  The identifier of the application to get
    a property for.
    \param property       The name of the property.
    \retval value         The returned property value.
    
    \exception TToiInvalidArgumentException
    Raised if a parameter is invalid.
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write property
    __request.push_string(property)
    __response = self.call(__request, "IToiApplicationService", "GetBoolProperty")
    __result = []
    # Read value
    __result.append(__response.pop_bool())
    return __result[0]

  def GetIntProperty(self, applicationId, property):
    """Returns the value of an integer property for an application.
    
    \param applicationId  The identifier of the application to get
    a property for.
    \param property       The name of the property.
    \retval value         The returned property value.
    
    \exception TToiInvalidArgumentException
    Raised if a parameter is invalid.
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write property
    __request.push_string(property)
    __response = self.call(__request, "IToiApplicationService", "GetIntProperty")
    __result = []
    # Read value
    __result.append(__response.pop_int32())
    return __result[0]

  def GetStringProperty(self, applicationId, property):
    """Returns the value of a string property for an application.
    
    \param applicationId  The identifier of the application to get
    a property for.
    \param property       The name of the property.
    \retval value         The returned property value.
    
    \exception TToiInvalidArgumentException
    Raised if a parameter is invalid.
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write property
    __request.push_string(property)
    __response = self.call(__request, "IToiApplicationService", "GetStringProperty")
    __result = []
    # Read value
    __result.append(__response.pop_string())
    return __result[0]

  def GetStringSequenceProperty(self, applicationId, property):
    """Returns the values of a sequence property for an application.
    Unlike the other property functions it does not throw an
    exception if the property does not exist, instead it returns an
    empty sequence.
    
    \param applicationId   The identifier of the application to get
    a property for.
    \param property        The name of the property.
    \retval values         The returned property values.
    
    \exception TToiInvalidArgumentException
    Raised if a parameter is invalid.
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write property
    __request.push_string(property)
    __response = self.call(__request, "IToiApplicationService", "GetStringSequenceProperty")
    __result = []
    # Read values
    values = []
    for k in range(__response.pop_int32()):
      values.append(__response.pop_string())
    __result.append(values)
    return __result[0]

  def GetInfo(self, applicationId):
    """Returns information about the application.
    
    \param applicationId  The identifier of the application to get
    a property for.
    \retval info          The returned information struct.
    
    \exception TToiInvalidArgumentException
    Raised if the specified application
    identifier is invalid.
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    __response = self.call(__request, "IToiApplicationService", "GetInfo")
    __result = []
    # Read info
    info = {}
    info["State"] = __response.pop_int32()
    info["ApplicationName"] = __response.pop_string()
    info["ApplicationId"] = __response.pop_int32()
    __result.append(info)
    return __result[0]

  def LoadUri(self, uri, mimeType):
    """Call this method to tell the Application Service to load a
    specific URI. If an application is registered to handle the type
    of URI, it will be activated and \ref IToiApplication::LoadUri() will
    be called.
    
    \param uri       The URI to load.
    \param mimeType  The MIME type is used to hint the loader about which
    application to use for loading the URI.
    \exception TToiUnhandledValueException
    Raised if there is no application that handles the
    URI or the MIME type.
    
    \see IToiApplication::LoadUri()
    """
    __request = BinIpcParameterStack()
    # Write uri
    __request.push_string(uri)
    # Write mimeType
    __request.push_string(mimeType)
    self.call(__request, "IToiApplicationService", "LoadUri")

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiApplicationObserver.
    The observer will receive events sent by this IToiApplicationService instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiApplicationService can find an
    IToiApplicationObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiApplicationObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiApplicationService", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiApplicationService
    can find the registered IToiApplicationObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiApplicationObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiApplicationService", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiApplicationObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiApplicationService", "RegisterObserver")
    return observer
