# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiAudioOutputObserver import ToiAudioOutputObserver


class ToiAudioOutputService(KreaTvIpcCaller):

  # Enum TToiAudioConnectionType
  AUDIO_CONNECTION_TYPE_ANALOG = 0
  AUDIO_CONNECTION_TYPE_SPDIF = 1
  AUDIO_CONNECTION_TYPE_HDMI = 2
  AUDIO_CONNECTION_TYPE_DECODER = 3
  AUDIO_CONNECTION_TYPE_BUFFER = 4
  AUDIO_CONNECTION_TYPE_I2S = 5

  # Enum TToiAudioConnectionDirection
  AUDIO_CONNECTION_DIRECTION_INPUT = 0
  AUDIO_CONNECTION_DIRECTION_OUTPUT = 1

  def __init__(self, ipcproxy):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.lookup("AudioOutputService")

  def GetConnections(self):
    """Returns available audio connections.
    
    \retval connectionList  The list available audio connections.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiAudioOutputService", "GetConnections")
    __result = []
    # Read connectionList
    connectionList = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["IsLatencyAdjustable"] = __response.pop_bool()
      temp_l["IsAdjustable"] = __response.pop_bool()
      temp_l["IsMutable"] = __response.pop_bool()
      temp_l["Index"] = __response.pop_uint32()
      temp_l["Direction"] = __response.pop_int32()
      temp_l["Type"] = __response.pop_int32()
      temp_l["Id"] = __response.pop_int32()
      connectionList.append(temp_l)
    __result.append(connectionList)
    return __result[0]

  def GetConfiguration(self):
    """Returns the audio output configuration.
    
    \retval address  The IToiAudioOutputConfiguration address.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiAudioOutputService", "GetConfiguration")
    __result = []
    # Read address
    __result.append(__response.pop_string())
    return __result[0]

  def CreateConfigurationSession(self, applicationId):
    """Creates and returns a session to change the audio output
    configuration. The session is active between this call and
    either \ref IToiAudioOutputConfigurationSession::Apply() or
    \ref IToiAudioOutputConfigurationSession::Revert().
    
    \param applicationId  The identifier of the calling application.
    
    \retval address       The IToiAudioOutputConfigurationSession
    address.
    
    \exception TToiInvalidArgumentException
    Raised if the application identifier is invalid.
    
    \exception TToiInvalidPreconditionException
    Raised if another session is active.
    
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    __response = self.call(__request, "IToiAudioOutputService", "CreateConfigurationSession")
    __result = []
    # Read address
    __result.append(__response.pop_string())
    return __result[0]

  def SetVolume(self, connectionId, level):
    """Set an audio connection volume level. The level can be adjusted between
    0 and 100 where 0 is the lowest volume and 100 the highest volume.
    The volume is only affected if the content is decoded or non compressed.
    
    \param connectionId  The identifier of the connection for which
    the volume will be changed.
    
    \param level         The desired volume level.
    
    \exception TToiInvalidArgumentException
    Raised if there is no such connection or if the level
    is outside the allowed interval.
    
    \exception TToiOperationNotSupportedException
    Raised if the volume for the connection is not
    adjustable.
    """
    __request = BinIpcParameterStack()
    # Write connectionId
    __request.push_int32(connectionId)
    # Write level
    __request.push_uint32(level)
    self.call(__request, "IToiAudioOutputService", "SetVolume")

  def GetVolume(self, connectionId):
    """Returns the audio volume level of a connection. Possible level range
    from 0 and 100 where 0 is the lowest volume and 100 the highest volume.
    
    \param connectionId  The identifier of the connection to query.
    
    \retval level        The volume level.
    
    \exception TToiInvalidArgumentException
    Raised if there is no such connection.
    
    \exception TToiOperationNotSupportedException
    Raised if the volume for the connection is not
    adjustable.
    """
    __request = BinIpcParameterStack()
    # Write connectionId
    __request.push_int32(connectionId)
    __response = self.call(__request, "IToiAudioOutputService", "GetVolume")
    __result = []
    # Read level
    __result.append(__response.pop_uint32())
    return __result[0]

  def SetMuteState(self, connectionId, state):
    """Sets the mute state of a connection.
    
    \param connectionId  The identifier of the connection to query.
    
    \param state         The desired mute state.
    
    \exception TToiInvalidArgumentException
    Raised if there is no such connection.
    
    \exception TToiOperationNotSupportedException
    Raised if the connection is not mutable.
    """
    __request = BinIpcParameterStack()
    # Write connectionId
    __request.push_int32(connectionId)
    # Write state
    __request.push_bool(state)
    self.call(__request, "IToiAudioOutputService", "SetMuteState")

  def GetMuteState(self, connectionId):
    """Returns the mutes state of a connection.
    
    \param connectionId  The identifier of the connection to query.
    
    \retval state        The mute state.
    
    \exception TToiInvalidArgumentException
    Raised if there is no such connection.
    
    \exception TToiOperationNotSupportedException
    Raised if the connection is not mutable.
    """
    __request = BinIpcParameterStack()
    # Write connectionId
    __request.push_int32(connectionId)
    __response = self.call(__request, "IToiAudioOutputService", "GetMuteState")
    __result = []
    # Read state
    __result.append(__response.pop_bool())
    return __result[0]

  def SetLatency(self, connectionId, latency):
    """\techpreview This method is a TECHNICAL PREVIEW, it may change.
    Set an audio connection audio latency. latency is in milliseconds.
    This should affect all audio formats. If the latency parameter is
    out of range, the closest supported value is set.
    Setting the latency for one connection may impact the latency
    for other connections. In this case, the
    \ref IToiAudioOutputObserver::OnLatencyChanged() will be
    called for all the impacted connections.
    
    \param connectionId  The identifier of the connection for which
    the audio latency will be changed.
    
    \param latency       The desired audio latency in milliseconds.
    
    \exception TToiInvalidArgumentException
    Raised if there is no such connection.
    
    \exception TToiOperationNotSupportedException
    Raised if the latency for the connection is not
    adjustable.
    """
    __request = BinIpcParameterStack()
    # Write connectionId
    __request.push_int32(connectionId)
    # Write latency
    __request.push_int32(latency)
    self.call(__request, "IToiAudioOutputService", "SetLatency")

  def GetLatency(self, connectionId):
    """\techpreview This method is a TECHNICAL PREVIEW, it may change.
    Returns the audio latency of a connection.
    
    \param connectionId  The identifier of the connection to query.
    
    \retval latency      The audio latency in milliseconds.
    
    \exception TToiInvalidArgumentException
    Raised if there is no such connection.
    
    \exception TToiOperationNotSupportedException
    Raised if the latency for the connection is not
    adjustable.
    """
    __request = BinIpcParameterStack()
    # Write connectionId
    __request.push_int32(connectionId)
    __response = self.call(__request, "IToiAudioOutputService", "GetLatency")
    __result = []
    # Read latency
    __result.append(__response.pop_int32())
    return __result[0]

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiAudioOutputObserver.
    The observer will receive events sent by this IToiAudioOutputService instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiAudioOutputService can find an
    IToiAudioOutputObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiAudioOutputObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiAudioOutputService", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiAudioOutputService
    can find the registered IToiAudioOutputObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiAudioOutputObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiAudioOutputService", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiAudioOutputObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiAudioOutputService", "RegisterObserver")
    return observer
