# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiChannelObserver import ToiChannelObserver


class ToiChannelService(KreaTvIpcCaller):

  # Enum TToiPropertyType
  TYPE_BOOLEAN = 0
  TYPE_STRING = 1
  TYPE_INTEGER = 2

  # Enum TToiConditionOperation
  EQUALS = 0
  NOT_EQUAL = 1

  # Const OP_RESULT_CHANNEL_INFO
  OP_RESULT_CHANNEL_INFO = 1

  # Const PROPERTY_MODULATION_SYSTEM_TYPE
  PROPERTY_MODULATION_SYSTEM_TYPE = "system.modulationsystemtype"

  # Const PROPERTY_NETWORK_NAME
  PROPERTY_NETWORK_NAME = "system.networkname"

  # Const PROPERTY_NETWORK_ID
  PROPERTY_NETWORK_ID = "system.networkid"

  # Const PROPERTY_SERVICE_TYPE
  PROPERTY_SERVICE_TYPE = "system.servicetype"

  # Const PROPERTY_SERVICE_PROVIDER
  PROPERTY_SERVICE_PROVIDER = "system.serviceprovider"

  # Const PROPERTY_SERVICE_ID
  PROPERTY_SERVICE_ID = "system.serviceid"

  # Const PROPERTY_SERVICE_NAME
  PROPERTY_SERVICE_NAME = "system.servicename"

  # Const PROPERTY_LOGICAL_CHANNEL_NUMBER
  PROPERTY_LOGICAL_CHANNEL_NUMBER = "system.logicalchannelnumber"

  # Const PROPERTY_VISIBLE
  PROPERTY_VISIBLE = "system.visible"

  # Const PROPERTY_FREE
  PROPERTY_FREE = "system.free"

  # Const PROPERTY_ORIGINAL_NETWORK_ID
  PROPERTY_ORIGINAL_NETWORK_ID = "system.originalnetworkid"

  # Const PROPERTY_TRANSPORT_STREAM_ID
  PROPERTY_TRANSPORT_STREAM_ID = "system.transportstreamid"

  # Const PROPERTY_DELIVERY_SYSTEM_DESCRIPTOR
  PROPERTY_DELIVERY_SYSTEM_DESCRIPTOR = "system.deliverysystemdescriptor"

  # Const PROPERTY_URI
  PROPERTY_URI = "system.uri"

  # Const PROPERTY_SOURCE_ID
  PROPERTY_SOURCE_ID = "system.sourceid"

  # Const PROPERTY_FREQUENCY
  PROPERTY_FREQUENCY = "system.frequency"

  # Const PROPERTY_MODULATION_FORMAT
  PROPERTY_MODULATION_FORMAT = "system.modulationformat"

  def __init__(self, ipcproxy):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.lookup("ChannelService")

  def GetOperationManager(self):
    """Returns the operation manager that handles asynchronous operations for
    all services derived from the Frontend service. The procedure of
    executing asynchronous operations is generic and looks like this:
    <ul>
    <li>Get a handle to the operation manager.</li>
    <li>Register an observer to get operation events from the operation
    manager.</li>
    <li>Create an operation in order to get an operation identifier.</li>
    <li>Call a asynchronous operation using the operation identifier.</li>
    <li>Receive progress events for the asynchronous operation.</li>
    <li>When the operation is completed, get result data (if any).</li>
    <li>Release the operation or reuse the identifier for another
    asynchronous call.</li>
    </ul>
    
    \retval address The address to the \ref IToiOperationManager.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiChannelService", "GetOperationManager")
    __result = []
    # Read address
    __result.append(__response.pop_string())
    return __result[0]

  def GetPropertyDefinition(self, propertyId):
    """Returns the property definition for a given property identifier.
    
    \param propertyId  The property identifier.
    
    \retval definition The property definition.
    
    \exception TToiInvalidArgumentException
    Raised if the property identifier is invalid.
    
    \exception TToiNotEnoughResourcesException
    Raised if there were not enough system resources to
    perform the operation.
    """
    __request = BinIpcParameterStack()
    # Write propertyId
    __request.push_string(propertyId)
    __response = self.call(__request, "IToiChannelService", "GetPropertyDefinition")
    __result = []
    # Read definition
    definition = {}
    definition["IsSystemDefined"] = __response.pop_bool()
    definition["Type"] = __response.pop_int32()
    definition["PropertyId"] = __response.pop_string()
    __result.append(definition)
    return __result[0]

  def RunQuery(self, operationId, queryConditions, requestedProperties):
    """Returns information of all channels that match the given query
    conditions. This is an asynchronous operation. When the application
    receives a \ref IToiOperationObserver::OnOperationResult() call with the
    operation result \ref OP_RESULT_CHANNEL_INFO, then the application
    may get the results by calling the \ref GetChannelInfoResult()
    function. The resulting \ref TToiChannelInfo will only include property
    values for the requested properties.
    
    \param operationId         The operation identifier created by \ref
    IToiOperationManager.
    
    \param queryConditions     This sequence specifies the search criteria
    for which channels to search for. Using an
    empty sequence will return all stored
    channels.
    
    \param requestedProperties The requested properties that will be included
    in the result.  Using an empty sequence will
    return all properties.  If some requested
    property does not exist, this property will
    not be in the channel info.
    
    \exception TToiInvalidArgumentException
    Raised if any of the conditions are invalid,
    or raised if the operation identifier is
    illegal.
    \exception TToiNotEnoughResourcesException
    Raised if there were not enough system
    resources to perform the query.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    # Write queryConditions
    for iter_k in queryConditions[::-1]:
      __request.push_string(iter_k["Operand"]["Id"])
      __request.push_string(iter_k["Operand"]["Value"])
      __request.push_int32(iter_k["Operation"])
    __request.push_int32(len(queryConditions))
    # Write requestedProperties
    for iter_k in requestedProperties[::-1]:
      __request.push_string(iter_k)
    __request.push_int32(len(requestedProperties))
    self.call(__request, "IToiChannelService", "RunQuery")

  def GetChannelsInfo(self, channels, requestedProperties):
    """Returns information for all requested channels.
    
    \param channels            A sequence of channels ids. Using an empty
    sequence will return all stored channels.
    
    \param requestedProperties The requested properties that will be included
    in the result.  Using an empty sequence will
    return all properties.  If some requested
    property does not exist, this property will
    not be in the channel info.
    
    \retval channelsInfo       The information of all requested channels.
    
    \exception TToiInvalidArgumentException
    Raised if any channel id is invalid,
    or raised if the number of channels exceed
    the maximum supported number.
    
    \exception TToiNotEnoughResourcesException
    Raised if there were not enough system
    resources to perform the query.
    /** \note The total buffer can extend the maximum ipc message size. 
    """
    __request = BinIpcParameterStack()
    # Write channels
    for iter_k in channels[::-1]:
      __request.push_int32(iter_k)
    __request.push_int32(len(channels))
    # Write requestedProperties
    for iter_k in requestedProperties[::-1]:
      __request.push_string(iter_k)
    __request.push_int32(len(requestedProperties))
    __response = self.call(__request, "IToiChannelService", "GetChannelsInfo")
    __result = []
    # Read channelsInfo
    channelsInfo = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Properties"] = []
      for l in range(__response.pop_int32()):
        temp_m = {}
        temp_m["Value"] = __response.pop_string()
        temp_m["Id"] = __response.pop_string()
        temp_l["Properties"].append(temp_m)
      temp_l["Name"] = __response.pop_string()
      temp_l["Id"] = __response.pop_int32()
      channelsInfo.append(temp_l)
    __result.append(channelsInfo)
    return __result[0]

  def GetChannelInfoResult(self, operationId):
    """Used to get the query results once the applications receive
    \ref IToiOperationObserver::OnOperationResult()
    and the result parameter is equal to \ref OP_RESULT_CHANNEL_INFO.
    
    \param operationId The operation identifier used when calling
    \ref RunQuery().
    
    \retval results    The resulting channel information.
    
    \exception TToiInvalidArgumentException
    Raised if the operation identifier is invalid
    
    \exception TToiNoDataException
    Raised if no result is available.
    /** \note The total buffer can extend the maximum ipc message size. 
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    __response = self.call(__request, "IToiChannelService", "GetChannelInfoResult")
    __result = []
    # Read results
    results = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Properties"] = []
      for l in range(__response.pop_int32()):
        temp_m = {}
        temp_m["Value"] = __response.pop_string()
        temp_m["Id"] = __response.pop_string()
        temp_l["Properties"].append(temp_m)
      temp_l["Name"] = __response.pop_string()
      temp_l["Id"] = __response.pop_int32()
      results.append(temp_l)
    __result.append(results)
    return __result[0]

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiChannelObserver.
    The observer will receive events sent by this IToiChannelService instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiChannelService can find an
    IToiChannelObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiChannelObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiChannelService", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiChannelService
    can find the registered IToiChannelObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiChannelObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiChannelService", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiChannelObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiChannelService", "RegisterObserver")
    return observer
