# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiDlnaRendererObserver import ToiDlnaRendererObserver


class ToiDlnaRendererManager(KreaTvIpcCaller):

  # Enum TToiRendererState
  STATE_SESSION_REQUEST = 0
  STATE_SESSION_ACTIVE = 1
  STATE_SESSION_CLOSED = 2

  # Enum TToiRendererStatus
  RENDERER_SUCCESS = 0
  RENDERER_UNAVAILABLE = 1
  RENDERER_FAILURE = 2

  def __init__(self, ipcproxy, object_address):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.set_object_address(object_address)

  def UpdateSessionStatus(self, id, status):
    """
    Notifies the DMR of an update to the session.  This typically will occur
    when 1, we just received a request, so we will activate a session (if successful)
    or notify a failure to renderer (if unsuccessful); 2, if the rendering resouce
    becomes unavailable during rendering; or 3, for any other error while rendering.
    This may cause a change in State, from REQUEST to ACTIVE (on successful REQUEST),
    from REQUEST to CLOSED (on unsuccessful REQUEST), or from ACTIVE to CLOSED
    (on error or resource unavailability).
    
    \param id      The session ID being updated.
    
    \param status The status for this session's action.
    
    \exception TToiInvalidArgumentException
    Raised if the session id is invalid, or the status is invalid
    for the state of the session.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    # Write status
    __request.push_int32(status)
    self.call(__request, "IToiDlnaRendererManager", "UpdateSessionStatus")

  def GetSessionInfo(self, id):
    """
    Retreives all the information associated with the current media being
    rendered for the session id provided.
    
    \param id    The session ID associated with the media in question.
    
    \retval info The information associated with the given session id.
    
    \exception TToiInvalidArgumentException
    Raised if the session id is invalid.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    __response = self.call(__request, "IToiDlnaRendererManager", "GetSessionInfo")
    __result = []
    # Read info
    info = {}
    info["MediaProperties"] = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Value"] = __response.pop_string()
      temp_l["SubIndex"] = __response.pop_int32()
      temp_l["Id"] = __response.pop_int32()
      info["MediaProperties"].append(temp_l)
    info["MimeType"] = __response.pop_string()
    info["Uri"] = __response.pop_string()
    info["State"] = __response.pop_int32()
    info["Id"] = __response.pop_int32()
    __result.append(info)
    return __result[0]

  def GetRendererSessions(self):
    """
    Retreives all the session id's in REQUEST or ACTIVE states
    
    \retval ids The list of active session ids.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiDlnaRendererManager", "GetRendererSessions")
    __result = []
    # Read ids
    ids = []
    for k in range(__response.pop_int32()):
      ids.append(__response.pop_int32())
    __result.append(ids)
    return __result[0]

  def ReleaseInstance(self):
    """When an application is done using the Renderer Manager it should
    gracefully release it by calling this function.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiDlnaRendererManager", "ReleaseInstance")

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiDlnaRendererObserver.
    The observer will receive events sent by this IToiDlnaRendererManager instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiDlnaRendererManager can find an
    IToiDlnaRendererObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiDlnaRendererObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiDlnaRendererManager", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiDlnaRendererManager
    can find the registered IToiDlnaRendererObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiDlnaRendererObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiDlnaRendererManager", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiDlnaRendererObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiDlnaRendererManager", "RegisterObserver")
    return observer
