# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiDlnaObserver import ToiDlnaObserver


class ToiDlnaService(KreaTvIpcCaller):

  # Enum TToiServiceType
  SERVICE_TYPE_CONTENT_DIRECTORY = 0

  # Enum TToiMediaFormatProfile
  MEDIA_FORMAT_PROFILE_JPEG_LRG_ICO = 0
  MEDIA_FORMAT_PROFILE_JPEG_SM_ICO = 1
  MEDIA_FORMAT_PROFILE_PNG_LRG_ICO = 2
  MEDIA_FORMAT_PROFILE_PNG_SM_ICO = 3

  def __init__(self, ipcproxy):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.lookup("DlnaService")

  def GetMediaServers(self):
    """Returns a list with device identifiers for the DLNA devices present
    in the network.
    
    \retval uuids  The DLNA media server uuid identifiers.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiDlnaService", "GetMediaServers")
    __result = []
    # Read uuids
    uuids = []
    for k in range(__response.pop_int32()):
      uuids.append(__response.pop_string())
    __result.append(uuids)
    return __result[0]

  def GetMediaServerInfo(self, uuid):
    """Returns a list with device identifiers for the DLNA devices present
    in the network.
    
    \param uuid   The DLNA media server identifier.
    
    \retval info  The DLNA media server information.
    
    \exception TToiInvalidArgumentException
    Raised if the uuid does not match any DLNA server.
    """
    __request = BinIpcParameterStack()
    # Write uuid
    __request.push_string(uuid)
    __response = self.call(__request, "IToiDlnaService", "GetMediaServerInfo")
    __result = []
    # Read info
    info = {}
    info["Icons"] = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Profile"] = __response.pop_int32()
      temp_l["Url"] = __response.pop_string()
      info["Icons"].append(temp_l)
    info["IsLocal"] = __response.pop_bool()
    info["SupportedServices"] = []
    for k in range(__response.pop_int32()):
      info["SupportedServices"].append(__response.pop_int32())
    info["Vendor"] = __response.pop_string()
    info["Name"] = __response.pop_string()
    info["Uuid"] = __response.pop_string()
    __result.append(info)
    return __result[0]

  def CreateContentDirectoryInstance(self, applicationId, uuid):
    """This function creates a new Content Directory Service control point for
    a specific DLNA media server and associates it with the calling
    application.
    
    When the application no longer needs the service, it should release
    it by calling \ref IToiDlnaContentDirectoryService::ReleaseInstance().
    
    \param applicationId The application identifier of the calling
    application.
    
    \param uuid   The DLNA media server identifier.
    
    \retval address      The address to the Content Directory Service
    interface.
    
    \exception TToiInvalidArgumentException
    Raised if the application identifier is invalid.
    
    \exception TToiNotEnoughResourcesException
    Raised if the system cannot create a new instance
    of the service.
    
    \exception TToiOperationNotSupportedException
    Raised if the DLNA server does not support the
    Content Directory Service.
    
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write uuid
    __request.push_string(uuid)
    __response = self.call(__request, "IToiDlnaService", "CreateContentDirectoryInstance")
    __result = []
    # Read address
    __result.append(__response.pop_string())
    return __result[0]

  def CreateRendererInstance(self, applicationId):
    """\techpreview method CreateRendererInstance is a TECHNICAL PREVIEW, it may change.
    This function creates a new DLNA Media Renderer Manager  for control
    via an external DMC. This also notifies the internal DLNA stack there
    is a renderer available. When the application no longer needs the
    service, it should release it by
    calling \ref IToiDlnaRendererManager::ReleaseInstance()
    
    \param applicationId The application identifier of the calling
    application.
    
    \retval address      The address to the Dlna Renderer Manager
    interface.
    
    \exception TToiInvalidArgumentException
    Raised if the application identifier is invalid.
    
    \exception TToiNotEnoughResourcesException
    Raised if the system cannot create a new instance
    of the manager.
    
    \exception TToiOperationNotSupportedException
    Raised if the platform does not support the
    Dlna Renderer manager.
    
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    __response = self.call(__request, "IToiDlnaService", "CreateRendererInstance")
    __result = []
    # Read address
    __result.append(__response.pop_string())
    return __result[0]

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiDlnaObserver.
    The observer will receive events sent by this IToiDlnaService instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiDlnaService can find an
    IToiDlnaObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiDlnaObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiDlnaService", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiDlnaService
    can find the registered IToiDlnaObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiDlnaObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiDlnaService", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiDlnaObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiDlnaService", "RegisterObserver")
    return observer
