# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiDvbEitCacheObserver import ToiDvbEitCacheObserver


class ToiDvbEitCache(KreaTvIpcCaller):

  # Const NO_EVENT_ID
  NO_EVENT_ID = 2147483647

  # Const FILTER_MODE_PF_ONLY
  FILTER_MODE_PF_ONLY = 1

  # Const FILTER_MODE_PF_AND_SCHEDULE
  FILTER_MODE_PF_AND_SCHEDULE = 2

  def __init__(self, ipcproxy, object_address):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.set_object_address(object_address)

  def SetFilterMode(self, filterMode):
    """Start the filtering mode of EIT events.
    
    \param filterMode     The filter to use for EIT data, see filtermode
    flags above.
    
    \exception IToiTypes::TInvalidArgumentException
    Raised if any of the parameters to the call are
    invalid.
    """
    __request = BinIpcParameterStack()
    # Write filterMode
    __request.push_int32(filterMode)
    self.call(__request, "IToiDvbEitCache", "SetFilterMode")

  def ClearServices(self):
    """Clears the list of services to be cached. */
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiDvbEitCache", "ClearServices")

  def ClearEvents(self):
    """Clears the list of events for all services.  Note that if there
    are several instances of the EIT cache, the events for all
    instances will be cleared.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiDvbEitCache", "ClearEvents")

  def AddService(self, service):
    """Adds a service to the list of services to be cached. The service
    is identified uniquely by three parameters: original network
    identifier, transport stream identifier and service identifier.
    
    \param service        The service to be added.
    """
    __request = BinIpcParameterStack()
    # Write service
    __request.push_int32(service["OriginalNetworkId"])
    __request.push_int32(service["TransportStreamId"])
    __request.push_int32(service["ServiceId"])
    self.call(__request, "IToiDvbEitCache", "AddService")

  def GetEvents(self, service, time, duration):
    """Gets events from the cache for a specific service and a specific time
    interval.
    
    The returned events are sorted in cronological order.
    Due to limitations in the platform, the entire interval might
    not be able to fit into the sequence. In this case, it is up to the user
    to see how much is received and request the rest of the interval in a new
    request. Note then that the date, time and duration parameters need to
    be set the point to the missing interval.
    
    \param service        The service to get informaiont for.
    \param time           The start time of the interval, given in seconds
    since the Unix epoch.
    \param duration       The length of the interval in seconds.
    \retval infoSequence  The sequence of EIT events
    \retval more          Set to true if entire interval could not fit in
    sequence. False otherwise.
    
    \exception TToiInvalidArgumentException
    Raised if any of the parameters to the call are
    invalid.
    """
    __request = BinIpcParameterStack()
    # Write service
    __request.push_int32(service["OriginalNetworkId"])
    __request.push_int32(service["TransportStreamId"])
    __request.push_int32(service["ServiceId"])
    # Write time
    __request.push_uint32(time)
    # Write duration
    __request.push_int32(duration)
    __response = self.call(__request, "IToiDvbEitCache", "GetEvents")
    __result = []
    # Read more
    __result.append(__response.pop_bool())
    # Read infoSequence
    infoSequence = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["FreeCaMode"] = __response.pop_bool()
      temp_l["ParentalRating"] = __response.pop_int32()
      temp_l["ContentNibbles"] = __response.pop_int32()
      temp_l["Duration"] = __response.pop_int32()
      temp_l["Time"] = __response.pop_uint32()
      temp_l["Name"] = __response.pop_string()
      temp_l["EventId"] = __response.pop_int32()
      infoSequence.append(temp_l)
    __result.append(infoSequence)
    return tuple(__result)

  def GetPresentEvent(self, service):
    """Gets present event from the cache for a specific service.
    
    \param service        The service to get information for.
    \retval event         The present EIT event.
    
    \exception TToiInvalidArgumentException
    Raised if any of the parameters to the call are
    invalid.
    """
    __request = BinIpcParameterStack()
    # Write service
    __request.push_int32(service["OriginalNetworkId"])
    __request.push_int32(service["TransportStreamId"])
    __request.push_int32(service["ServiceId"])
    __response = self.call(__request, "IToiDvbEitCache", "GetPresentEvent")
    __result = []
    # Read event
    event = {}
    event["FreeCaMode"] = __response.pop_bool()
    event["ParentalRating"] = __response.pop_int32()
    event["ContentNibbles"] = __response.pop_int32()
    event["Duration"] = __response.pop_int32()
    event["Time"] = __response.pop_uint32()
    event["Name"] = __response.pop_string()
    event["EventId"] = __response.pop_int32()
    __result.append(event)
    return __result[0]

  def GetFollowingEvent(self, service):
    """Gets following event from the cache for a specific service.
    
    \param service        The service to get information for.
    \retval event         The following EIT event.
    
    \exception TToiInvalidArgumentException
    Raised if any of the parameters to the call are
    invalid.
    """
    __request = BinIpcParameterStack()
    # Write service
    __request.push_int32(service["OriginalNetworkId"])
    __request.push_int32(service["TransportStreamId"])
    __request.push_int32(service["ServiceId"])
    __response = self.call(__request, "IToiDvbEitCache", "GetFollowingEvent")
    __result = []
    # Read event
    event = {}
    event["FreeCaMode"] = __response.pop_bool()
    event["ParentalRating"] = __response.pop_int32()
    event["ContentNibbles"] = __response.pop_int32()
    event["Duration"] = __response.pop_int32()
    event["Time"] = __response.pop_uint32()
    event["Name"] = __response.pop_string()
    event["EventId"] = __response.pop_int32()
    __result.append(event)
    return __result[0]

  def GetPrivateData(self, service, eventId, privDataMaxSize):
    """Gets private data about an EIT event from a specific service which is
    identified uniquely by three parameters: original network identifier,
    transport stream identifier and service identifier.
    
    \param service        The service to get extended EIT event for.
    \param eventId        The event identifier to get information for.
    \retval privDataDataSize	The buffer size of privData
    \param privDataMaxSize	The maximum size of privData
    \retval privData      The private data extracted from EIT.
    \exception TToiInvalidArgumentException
    Raised if any of the parameters to the call are
    invalid.
    """
    __request = BinIpcParameterStack()
    # Write service
    __request.push_int32(service["OriginalNetworkId"])
    __request.push_int32(service["TransportStreamId"])
    __request.push_int32(service["ServiceId"])
    # Write eventId
    __request.push_int32(eventId)
    # Write privDataMaxSize
    __request.push_int32(privDataMaxSize)
    __response = self.call(__request, "IToiDvbEitCache", "GetPrivateData")
    __result = []
    # Read privDataDataSize
    __result.append(__response.pop_int32())
    # Read privData
    privData = __response.pop_binary(3)
    return tuple(__result)

  def GetExtendedEventInfo(self, service, eventId):
    """Gets extended information about a EIT eventi from a specific service.
    
    \param service        The service to get information for.
    \param eventId        The event identifier to get information for.
    \retval eventInfo     The extended EIT event info.
    
    \exception TToiInvalidArgumentException
    Raised if any of the parameters to the call are
    invalid.
    """
    __request = BinIpcParameterStack()
    # Write service
    __request.push_int32(service["OriginalNetworkId"])
    __request.push_int32(service["TransportStreamId"])
    __request.push_int32(service["ServiceId"])
    # Write eventId
    __request.push_int32(eventId)
    __response = self.call(__request, "IToiDvbEitCache", "GetExtendedEventInfo")
    __result = []
    # Read eventInfo
    eventInfo = {}
    eventInfo["ExtendedInfo"] = __response.pop_string()
    eventInfo["ShortInfo"] = __response.pop_string()
    eventInfo["EventId"] = __response.pop_int32()
    __result.append(eventInfo)
    return __result[0]

  def GetEventItemInfo(self, service, eventId):
    """Gets the items in extended event desciptor for a specific event
    from a specific service.
    
    \param service        The service to get information for.
    \param eventId        The event identifier to get information for.
    \retval eventItems    The sequence of TEventItemInfo.
    
    \exception TToiInvalidArgumentException
    Raised if any of the parameters to the call are
    invalid.
    """
    __request = BinIpcParameterStack()
    # Write service
    __request.push_int32(service["OriginalNetworkId"])
    __request.push_int32(service["TransportStreamId"])
    __request.push_int32(service["ServiceId"])
    # Write eventId
    __request.push_int32(eventId)
    __response = self.call(__request, "IToiDvbEitCache", "GetEventItemInfo")
    __result = []
    # Read eventItems
    eventItems = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Value"] = __response.pop_string()
      temp_l["Description"] = __response.pop_string()
      eventItems.append(temp_l)
    __result.append(eventItems)
    return __result[0]

  def GetVideoComponentList(self, service, eventId):
    """Gets the video components in component descriptor for a specific event
    from a specific service.
    
    \param service        The service to get information for.
    \param eventId        The event identifier to get information for.
    \retval components    The sequence of TToiComponent.
    
    \exception TToiInvalidArgumentException
    Raised if any of the parameters to the call are
    invalid.
    """
    __request = BinIpcParameterStack()
    # Write service
    __request.push_int32(service["OriginalNetworkId"])
    __request.push_int32(service["TransportStreamId"])
    __request.push_int32(service["ServiceId"])
    # Write eventId
    __request.push_int32(eventId)
    __response = self.call(__request, "IToiDvbEitCache", "GetVideoComponentList")
    __result = []
    # Read components
    components = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Description"] = __response.pop_string()
      temp_l["LanguageCode"] = __response.pop_string()
      temp_l["Tag"] = __response.pop_int32()
      temp_l["ComponentType"] = __response.pop_int32()
      temp_l["StreamContent"] = __response.pop_int32()
      components.append(temp_l)
    __result.append(components)
    return __result[0]

  def GetAudioComponentList(self, service, eventId, languageCode):
    """Gets the audio components of specific langauge in component descriptor
    for a specific event from a specific service. If languageCode is empty,
    all audio components for the event are fetched.
    
    \param service        The service to get information for.
    \param eventId        The event identifier to get information for.
    \param languageCode   The ISO 639 langauge code (a 3-character code);
    If empty, it means any language.
    \retval components    The sequence of TToiComponent.
    
    \exception TToiInvalidArgumentException
    Raised if any of the parameters to the call are
    invalid.
    """
    __request = BinIpcParameterStack()
    # Write service
    __request.push_int32(service["OriginalNetworkId"])
    __request.push_int32(service["TransportStreamId"])
    __request.push_int32(service["ServiceId"])
    # Write eventId
    __request.push_int32(eventId)
    # Write languageCode
    __request.push_string(languageCode)
    __response = self.call(__request, "IToiDvbEitCache", "GetAudioComponentList")
    __result = []
    # Read components
    components = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Description"] = __response.pop_string()
      temp_l["LanguageCode"] = __response.pop_string()
      temp_l["Tag"] = __response.pop_int32()
      temp_l["ComponentType"] = __response.pop_int32()
      temp_l["StreamContent"] = __response.pop_int32()
      components.append(temp_l)
    __result.append(components)
    return __result[0]

  def GetSubtitleComponentList(self, service, eventId, languageCode):
    """Gets the subtitle components of specific langauge in component descriptor
    for a specific event from a specific service. If languageCode is empty,
    all subtitle components for the event are fetched.
    
    \param service        The service to get information for.
    \param eventId        The event identifier to get information for.
    \param languageCode   The ISO 639 langauge code (a 3-character code);
    If empty, it means any language.
    \retval components    The sequence of TToiComponent.
    
    \exception TToiInvalidArgumentException
    Raised if any of the parameters to the call are
    invalid.
    """
    __request = BinIpcParameterStack()
    # Write service
    __request.push_int32(service["OriginalNetworkId"])
    __request.push_int32(service["TransportStreamId"])
    __request.push_int32(service["ServiceId"])
    # Write eventId
    __request.push_int32(eventId)
    # Write languageCode
    __request.push_string(languageCode)
    __response = self.call(__request, "IToiDvbEitCache", "GetSubtitleComponentList")
    __result = []
    # Read components
    components = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Description"] = __response.pop_string()
      temp_l["LanguageCode"] = __response.pop_string()
      temp_l["Tag"] = __response.pop_int32()
      temp_l["ComponentType"] = __response.pop_int32()
      temp_l["StreamContent"] = __response.pop_int32()
      components.append(temp_l)
    __result.append(components)
    return __result[0]

  def ReleaseInstance(self):
    """When an application is done using a EIT cache instance it should
    gracefully release it by calling this function.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiDvbEitCache", "ReleaseInstance")

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiDvbEitCacheObserver.
    The observer will receive events sent by this IToiDvbEitCache instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiDvbEitCache can find an
    IToiDvbEitCacheObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiDvbEitCacheObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiDvbEitCache", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiDvbEitCache
    can find the registered IToiDvbEitCacheObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiDvbEitCacheObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiDvbEitCache", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiDvbEitCacheObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiDvbEitCache", "RegisterObserver")
    return observer
