# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack


class ToiFrontendDownloader(KreaTvIpcCaller):

  # Enum TToiDownloadType
  DOWNLOAD_TYPE_SOFTWARE = 0
  DOWNLOAD_TYPE_SPLASH = 1

  # Enum TToiDownloadAvailability
  DOWNLOAD_NOT_AVAILABLE = 0
  DOWNLOAD_AVAILABLE = 1

  # Const OP_RESULT_DOWNLOAD_INFO
  OP_RESULT_DOWNLOAD_INFO = 1

  def __init__(self, ipcproxy, object_address):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.set_object_address(object_address)

  def StartSearch(self, operationId, downloadType):
    """Start a search for upgrades of a certain software type.
    
    This is an asynchronous operation. When the application receives
    a \ref IToiOperationObserver::OnOperationResult() call with the
    operation result \ref IToiOperationManager::OP_RESULT_NONE,
    then the application can get the search progess, its state and more from
    \ref IToiOperationManager::TToiOperationInfo. When the application
    receives a \ref IToiOperationObserver::OnOperationResult() call with the
    operation result \ref OP_RESULT_DOWNLOAD_INFO, then the application
    can get the download information by calling \ref GetDownloadInfoResult().
    
    Calling \ref IToiOperationManager::CancelOperation() will
    immediatly stop an ongoing search operation.
    Calling \ref IToiOperationManager::ReleaseOperation() will remove
    the operation identified by operationId. If the search is in progress,
    calling this method will immediatly stop it.
    
    \param operationId  The operation identifier recived from
    \ref IToiOperationManager::CreateOperation() upon
    creation of the operation.
    
    \param downloadType The type of software release.
    
    \exception TToiInvalidArgumentException
    Raised if the operation identifier or the download
    type is invalid.
    
    \exception TToiInvalidPreconditionException
    Raised if one search/download/install
    operation is active.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    # Write downloadType
    __request.push_int32(downloadType)
    self.call(__request, "IToiFrontendDownloader", "StartSearch")

  def StartDownload(self, operationId, downloadType):
    """Downloads an update of the specified software. The software will not be
    used until the set-top box has been rebooted, e.g. by calling
    any of the reboot functions in \ref IToiPlatformService.
    
    When the application receives
    a \ref IToiOperationObserver::OnOperationResult() call with the
    operation result \ref IToiOperationManager::OP_RESULT_NONE,
    then the application can get the download progress from \ref
    IToiOperationManager::TToiOperationInfo&.
    
    Calling \ref IToiOperationManager::CancelOperation() will
    immediatly stop an ongoing download operation.
    Calling \ref IToiOperationManager::ReleaseOperation() will remove
    the operation identified by operationId. If a download is in progress,
    calling this method will immediatly stop it.
    
    \param operationId  The operation identifier recived from
    \ref IToiOperationManager::CreateOperation() upon
    creation of the operation.
    
    \param downloadType The type of software release.
    
    \exception TToiInvalidArgumentException
    Raised if the operation identifier or the download
    type is invalid.
    
    \exception TToiInvalidPreconditionException
    If no search has been performed that has found new
    software of the specified download type or one
    search/download/install operation is active.
    
    \see StartSearch()
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    # Write downloadType
    __request.push_int32(downloadType)
    self.call(__request, "IToiFrontendDownloader", "StartDownload")

  def Install(self, operationId):
    """Install the downloaded software or splash.
    
    When the application receives
    a \ref IToiOperationObserver::OnOperationResult() call with the
    operation result \ref IToiOperationManager::OP_RESULT_NONE,
    then the application can get the install progress from \ref
    IToiOperationManager::TToiOperationInfo&.
    
    The install operation can not be cancelled once it has been started.
    Calling \ref IToiOperationManager::CancelOperation() or
    \ref IToiOperationManager::ReleaseOperation() will do nothing.
    
    \param operationId  The operation identifier recived from
    \ref IToiOperationManager::CreateOperation() upon
    creation of the operation.
    
    \exception TToiInvalidArgumentException
    Raised if the operation identifier or the download
    type is invalid.
    
    \exception TToiInvalidPreconditionException
    Raised if no download has been performed before
    calling this method or one search/download/install
    operation is active.
    
    \see StartDownload()
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    self.call(__request, "IToiFrontendDownloader", "Install")

  def GetDownloadInfoResult(self, operationId):
    """Used to get the search result once the application recieve
    \ref IToiOperationObserver::OnOperationResult()
    and the result parameter is equal to \ref OP_RESULT_DOWNLOAD_INFO.
    
    \param operationId   The operation identifier recived from
    \ref IToiOperationManager::CreateOperation() upon
    creation of the operation.
    
    \retval downloadInfo The resulting download information.
    
    \exception TToiInvalidArgumentException
    Raised if the operation identifier is invalid.
    
    \exception TToiNoDataException
    Raised if no result is available.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    __response = self.call(__request, "IToiFrontendDownloader", "GetDownloadInfoResult")
    __result = []
    # Read downloadInfo
    downloadInfo = {}
    downloadInfo["DownloadVersion"] = __response.pop_string()
    downloadInfo["DownloadAvailability"] = __response.pop_int32()
    downloadInfo["DownloadType"] = __response.pop_int32()
    __result.append(downloadInfo)
    return __result[0]

  def ReleaseInstance(self):
    """This function releases the resources used by this instance and
    removes all registered observers. Upon return this object will
    no longer be available and must not be called.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiFrontendDownloader", "ReleaseInstance")

