# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiMediaConsumerObserver import ToiMediaConsumerObserver


class ToiMediaConsumer(KreaTvIpcCaller):

  # Enum TToiState
  STATE_IDLE = 0
  STATE_CONNECTING = 1
  STATE_PAUSED = 2
  STATE_CONSUMING = 3
  STATE_FAILED = 4

  def __init__(self, ipcproxy, object_address):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.set_object_address(object_address)

  def GetState(self):
    """Returns the current state of the media consumer instance.
    
    \retval state     The current consumer state.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaConsumer", "GetState")
    __result = []
    # Read state
    __result.append(__response.pop_int32())
    return __result[0]

  def GetSessionId(self):
    """Returns the session identifier. This identifier can be used to open a
    session object with \ref IToiMediaService::OpenSession().
    
    \retval id        The session identifier.
    
    \exception TToiSessionViolationException
    If the consumer is in \ref STATE_IDLE a session violation
    exception is raised.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaConsumer", "GetSessionId")
    __result = []
    # Read id
    __result.append(__response.pop_int32())
    return __result[0]

  def GetUrl(self):
    """Returns the URL for the media stream.
    
    \retval url       The URL for the media stream.
    
    \exception TToiSessionViolationException
    If the consumer is in \ref STATE_IDLE a session violation
    exception is raised.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaConsumer", "GetUrl")
    __result = []
    # Read url
    __result.append(__response.pop_string())
    return __result[0]

  def Open(self, url):
    """This function starts a new stream session by trying to establish
    a connection to a media source. If this function returns
    successfully, the consumer state has changed to \ref STATE_CONNECTING.
    If an exception is raised, the consumer state is unaffected.
    
    Depending on the success of the connection attempt to the media
    source, the consumer will make a second state transition to either
    \ref STATE_PAUSED or \ref STATE_FAILED, on success and failure,
    respectively. In any respect, the application will be notified
    about this event by a call to its
    \ref IToiMediaConsumerObserver::OnStateChanged() function.
    
    \param url        The URL identifying the media source. The URL
    string may contain additional parameter
    information in accordance to the extended URL
    format.
    
    \exception TToiSessionViolationException
    If the consumer is not in idle state at the time of the call
    to \ref Open(), a session violation exception is raised.
    
    \exception TToiInvalidArgumentException
    If the URL given as argument does not follow the extended
    URL format, an invalid argument exception is raised.
    
    \exception TToiUnavailableException
    \ref TToiUnavailableException is raised if the consumer is
    temporarily unavailable due to platform resource
    management.
    
    \see Close(), TToiState
    """
    __request = BinIpcParameterStack()
    # Write url
    __request.push_string(url)
    self.call(__request, "IToiMediaConsumer", "Open")

  def Close(self):
    """This function ends the current session by disconnecting from
    the media source. For example, if the source is a video server it ends
    the server session and disconnects, or if the source is a multicast
    channel it leaves that multicast group. If this function returns
    successfully, the consumer state has changed to \ref STATE_IDLE.
    If an exception is raised, the consumer state is unaffected.
    
    \exception TToiSessionViolationException
    If the consumer is in idle state a session violation
    exception is raised.
    
    \exception TToiUnavailableException
    \ref TToiUnavailableException is raised if the consumer is
    temporarily unavailable due to platform resource
    management.
    
    \see Open(), TToiState
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiMediaConsumer", "Close")

  def Consume(self):
    """This function starts consuming the stream. If this function
    returns successfully, the consumer state has changed to
    \ref STATE_CONSUMING. If an exception is raised, the consumer state
    is unaffected.
    
    \exception TToiSessionViolationException
    If the consumer is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiOperationNotSupportedException
    If the consumer is in a state is which this operation is not
    supported, an operation not supported exception is raised.
    This is the case when in \ref STATE_FAILED, for example.
    
    \exception TToiUnavailableException
    \ref TToiUnavailableException is raised if the consumer is
    temporarily unavailable due to platform resource
    management.
    
    \see Open(), TToiState
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiMediaConsumer", "Consume")

  def ReleaseInstance(self):
    """This function releases the resources used by this instance.
    When this function returns all observers has also been released.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiMediaConsumer", "ReleaseInstance")

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiMediaConsumerObserver.
    The observer will receive events sent by this IToiMediaConsumer instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiMediaConsumer can find an
    IToiMediaConsumerObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiMediaConsumerObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiMediaConsumer", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiMediaConsumer
    can find the registered IToiMediaConsumerObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiMediaConsumerObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiMediaConsumer", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiMediaConsumerObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiMediaConsumer", "RegisterObserver")
    return observer
