# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack


from ToiMediaPlayerBase import ToiMediaPlayerBase

class ToiMediaPlayer(ToiMediaPlayerBase):

  def __init__(self, ipcproxy, object_address):
    ToiMediaPlayerBase.__init__(self, ipcproxy, object_address)

  def Open(self, url):
    """This function starts a new stream session by trying to establish
    a connection to a media source. If this function returns successfully,
    the player state will change to STATE_CONNECTING. If an exception is
    raised, the player state is unaffected.
    
    Depending on the success of the connection attempt to the media
    source, the player will make a second state transition to either
    \ref STATE_PAUSED or \ref STATE_FAILED, on success and failure,
    respectively. In any respect, the application will be notified
    with the event \ref IToiMediaPlayerObserver::OnStateChanged().
    
    \param url        The URL identifying the media source. The URL
    string may contain additional parameter
    information in accordance to the extended URL
    format.
    
    \exception TToiSessionViolationException If the player is not in
    idle state at the time of the call to \ref Open(), a session
    violation exception is raised.
    
    \exception TToiInvalidArgumentException
    If the URL given as argument does not follow the extended
    URL format, an invalid argument exception is raised.
    
    \exception TToiUnavailableException
    Raised if the player is temporarily unavailable due to
    platform resource management.
    
    \see Close()
    \see TToiState
    """
    __request = BinIpcParameterStack()
    # Write url
    __request.push_string(url)
    self.call(__request, "IToiMediaPlayer", "Open")

  def Close(self):
    """This function ends the current session by stopping any playback
    and disconnecting from the media source. For example, if the
    source is a video server it ends the server session and
    disconnects, or if the source is a multicast channel it leaves
    that multicast group. If this function returns successfully, the
    player state will change to STATE_IDLE. If an exception is
    raised, the player state is unaffected.
    
    \exception TToiSessionViolationException
    If the player is in idle state a session violation
    exception is raised.
    
    \exception TToiUnavailableException
    Raised if the player is temporarily unavailable due to
    platform resource management.
    
    \see Open()
    \see TToiState
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiMediaPlayer", "Close")

  def Play(self, pace):
    """This function controls playback of the stream. If this function
    returns successfully, the player state will change to one of
    four states: \ref STATE_PLAYING, \ref STATE_PAUSED,
    \ref STATE_FASTFORWARDING, and \ref STATE_REWINDING. Different values
    pace result one of these different states with the corresponding effect.
    In the case of pausing, it keeps the session alive, i.e. it remains
    connected to video servers and joined in multicast groups. If an
    exception is raised, the player state is unaffected.
    
    \param pace      The pace at which the position will change, given
    as a multiplier of normal playback times
    1000. Zero will pause the stream. 1000 is normal
    speed (playing). Values above 1000 will result in
    fast  forward, and below -1000 will result in rewind.
    This are desired paces, the actual implementation
    may not set the desired speed, but the different
    states will change according to this description.
    
    \exception TToiSessionViolationException
    Raised if the player does not have an active session and will not
    have one soon (due to a recent call to Open) either.
    
    \exception TToiOperationNotSupportedException
    If the player is in a state is which this operation is not
    supported, an operation not supported exception is raised.
    This is the case when in \ref STATE_FAILED, for example. What
    is supported in which state may in other cases vary
    depending on which media source is used.
    
    \exception TToiUnavailableException
    Raised if the player is temporarily unavailable due to
    platform resource management.
    
    \see Open()
    \see TToiState
    """
    __request = BinIpcParameterStack()
    # Write pace
    __request.push_int32(pace)
    self.call(__request, "IToiMediaPlayer", "Play")

  def PlayFromPosition(self, position, pace):
    """This function moves the playback position of the current stream
    and resumes playback at the new position with the desired pace.
    Streams without a well defined notion of beginning and end, like
    multicast channels, generally do not support this operation.
    A position value greater than the length of the stream will be clamped.
    
    If this function returns successfully, the player state will change
    to one of four states: STATE_PLAYING, STATE_PAUSED,
    STATE_FASTFORWARDING, and STATE_REWINDING. Different values of
    pace results in the different states. In the case of pausing, it
    keeps the session alive, i.e. it remains connected to the video
    server or the joined in multicast group. If an exception is
    raised, the player state is unaffected.
    
    \param position   The position, specified as the number of
    milliseconds from the beginnig of the stream,
    to which the playback position is to be
    moved. The length of the stream can be
    obtained by calling the \ref GetStreamInfo() function.
    
    When playing from the timeshift buffer, the constant
    \ref POSITION_LIVE can be used to stop buffer playback
    and start playback of the live stream.
    
    \param pace       The pace at which the position will change, given
    as a multiplier of normal playback times
    1000. Zero will pause the stream. 1000 is normal
    speed (playing). Values above 1000 will result in
    fast  forward, and below -1000 will result in rewind.
    This are desired paces, the actual implementation
    may not set the desired speed, but the different
    states will change according to this description.
    
    \exception TToiSessionViolationException
    Raised if the player does not have an active session and will not
    have one soon (due to a recent call to Open) either .
    
    \exception TToiOperationNotSupportedException
    If the player is in a state is which this operation is not
    supported, an operation not supported exception is raised.
    This is the case when in \ref STATE_FAILED, for example. What
    is supported in which state may in other cases vary
    depending on which media source is used. The client may
    investigate this in advance by checking capabilities with
    \ref GetCapabilities().
    
    \exception TToiInvalidArgumentException
    Raised if the position is negative.
    
    \exception TToiUnavailableException
    Raised if the player is temporarily unavailable due to
    platform resource management.
    """
    __request = BinIpcParameterStack()
    # Write position
    __request.push_int32(position)
    # Write pace
    __request.push_int32(pace)
    self.call(__request, "IToiMediaPlayer", "PlayFromPosition")

  def StartTimeshiftBuffering(self, maxBufferLength):
    """Starts background buffering of the currently viewed stream
    and therefore enables timeshift operations. The media player
    will continue to display the live stream until either pause or
    rewind is performed. The timeshift buffering is stopped either
    by an excplicit call to \ref StopTimeshiftBuffering() or when the
    buffer is full. Timeshift operations are disabled by calling
    \ref DiscardTimeshiftBuffer(). If this function returns successfully,
    the timeshift state has changed to \ref TIMESHIFT_STATE_BUFFERING.
    The media player should be in \ref STATE_PLAYING when calling this.
    
    \param maxBufferLength
    The maximum size of the timeshift buffer in seconds.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiInvalidArgumentException
    If maxBufferLength is <= 0 an invalid argument
    exception is raised.
    
    \exception TToiInvalidPreconditionException
    If the timeshift state is not \ref TIMESHIFT_STATE_INACTIVE or
    the media player is not in \ref STATE_PLAYING then an
    invalid precondition exception is raised.
    
    \exception TToiOperationNotSupportedException
    If the current media source does not have the timeshift
    capability an operation not supported exception is raised.
    
    \exception TToiUnavailableException
    Raised if the player is temporarily unavailable due to
    platform resource management.
    """
    __request = BinIpcParameterStack()
    # Write maxBufferLength
    __request.push_int32(maxBufferLength)
    self.call(__request, "IToiMediaPlayer", "StartTimeshiftBuffering")

  def StopTimeshiftBuffering(self):
    """Stops the timeshift buffering but preserves the content in the buffer
    until the function \ref DiscardTimeshiftBuffer() is called. The player
    will continue to play from the current media source. If this function
    returns successfully, the timeshift state has changed to
    \ref TIMESHIFT_STATE_STOPPED.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiInvalidPreconditionException
    If the timeshift state is not \ref TIMESHIFT_STATE_BUFFERING an
    invalid precondition exception is raised.
    
    \exception TToiUnavailableException
    Raised if the player is temporarily unavailable due to
    platform resource management.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiMediaPlayer", "StopTimeshiftBuffering")

  def DiscardTimeshiftBuffer(self):
    """Discards the timeshift buffer and resumes live stream playback,
    if not already playing live. This disables further timeshift
    operations. If this function returns successfully,
    the timeshift state has changed to \ref TIMESHIFT_STATE_INACTIVE.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiInvalidPreconditionException
    If the timeshift state is not \ref TIMESHIFT_STATE_STOPPED or
    \ref TIMESHIFT_STATE_BUFFERING_FAILED an invalid precondition
    exception is raised.
    
    \exception TToiUnavailableException
    Raised if the player is temporarily unavailable due to
    platform resource management.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiMediaPlayer", "DiscardTimeshiftBuffer")

  def ActivateComponent(self, mediaComponent):
    """This function selects a media component to be used. The player
    state is not affected.
    
    \param mediaComponent The media component entry to be activated.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiInvalidArgumentException
    If the component argument does not identify a valid component,
    an invalid argument exception is raised.
    
    \exception TToiUnavailableException
    \ref TToiUnavailableException is raised if the player is
    temporarily unavailable due to platform resource
    management.
    
    \see  GetComponentInfo()
    """
    __request = BinIpcParameterStack()
    # Write mediaComponent
    __request.push_int32(mediaComponent["Type"])
    __request.push_int32(mediaComponent["Id"])
    self.call(__request, "IToiMediaPlayer", "ActivateComponent")

  def DeactivateComponent(self, mediaType):
    """This function deactivates a media component type.
    
    \param mediaType The media component type to be deactivated.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiInvalidArgumentException
    Raised if the component argument does not identify a valid
    component.
    
    \exception TToiUnavailableException
    Raised if the player is temporarily unavailable due to
    platform resource management.
    """
    __request = BinIpcParameterStack()
    # Write mediaType
    __request.push_int32(mediaType)
    self.call(__request, "IToiMediaPlayer", "DeactivateComponent")

  def SetAudioChannelMode(self, streamId, channelMode):
    """\techpreview This method is a TECHNICAL PREVIEW, it may change.
    
    This function sets the audio channel mode for a given stream
    identifier. For the active stream the mode will be set
    instantly.
    
    \param streamId    The audio stream identifier.
    
    \param channelMode The audio channel mode.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiInvalidArgumentException
    An invalid argument exception is raised if the stream
    identifier argument does not identify a valid audio stream
    or the channel mode is not supported.
    """
    __request = BinIpcParameterStack()
    # Write streamId
    __request.push_int32(streamId)
    # Write channelMode
    __request.push_int32(channelMode)
    self.call(__request, "IToiMediaPlayer", "SetAudioChannelMode")

  def SetParameter(self, name, value):
    """This function sets the value of a parameter. Depending on the
    current state, the new value is used differently. If the
    function is called when in idle state, the new value is used for
    all sessions started after the parameter is set. If the function
    is called in any other state but the idle state, the new value
    is applied to that session only and is discarded automatically
    when the session ends.
    
    \param name       The name of the parameter to set.
    
    \param value      The value to set the parameter to.
    
    \exception TToiInvalidArgumentException
    The parameter name is invalid or not
    recognized.
    
    \exception TToiPermissionDeniedException
    The application does not have permission to
    change the value of the parameter.
    
    \exception TToiUnavailableException
    Raised if the player is temporarily
    unavailable due to platform resource
    management.
    
    \see UnsetParameter()
    \see GetParameter()
    """
    __request = BinIpcParameterStack()
    # Write name
    __request.push_string(name)
    # Write value
    __request.push_string(value)
    self.call(__request, "IToiMediaPlayer", "SetParameter")

  def UnsetParameter(self, name):
    """This function restores the value of a parameter to the default.
    Depending on the current state, the new value is used
    differently.  If the function is called when in idle state, all
    sessions started after the parameter is restored will use the
    default value. If the function is called in any other state but
    the idle state, the value is changed for that session only.
    
    \param name       The name of the parameter for which the value
    is to be restored to the default.
    
    \exception TToiInvalidArgumentException
    The parameter name is invalid or not
    recognized.
    
    \exception TToiPermissionDeniedException
    The application does not have permission to
    change the value of the parameter.
    
    \exception TToiUnavailableException
    Raised if the player is temporarily
    unavailable due to platform resource
    management.
    
    \see SetParameter(), GetParameter()
    """
    __request = BinIpcParameterStack()
    # Write name
    __request.push_string(name)
    self.call(__request, "IToiMediaPlayer", "UnsetParameter")

