# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack


from ToiMediaRecorderBase import ToiMediaRecorderBase

class ToiMediaRecorder(ToiMediaRecorderBase):

  def __init__(self, ipcproxy, object_address):
    ToiMediaRecorderBase.__init__(self, ipcproxy, object_address)

  def Open(self, url, assetId):
    """This function starts a new recording session by trying to
    establish a connection to a media source. If this function
    returns successfully, the recorder state has changed to
    \ref STATE_CONNECTING. If an exception is raised, the recorder state
    is unaffected.
    
    Depending on the success of the connection attempt to the media
    source, the recorder will make a second state transition to
    either \ref STATE_PAUSED or \ref STATE_FAILED, on success and failure,
    respectively. In any respect, the application will be notified
    about this event by a call to its
    \ref IToiMediaRecorderObserver::OnStateChanged() function.
    
    The application ends the session by calling \ref Close(),
    which makes the recorder change state to \ref STATE_IDLE.
    
    \param url
    The URL identifying the media source. The URL string may
    contain additional parameter information in accordance to the
    extended URL format.
    
    \param assetId
    Identifies the asset to record to.
    
    \exception TToiSessionViolationException
    If the recorder is not in \ref STATE_IDLE at the time of the call
    to \ref Open(), a session violation exception is raised.
    
    \exception TToiInvalidArgumentException
    If the URL given as argument does not follow the extended
    URL format, an invalid argument exception is raised.
    
    \exception TToiBusyException
    If the asset is locked and used by another recorder.
    
    \exception TToiUnavailableException
    Raised if the recorder is temporarily unavailable due to
    platform resource management.
    
    \exception TToiInvalidPreconditionException
    Raised if the asset used for recording already exists and
    is not empty (has already been recorded to).
    
    \see Close()
    \see TToiMediaRecorderState
    """
    __request = BinIpcParameterStack()
    # Write url
    __request.push_string(url)
    # Write assetId
    __request.push_string(assetId)
    self.call(__request, "IToiMediaRecorder", "Open")

  def Close(self):
    """This function ends the current session by stopping any recording
    and disconnecting from the media source. For example, if the
    source is a video server it ends the server session and
    disconnects, or if the source is a multicast channel it leaves
    that multicast group. If this function returns successfully, the
    recorder state has changed to \ref STATE_IDLE. If an exception is
    raised, the recorder state is unaffected.
    
    \note The only way to leave the \ref STATE_FAILED state is a
    call to \ref Close().
    
    \exception TToiSessionViolationException
    If the recorder is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \see Open()
    \see RecordAt()
    \see Record()
    \see TToiMediaRecorderState
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiMediaRecorder", "Close")

  def Record(self):
    """This function starts a recording of the stream. If this function
    returns successfully, the recorder state has changed to
    \ref STATE_RECORDING. If an exception is raised, the recorder state
    is unaffected.
    
    \exception TToiSessionViolationException
    If the recorder is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiOperationNotSupportedException
    Raised if the recorder is in a state in which this
    operation is not supported.  This is the case when in
    \ref STATE_FAILED, for example. What is supported in which
    state may in other cases vary depending on which media
    source is used.
    
    \see TToiMediaRecorderState
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiMediaRecorder", "Record")

  def RecordAt(self, offset):
    """\techpreview This method is a TECHNICAL PREVIEW, it may change.
    This function starts a new recording session and may reuse
    existing recorded content from another ongoing recording session,
    including a possible timeshift session, sharing the same source.
    
    Depending on the success of the connection attempt to the media
    source, the recorder will change state to \ref STATE_RECORDING.
    If something goes wrong it will end up in state \ref STATE_FAILED.
    In any respect, the application will be notified about those
    state transistions by calls to its
    \ref IToiMediaRecorderObserver::OnStateChanged() function.
    
    \param offset
    The offset (in milliseconds) relative to the current position,
    i.e. a value of 1000 means that one second back in time will be
    reused. If the specified offset time is not available less data
    than requested will be given.
    
    \exception TToiSessionViolationException
    If the recorder is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiOperationNotSupportedException
    Raised if the recorder is in a state in which this
    operation is not supported.  This is the case when in
    \ref STATE_FAILED, for example. What is supported in which
    state may in other cases vary depending on which media
    source is used.
    
    \see Close()
    \see TToiMediaRecorderState
    """
    __request = BinIpcParameterStack()
    # Write offset
    __request.push_uint32(offset)
    self.call(__request, "IToiMediaRecorder", "RecordAt")

  def SetParameter(self, name, value):
    """This function sets the value of a parameter. Depending on the
    current state, the new value is used differently. If the
    function is called when in idle state, the new value is used for
    all sessions started after the parameter is set. If the function
    is called in any other state but the idle state, the new value
    is applied to that session only and is discarded automatically
    when the session ends.
    
    \param name       The name of the parameter to set.
    
    \param value      The value to set the parameter to.
    
    \exception TToiInvalidArgumentException
    The parameter name is invalid or not
    recognized.
    
    \exception TToiPermissionDeniedException
    The application does not have permission to
    change the value of the parameter.
    
    \see UnsetParameter()
    \see GetParameter()
    """
    __request = BinIpcParameterStack()
    # Write name
    __request.push_string(name)
    # Write value
    __request.push_string(value)
    self.call(__request, "IToiMediaRecorder", "SetParameter")

  def UnsetParameter(self, name):
    """This function restores the value of a parameter to the default.
    Depending on the current state, the new value is used
    differently.  If the function is called when in idle state, all
    sessions started after the parameter is restored will use the
    default value. If the function is called in any other state but
    the idle state, the value is changed for that session only.
    
    \param name       The name of the parameter for which the value
    is to be restored to the default.
    
    \exception TToiInvalidArgumentException
    The parameter name is invalid or not
    recognized.
    
    \exception TToiPermissionDeniedException
    The application does not have permission to
    change the value of the parameter.
    
    \see SetParameter()
    \see GetParameter()
    """
    __request = BinIpcParameterStack()
    # Write name
    __request.push_string(name)
    self.call(__request, "IToiMediaRecorder", "UnsetParameter")

