# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiMediaRecorderObserver import ToiMediaRecorderObserver


class ToiMediaRecorderBase(KreaTvIpcCaller):

  # Enum TToiMediaRecorderState
  STATE_IDLE = 0
  STATE_CONNECTING = 1
  STATE_PAUSED = 2
  STATE_RECORDING = 3
  STATE_FAILED = 4

  def __init__(self, ipcproxy, object_address):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.set_object_address(object_address)

  def GetState(self):
    """Returns the current state of the media recorder instance.
    
    \retval state The current recorder state.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaRecorderBase", "GetState")
    __result = []
    # Read state
    __result.append(__response.pop_int32())
    return __result[0]

  def GetSessionId(self):
    """Returns the session identifier. This identifier can be used to open a
    session object with \ref IToiMediaService::OpenSession().
    
    \retval id        The session identifier.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \see IToiMediaService::OpenSession()
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaRecorderBase", "GetSessionId")
    __result = []
    # Read id
    __result.append(__response.pop_int32())
    return __result[0]

  def GetUrl(self):
    """Returns the URL for the media stream.
    
    \retval url The URL for the media stream.
    
    \exception TToiSessionViolationException
    If the recorder is in idle state a session violation
    exception is raised.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaRecorderBase", "GetUrl")
    __result = []
    # Read url
    __result.append(__response.pop_string())
    return __result[0]

  def GetAssetId(self):
    """Returns the asset used for recording.
    
    \retval assetId The asset identifier.
    
    \exception TToiSessionViolationException
    If the recorder is in idle state a session violation
    exception is raised.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaRecorderBase", "GetAssetId")
    __result = []
    # Read assetId
    __result.append(__response.pop_string())
    return __result[0]

  def GetParameter(self, name):
    """Get the current value of a parameter.
    
    \param name         Name of the requested parameter.
    
    \retval value       The current value of the parameter.
    
    \exception TToiInvalidArgumentException
    The parameter name is invalid or not
    recognized.
    
    \see IToiMediaRecorder::SetParameter()
    \see IToiMediaRecorder::UnsetParameter()
    \see SubscribeParameter()
    """
    __request = BinIpcParameterStack()
    # Write name
    __request.push_string(name)
    __response = self.call(__request, "IToiMediaRecorderBase", "GetParameter")
    __result = []
    # Read value
    __result.append(__response.pop_string())
    return __result[0]

  def SubscribeParameter(self, address, name):
    """This function makes the application subscribe to updates of a
    specific named parameter. When the subscription is made there
    will be an immediate call to
    \ref IToiMediaRecorderObserver::OnParameterChanged() if there is a
    value available.
    
    \param address    The observer address of the parameter that is
    subscribed.
    
    \param name       The name of the parameter for which changes
    the application wants to subscribe to.
    
    \exception TToiInvalidArgumentException
    The parameter name is invalid or not
    recognized.
    
    \see UnsubscribeParameter()
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    # Write name
    __request.push_string(name)
    self.call(__request, "IToiMediaRecorderBase", "SubscribeParameter")

  def UnsubscribeParameter(self, address, name):
    """This function undoes the effect of a previous call to
    \ref SubscribeParameter().
    
    \param address    The observer address of the parameter that is
    unsubscribed.
    
    \param name        The name of the parameter that the
    application is no longer interested in.
    
    \exception TToiInvalidArgumentException
    The parameter name is invalid or not
    recognized.
    
    \see SubscribeParameter()
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    # Write name
    __request.push_string(name)
    self.call(__request, "IToiMediaRecorderBase", "UnsubscribeParameter")

  def ReleaseInstance(self):
    """This function releases the resources used by this instance.
    When this function returns all observers has also been released.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiMediaRecorderBase", "ReleaseInstance")

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiMediaRecorderObserver.
    The observer will receive events sent by this IToiMediaRecorderBase instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiMediaRecorderBase can find an
    IToiMediaRecorderObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiMediaRecorderObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiMediaRecorderBase", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiMediaRecorderBase
    can find the registered IToiMediaRecorderObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiMediaRecorderObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiMediaRecorderBase", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiMediaRecorderObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiMediaRecorderBase", "RegisterObserver")
    return observer
