# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiPlatformObserver import ToiPlatformObserver


class ToiPlatformService(KreaTvIpcCaller):

  # Enum TToiResetFlag
  REFORMAT_INTERNAL_HDD = 0x1
  REMOVE_PERSISTENT_COOKIES = 0x2
  REMOVE_PORTAL_DATABASES = 0x4
  REMOVE_PERSISTENT_INFORMATION_OBJECTS = 0x8
  RESET_TR069_DATA_MODEL = 0x10
  CUSTOM_POST_PROCEDURE = 0x20
  REMOVE_CHANNELS = 0x40
  FULL_RESET = 0xff

  # Enum TToiComponentState
  COMPONENT_STATE_INACTIVE = 0x1
  COMPONENT_STATE_PENDING_INACTIVE = 0x2
  COMPONENT_STATE_PENDING_ACTIVE = 0x3
  COMPONENT_STATE_ACTIVE = 0x4

  def __init__(self, ipcproxy):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.lookup("PlatformService")

  def RebootNow(self):
    """Reboot the platform immediately in a controlled way. This is
    useful to force the usage of a new boot image after upgrade.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiPlatformService", "RebootNow")

  def RebootAtNextStandby(self):
    """Reboot the platform the next time the system goes from active to
    standby state. This is used for starting a new boot image in a
    way that is less disturbing to the end user.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiPlatformService", "RebootAtNextStandby")

  def SetStandby(self, mode):
    """Enter or leave standby. In standby all video and audio outputs
    are disabled. Applications are stopped or left running depending
    on the system configuration. If the system is already in the
    requested mode the call will have no effect.
    
    \param mode        The requested mode. True corresponds to
    standby, while false corresponds to normal
    operation.
    """
    __request = BinIpcParameterStack()
    # Write mode
    __request.push_bool(mode)
    self.call(__request, "IToiPlatformService", "SetStandby")

  def FactoryReset(self, flags):
    """Perform factory reset. Which parts of the persistent storage that
    is reset to factory state is dependent on which flags that are given
    to the function call.
    After the reset is perform, the STB will reboot to ensure that it boots
    up in a factory reset state.
    
    \param flags      Flags indicating which parts to reset.
    \see TToiResetFlag.
    
    \exception TToiOperationNotSupportedException
    Raised if some error occured.
    
    """
    __request = BinIpcParameterStack()
    # Write flags
    __request.push_int32(flags)
    self.call(__request, "IToiPlatformService", "FactoryReset")

  def RegisterComponentUsage(self, applicationId, componentName):
    """\techpreview This method is a TECHNICAL PREVIEW, it may change.
    
    Registers interest in a dynamic component. If not already active this
    method activates the dynamic component with the given
    identifier. The calling application should register itself as an
    IToiPlatformObserver and listen to
    \ref IToiPlatformObserver::OnComponentInfoChanged() to know when the
    component is activated. If the component has already been activated
    by another user (indicated by the active flag) no event will be sent.
    \see IToiPlatformObserver
    
    \param applicationId  The identifier of the calling application.
    
    \param componentName  The name of the dynamic component.
    
    \retval active        A flag indicating if the component is already
    active, i.e. no need to wait for an active state
    event.
    
    \exception TToiInvalidArgumentException
    Raised if the component name is not valid.
    
    \exception TToiInvalidPreconditionException
    Raised if the component is in state
    \ref COMPONENT_STATE_PENDING_INACTIVE.
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write componentName
    __request.push_string(componentName)
    __response = self.call(__request, "IToiPlatformService", "RegisterComponentUsage")
    __result = []
    # Read active
    __result.append(__response.pop_bool())
    return __result[0]

  def ReleaseComponentUsage(self, applicationId, componentName):
    """\techpreview This method is a TECHNICAL PREVIEW, it may change.
    
    Releases interest in a dynamic component with the given identifier.
    All resources allocated by the dynamic component will be
    released when it becomes inactive. If another user still has interest
    in the component it will not be deactivated (indicated by the active
    flag).
    
    \see IToiPlatformObserver
    
    \param applicationId  The identifier of the calling application.
    
    \param componentName    The name of the dynamic component.
    
    \retval active        A flag indicating if the component is still active
    after release due to another user, i.e. the
    component will be inactivated only when all users
    have released the component.
    
    \exception TToiInvalidArgumentException
    Raised if the component name is not valid.
    
    \exception TToiInvalidPreconditionException
    Raised if the user has not called
    \ref IToiPlatformService::RegisterComponentUsage().
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write componentName
    __request.push_string(componentName)
    __response = self.call(__request, "IToiPlatformService", "ReleaseComponentUsage")
    __result = []
    # Read active
    __result.append(__response.pop_bool())
    return __result[0]

  def GetComponentsInfo(self):
    """\techpreview This method is a TECHNICAL PREVIEW, it may change.
    
    Returns information of available platform components.
    
    \retval infoSequence  A sequence of component information.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiPlatformService", "GetComponentsInfo")
    __result = []
    # Read infoSequence
    infoSequence = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["State"] = __response.pop_int32()
      temp_l["IsDynamic"] = __response.pop_bool()
      temp_l["Name"] = __response.pop_string()
      infoSequence.append(temp_l)
    __result.append(infoSequence)
    return __result[0]

  def GetComponentInfo(self, componentName):
    """\techpreview This method is a TECHNICAL PREVIEW, it may change.
    
    Returns information for a platform component.
    
    \param componentName   The name of the dynamic component.
    
    \retval info            Component information.
    
    \exception TToiInvalidArgumentException
    Raised if the component name is not valid.
    """
    __request = BinIpcParameterStack()
    # Write componentName
    __request.push_string(componentName)
    __response = self.call(__request, "IToiPlatformService", "GetComponentInfo")
    __result = []
    # Read info
    info = {}
    info["State"] = __response.pop_int32()
    info["IsDynamic"] = __response.pop_bool()
    info["Name"] = __response.pop_string()
    __result.append(info)
    return __result[0]

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiPlatformObserver.
    The observer will receive events sent by this IToiPlatformService instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiPlatformService can find an
    IToiPlatformObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiPlatformObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiPlatformService", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiPlatformService
    can find the registered IToiPlatformObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiPlatformObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiPlatformService", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiPlatformObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiPlatformService", "RegisterObserver")
    return observer
