# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiSchedulerObserver import ToiSchedulerObserver


class ToiSchedulerService(KreaTvIpcCaller):

  # Enum TToiConflictStrategy
  CONFLICT_STRATEGY_FAIL = 0
  CONFLICT_STRATEGY_SKIP_BOOKINGS = 1

  # Enum TToiSolutionType
  SOLUTION_TYPE_NONE = 0
  SOLUTION_TYPE_REMOVE = 1

  # Const BOOKING_ID_NONE
  BOOKING_ID_NONE = 0

  # Const SERIES_ID_NONE
  SERIES_ID_NONE = 0

  # Const TIME_NOW
  TIME_NOW = -1

  # Const WEEKDAY_MONDAY
  WEEKDAY_MONDAY = 1

  # Const WEEKDAY_TUESDAY
  WEEKDAY_TUESDAY = 2

  # Const WEEKDAY_WEDNESDAY
  WEEKDAY_WEDNESDAY = 4

  # Const WEEKDAY_THURSDAY
  WEEKDAY_THURSDAY = 8

  # Const WEEKDAY_FRIDAY
  WEEKDAY_FRIDAY = 16

  # Const WEEKDAY_SATURDAY
  WEEKDAY_SATURDAY = 32

  # Const WEEKDAY_SUNDAY
  WEEKDAY_SUNDAY = 64

  # Const WEEKDAY_ALL_WEEKDAYS
  WEEKDAY_ALL_WEEKDAYS = 127

  def __init__(self, ipcproxy):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.lookup("SchedulerService")

  def Schedule(self, category, activity, start, duration):
    """Requests scheduling of a new booking.
    \param category
    The category info for the booking.
    \param activity
    The activity type for the booking.
    \param start
    The start time for the booking. Seconds since the
    Unix epoch.
    \param duration
    The duration time in seconds for the booking.
    \retval id
    A generated booking identifier that will be
    used in further communication with the
    client. If the request fails due to conflicts
    with other bookings this value will be set to
    \ref BOOKING_ID_NONE.
    \exception TToiInvalidArgumentException
    The exception will be thrown if the activity type does
    not exist or if start or time is in the past.
    \exception TToiNotEnoughResourcesException
    The exception will be thrown if the schedule request
    fails because requested resources exceeds the amount of
    resources available on the system.
    
    """
    __request = BinIpcParameterStack()
    # Write category
    __request.push_string(category)
    # Write activity
    __request.push_string(activity)
    # Write start
    __request.push_int32(start)
    # Write duration
    __request.push_int32(duration)
    __response = self.call(__request, "IToiSchedulerService", "Schedule")
    __result = []
    # Read id
    __result.append(__response.pop_int32())
    return __result[0]

  def ScheduleSeries(self, category, activity, start, duration, weekdays, numberOfOccurrences, conflictStrategy):
    """\techpreview This method is a TECHNICAL PREVIEW, it may change.
    Requests scheduling of a recurrent booking.
    \param category
    The category info for the series.
    \param activity
    The activity type for the series.
    \param start
    The start time for the first occurence of the
    series. Seconds since the Unix epoch.
    \param duration
    The duration time in seconds for each occurence
    of the series.
    \param weekdays
    Weekday mask indicating which days the recurring
    series shall happen.
    \param numberOfOccurrences
    The number of occurrences for the series.
    \param conflictStrategy
    The type of conflict handling strategy to use if a
    conflict with previously scheduled bookings is detected.
    \retval id
    A generated series identifier that will be used in
    further communication with the client. If the request
    fails due to conflicts with other bookings this value
    will be set to \ref SERIES_ID_NONE.
    \exception TToiInvalidArgumentException
    The exception will be thrown if the activity type does
    not exist or if start or time is in the past.
    \exception TToiNotEnoughResourcesException
    The exception will be thrown if the schedule request
    fails because requested resources exceeds the amount of
    resources available on the system.
    """
    __request = BinIpcParameterStack()
    # Write category
    __request.push_string(category)
    # Write activity
    __request.push_string(activity)
    # Write start
    __request.push_int32(start)
    # Write duration
    __request.push_int32(duration)
    # Write weekdays
    __request.push_int32(weekdays)
    # Write numberOfOccurrences
    __request.push_int32(numberOfOccurrences)
    # Write conflictStrategy
    __request.push_int32(conflictStrategy)
    __response = self.call(__request, "IToiSchedulerService", "ScheduleSeries")
    __result = []
    # Read id
    __result.append(__response.pop_int32())
    return __result[0]

  def Reschedule(self, id, category, activity, start, duration):
    """Requests changing and rescheduling of a booking.
    \param id
    The identifier of the booking which needs to
    be rescheduled.
    \param category
    The new category info for the booking.
    \param activity
    The new activity type for the booking.
    \param start
    The start time for the booking. Seconds since the
    Unix epoch.
    \param duration
    The duration time in seconds for the booking.
    \exception TToiInvalidArgumentException
    The exception will be thrown if the booking identifier
    does not exist, if changing to a time in the past, or if
    the new activity used other resources.
    \exception TToiNotEnoughResourcesException
    The exception will be thrown if the reschedule request
    fails due to resource conflict.
    \note
    If a booking has been started, and the reschedule
    function is called, the scheduler does not care about
    the start time as long as the end time is after current
    time. The new end time is calculated based on the newly
    provided start time and duration.
    When \ref GetBooking() is called after the reschedule
    of a started booking, the start time is the same as
    when the booking start event was fired, and only the
    duration is updated and calculated from the first start
    time and the new end time.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    # Write category
    __request.push_string(category)
    # Write activity
    __request.push_string(activity)
    # Write start
    __request.push_int32(start)
    # Write duration
    __request.push_int32(duration)
    self.call(__request, "IToiSchedulerService", "Reschedule")

  def Remove(self, id):
    """Remove a booking.
    \param id
    The identifier of the booking to be removed.
    \exception TToiInvalidArgumentException
    The exception will be thrown if the booking
    identifier does not exist.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    self.call(__request, "IToiSchedulerService", "Remove")

  def RemoveSeries(self, id):
    """\techpreview This method is a TECHNICAL PREVIEW, it may change.
    Remove a booked recurrent scheduling.
    \param id
    The identifier of the recurrent series to be removed.
    \exception TToiInvalidArgumentException
    The exception will be thrown if the series identifier
    does not exist.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    self.call(__request, "IToiSchedulerService", "RemoveSeries")

  def GetConflicts(self, id, category, activity, start, duration):
    """Detect conflicts without actually createing a booking.
    \param id
    The identifier of the booking which need to
    be tested (in case of a rescheduling to be performed) or
    \ref BOOKING_ID_NONE (in case of a new booking to be
    performed).
    \param category
    The category info for the attempted booking.
    \param activity
    The activity type for the attempted booking.
    \param start
    The start time for the attempted booking. Seconds since
    the Unix epoch.
    \param duration
    The duration time in seconds for the attempted booking.
    \retval conflicts
    If the attempt fails, this parameter will
    contain lists of existing bookings which have
    conflicts with the schedule request. Time and
    resources are considered here to determine
    whether conflicts exist.
    
    \exception TToiInvalidArgumentException
    The exception will be thrown if the identifer cannot be
    found.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    # Write category
    __request.push_string(category)
    # Write activity
    __request.push_string(activity)
    # Write start
    __request.push_int32(start)
    # Write duration
    __request.push_int32(duration)
    __response = self.call(__request, "IToiSchedulerService", "GetConflicts")
    __result = []
    # Read conflicts
    conflicts = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["AffectedBookings"] = []
      for l in range(__response.pop_int32()):
        temp_l["AffectedBookings"].append(__response.pop_int32())
      temp_l["Type"] = __response.pop_int32()
      conflicts.append(temp_l)
    __result.append(conflicts)
    return __result[0]

  def GetSeriesConflicts(self, id, category, activity, start, duration, weekdays, occurrence):
    """\techpreview This method is a TECHNICAL PREVIEW, it may change.
    Detect conflicts for a new series booking or a series booking
    about to be rescheduled, but without actually booking any activities.
    
    \param id
    The identifier of the series which need to be
    tested (in case of a rescheduling to be
    performed) or \ref SERIES_ID_NONE (in case of a new
    scheduling to be performed).
    \param category
    The category info for the attempted series.
    \param activity
    The activity type for the attempted series.
    \param start
    The start time for the attempted
    series. Seconds since the Unix epoch.
    \param duration
    The duration time in seconds for each occurence
    in the attempted series.
    \param weekdays
    Weekday mask indicating which days the series
    shall happen.
    \param occurrence
    The number occurrences to be tested.  Must be
    greater than zero.
    \retval conflicts
    If the attempt fails, this parameter will
    contain lists of existing bookings which have conflicts
    with the schedule request. Time and resources are
    considered here to determine whether conflicts exist.
    \exception TToiInvalidArgumentException
    The exception will be thrown if the series can not be found.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    # Write category
    __request.push_string(category)
    # Write activity
    __request.push_string(activity)
    # Write start
    __request.push_int32(start)
    # Write duration
    __request.push_int32(duration)
    # Write weekdays
    __request.push_int32(weekdays)
    # Write occurrence
    __request.push_int32(occurrence)
    __response = self.call(__request, "IToiSchedulerService", "GetSeriesConflicts")
    __result = []
    # Read conflicts
    conflicts = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["AffectedBookings"] = []
      for l in range(__response.pop_int32()):
        temp_l["AffectedBookings"].append(__response.pop_int32())
      temp_l["Type"] = __response.pop_int32()
      conflicts.append(temp_l)
    __result.append(conflicts)
    return __result[0]

  def GetNextStartTime(self, categoryExpression):
    """Get the expected starting time for next scheduled booking with matching
    category.
    \param categoryExpression
    The category to use as filter. Wildcard "*" is partly
    supported. \ref SetCategorySubscription().
    \retval nextStart
    The expected starting time of next scheduled booking. If
    a booking is currently ongoing, 0 will be returned.
    """
    __request = BinIpcParameterStack()
    # Write categoryExpression
    __request.push_string(categoryExpression)
    __response = self.call(__request, "IToiSchedulerService", "GetNextStartTime")
    __result = []
    # Read nextStart
    __result.append(__response.pop_int32())
    return __result[0]

  def GetBooking(self, id):
    """Get information about a booking.
    \param id
    The booking identifier to find information about.
    \retval booking
    The booking information.
    \exception TToiInvalidArgumentException
    The exception will be thrown if can not find
    a booking with the identifier.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    __response = self.call(__request, "IToiSchedulerService", "GetBooking")
    __result = []
    # Read booking
    booking = {}
    booking["SeriesId"] = __response.pop_int32()
    booking["Duration"] = __response.pop_int32()
    booking["Start"] = __response.pop_int32()
    booking["ActivityType"] = __response.pop_string()
    booking["Category"] = __response.pop_string()
    booking["Id"] = __response.pop_int32()
    __result.append(booking)
    return __result[0]

  def GetSeries(self, id):
    """\techpreview This method is a TECHNICAL PREVIEW, it may change.
    Get the info of the series with a certain series identifier
    \param id
    The series identifier to find information about.
    \retval series
    The info of the scheduled series
    \exception TToiInvalidArgumentException
    The exception will be thrown if the series can not be
    found.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    __response = self.call(__request, "IToiSchedulerService", "GetSeries")
    __result = []
    # Read series
    series = {}
    series["OriginalNumberOfOccurences"] = __response.pop_int32()
    series["Weekdays"] = __response.pop_int32()
    series["Duration"] = __response.pop_int32()
    series["Start"] = __response.pop_int32()
    series["ActivityType"] = __response.pop_string()
    series["Category"] = __response.pop_string()
    series["Id"] = __response.pop_int32()
    __result.append(series)
    return __result[0]

  def GetBookingIds(self, categoryExpression, start, stop):
    """Get the bookings during start and stop time, with the same category.
    \param categoryExpression
    The category to use as filter. Wildcard "*" is partly
    supported. \ref SetCategorySubscription().
    \param start
    Start time of the booking in seconds since the Unix
    epoch. A value of 0 means no limit.
    \param stop
    End time of the booking in seconds since the Unix epoch.
    A value of 0 means no limit.
    \retval bookings
    The sequence of booking identifiers found.
    \exception TToiInvalidArgumentException
    The exception will be thrown if start is later than stop.
    /** \note The total buffer can extend the maximum ipc message size. 
    """
    __request = BinIpcParameterStack()
    # Write categoryExpression
    __request.push_string(categoryExpression)
    # Write start
    __request.push_int32(start)
    # Write stop
    __request.push_int32(stop)
    __response = self.call(__request, "IToiSchedulerService", "GetBookingIds")
    __result = []
    # Read bookings
    bookings = []
    for k in range(__response.pop_int32()):
      bookings.append(__response.pop_int32())
    __result.append(bookings)
    return __result[0]

  def GetSeriesIds(self, categoryExpression):
    """\techpreview This method is a TECHNICAL PREVIEW, it may change.
    Get the all series identifiers with the given category.
    \param categoryExpression
    The category to use as filter. Wildcard "*" is partly
    supported. \ref SetCategorySubscription().
    
    \retval seriesIds
    The sequence of series identifiers found.
    """
    __request = BinIpcParameterStack()
    # Write categoryExpression
    __request.push_string(categoryExpression)
    __response = self.call(__request, "IToiSchedulerService", "GetSeriesIds")
    __result = []
    # Read seriesIds
    seriesIds = []
    for k in range(__response.pop_int32()):
      seriesIds.append(__response.pop_int32())
    __result.append(seriesIds)
    return __result[0]

  def GetBookingIdsForSeries(self, seriesId):
    """\techpreview This method is a TECHNICAL PREVIEW, it may change.
    Get the bookings associated with a series.
    \param seriesId
    The identifier of the series to retrieve bookings for
    \retval bookings
    The sequence of booking identifiers found.
    \exception TToiInvalidArgumentException
    Raised if the identifier does not exist.
    """
    __request = BinIpcParameterStack()
    # Write seriesId
    __request.push_int32(seriesId)
    __response = self.call(__request, "IToiSchedulerService", "GetBookingIdsForSeries")
    __result = []
    # Read bookings
    bookings = []
    for k in range(__response.pop_int32()):
      bookings.append(__response.pop_int32())
    __result.append(bookings)
    return __result[0]

  def AddTrigger(self, id, delta):
    """Add triggers to a booking. For information on triggers, refer
    to the comments in \ref IToiSchedulerObserver.
    \param id
    The booking identifier.
    \param delta
    The delta time in seconds since start point of an
    booking, may be negative to trigger a callback
    before the booking shall occur.
    \exception TToiInvalidArgumentException
    Raised if the booking does not exist.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    # Write delta
    __request.push_int32(delta)
    self.call(__request, "IToiSchedulerService", "AddTrigger")

  def RemoveTrigger(self, id, delta):
    """Remove trigger from a booking.
    \param id
    The booking identifier.
    \param delta
    The delta time in seconds since start point of an
    booking, may be negative to trigger a callback
    before the booking shall occur.
    \exception TToiInvalidArgumentException
    Raised if the booking does not exist.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    # Write delta
    __request.push_int32(delta)
    self.call(__request, "IToiSchedulerService", "RemoveTrigger")

  def AddTriggerOnBookings(self, ids, delta):
    """Add triggers to a number of bookings. If such a trigger is already
    define for a booking, no new trigger will be added.
    \param ids
    A sequence of booking identifiers that should use the
    new trigger.
    \param delta
    The delta time in seconds since start point of a
    booking.
    \exception TToiInvalidArgumentException
    Raised if one or more bookings can not be found.
    """
    __request = BinIpcParameterStack()
    # Write ids
    for iter_k in ids[::-1]:
      __request.push_int32(iter_k)
    __request.push_int32(len(ids))
    # Write delta
    __request.push_int32(delta)
    self.call(__request, "IToiSchedulerService", "AddTriggerOnBookings")

  def RemoveTriggerOnBookings(self, ids, delta):
    """Remove triggers from a number of bookings. If there is no trigger with
    the specified delta - nothing will be changed for that booking.
    \param ids
    A sequence of booking identifiers that should remove
    the trigger.
    \param delta
    The delta time in seconds since start point of an
    booking.
    \exception TToiInvalidArgumentException
    Raised if one or more bookings can not be found.
    """
    __request = BinIpcParameterStack()
    # Write ids
    for iter_k in ids[::-1]:
      __request.push_int32(iter_k)
    __request.push_int32(len(ids))
    # Write delta
    __request.push_int32(delta)
    self.call(__request, "IToiSchedulerService", "RemoveTriggerOnBookings")

  def GetTriggers(self, id):
    """Get triggers associated with a booking.
    \param id
    The booking identifier.
    \retval deltas
    Sequences of trigger delta times associated with the
    booking.
    \exception TToiInvalidArgumentException
    Raised if the booking does not exist.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    __response = self.call(__request, "IToiSchedulerService", "GetTriggers")
    __result = []
    # Read deltas
    deltas = []
    for k in range(__response.pop_int32()):
      deltas.append(__response.pop_int32())
    __result.append(deltas)
    return __result[0]

  def SetParameter(self, id, name, value):
    """Add user defined parameters to a booking.
    \param id
    The booking identifier.
    \param name
    The name of the parameter to set. If a parameter with the
    same name already exists, it will be overwritten.
    \param value
    The value of the parameter to set.
    \exception TToiInvalidArgumentException
    Raised if the booking does not exist.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    # Write name
    __request.push_string(name)
    # Write value
    __request.push_string(value)
    self.call(__request, "IToiSchedulerService", "SetParameter")

  def UnsetParameter(self, id, name):
    """Remove user defined parameters from a booking.
    \param id
    The booking identifier.
    \param name
    The name of the parameter to remove.
    \exception TToiInvalidArgumentException
    Raised if the booking does not exist.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    # Write name
    __request.push_string(name)
    self.call(__request, "IToiSchedulerService", "UnsetParameter")

  def GetParameter(self, id, name):
    """Get user defined parameters associated with a booking.
    
    \param id
    The booking identifier.
    \param name
    The name of the parameter.
    \retval value
    The value of the parameter.
    \exception TToiInvalidArgumentException
    Raised if the booking does not exist.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    # Write name
    __request.push_string(name)
    __response = self.call(__request, "IToiSchedulerService", "GetParameter")
    __result = []
    # Read value
    __result.append(__response.pop_string())
    return __result[0]

  def GetParameterNames(self, id):
    """Get all user defined parameters names of a booking
    \param id
    The booking identifier.
    \retval names
    A sequence with names of the parameters.
    \exception TToiInvalidArgumentException
    Raised if the booking does not exist.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    __response = self.call(__request, "IToiSchedulerService", "GetParameterNames")
    __result = []
    # Read names
    names = []
    for k in range(__response.pop_int32()):
      names.append(__response.pop_string())
    __result.append(names)
    return __result[0]

  def SetParametersOnBookings(self, ids, names, values):
    """Add user defined parameters to a set of bookings.
    \param ids
    A sequence of booking identifiers to update with parameters.
    \param names
    A sequence of parameter names to be set. Existing parameters
    with matching names will be overwritten.
    \param values
    A sequence of parameter values to be set,
    matching the parameter names.
    \exception TToiInvalidArgumentException
    Raised if any of the bookings do not exist.
    """
    __request = BinIpcParameterStack()
    # Write ids
    for iter_k in ids[::-1]:
      __request.push_int32(iter_k)
    __request.push_int32(len(ids))
    # Write names
    for iter_k in names[::-1]:
      __request.push_string(iter_k)
    __request.push_int32(len(names))
    # Write values
    for iter_k in values[::-1]:
      __request.push_string(iter_k)
    __request.push_int32(len(values))
    self.call(__request, "IToiSchedulerService", "SetParametersOnBookings")

  def SetSeriesParameter(self, id, name, value):
    """\techpreview This method is a TECHNICAL PREVIEW, it may change.
    Add user defined parameters to a scheduled series.
    \param id
    The identifier of the series.
    \param name
    The name of the parameter to set. If a parameter with the
    same name already exists, it will be overwritten.
    \param value
    The value of the parameter to set.
    \exception TToiInvalidArgumentException
    Raised if the series does not exist.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    # Write name
    __request.push_string(name)
    # Write value
    __request.push_string(value)
    self.call(__request, "IToiSchedulerService", "SetSeriesParameter")

  def UnsetSeriesParameter(self, id, name):
    """\techpreview This method is a TECHNICAL PREVIEW, it may change.
    Remove user defined parameters from a scheduled series.
    \param id
    The identifier of the series.
    \param name
    The name of the parameter to remove.
    \exception TToiInvalidArgumentException
    Raised if the series does not exist.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    # Write name
    __request.push_string(name)
    self.call(__request, "IToiSchedulerService", "UnsetSeriesParameter")

  def GetSeriesParameter(self, id, name):
    """\techpreview This method is a TECHNICAL PREVIEW, it may change.
    Get user defined parameters associated with a series.
    \param id
    The identifier of the series.
    \param name
    The name of the parameter.
    \retval value
    The value of the parameter.
    \exception TToiInvalidArgumentException
    Raised if the series does not exist.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    # Write name
    __request.push_string(name)
    __response = self.call(__request, "IToiSchedulerService", "GetSeriesParameter")
    __result = []
    # Read value
    __result.append(__response.pop_string())
    return __result[0]

  def GetSeriesParameterNames(self, id):
    """\techpreview This method is a TECHNICAL PREVIEW, it may change.
    Get all user defined parameters names associated with a series.
    \param id
    The identifier of the series.
    \retval names
    A sequence of names of the parameters.
    \exception TToiInvalidArgumentException
    Raised if the series does not exist.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_int32(id)
    __response = self.call(__request, "IToiSchedulerService", "GetSeriesParameterNames")
    __result = []
    # Read names
    names = []
    for k in range(__response.pop_int32()):
      names.append(__response.pop_string())
    __result.append(names)
    return __result[0]

  def SetCategorySubscription(self, address, categoryExpression):
    """This method is used to filter which categories the application is
    notified about.
    \param address
    The address of an already registered
    \ref IToiSchedulerObserver.
    \param categoryExpression The category expression of the bookings to be
    notified about. Wildcard "*" is supported when it is at the
    beginning or end of a string. And a single "*" match all category.
    Supported wildcard is like that "*dvr", "dvr*", "*". Otherwise it
    is treated as an ordinary character.
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    # Write categoryExpression
    __request.push_string(categoryExpression)
    self.call(__request, "IToiSchedulerService", "SetCategorySubscription")

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiSchedulerObserver.
    The observer will receive events sent by this IToiSchedulerService instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiSchedulerService can find an
    IToiSchedulerObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiSchedulerObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiSchedulerService", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiSchedulerService
    can find the registered IToiSchedulerObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiSchedulerObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiSchedulerService", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiSchedulerObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiSchedulerService", "RegisterObserver")
    return observer
