# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiSoftwareObserver import ToiSoftwareObserver


class ToiSoftwareService(KreaTvIpcCaller):

  # Enum TToiDependencyType
  ROOTDISK = 0
  EXECUTION = 1

  def __init__(self, ipcproxy):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.lookup("SoftwareService")

  def AddIips(self, iipDistributionUrl):
    """This method makes it possible to add IIPs at runtime. Note that
    the IIPs are added as information entries, they are not
    installed. A callback is sent to all registered observers via the
    \ref IToiSoftwareObserver::OnAddIipsResponse() method with information
    on if the adding of the IIPs was successful or not. More about
    IIP distributions can be found in the Operation and Maintainance
    Guide.
    
    \param iipDistributionUrl The URL of the IIP distribution.
    
    \exception TToiInvalidArgumentException Raised when the
    URL is invalid.
    
    \see IToiSoftwareObserver::OnAddIipsResponse
    """
    __request = BinIpcParameterStack()
    # Write iipDistributionUrl
    __request.push_string(iipDistributionUrl)
    self.call(__request, "IToiSoftwareService", "AddIips")

  def SetIipFilter(self, address, iip):
    """Specifies the IIPs that callbacks will be receieved for. In order to
    get callbacks a call to RegisterObserver must be done. Only registered
    observers are allowed to set an IIP filter.
    
    \param address   The address of a registered observer for which IIPs
    should be filtered.
    
    \param iip       The name of the IIP for which to get callbacks.
    A wildcard (*) is allowed at the end of the iip
    filter string  which will result in callbacks
    concerning all matching IIPs.
    
    \exception TToiInvalidArgumentException
    Raised if the specified address is illegal.
    
    \see RegisterObserver()
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    # Write iip
    __request.push_string(iip)
    self.call(__request, "IToiSoftwareService", "SetIipFilter")

  def AreDependenciesFulfilled(self, iipName, dependencyType):
    """This method can be called to find out if the dependencies of the
    IIP are fulfilled, i.e. if all of the IIPs that the IIP depends
    on (of a certain dependency type) have been successfully
    addreffed / are installed. Can e.g. be used to find out whether
    the IIP is ready for execution.
    
    \param iipName The name of the IIP.
    
    \param dependencyType The dependency type of the IIP.
    
    \retval fulfilled Set to <i>true</i> if the IIPs dependencies
    are fulfilled.
    
    \exception TToiInvalidArgumentException Raised if
    iipName is unknown.
    """
    __request = BinIpcParameterStack()
    # Write iipName
    __request.push_string(iipName)
    # Write dependencyType
    __request.push_int32(dependencyType)
    __response = self.call(__request, "IToiSoftwareService", "AreDependenciesFulfilled")
    __result = []
    # Read fulfilled
    __result.append(__response.pop_bool())
    return __result[0]

  def AddRef(self, iipName, dependencyType):
    """Call this method to add a reference to an IIP. Adding references
    is only allowed on IIPs that has not been "addreffed" (with this
    dependencyType) before. If the IIP has not been installed on the
    KreaTV IP-STB, it and all its dependencies will be when this
    method is called. This method returns immediately and the
    installation is executed in the background. All IIPs required by
    this IIP that has not been installed will also be installed. If
    an IIP status observer has been registered with the
    \ref IToiSoftwareService::RegisterObserver() method a callback with
    info on if the \ref AddRef() was successful or not is sent via the
    \ref IToiSoftwareObserver::OnAddRefResponse() method.
    
    \param iipName The name of the IIP.
    
    \param dependencyType The type of dependency that the AddRef of
    the IIP will follow.
    
    \exception TToiInvalidArgumentException Raised if
    iipName is unknown.
    
    \see IToiSoftwareObserver::OnAddRefResponse
    """
    __request = BinIpcParameterStack()
    # Write iipName
    __request.push_string(iipName)
    # Write dependencyType
    __request.push_int32(dependencyType)
    self.call(__request, "IToiSoftwareService", "AddRef")

  def Release(self, iipName, dependencyType):
    """Call this method to release an IIP (remove a reference from an
    IIP). Releasing is only allowed on IIPs that has been
    "addreffed" with the \ref IToiSoftwareService::AddRef() method
    before. If the IIP is not referenced anymore by any IIP after
    releasing it, it will be uninstalled from the KreaTV
    IP-STB. The same is valid for any of its dependencies. This
    method returns immediately and the uninstallation is executed in
    the background. If an IIP status observer has been registered
    with the \ref IToiSoftwareService::RegisterObserver() method a
    callback with info on if the release was successful or not is
    sent via the \ref IToiSoftwareObserver::OnReleaseResponse() method.
    
    \param iipName    The name of the IIP.
    
    \param dependencyType The type of dependency that the release of
    the IIP and its dependencies will follow.
    
    \exception TToiInvalidArgumentException Raised if
    iipName is unknown.
    
    \see IToiSoftwareObserver::OnReleaseResponse
    """
    __request = BinIpcParameterStack()
    # Write iipName
    __request.push_string(iipName)
    # Write dependencyType
    __request.push_int32(dependencyType)
    self.call(__request, "IToiSoftwareService", "Release")

  def GetContentVersion(self, iipName):
    """This method retrieves the content version of an IIP. The content
    version is the version of the IIP's content, i.e. e.g. binaries
    or firmware. Is e.g. used for DBL IIPs.
    
    \param iipName The name of the IIP.
    
    \retval installedVersion The version of the currently installed
    IIP's content.
    
    \retval availableVersion The version of the content of the IIP
    that is available for download.
    
    \exception TToiInvalidArgumentException Raised if
    iipName is unknown.
    """
    __request = BinIpcParameterStack()
    # Write iipName
    __request.push_string(iipName)
    __response = self.call(__request, "IToiSoftwareService", "GetContentVersion")
    __result = []
    # Read availableVersion
    __result.append(__response.pop_string())
    # Read installedVersion
    __result.append(__response.pop_string())
    return tuple(__result)

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiSoftwareObserver.
    The observer will receive events sent by this IToiSoftwareService instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiSoftwareService can find an
    IToiSoftwareObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiSoftwareObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiSoftwareService", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiSoftwareService
    can find the registered IToiSoftwareObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiSoftwareObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiSoftwareService", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiSoftwareObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiSoftwareService", "RegisterObserver")
    return observer
