# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack


class ToiStorageDevice(KreaTvIpcCaller):

  # Enum TToiFileSystemStatus
  FS_STATUS_OK = 0
  FS_STATUS_ERRORS_CORRECTED = 1
  FS_STATUS_ERRORS_LEFT_UNCORRECTED = 2
  FS_STATUS_MAJOR_FAILURE = 3

  def __init__(self, ipcproxy, object_address):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.set_object_address(object_address)

  def GetStorageInfo(self):
    """Returns storage info for this device.
    
    \retval info     The storage info.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiStorageDevice", "GetStorageInfo")
    __result = []
    # Read info
    info = {}
    info["IsApproved"] = __response.pop_bool()
    info["Temperature"] = __response.pop_int32()
    info["FreeCapacity"] = __response.pop_int64()
    info["Capacity"] = __response.pop_int64()
    info["Partitions"] = []
    for k in range(__response.pop_int32()):
      info["Partitions"].append(__response.pop_uint32())
    info["IsOverheated"] = __response.pop_bool()
    info["IsSpinning"] = __response.pop_bool()
    info["IsConnected"] = __response.pop_bool()
    info["IsInternal"] = __response.pop_bool()
    info["SerialNumber"] = __response.pop_string()
    info["Id"] = __response.pop_uint32()
    __result.append(info)
    return __result[0]

  def GetPartitionInfo(self, partitionId):
    """Returns partition info for a partition.
    
    \param partitionId Partition identifier.
    
    \retval info The partition info.
    
    \exception TToiInvalidArgumentException
    Raised if the partitionId is wrong.
    
    \exception TToiInvalidPreconditionException
    Raised if the storage is disconnected.
    """
    __request = BinIpcParameterStack()
    # Write partitionId
    __request.push_uint32(partitionId)
    __response = self.call(__request, "IToiStorageDevice", "GetPartitionInfo")
    __result = []
    # Read info
    info = {}
    info["IsReadWrite"] = __response.pop_bool()
    info["FsStatus"] = __response.pop_int32()
    info["FreeCapacity"] = __response.pop_int64()
    info["Capacity"] = __response.pop_int64()
    info["MountPoint"] = __response.pop_string()
    info["IsMounted"] = __response.pop_bool()
    info["UniqueId"] = __response.pop_string()
    info["Id"] = __response.pop_uint32()
    __result.append(info)
    return __result[0]

  def MountPartition(self, operationId, partitionId, mountPoint):
    """Mounts a partition at a specified path.
    
    \param operationId Operation identifier for this operation.
    
    \param partitionId Partition identifier.
    
    \param mountPoint  Mount point path.
    
    \exception TToiInvalidArgumentException
    Raised if the operationId or partitionId is wrong.
    
    \exception TToiInvalidPreconditionException
    Raised if the partition is already mounted
    or the storage is disconnected.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    # Write partitionId
    __request.push_uint32(partitionId)
    # Write mountPoint
    __request.push_string(mountPoint)
    self.call(__request, "IToiStorageDevice", "MountPartition")

  def UnmountPartition(self, operationId, partitionId):
    """Unmounts a partition.
    
    \param operationId Operation identifier for this operation.
    
    \param partitionId Partition identifier.
    
    \exception TToiInvalidArgumentException
    Raised if the operationId or partitionId is wrong.
    
    \exception TToiInvalidPreconditionException
    Raised if the partition has not been mounted
    or the storage is disconnected.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    # Write partitionId
    __request.push_uint32(partitionId)
    self.call(__request, "IToiStorageDevice", "UnmountPartition")

  def Reformat(self, operationId):
    """Reformat a device. This operation is only supported for external
    storage devices.
    
    \param operationId Operation identifier for this operation.
    
    \exception TToiInvalidArgumentException
    Raised if the operationId is wrong.
    
    \exception TToiInvalidPreconditionException
    Raised if the storage is disconnected.
    
    \exception TToiOperationNotSupportedException
    Raised if the operation is not supported on this device.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    self.call(__request, "IToiStorageDevice", "Reformat")

  def ScheduleReformatOnNextReboot(self, partitionId):
    """Schedule a reformat of a partition at next reboot. This
    operation is only supported for internal storage devices.
    
    \param partitionId The partition to format.
    
    \exception TToiInvalidArgumentException
    Raised if the partition does not exist.
    
    \exception TToiOperationNotSupportedException
    Raised if the operation is not supported on this device.
    """
    __request = BinIpcParameterStack()
    # Write partitionId
    __request.push_uint32(partitionId)
    self.call(__request, "IToiStorageDevice", "ScheduleReformatOnNextReboot")

  def ScheduleFileSystemRepairOnNextReboot(self, partitionId):
    """Schedule a forced file system check of a partition and repair it at next
    reboot. This operation is only supported for internal storage devices.
    
    \param partitionId The partition to be checked and repaired.
    
    \exception TToiInvalidArgumentException
    Raised if the partition does not exist.
    
    \exception TToiOperationNotSupportedException
    Raised if the operation is not supported on this device.
    """
    __request = BinIpcParameterStack()
    # Write partitionId
    __request.push_uint32(partitionId)
    self.call(__request, "IToiStorageDevice", "ScheduleFileSystemRepairOnNextReboot")

  def SpinDown(self):
    """Requests a spin down of a disk device. The platform may decide not to
    do this if the disk is used by platform.
    
    \exception TToiInvalidPreconditionException
    Raised if the storage is disconnected, already in spindown mode
    or if the device is not approved.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiStorageDevice", "SpinDown")

  def SpinUp(self):
    """Spins up a disk device.
    
    \exception TToiInvalidPreconditionException
    Raised if the storage is disconnected, already in spinup mode
    or if the device is not approved.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiStorageDevice", "SpinUp")

