# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiStorageObserver import ToiStorageObserver


class ToiStorageService(KreaTvIpcCaller):

  def __init__(self, ipcproxy):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.lookup("StorageService")

  def GetStorageOperationManager(self):
    """Returns the operation manager that handles asynchronous operations for
    this storage device.
    
    \retval address     Address to the \ref IToiOperationManager.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiStorageService", "GetStorageOperationManager")
    __result = []
    # Read address
    __result.append(__response.pop_string())
    return __result[0]

  def GetStorageDeviceIds(self):
    """Returns the identifiers for the currently connected storage devices.
    
    \retval identifiers    The storage identifiers.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiStorageService", "GetStorageDeviceIds")
    __result = []
    # Read identifiers
    identifiers = []
    for k in range(__response.pop_int32()):
      identifiers.append(__response.pop_uint32())
    __result.append(identifiers)
    return __result[0]

  def GetStorageDevice(self, id):
    """Returns the address to the storage device handler for a given
    storage identifier.
    
    \param id           Storage identifier.
    
    \retval address     Address to the \ref IToiStorageDevice.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_uint32(id)
    __response = self.call(__request, "IToiStorageService", "GetStorageDevice")
    __result = []
    # Read address
    __result.append(__response.pop_string())
    return __result[0]

  def Copy(self, operationId, source, target):
    """Copies a file from source to target.
    
    \param operationId
    Operation identifier, created by the operation manager.
    
    \param source  Source file path.
    
    \param target  Target path.
    
    \exception TToiInvalidArgumentException
    Raised if the operationId is illegal.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    # Write source
    __request.push_string(source)
    # Write target
    __request.push_string(target)
    self.call(__request, "IToiStorageService", "Copy")

  def Move(self, operationId, source, target):
    """Moves a file from source to target.
    
    \param operationId
    Operation identifier, created by operation manager.
    
    \param source  Source file path.
    
    \param target  Target path.
    
    \exception TToiInvalidArgumentException
    Raised if the operationId is illegal.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    # Write source
    __request.push_string(source)
    # Write target
    __request.push_string(target)
    self.call(__request, "IToiStorageService", "Move")

  def CreateFile(self, applicationId, fileName, fileType):
    """Creates a file. A limited number of simultaneous \ref IToiStorageFile
    objects are allowed. This functionality is primarily interesting for
    non C++ appliations (such as portal) that do not have access to the file
    system directly.
    
    \param applicationId
    The identifier of the calling application.
    
    \param fileName     The name of the file.
    Note: If the filetype is
    \ref IToiStorageFile::FILE_TYPE_TEMPORARY,
    a unique random string is automatically added to
    the end of the filename (but before any extension).
    The purpose of the filename in such case would merely
    be to describe its file-extension (e.g. '.jpeg').
    
    \param fileType     The type of file to be created.
    
    \retval address     The address to the \ref IToiStorageFile instance.
    
    \exception TToiInvalidArgumentException
    Raised if the fileType is invalid.
    \exception TToiFileException
    Raised if files cannot be created, i.e. no
    internal drive, no directory set up for
    temporary files, or the maximum number of
    simultaneous file objects is exceeded.
    
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write fileName
    __request.push_string(fileName)
    # Write fileType
    __request.push_int32(fileType)
    __response = self.call(__request, "IToiStorageService", "CreateFile")
    __result = []
    # Read address
    __result.append(__response.pop_string())
    return __result[0]

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiStorageObserver.
    The observer will receive events sent by this IToiStorageService instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiStorageService can find an
    IToiStorageObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiStorageObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiStorageService", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiStorageService
    can find the registered IToiStorageObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiStorageObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiStorageService", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiStorageObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiStorageService", "RegisterObserver")
    return observer
