# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack


from ToiVideoOutputConfiguration import ToiVideoOutputConfiguration

class ToiVideoOutputConfigurationSession(ToiVideoOutputConfiguration):

  def __init__(self, ipcproxy, object_address):
    ToiVideoOutputConfiguration.__init__(self, ipcproxy, object_address)

  def SetDisplayInfo(self, outputId, modes, aspectRatio):
    """Sets information about the display connected to a video output.
    This is primarily used to give information about displays that
    cannot provide this information themselves, but can also be used
    to override the video output capabilites of a display, e.g. if a
    TV set supports other settings than the EDID indicates.
    
    \param outputId          Video output identifier.
    \param modes             The supported video modes.
    \param aspectRatio       The aspect ratio of the display.
    
    \exception               TToiInvalidArgumentException
    Raised if outputId is an invalid output.
    """
    __request = BinIpcParameterStack()
    # Write outputId
    __request.push_uint32(outputId)
    # Write modes
    for iter_k in modes[::-1]:
      __request.push_int32(iter_k)
    __request.push_int32(len(modes))
    # Write aspectRatio
    __request.push_int32(aspectRatio)
    self.call(__request, "IToiVideoOutputConfigurationSession", "SetDisplayInfo")

  def ResetDisplayInfo(self, outputId):
    """Resets display information that has been overridden using
    \ref SetDisplayInfo(). The overridden values are reset to values
    retrieved from the EDID if one is available, otherwise default values.
    
    \param outputId          Video output identifier.
    """
    __request = BinIpcParameterStack()
    # Write outputId
    __request.push_uint32(outputId)
    self.call(__request, "IToiVideoOutputConfigurationSession", "ResetDisplayInfo")

  def SetDefaultVideoMode(self, outputId, mode):
    """Configures the default video output mode. This will be used when
    no adaptive video mode rules are applicable.
    
    \param outputId        Output identifier.
    
    \param mode            The new output mode.
    
    \exception             TToiOperationNotSupportedException
    Raised if output can not be set.
    
    \exception             TToiInvalidArgumentException
    Raised if outputId is an invalid output.
    """
    __request = BinIpcParameterStack()
    # Write outputId
    __request.push_uint32(outputId)
    # Write mode
    __request.push_int32(mode)
    self.call(__request, "IToiVideoOutputConfigurationSession", "SetDefaultVideoMode")

  def SetColorSpace(self, outputId, colorSpace):
    """Configures the color space of the video.
    
    \param outputId        Output identifier.
    
    \param colorSpace      The new color space.
    
    \exception             TToiOperationNotSupportedException
    Raised if output can not be set.
    
    \exception             TToiInvalidArgumentException
    Raised if outputId is an invalid output.
    """
    __request = BinIpcParameterStack()
    # Write outputId
    __request.push_uint32(outputId)
    # Write colorSpace
    __request.push_int32(colorSpace)
    self.call(__request, "IToiVideoOutputConfigurationSession", "SetColorSpace")

  def SetScartMode(self, outputId, mode):
    """Sets the SCART mode for a video output. Only supported for SCART outputs.
    
    \param outputId        Output identifier.
    
    \param mode            The new SCART mode.
    
    \exception             TToiOperationNotSupportedException
    Raised if output can not be set.
    
    \exception             TToiInvalidArgumentException
    Raised if outputId is an invalid output.
    """
    __request = BinIpcParameterStack()
    # Write outputId
    __request.push_uint32(outputId)
    # Write mode
    __request.push_int32(mode)
    self.call(__request, "IToiVideoOutputConfigurationSession", "SetScartMode")

  def SetScartLoopThroughMode(self, outputId, mode):
    """Sets the SCART standby loop through mode for a video output.
    Only supported for SCART outputs.
    
    \param outputId        Output identifier.
    
    \param mode            The new SCART standby loop through mode.
    
    \exception             TToiOperationNotSupportedException
    Raised if output can not be set.
    
    \exception             TToiInvalidArgumentException
    Raised if outputId is an invalid output.
    """
    __request = BinIpcParameterStack()
    # Write outputId
    __request.push_uint32(outputId)
    # Write mode
    __request.push_int32(mode)
    self.call(__request, "IToiVideoOutputConfigurationSession", "SetScartLoopThroughMode")

  def SetDviMode(self, outputId, mode):
    """Sets the DVI mode for a video output. Only supported for HDMI outputs.
    
    \param outputId        Output identifier.
    
    \param mode            The new DVI mode.
    
    \exception             TToiOperationNotSupportedException
    Raised if output can not be set.
    
    \exception             TToiInvalidArgumentException
    Raised if outputId is an invalid output.
    """
    __request = BinIpcParameterStack()
    # Write outputId
    __request.push_uint32(outputId)
    # Write mode
    __request.push_int32(mode)
    self.call(__request, "IToiVideoOutputConfigurationSession", "SetDviMode")

  def SetColorSystem(self, colorSystem):
    """Set color system, e.g. NTSC or PAL.
    
    \param colorSystem     Color system setting.
    
    \exception             TToiOperationNotSupportedException
    Raised if color system can not be changed.
    
    \exception             TToiInvalidArgumentException
    Raised if colorSystemSetting not is supported.
    """
    __request = BinIpcParameterStack()
    # Write colorSystem
    __request.push_int32(colorSystem)
    self.call(__request, "IToiVideoOutputConfigurationSession", "SetColorSystem")

  def Revert(self):
    """Reverts a configuration session to the state it had when first
    being created, even if changes have been applied. The reverted
    configuration is not applied until \ref Apply() is called.
    
    \exception             TToiOperationNotSupportedException
    Raised if a revert cannot be performed.
    
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiVideoOutputConfigurationSession", "Revert")

  def Clear(self):
    """Clears this configuration session to a clean state.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiVideoOutputConfigurationSession", "Clear")

  def Apply(self):
    """Applies the changes made in the configuration session and makes
    the changes permanent.
    
    \exception             TToiOperationNotSupportedException
    Raised if apply cannot be performed.
    
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiVideoOutputConfigurationSession", "Apply")

  def ReleaseInstance(self):
    """This function releases the resources used by this instance and ends the
    configuration session.
    If \ref Apply() has not been called, the session configuration is lost.
    When this function returns all observers has also been released.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiVideoOutputConfigurationSession", "ReleaseInstance")

