# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiVideoOutputObserver import ToiVideoOutputObserver


class ToiVideoOutputService(KreaTvIpcCaller):

  def __init__(self, ipcproxy):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.lookup("VideoOutputService")

  def GetVideoConfiguration(self):
    """This method returns an address to the current video output
    configuration.
    
    \retval address Address to an \ref IToiVideoOutputConfiguration
    interface.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiVideoOutputService", "GetVideoConfiguration")
    __result = []
    # Read address
    __result.append(__response.pop_string())
    return __result[0]

  def CreateVideoConfigurationSession(self, applicationId):
    """This method creates a session that makes it possible to change
    the video output configuration.
    
    \param applicationId Identifier of the calling application.
    
    \retval address      Address to an
    \ref IToiVideoOutputConfigurationSession
    interface.
    
    \exception           TToiInvalidArgumentException
    Raised if the specified application identifier
    is invalid.
    
    \exception           TToiInvalidPreconditionException
    Raised if another session is active.
    
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    __response = self.call(__request, "IToiVideoOutputService", "CreateVideoConfigurationSession")
    __result = []
    # Read address
    __result.append(__response.pop_string())
    return __result[0]

  def SetVideoSafeMode(self):
    """Sets the video outputs into a safe mode configuration.
    
    \exception           TToiInvalidPreconditionException
    Raised if a configuration session is active.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiVideoOutputService", "SetVideoSafeMode")

  def LoadAdaptiveRuleSets(self, adaptiveRuleSets):
    """Loads a sequence of adaptive video rule sets based on their names.
    An empty sequence will unload all currently loaded rule sets.
    Each rule within a loaded rule set is evaluated in the order it is placed
    and all rule sets are evaluated in parallel.
    
    \param adaptiveRuleSets  A sequence of rule set names.
    
    \exception               TToiNotEnoughResourcesException
    Raised if necessary resource can not
    be allocated.
    
    \exception               TToiInvalidArgumentException
    Raised if the ruleset is unavailable.
    """
    __request = BinIpcParameterStack()
    # Write adaptiveRuleSets
    for iter_k in adaptiveRuleSets[::-1]:
      __request.push_string(iter_k)
    __request.push_int32(len(adaptiveRuleSets))
    self.call(__request, "IToiVideoOutputService", "LoadAdaptiveRuleSets")

  def GetAvailableAdaptiveRuleSetNames(self):
    """Get a sequence of rule set names based on the available
    *  adaptive video rule sets that can be loaded.
    
    \retval adaptiveRuleSets A sequence of rule sets names.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiVideoOutputService", "GetAvailableAdaptiveRuleSetNames")
    __result = []
    # Read adaptiveRuleSets
    adaptiveRuleSets = []
    for k in range(__response.pop_int32()):
      adaptiveRuleSets.append(__response.pop_string())
    __result.append(adaptiveRuleSets)
    return __result[0]

  def GetLoadedAdaptiveRuleSetNames(self):
    """Get a sequence of rule set names based on the adaptive
    *  video rule sets that have been loaded.
    
    \retval adaptiveRuleSets A sequence of rule sets names.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiVideoOutputService", "GetLoadedAdaptiveRuleSetNames")
    __result = []
    # Read adaptiveRuleSets
    adaptiveRuleSets = []
    for k in range(__response.pop_int32()):
      adaptiveRuleSets.append(__response.pop_string())
    __result.append(adaptiveRuleSets)
    return __result[0]

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiVideoOutputObserver.
    The observer will receive events sent by this IToiVideoOutputService instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiVideoOutputService can find an
    IToiVideoOutputObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiVideoOutputObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiVideoOutputService", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiVideoOutputService
    can find the registered IToiVideoOutputObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiVideoOutputObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiVideoOutputService", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiVideoOutputObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiVideoOutputService", "RegisterObserver")
    return observer
