// Copyright (c) 2010 Motorola, Inc. All Rights Reserved.

#include "TApplication.h"
#include "TMedia.h"
#include "TSocket.h"

#include "debug/TLogTracker.h"
#include "ipc/TIpcClient.h"
#include "ipc/TIpcServer.h"
#include "interface/ToiObjectNames.h"
#include "interface/TToiFrontPanelServiceCaller.h"
#include "interface/TToiNameServiceCaller.h"

#include <csignal>
#include <cstdlib>
#include <string>

#define SOCKET_PATH "/tmp/mediaSocket"

using namespace std;

static TIpcServer IpcServer;
static TLogTracker Tracker;

TTracker* TTracker::GetProcessTracker()
{
  return &Tracker;
}

void SignalHandler(int signum)
{
  Tracker.Note("Got signal %d, exiting", signum);
  IpcServer.Stop();
}

int main()
{
  TIpcClient IpcClient;

  try {
    // Get the address to the name service
    const char* nameServiceAddress = getenv("NAMESERVICE_ADDRESS");
    if (nameServiceAddress == NULL) {
      throw TException("NAMESERVICE_ADDRESS variable is not defined");
    }
    Tracker.Debug("Address to name service is %s",  nameServiceAddress);

    // Connect to the name service
    TToiNameServiceCaller nameService(&IpcClient, nameServiceAddress);

    // Read the application ID
    const char* applicationIdString = getenv("APPLICATION_ID");
    if (applicationIdString == NULL) {
      throw TException("APPLICATION_ID environment variable is not defined");
    }
    char* endPtr;
    IToiApplicationService::TToiApplicationId  applicationId =
      strtol(applicationIdString, &endPtr, 10);
    if (applicationIdString[0] == '\0' || endPtr[0] != '\0') {
      throw TException("APPLICATION_ID string has illegal format");
    }
    Tracker.Debug("Application ID is %d", applicationId);

    // Setup application service handling
    TApplication application(IpcClient, IpcServer, nameService, applicationId);

    // Setup media session handling
    TMedia media(IpcClient, IpcServer, nameService, applicationId,
                 SOCKET_PATH);

    // Create socket and add it to the event loop
    TSocket socket(media, SOCKET_PATH);
    IpcServer.GetEventLoop()->
      AddDescriptor(socket.GetSocket(), IIOEventLoop::EVENT_READ,
                    IIOEventLoop::PRIORITY_NORMAL, &socket);

    // Create front panel
    string frontPanelAddress;
    nameService.LookupObject(TOI_FRONT_PANEL_SERVICE_NAME, frontPanelAddress);
    TToiFrontPanelServiceCaller frontPanel(&IpcClient, frontPanelAddress);

    // Start flashing the LED
    frontPanel.
      SetLedState(0, IToiFrontPanelService::
                  TToiLedState(IToiFrontPanelService::COLOR_RED, 5));

    // Start the IPC event loop
    signal(SIGTERM, &SignalHandler);
    IpcServer.Run();

    // Cleanup
    frontPanel.
      SetLedState(0, IToiFrontPanelService::
                  TToiLedState(IToiFrontPanelService::COLOR_OFF, 0));
  }
  catch (TException& e) {
    Tracker.Fatal("Fatal error: %s", e.GetText());
  }
  catch (...) {
    Tracker.Fatal("Fatal error: Unknown exception");
  }

  return 0;
}
