/*
 *--------------------------------------------------------------------
 *
 * TBinIpcParameterStack.h
 *
 * This class stores and reads parameters of different types into
 * a buffer
 * 
 * Copyright (c) 2004 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TBINIPCPARAMETERSTACK_H
#define TBINIPCPARAMETERSTACK_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>

/* Parameter stack format and functionality:
   All params are stored in the internal buffer.
   It starts with an integer, which describes the total size of the message.
   Push/Pop functions may be used any number of times. If the buffer is too
   small for a 'push' or if its too small for a 'pop', the stack goes
   invalid. If the stack is invalid, all following push/pops are ignored.

   Note: no type checking is performed. The user is responsible for pushing
   and popping parameters of the same type in the right order.

   Note, it might be obvious from the 'class' name, but still note: Parameters
   are pushed / popped in last-in-first-out order.
*/

struct TBinIpcParameterStackHeader
{
  int Size;
};

typedef struct TBinIpcParameterStackHeader TBinIpcParameterStackHeader;
  
struct TBinIpcParameterStack
{
  char* Buffer;
  TBinIpcParameterStackHeader* Header;

  int BufferSize;
  bool Valid;
};

typedef struct TBinIpcParameterStack TBinIpcParameterStack;

/*
  To create a message from scratch:
  1. Call Init().
  2. Push/Pop data.

  To read stack data into an existing stack:
  1. Call Init().
  2. Copy data into buffer pointed out by GetBuffer(). Buffer is
     GetBufferSize() bytes large.
  3. Call Parse() to check if an entire message has been read.
  4. Push/Pop data.

  To use an existing buffer as a stack:
  1. Read data into buffer.
  2. Call Attach().
  3. Call Parse().
  4. Push/Pop data.
*/

void TBinIpcParameterStack_Init(TBinIpcParameterStack* _this,
                                char* buffer, int bufferSize);
void TBinIpcParameterStack_Attach(TBinIpcParameterStack* _this,
                                  char* buffer, int bufferSize);
char* TBinIpcParameterStack_GetBuffer(TBinIpcParameterStack* _this);
int TBinIpcParameterStack_GetBufferSize(TBinIpcParameterStack* _this);
bool TBinIpcParameterStack_Parse(TBinIpcParameterStack* _this,
                                 int validBytes);

/* The stack is valid after Parse has been called and has
   succeeded, and until an invalid push/pop has been performed.
   If the stack is invalid, the the previous Push/Pop call has failed.
*/
bool TBinIpcParameterStack_IsValid(TBinIpcParameterStack* _this);

/* Returns the size of the stack, i.e. the number of bytes in the buffer
   in use by the stack.
*/
int TBinIpcParameterStack_GetSize(TBinIpcParameterStack* _this);

/* Returns true if no parameters are left in the stack */
bool TBinIpcParameterStack_IsEmpty(TBinIpcParameterStack* _this);

/* Push functions copy data from the source into the stack buffer */
void TBinIpcParameterStack_PushBinary(TBinIpcParameterStack* _this,
                                      const uint8_t* value, int size);
void TBinIpcParameterStack_PushBool(TBinIpcParameterStack* _this, bool value);

void TBinIpcParameterStack_PushDouble(TBinIpcParameterStack* _this,
                                      double value);

void TBinIpcParameterStack_PushInt(TBinIpcParameterStack* _this, int value);

void TBinIpcParameterStack_PushUint(TBinIpcParameterStack* _this,
                                    unsigned int value);

void TBinIpcParameterStack_PushInt16(TBinIpcParameterStack* _this,
                                    int16_t value);

void TBinIpcParameterStack_PushUint16(TBinIpcParameterStack* _this,
                                    uint16_t value);

void TBinIpcParameterStack_PushInt64(TBinIpcParameterStack* _this,
                                     int64_t value);

void TBinIpcParameterStack_PushUint64(TBinIpcParameterStack* _this,
                                      uint64_t value);

void TBinIpcParameterStack_PushString(TBinIpcParameterStack* _this,
                                      const char* value);

/* Pop functions copy data from the stack buffer to the target, unless
   the value params are pointers-to-pointers, in which case a pointer
   into the stack buffer is returned. The value pointer may be set to
   NULL.

   Note! The return value from the value returning functions is undefined
   if the stack isn't valid!
*/
const uint8_t*
TBinIpcParameterStack_PopBinary(TBinIpcParameterStack* _this,
                                int size, const uint8_t** value);
bool TBinIpcParameterStack_PopBool(TBinIpcParameterStack* _this, bool* value);

double TBinIpcParameterStack_PopDouble(TBinIpcParameterStack* _this,
                                     double* value);

int TBinIpcParameterStack_PopInt(TBinIpcParameterStack* _this, int* value);

unsigned int TBinIpcParameterStack_PopUint(TBinIpcParameterStack* _this,
                                           unsigned int* value);

int16_t TBinIpcParameterStack_PopInt16(TBinIpcParameterStack* _this,
                                       int16_t* value);

uint16_t TBinIpcParameterStack_PopUint16(TBinIpcParameterStack* _this,
                                       uint16_t* value);

int64_t TBinIpcParameterStack_PopInt64(TBinIpcParameterStack* _this,
                                       int64_t* value);

uint64_t TBinIpcParameterStack_PopUint64(TBinIpcParameterStack* _this,
                                         uint64_t* value);

const char* TBinIpcParameterStack_PopString(TBinIpcParameterStack* _this,
                                            const char** value);

/* Functions for testing only! */
int TBinIpcParameterStack_GetMaxStackSize(TBinIpcParameterStack* _this);

/* Write functions copy data from the source to a file */
void TBinIpcParameterFile_WriteBool(FILE* _f, bool value);

void TBinIpcParameterFile_WriteDouble(FILE* _f, double value);

void TBinIpcParameterFile_WriteInt(FILE* _f, int value);

void TBinIpcParameterFile_WriteUint(FILE* _f, unsigned int value);

void TBinIpcParameterFile_WriteInt16(FILE* _f, int16_t value);

void TBinIpcParameterFile_WriteUint16(FILE* _f, uint16_t value);

void TBinIpcParameterFile_WriteInt64(FILE* _f, int64_t value);

void TBinIpcParameterFile_WriteUint64(FILE* _f, uint64_t value);

void TBinIpcParameterFile_WriteString(FILE* _f, const char* value);

/* Read functions read data from file to variable */
bool TBinIpcParameterFile_ReadBool(FILE* _f);

double TBinIpcParameterFile_ReadDouble(FILE* _f);

int TBinIpcParameterFile_ReadInt(FILE* _f);

unsigned int TBinIpcParameterFile_ReadUint(FILE* _f);

int16_t TBinIpcParameterFile_ReadInt16(FILE* _f);

uint16_t TBinIpcParameterFile_ReadUint16(FILE* _f);

int64_t TBinIpcParameterFile_ReadInt64(FILE* _f);

uint64_t TBinIpcParameterFile_ReadUint64(FILE* _f);

const char* TBinIpcParameterFile_ReadString(FILE* _f,
                                            int length,
                                            char** value);
#ifdef __cplusplus
}
#endif

#endif
