/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * TTracker.h
 *
 * A generic class for trace output.
 *
 * Copyright (c) 2002 Kreatel Communications AB. All Rights Reserved.
 * Copyright (c) 2007 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef DEBUG_TTRACKER_H
#define DEBUG_TTRACKER_H

#include "debug/TLogLevel.h"

#include <cstdarg>

#ifdef __GNUC__
# define TTRACKER_CHECK_FORMAT(x, y) __attribute__ ((format (printf, x, y)))
#else
# define TTRACKER_CHECK_FORMAT(x, y)
#endif

#ifdef __GNUC__
# define TTRACKER_DEPRECATED() __attribute__ ((deprecated))
#else
# define TTRACKER_DEPRECATED()
#endif

/** Some general tips for using the tracker

    1. Be concise
    Logs should be informative and easy to understand. Prefer short expressive
    statements to longer, grammatically flawless sentences. Avoid capital
    letters and special characters just to get attention.

    2. Avoid spamming
    Do not repeat yourself unless there is something new to report. It is often
    worth a few extra lines of code to avoid log spamming. Think one extra time
    before adding log lines to heavily shared code.

    3. Use the correct levels
    Understand what level is appropriate for each log line. Remember that the
    failure of some functionality may be the result of an explicit
    configuration and thus could be expected. Keep in mind that traces often
    are filtered out and no warnings/errors/fatals should be shown when the box
    operates as expected.
 */
class TTracker
{
  friend class TSwitchableTracker;

public:
  enum TLevel {
    TrackNone = LOG_LEVEL_NONE,
    TrackFatal = LOG_LEVEL_FATAL,
    TrackError = LOG_LEVEL_ERROR,
    TrackWarning = LOG_LEVEL_WARNING,
    TrackNote = LOG_LEVEL_NOTE,
    TrackDebug = LOG_LEVEL_DEBUG,
    TrackAll = LOG_LEVEL_ALL
  };

protected:
  TLevel Level;

  /** Derived classes implement their specific output handling by overriding
      this function.
   */
  virtual void Output(TLevel level, const char* format,
                      va_list ap) throw () = 0;

public:
  /** Implement this function to get a single access point for
      the process's tracker object.
  */
  static TTracker* GetProcessTracker();

  /** This method returns a textual representation of a track level.

      \param level The level value
      \return The string representing the given level
  */
  static const char* LevelToString(TTracker::TLevel level);

  /** Sets the default tracker.
  */
  static void SetDefaultTracker(TTracker* tracker);

  /** This method converts a string representation of a track level to
      a TLevel. The comparison is case-insensitive.

      \param level
      Valid values: none, fatal, error, warning, note, debug, all
      Invalid inputs will return TLevel::TrackNone
  */
  static TLevel StringToLevel(const char* level);

  TTracker() throw ();
  TTracker(TLevel level) throw ();
  virtual ~TTracker() { /* Empty */ };

  /** These methods allow the user to pass a level argument together with
      the message.
      Where possible the specialized methods should be used instead.

      \param level
      The severity of the message.

      \param format
      A printf style variable argument format string.
  */
  void Track(TLevel level, const char* format...) throw ()
    TTRACKER_CHECK_FORMAT(3, 4);
  void Track(TLevel level, const char* format, va_list ap) throw ()
    TTRACKER_CHECK_FORMAT(3, 0);

  /** Used to provide information that can be used in debugging without
      requiring specific knowledge about the code internals. It could be minor
      state changes or extra information in addition to another log message. No
      pure development debugging messages should be enabled in production code.

      Ex: "IP address renewed"
          "Initialization done, entering event loop"

      \param format
      A printf style variable argument format string.
  */
  void Debug(const char* format...) throw () TTRACKER_CHECK_FORMAT(2, 3);

  /** Trace is deprecated use Debug instead.
   */
  void Trace(const char* format...) throw () TTRACKER_DEPRECATED();

  /** The code has reached a point where it is informative to put a message in
      the log. It could be triggered by a state change, an event or user input
      that provides useful information to a working system.

      Ex: "HDMI connected"
          "Entering standby"

      \param format
      A printf style variable argument format string.
  */
  void Note(const char* format...) throw () TTRACKER_CHECK_FORMAT(2, 3);

  /** Something out of the ordinary has happened but does not have any direct
      impact on the current task. It may be caused by input that is missing or
      out of bounds. It can be the symptom of bugs in the calling code.

      Ex: "Missing configuration, using default"
          "Resource already open"

      \param format
      A printf style variable argument format string.
  */
  void Warning(const char* format...) throw () TTRACKER_CHECK_FORMAT(2, 3);

  /** An error has occurred that affects some core functionality but can be
      handled and does not prevent further execution. Errors may be the
      symptoms of bugs, but may also be the result of problems in the
      environment/configuration/input.

      Ex: "VoD server not found"
          "Child process exited unexpectedly"

      \param format
      A printf style variable argument format string.
  */
  void Error(const char* format...) throw () TTRACKER_CHECK_FORMAT(2, 3);

  /** A fatal error has occurred that can not be handled and prevents further
      execution. This log line should always be followed by a program
      termination, never use one without the other. Fatals should never be
      reached in production code and always indicate a bug somewhere in the
      system.

      Ex: "Could not connect to name service"
          "Caught an unknown exception"

      \param format
      A printf style variable argument format string.
  */
  void Fatal(const char* format...) throw () TTRACKER_CHECK_FORMAT(2, 3);
};

#undef TTRACKER_CHECK_FORMAT

inline TTracker::TTracker() throw () :
  Level(TrackNone)
{
  
}

inline TTracker::TTracker(TLevel level) throw () :
  Level(level)
{
  
}

inline void TTracker::Track(TLevel level, const char* format...) throw ()
{
  if (Level >= level) {
    va_list ap;
    va_start(ap, format);
    Output(level, format, ap);
    va_end(ap);
  }
}

inline void TTracker::Track(TLevel level, const char* format,
                            va_list ap) throw ()
{
  if (Level >= level) {
    Output(level, format, ap);
  }
}

inline void TTracker::Fatal(const char* format...) throw ()
{
  if (Level >= TrackFatal) {
    va_list ap;
    va_start(ap, format);
    Output(TrackFatal, format, ap);
    va_end(ap);
  }
}

inline void TTracker::Error(const char* format...) throw ()
{
  if (Level >= TrackError) {
    va_list ap;
    va_start(ap, format);
    Output(TrackError, format, ap);
    va_end(ap);
  }
}

inline void TTracker::Warning(const char* format...) throw ()
{
  if (Level >= TrackWarning) {
    va_list ap;
    va_start(ap, format);
    Output(TrackWarning, format, ap);
    va_end(ap);
  }
}

inline void TTracker::Note(const char* format...) throw ()
{
  if (Level >= TrackNote) {
    va_list ap;
    va_start(ap, format);
    Output(TrackNote, format, ap);
    va_end(ap);
  }
}

inline void TTracker::Debug(const char* format...) throw ()
{
  if (Level >= TrackDebug) {
    va_list ap;
    va_start(ap, format);
    Output(TrackDebug, format, ap);
    va_end(ap);
  }
}

inline void TTracker::Trace(const char* format...) throw ()
{
   if (Level >= TrackDebug) {
    va_list ap;
    va_start(ap, format);
    Output(TrackDebug, format, ap);
    va_end(ap);
  }
}
#endif 
