/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * IIOEventLoop.h
 *
 * Copyright (c) 2002 Kreatel Communications AB. All Rights Reserved.
 * Copyright (c) 2007 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef IIOEVENTLOOP_H
#define IIOEVENTLOOP_H

#include <stdint.h>

class IIOEventHandler;
class IIOTimeoutHandler;

/** The event loop interface is used to add and remove file
    descriptors that should be monitored for reading or writing. When
    an event occurs the registered callback is invoked. It is also
    possible to create timers.

    How the event loop is started and stopped is implementation-specific.
*/
class IIOEventLoop
{
protected:
  virtual ~IIOEventLoop() { /* Empty */ }
public:
  /** Enumeration of possible events that can be monitored. */
  enum TEvent
  {
    /** Reading from the descriptor will not block, i. e. data is
        available. This event will also be sent on end of file but
        read() will then return 0. */
    EVENT_READ = 0x00000001,
    /** Writing to the descriptor will not block. Note that if a
        socket or pipe has been closed at the other end write() will
        generate SIGPIPE and eventually return -1 with errno EPIPE if
        the signal is caught.  An alternative for sockets is to use
        send() together with the flag MSG_NOSIGNAL. */
    EVENT_WRITE = 0x00000002
  };

  /** Predefined priority levels for compatibility. */
  enum
  {
    /** High priority. */
    PRIORITY_HIGH = -100,

    /** Normal priority. This is the priority level used by GTK+ and
        the IPC framework. */
    PRIORITY_NORMAL = 0,

    /** Low priority. */
    PRIORITY_LOW = 300
  };

  typedef int32_t TPriority;


  /** Add a file descriptor to the set of descriptors that are
      monitored for events.

      \param fd         File descriptor number.

      \param eventMask  Bitmask with events that should be monitored.

      \param priority   Priority of the event or events. Events with
                        the lowest numbers have the highest
                        priority. Zero corresponds to normal priority
                        and is used for descriptors added by the
                        framework and standard libraries. For events
                        with the same priority round-robin scheduling
                        is used.

      \param handler    Pointer to an interface with a callback
                        method. The callback is invoked whenever an
                        event is signalled on the descriptor. Note
                        that unless the client reads from or writes to
                        the descriptor (for read and write events
                        respectively), the handler will be invoked again
                        after it has returned.
  */
  virtual void AddDescriptor(int fd,
                             uint32_t eventMask,
                             TPriority priority,
                             IIOEventHandler* handler) throw () = 0;

  /** Stop monitoring a previously added file descriptor.

      \param fd         File descriptor number.
  */
  virtual void RemoveDescriptor(int fd) throw () = 0;

  /** Create a new interval timer.

      \param interval   Timer interval in milliseconds.

      \param handler    Pointer to an interface with a callback
                        method. The timeout handler is invoked at
                        regular intervals until the timer is removed.

      \return           Integer uniquely identifying the timer. This
                        value is always non-negative, making it safe
                        to use -1 as a null value indicating whether a
                        timer has been added or not.
  */
  virtual int AddTimer(uint32_t interval,
                       IIOTimeoutHandler* handler) throw () = 0;

  /** Remove a timer. Call this method from the timeout handler to
      emulate a single-shot timer.

      \param timerId    Integer identifying the timer that should be
                        removed.
  */
  virtual void RemoveTimer(int timerId) throw () = 0;
};

#endif
