








#ifndef TDEFERREDMETHODCALLER_H
#define TDEFERREDMETHODCALLER_H

#include "event/IIOEventHandler.h"
#include "event/IIOTimeoutHandler.h"
#include "TPollableMessageQueue.h"
#include <boost/function.hpp>
#include <map>
#include <queue>

class IIOEventLoop;

/** Class implementing deferred method calls. Functions objects 
    pushed to the caller will be executed "later". 
 */
class TDeferredMethodCaller : public IIOEventHandler,
                              public IIOTimeoutHandler
{
public:
  typedef boost::function<void ()> TMethodRunner;

private:
  IIOEventLoop& EventLoop;

  typedef std::queue<TMethodRunner> TMethodQueue;
  TMethodQueue MethodQueue;
  TPollableMessageQueue MessageQueue;
  
  typedef std::map<int, TMethodRunner> TTimerMethodMap;
  TTimerMethodMap TimerMethodMap;

  void HandleEvent(int fd, uint32_t event) throw ();
  void HandleTimeout(int timerId) throw ();

protected:
  virtual void InsertMethodRunner(int timerId, const TMethodRunner& runner);
  virtual TMethodRunner ExtractMethodRunner(int timerId);
  virtual TMethodRunner ExtractMethodRunner();

public:

  /** Constructor. The object will keep a reference to the event loop 
      object and the caller is responsible for assuring deletion order.
      \param eventLoop The eventloop object to use
   */
  TDeferredMethodCaller(IIOEventLoop& eventLoop);
  
  /** Destructor.
   */
  virtual ~TDeferredMethodCaller();

  /** Queues a function call that should be called "later".
      The function object may refer to static functions as well
      as methods in objects. boost::bind can be used to create the
      function objects.
      \runner A function object with the signature "void func)()".
   */
  virtual void Push(const TMethodRunner& runner);

  /** Hands over a function call that should be executed upon timeout.
      The function will be executed once.
      The function object may refer to static functions as well
      as methods in objects. boost::bind can be used to create the
      function objects.
      \param milliseconds Time in milliseconds when the runner is run.
      If milliseconds is 0, Push will be called.
      \param runner A function object with the signature "void func)()".
   */
  void PushAt(int milliseconds, const TMethodRunner& runner);

  
  void PopAll();
};

#endif
