







/** \file */

#ifndef ITELETEXTRENDEREXTENSION_H
#define ITELETEXTRENDEREXTENSION_H

#include "interface/kreatvgfx/IKreaTvGfx.h"
#include "interface/kreatvgfx/IKreaTvGfxExtension.h"
#include "interface/kreatvgfx/IKreaTvGfxExtensionObserver.h"

#include "event/IIOEventLoop.h"

#include <boost/shared_ptr.hpp>

/** Interface to be used by an application to control the teletext renderer. */
class IKreaTvGfxTeletextExtension : public IKreaTvGfxExtension
{
protected:
  virtual ~IKreaTvGfxTeletextExtension() throw () { /* Empty */ }

public:
  /** Start the extension. */
  virtual TKreaTvGfxResult Start() = 0;
  /** Stop the extension. */
  virtual TKreaTvGfxResult Stop() = 0;
  /** Method to be called when the user presses the index key page or when the
      software wishes to restore the teletext  component to the index page.
      In most cases this is page 100. */
  virtual TKreaTvGfxResult GetIndexPage() = 0;
  /** Fetches the previous page available in the cache. */
  virtual TKreaTvGfxResult PrevPage() = 0;
  /** Fetches the next page available in the cache. */
  virtual TKreaTvGfxResult NextPage() = 0;
  /** Method to be called when the user presses a number key on the remote
      control. When three digits have been received, the corresponding page
      is displayed. */
  virtual TKreaTvGfxResult InputDigit(char key) = 0;
  /** Turns on the cached subpage function. */
  virtual TKreaTvGfxResult ShowSubpages() = 0;
  /** Turns off the cached subpage function. */
  virtual TKreaTvGfxResult HideSubpages() = 0;
  /** Method to be called when the user presses the "previous subpage" key.
      Often implemented by using the arrow keys on the remote control. Fetches
      the previous subpage available in the cache. */
  virtual TKreaTvGfxResult PrevSubpage() = 0;
  /** Method to be called when the user presses the "next subpage" key.
      Often implemented by using the arrow keys on the remote control. Fetches
      the next subpage available in the cache. */
  virtual TKreaTvGfxResult NextSubpage() = 0;
  /** Toggles the display of hidden text. */
  virtual TKreaTvGfxResult ToggleReveal() = 0;
  /** Method for toggling the background between black and transparent. */
  virtual TKreaTvGfxResult ToggleTransparent() = 0;
  /** Method for toggling hold function. When the hold state is active, subpage
      updates will be ignored. */
  virtual TKreaTvGfxResult ToggleHold() = 0;
  /** Method for toggling the zooming state. If starting from the normal state,
      this will first display the upper half of the current page in double
      size. On the next call it will display the lower half of the current page
      in double size. On the third call it will restore normal size. */
  virtual TKreaTvGfxResult ToggleZoom() = 0;
  /** Method to be called when the user presses the red FLOF key on the remote
      control. Switches to the correct page according to the current FLOF
      state. */
  virtual TKreaTvGfxResult FlofRed() = 0;
  /** Method to be called when the user presses the green FLOF key on the
      remote control. Switches to the correct page according to the current
      FLOF state. */
  virtual TKreaTvGfxResult FlofGreen() = 0;
  /** Method to be called when the user presses the yellow FLOF key on the
      remote control. Switches to the correct page according to the current
      FLOF state. */
  virtual TKreaTvGfxResult FlofYellow() = 0;
  /** Method to be called when the user presses the cyan FLOF key on the remote
      control. Switches to the correct page according to the current FLOF
      state. */
  virtual TKreaTvGfxResult FlofCyan() = 0;
};

/** Pointer type for IKreaTvGfxTeletextExtension objects. Used for
    automatic reference counting.
 */
typedef boost::shared_ptr<IKreaTvGfxTeletextExtension>
  TKreaTvGfxTeletextExtensionPointer;

/** This function will create a teletext renderer that can draw
    teletexts as decided by the active media player and Information
    Service parameters. The application level graphics rendering
    engine can use this extension to draw teletexts in the graphics
    buffer to show on screen.

    \param[in] kreaTvGfx
                      KreaTvGfx instance that library will use to
                      create internally needed pixmaps.

    \param[in] applicationId
                      The id of the application.

    \param[in] eventLoop
                      Event loop needed to allow library to talk to
                      the media session.

    \param[in] observer
                      Observer interface where update events will be
                      signalled when new teletext graphics is
                      available.

    \param[out] teletextExtension
                      The entry point for the KreaTvGfx teletext extension.
 */
void CreateTeletextRenderExtension(
    TKreaTvGfxPointer kreaTvGfx,
    int applicationId,
    IIOEventLoop& eventLoop,
    IKreaTvGfxExtensionObserver* observer,
    TKreaTvGfxTeletextExtensionPointer& teletextExtension)
  throw ();

#endif
