/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * IKreaTvGfx.h
 *
 * Entry point to the KreaTV graphics library.
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

/** \file */

#ifndef IKREATVGFX_H
#define IKREATVGFX_H

#include "KreaTvGfxTypes.h"
#include "IKreaTvGfxDisplayObserver.h"

/** \class IKreaTvGfx IKreaTvGfx.h "kreatvgfx/IKreaTvGfx.h"

    This is the main interface of the KreaTV graphics library. An
    instance of this interface can be created with the global function
    CreateKreaTvGfx(). Only one instance can be created per application.
*/
class IKreaTvGfx
{
public:
  /** Do not call the destructor directly. Always use the reference
      counting pointer type.

      \see TKreaTvGfxPointer;
  */
  virtual ~IKreaTvGfx() { /* Empty */ }


  /** STBs usually have several video outputs, some of them are
      capable of only showing standard definition (SD) video and some
      are able to show high definition (HD) video. Since these types
      of outputs may want to show different content, e.g. no graphics
      but subtitling on SD, we use the concept of displays to separate
      these types of outputs. Observe that if an application wants to
      show the same things on SD and HD outputs, it must use both
      displays and perform the same operations on both.

      This function is used to get the different displays to allow
      applications to update video position and graphics.

      \param[in] displayType
                        The type of display to get (SD or HD).

      \param[out] display
                        The display object.

      \param[in] observer
                        A pointer to an object implementing the \ref
                        IKreaTvGfxDisplayObserver interface.

      \retval KGFX_RESULT_OK
                        On success.
      \retval KGFX_RESULT_INVALID_ARGUMENT
                        If the display type is not valid.

      \todo Add support for the following return value:
              KGFX_RESULT_OPERATION_NOT_SUPPORTED
                        If the display is not available on the STB model.
   */
  virtual TKreaTvGfxResult
  GetDisplay(TKreaTvGfxDisplayType displayType,
             TKreaTvGfxDisplayPointer& display,
             IKreaTvGfxDisplayObserver* observer) throw () = 0;


  /** Create a blitter object. A blitter is used to perform hardware
      accelerated fill and copy operations (with or without blending).

      \return           The blitter object.
  */
  virtual TKreaTvGfxBlitterPointer CreateBlitter() throw () = 0;


  /** Create a pixmap, which represents an area of graphics memory
      where the application can draw and blit. It is possible to show
      a pixmap on a display using the IKreaTvGfxDisplay::Update()
      function.

      Graphics memory is automatically allocated immediately when
      created.

      \param[in] width  The width of the pixmap.

      \param[in] height The height of the pixmap.

      \param[in] pixelFormat
                        The pixel format of the pixmap.

      \param[in] premultiplied
                        True to premultiply the pixel color components
                        with the pixel alpha value. This can only be
                        used with pixel formats that includes an alpha
                        channel. Not all chipsets support
                        premultiplication.

      \param[out] pixmap
                        The created pixmap.

      \retval KGFX_RESULT_OK
                        On success.
      \retval KGFX_RESULT_INVALID_ARGUMENT
                        If premultiplied is used with a pixel format
                        without alpha channel.
      \retval KGFX_RESULT_OPERATION_NOT_SUPPORTED
                        If the pixel format or the premultiplied property
                        are not supported in hardware.
      \retval KGFX_RESULT_NOT_ENOUGH_RESOURCES
                        If there were not enough graphics memory to
                        allocate to a pixmap.
  */
  virtual TKreaTvGfxResult
  CreatePixmap(uint32_t width,
               uint32_t height,
               TKreaTvGfxPixelFormat pixelFormat,
               bool premultiplied,
               TKreaTvGfxPixmapPointer& pixmap) throw () = 0;
};


/** Global entry point.

    \param[in] eventLoop
                      Event loop that KreaTvGfx will use to
                      monitor file descriptors and create timers.

    \param[out] kreaTvGfx
                      The entry point for KreaTV graphics layer. If
                      this function is called more than once, the
                      same underlying object will be returned.

    \todo (future) Add parameter to choose amount of graphics memory
                   to allocate to this application.
*/
void CreateKreaTvGfx(IIOEventLoop* eventLoop,
                     TKreaTvGfxPointer& kreaTvGfx) throw ();

#endif
