/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * IKreaTvGfxBlitter.h
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef IKREATVGFXBLITTER_H
#define IKREATVGFXBLITTER_H

#include "KreaTvGfxTypes.h"

/** \class IKreaTvGfxBlitter IKreaTvGfxBlitter.h "kreatvgfx/IKreaTvGfxBlitter.h"

    The blitter provides access to hardware accelerated operations on
    pixmaps. It is possible to fill a pixmap or copy a pixmap to
    another. Both operations support blending.
*/
class IKreaTvGfxBlitter
{
public:
  virtual ~IKreaTvGfxBlitter() { /* Empty */ }


  /** Set if blending should be enabled or disabled.

      \param[in] enable Set to true to enable blending or false to disable.
  */

  virtual void SetBlendEnabled(bool enable) throw () = 0;

  /** Check if blending is enabled or disabled.

      \retval true      If blending is enabled.
      \retval false     If blending is disabled.
   */
  virtual bool IsBlendEnabled() const throw () = 0;
 

  /** Fill an area of the pixmap with a color using the blitter. Blending
      operation is used or not depending on blitter settings.

      \param[in] destinationPixmap
                        The pixmap to fill.

      \param[in] destinationRectangle
                        The area of the pixmap to fill.

      \param[in] fillColor
                        The color to fill the pixmap with, in ARGB8888 format.

      \retval KGFX_RESULT_OK
                        On success.
      \retval KGFX_RESULT_INVALID_ARGUMENT
                        If destination rectangle is outside the pixmap or
                        the color value is invalid, e.g. a value larger
                        than 2^16 for a 16-bit pixel format.
      \retval KGFX_RESULT_OPERATION_NOT_SUPPORTED
                        If the configured blend mode is not supported or
                        if the chipset does not support filling with the
                        selected pixel format, e.g. filling a CLUT
                        pixmap may not be supported.
      \retval KGFX_RESULT_INVALID_PRECONDITION
                        If the destination pixmap is locked for manual
                        drawing (with IKreaTvGfxPixmap::Lock()).<br>
   */
  
  virtual TKreaTvGfxResult Fill(TKreaTvGfxPixmapPointer destinationPixmap,
                                const TKreaTvGfxRectangle& destinationRectangle,
                                uint32_t fillColor)
    throw () = 0;


  /** Copy an area of another pixmap over an area of this pixmap,
      using the blitter. Blending operation is used or not depending on blitter
      settings.

      \param[in] sourcePixmap
                        The source pixmap to copy.

      \param[in] sourceRectangle
                        The area of the source pixmap to copy.

      \param[in] destinationPixmap
                        The destination pixmap to copy to.

      \param[in] destinationRectangle
                        The area of the pixmap to copy to.

      \param[in] clipRectangle
                        The area in the destination rectangle to actually
                        update, same co-ordinate system as the destination
                        rectangle.
                        An empty rectangle indicates no clipping,
                        i.e., updating the entire destination rectangle.
                        NOTE: If you are using clipping and dynamic borders you
                        need to check that both width and height is nonzero so
                        you dont get a bigger non-clipped blit by accident.

      \param[in] modulationColor
                        A color (including alpha) in ARGB8888
                        format. All values in the source pixmap will
                        be modulated with this color before the
                        blending onto the destination pixmap is done.
                        For no modulation, use 0xFFFFFFFF.

      \retval KGFX_RESULT_OK
                        On success.
      \retval KGFX_RESULT_INVALID_ARGUMENT
                        If the source or destination rectangles are
                        outside of respective pixmap.
      \retval KGFX_RESULT_OPERATION_NOT_SUPPORTED
                        If the configured blend mode is not supported or
                        if the chipset does not support copying the
                        source pixmap over the destination pixmap,
                        e.g. pixel formats may be incompatible.
      \retval KGFX_RESULT_INVALID_PRECONDITION
                        If the destination pixmap is locked for manual
                        drawing (with IKreaTvGfxPixmap::Lock()).
      \retval KGFX_RESULT_SCALING_ERROR
                        If the blitter cannot scale according to the
                        factors decided by the rectangles. Different
                        chipsets have different constraints.
   */
  virtual TKreaTvGfxResult Copy(TKreaTvGfxPixmapPointer sourcePixmap,
                                const TKreaTvGfxRectangle& sourceRectangle,
                                TKreaTvGfxPixmapPointer destinationPixmap,
                                const TKreaTvGfxRectangle& destinationRectangle,
                                const TKreaTvGfxRectangle& clipRectangle =
                                  TKreaTvGfxRectangle(0, 0, 0, 0),
                                uint32_t modulationColor = 0xFFFFFFFF)
    throw () = 0;

  /** Tells if there are any pending blitter operations.
      
      \retval true      If there are no pending operations.
      \retval false     If there are pending operations.
  */
  virtual bool HasPendingOperations() const throw () = 0;

};

#endif
