







#ifndef IKREATVGFXDISPLAY_H
#define IKREATVGFXDISPLAY_H

#include "KreaTvGfxTypes.h"

class IKreaTvGfxVideoLayer;


/** \class IKreaTvGfxDisplay IKreaTvGfxDisplay.h "kreatvgfx/IKreaTvGfxDisplay.h"

    STBs usually have several video outputs, some of them are capable
    of only showing standard definition (SD) video and some are able
    to show high definition (HD) video. Since these types of outputs
    may want to show different content, e.g. no graphics but
    subtitling on SD, we use the concept of displays to separate these
    types of outputs. Observe that if an application wants to show the
    same things on SD and HD outputs, it must use both displays and
    perform the same operations on both.

    This class models an interface to a display. The displays can be
    fetched from the main interface, IKreaTvGfx.

    \see IKreaTvGfx::GetDisplay()
*/
class IKreaTvGfxDisplay
{
public:
  /** Do not call the destructor directly. Always use the reference
      counting pointer type.

      \see TKreaTvGfxDisplayPointer;
  */
  virtual ~IKreaTvGfxDisplay() throw () { /* Empty */ };


  /** Get parameters describing the current output video mode. This
      information is useful when setting up the transformation from
      application coordinates to display coordinates as well as
      scaling in the video windows.

      \param[out] videoMode
                        The current output video mode.

      \retval KGFX_RESULT_OK
                        On success.

      \todo Get this from video output service instead?
      \todo Is OK the only valid return code?
  */
  virtual TKreaTvGfxResult
  GetVideoMode(TKreaTvGfxVideoMode& videoMode) const throw () = 0;


  /** Enable or disable flicker filtering. Flicker filtering is used
      with interlaced video modes to get a smoother picture. Flicker
      filtering is never used for progressive video modes but you can
      still set the flicker filtering flag to true to enable flicker
      filtering if the video mode changes to interlaced. Enabling
      flicker filtering may affect stretching capabilities when calling
      Update(), see SetNextVisiblePixmap().

      \param[in] enabledFlag
                        True to enable flicker filtering and false to disable.

      \retval KGFX_RESULT_OK
                        On success.
      \retval KGFX_RESULT_OPERATION_NOT_SUPPORTED
                        If the chipset does not support flicker filtering.
  */
  virtual TKreaTvGfxResult SetFlickerFilter(bool enabledFlag) throw () = 0;


  /** Get the flicker filtering status.

      \retval true      Flicker filtering is enabled.
      \retval false     Flicker filtering is disabled.

      \see SetFlickerFilter()
   */
  virtual bool IsFlickerFilterEnabled() const throw () = 0;


  /** Get the number of available video layers for the display.

      \return           The number of video layers for the display.

      \see GetVideoLayer()
   */
  virtual uint32_t GetVideoLayerCount() const throw () = 0;


  /** Get a video layer. Use them to show video from different sources
      on the display.

      \param[in] index  The index of the video layer to get.

      \return           The video layer.
      \retval NULL      The index is out of range.
   */
  virtual IKreaTvGfxVideoLayer* GetVideoLayer(uint32_t index) throw () = 0;


  /** \techpreview Parameter "alpha" is a TECHNICAL PREVIEW, it may change.
 
      Set the next pixmap to be showed on this display. The pixmap
      will become visible at the next vertical blank after calling
      Update().

      Graphics that is shown this way will be shown above the video
      layers. The application is responsible for punching a hole in
      the graphics to make video visible, i.e. drawing a transparent
      area where video shall be visible. If possible, the entire pixmap,
      or area defined by srcRect, will be stretched to fit the screen.
      With some hardware, vertical stretch is not supported, with others it
      may be supported when flicker filtering is disabled. An error will be
      generated when calling Update(), if the given scaling is not supported.

      \param[in] pixmap  The pixmap to show on the display.
      \param[in] srcRect Optional rectangle for specifying which part
                         of the pixmap to display, in pixmap coordinates.
                         The entire pixmap will be displayed if not specified,
                         or setting the dimensions to zero.
      \param[in] alpha   Optional alpha value which will be used to
                         premultiply all the pixmap's individual pixel alpha
                         values at the moment of screen update.

      \retval KGFX_RESULT_INVALID_ARGUMENT
                         If srcRect is not within the pixmap boundaries.
   */
  virtual TKreaTvGfxResult
  SetNextVisiblePixmap(TKreaTvGfxPixmapPointer pixmap,
                       TKreaTvGfxRectangle srcRect = TKreaTvGfxRectangle(),
                       uint8_t alpha = 255)
    throw () = 0;


  /** Performs a flip of the pixmap and updates video layer positions,
      scalings and sources accoring to values set with functions in
      IKreaTvGfxVideoLayer and with SetNextVisiblePixmap(). All
      changes are synchronized and happens on the next vertcial blank.

      \retval KGFX_RESULT_OK
                        On success.
      \retval KGFX_RESULT_SCALING_ERROR
                        When the hardware is not able to scale
                        the pixmap to fit the screen. Different
                        hardware have different capabilities, see e.g.
                        SetFlickerFilter() and SetNextVisiblePixmap().
      \retval KGFX_RESULT_INVALID_PRECONDITION
                        When this function is called before then
                        OnUpdate event for the previous update.
      \retval KGFX_RESULT_DISPLAY_DISABLED
                        The display is disabled.

      \see IKreaTvGfxDisplayObserver::OnUpdate()
   */
  virtual TKreaTvGfxResult Update() throw () = 0;


  /** \techpreview This method is a TECHNICAL PREVIEW, it may change.

      Set the color that should be used for color keying of this display.
      Color keying is used to replace a certain color with full
      transparancy. The color key changes will be visible after next Update()
      call.

      Color keying overrides other alpha values set with global alpha or
      the alpha value of the pixel.

      \param[in] red
                        The red component of the color key,
                        as a value between 0 and maximum value
                        for current pixel format.

      \param[in] green
                        The green component of the color key,
                        as a value between 0 and maximum value
                        for current pixel format.

      \param[in] blue
                        The blue component of the color key,
                        as a value between 0 and maximum value
                        for current pixel format.

      \retval KGFX_RESULT_OK
                        On success.

      \retval KGFX_RESULT_INVALID_PRECONDITION
                        If the hardware does not support color keying.
  */
  virtual TKreaTvGfxResult SetColorKey(uint8_t red,
                                       uint8_t green,
                                       uint8_t blue) = 0;

  /** \techpreview This method is a TECHNICAL PREVIEW, it may change.

      Unset the color key. No colors will be replaced after next Update()
      call.
  */
  virtual void UnsetColorKey() = 0;
};



#endif
