/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * IKreaTvGfxExtension.h
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef IKREATVGFXEXTENSION_H
#define IKREATVGFXEXTENSION_H

#include "KreaTvGfxTypes.h"

/** \class IKreaTvGfxExtension IKreaTvGfxExtension.h "kreatvgfx/IKreaTvGfxExtension.h"

    This interface is used for extensions to the KreaTV graphics
    library, e.g. for subtitling. These extensions are not part of the
    KreaTV graphics library but are implemented in free-standing
    libraries.

    The interface exists to get consistent interfaces for all such
    extension, even if the interface is just base interface that needs
    to be extended with more functionality for each extension.

    \see IKreaTvGfxExtensionObserver
*/
class IKreaTvGfxExtension
{
public:
  virtual ~IKreaTvGfxExtension() throw () { /* Empty */ }


  /** Get the current damage of the extension, i.e what
      area the extension needs to redraw. The damage is reset
      for a specific destinationId when calling Draw for
      that destinationId. The extension cannot change its
      graphics or required damage, between the calls to
      GetDamage and Draw. Also, the user of the extension
      must not handle events, which may affect the extension,
      between the calls to GetDamage and Draw.

      \param[in] destinationRectangle
                        The rectangle where the extension is allowed
                        to draw.

      \param[out] damageRectangle
                        A rectangle describing which area of the
                        pixmap that the extension needs to redraw.

      \param[in] destinationId
                        Optional logical destination id.
                        When drawing to several destination pixmaps
                        (e.g. displays), this specifies which logical
                        pixmap is used (may not be the same pixmap
                        when double buffering). This is used for handling
                        damage correctly.

      \retval KGFX_RESULT_OK
                        On success.
      \retval KGFX_RESULT_INVALID_ARGUMENT
                        For instance, if the destination rectangle
                        is too small for a given extension.
  */
  virtual TKreaTvGfxResult
  GetDamage(TKreaTvGfxRectangle destinationRectangle,
            TKreaTvGfxRectangle& damageRectangle,
            uint32_t destinationId = 0) throw () = 0;

  /** Ask the extension to draw in a pixmap. The extension may use the
      blitter or manually draw, in which case it will lock the
      pixmap. Depending on whether the extension uses manual draw or
      blitter, the application must make sure there are currently no
      locks on the pixmap and no outstanding blit operations.

      \param[in] destinationPixmap
                        The pixmap the extension should draw in.

      \param[in] destinationRectangle
                        The rectangle where the extension is allowed
                        to draw.

      \param[in] damageRectangle
                        A rectangle describing which areas of the
                        destination pixmap that needs to be redrawn.
                        Only the area within the destinationRectangle
                        will be redrawn.

      \param[in] destinationId
                        Optional logical destination id.
                        When drawing to several destination pixmaps
                        (e.g. displays), this specifies which logical
                        pixmap is used (may not be the same pixmap
                        when double buffering). This is used for handling
                        damage correctly.

      \retval KGFX_RESULT_OK
                        On success.
      \retval KGFX_RESULT_INVALID_ARGUMENT
                        For instance, if the destination rectangle
                        is too small for a given extension.
      \retval KGFX_RESULT_BUSY_BLITTING
                        If the pixmap was busy with a blitter operation
                        so that the extension could not lock it, for
                        manual drawing.
      \retval KGFX_RESULT_INVALID_PRECONDITION
                        if the pixmap was locked for manual drawing.

      \see IKreaTvGfxExtensionObserver::OnGraphicsChanged()
   */
  virtual TKreaTvGfxResult
  Draw(TKreaTvGfxPixmapPointer destinationPixmap,
       TKreaTvGfxRectangle destinationRectangle,
       const TKreaTvGfxRectangle& damageRectangle,
       uint32_t destinationId = 0) throw () = 0;
};

#endif
