/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * IKreaTvGfxPixmap.h
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef IKGFXPIXMAP_H
#define IKGFXPIXMAP_H

#include "KreaTvGfxTypes.h"

/** \class IKreaTvGfxPixmap IKreaTvGfxPixmap.h "kreatvgfx/IKreaTvGfxPixmap.h"

    Pixmap objects represent a 2D array of pixels in graphics
    memory. Applications can access the contents of pixmaps
    directly or use the blitter.
 */
class IKreaTvGfxPixmap
{
public:
  /** Do not call the destructor directly. Always use the reference
      counting pointer type.

      \see TKreaTvGfxPixmapPointer;
  */
  virtual ~IKreaTvGfxPixmap() throw () { /* Empty */ }


  /** Get the width of the pixmap.

      \return           The width in pixels.
  */
  virtual uint32_t GetWidth() throw () = 0;


  /** Get the height of the pixmap.

      \return           The height in pixels.
  */
  virtual uint32_t GetHeight() throw () = 0;


  /** Get the pitch of the pixmap. The pitch is the number of bytes
      per line. This value is not always equal to the number bytes per
      pixel times the number of pixels, since padding may be required
      to meet the hardware alignment requirements. Applications must
      take this into account when drawing directly to the pixmap.
      The padding is otherwise unused.

      \return           The pitch in bytes.
  */
  virtual uint32_t GetPitch() throw () = 0;


  /** Get the pixel format.

      \return           The pixel format.
  */
  virtual TKreaTvGfxPixelFormat GetPixelFormat() throw () = 0;


  /** Get the premultiplied property.

      \retval true      The pixmap is premultiplied.
      \retval false     The pixmap is not premultiplied.
   */
  virtual bool IsPremultiplied() throw () = 0;


  /** Set a palette entry used by the pixmap. Palette entries can only
      be set for a pixmap in CLUT pixel format. Palette entries are
      given in ARGB pixel format.

      \param[in] index
                        The index of the palette entry to set.

      \param[in] value  The value for the palette entry, which
                        is given in ARGB pixel format.

      \retval KGFX_RESULT_OK
                        On success.
      \retval KGFX_RESULT_INVALID_PRECONDITION
                        If the pixel format of the pixmap is not
                        KGFX_PIXEL_FORMAT_CLUT8.
      \retval KGFX_RESULT_INVALID_ARGUMENT
                        If the given index is out of bounds,
                        where the valid range is from 0 to 255.
  */
  virtual TKreaTvGfxResult SetPaletteEntry(uint32_t index,
                                           uint32_t value) throw () = 0;


  /** Get a palette entry used by the pixmap. Palette entries can only
      be retrieved for a pixmap in CLUT pixel format. Palette entries are
      returned in ARGB pixel format. The returned value may differ slightly
      from the set value, due to conversions from internal pixel formats.

      \param[in] index
                        The index of the palette entry to set.

      \param[out] value The value of the palette entry, which
                        is returned in ARGB pixel format.

      \retval KGFX_RESULT_OK
                        On success.
      \retval KGFX_RESULT_INVALID_PRECONDITION
                        If the pixel format of the pixmap is not
                        KGFX_PIXEL_FORMAT_CLUT8.
      \retval KGFX_RESULT_INVALID_ARGUMENT
                        If the given index is out of bounds,
                        where the valid range is from 0 to 255.
  */
  virtual TKreaTvGfxResult GetPaletteEntry(uint32_t index,
                                           uint32_t& value) throw () = 0;


  /** Lock the graphics memory for a pixmap for reading and writing,
      e.g. "manual" drawing without blitter. When the pixmap is locked
      it cannot be used for blitting.

      \param[in] wait   Set to true if you want to wait until the pixmap
                        is available, i.e. not used by the blitter.

      \param[out] address
                        Address to the graphics memory area of the
                        pixmap.

      \retval KGFX_RESULT_OK
                        On success.
      \retval KGFX_RESULT_BUSY_BLITTING
                        If the pixmap is currently used by the blitter
                        and wait was false.
      \retval KGFX_RESULT_INVALID_PRECONDITION
                        If the pixmap is already locked.

      \see Unlock()
  */
  virtual TKreaTvGfxResult Lock(bool wait, uint8_t*& address) throw () = 0;


  /** Unlock the graphics memory for a pixmap. This has to be done before
      the blitter can access the pixmap.

      \retval KGFX_RESULT_OK
                        On success.
      \retval KGFX_RESULT_INVALID_PRECONDITION
                        If the address has not been locked.

      \see Lock()
  */
  virtual TKreaTvGfxResult Unlock() throw () = 0;

private:
  friend void intrusive_ptr_add_ref(IKreaTvGfxPixmap*);
  friend void intrusive_ptr_release(IKreaTvGfxPixmap*);
  virtual void AddRef() = 0;
  virtual void Release() = 0;
};


/** For internal use only, do not call directly. */
inline void intrusive_ptr_add_ref(IKreaTvGfxPixmap* pixmap)
{
  pixmap->AddRef();
}


/** For internal use only, do not call directly. */
inline void intrusive_ptr_release(IKreaTvGfxPixmap* pixmap)
{
  pixmap->Release();
}

#endif
