/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * IKreaTvGfxVideoLayer.h
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef IKREATVGFXVIDEOLAYER_H
#define IKREATVGFXVIDEOLAYER_H

#include "KreaTvGfxTypes.h"

/** \class IKreaTvGfxVideoLayer IKreaTvGfxVideoLayer.h "kreatvgfx/IKreaTvGfxVideoLayer.h"

    Each display can show a number of video streams. These are
    displayed in video layers, that are modeled with this
    interface. Each video layer can show a video from a different
    source, e.g. the main video decoder or the picture-in-picture
    video decoder.

    \see IKreaTvGfxDisplay::GetVideoLayer()

    \todo Somehow we need to get events when video size have
    changed. This should probably be events from the media manager,
    e.g. through IToiMediaSessionObserver.
*/
class IKreaTvGfxVideoLayer
{
protected:
  virtual ~IKreaTvGfxVideoLayer() throw () { /* Empty */ }

public:
  /** Enable or disable the video layer. If the video layer is
      disabled it will not be visible. All video layers are disabled
      by default.

      \param[in] enabledFlag
                        True to enable the video layer.

      \retval KGFX_RESULT_OK
                        On success.

      \todo Add support for the following return value:
              KGFX_RESULT_NOT_ENOUGH_RESOURCES
                        If there are hardware limitations, e.g.
                        the chipset may support picture-in-picture
                        only if a single display is enabled.
   */
  virtual TKreaTvGfxResult SetEnabled(bool enabledFlag) throw () = 0;


  /** Get the enabled status.

      \retval true      If the video layer is enabled.
      \retval false     If the video layer is disabled.

      \see SetEnabled()
  */
  virtual bool IsEnabled() const throw () = 0;


  /** Set the video layer z-index. A higher index means closer to the
      top of the video layer stack and a lower values means closer to
      the bottom of the stack. If two layers have the same z-index,
      the result is undefined.

      Graphics is always displayed on top of video and the application
      is responsible for "punching a hole" in the graphics to make the
      video visible, i.e. make the graphics transparent.

      \param[in] zIndex The new video layer z-index.

      \retval KGFX_RESULT_OK
                        On success.

      \todo Implement.
  */
  virtual void SetZIndex(uint32_t zIndex) throw () = 0;


  /** Get the video layer z-index.

      \return           The current video layer z-index.

      \see SetZIndex()

      \todo Implement.
  */
  virtual uint32_t GetZIndex() const throw () = 0;


  /** Set the source of the video layer.

      \param[in] sourceType
                        The source type.

      \retval KGFX_RESULT_OK
                        On success.
      \retval KGFX_RESULT_OPERATION_NOT_SUPPORTED
                        If the hardware doesn't support the source type.
  */
  virtual TKreaTvGfxResult
  SetSource(TKreaTvGfxVideoSourceType sourceType) throw () = 0;

  /** Set the position and size of the video window in the video
      layer, i.e. the area on the display to show the video on. The
      application is still responsible for punching a hole in the
      graphics to make the video visible.

      The co-ordinates are relative to the display, i.e. affected by
      the current video mode.

      Changes are applied on the next call to
      IKreaTvGfxDisplay::Update().

      \param[in] window
                        The area of the display to show the video in.

      \retval KGFX_RESULT_OK
                        On success,
      \retval KGFX_RESULT_INVALID_ARGUMENT
                        If the window rectangle is outside the
                        display.
  */
  virtual TKreaTvGfxResult
  SetVideoWindow(const TKreaTvGfxRectangle& window) throw () = 0;


  /** Set the area of the played video to show in the video window. If
      this function is called with a rectangle that is not empty
      (width or height non-zero), it will effectively override the
      adaptive video mode rules set up in IToiVideoOutputService to
      handle aspect ratio conversion. If the rectangle is empty but no
      adaptive video mode rule is applicable, the source rectangle
      will cover the entire video being played.

      The co-ordinates are relative to the current played video,
      i.e. affected by the resolution of the video clip.

      Changes are applied on the next call to
      IKreaTvGfxDisplay::Update().

      \param[in] rectangle
                        The area of the played video to show in the
                        video window.

      \retval KGFX_RESULT_OK
                        On success,
  */
  virtual TKreaTvGfxResult
  SetSourceRectangle(const TKreaTvGfxRectangle& rectangle) throw () = 0;

  /** Set the area of the video window to show the selected part of
      played video. If this function is called with a rectangle that is
      not empty (width or height non-zero), it will effectively
      override the adaptive video mode rules set up in
      IToiVideoOutputService to handle aspect ratio conversion. If the
      rectangle is empty but no adaptive video mode rule is
      applicable, the destination rectangle will cover the entire
      video window.

      The co-ordinates are relative to the currently set up video
      window, i.e. affected by the width and height of the video
      window. Position X=0,Y=0 refers to the top, left corner of the
      video window rather than the corner of the display.

      Changes are applied on the next call to
      IKreaTvGfxDisplay::Update().

      To accomplish pillar-boxing or letter-boxing, the destination
      rectangle can be set up to the relevant part of the video
      window.

      \param[in] rectangle
                        The area of the video window to show the video in.

      \retval KGFX_RESULT_OK
                        On success,
  */
  virtual TKreaTvGfxResult
  SetDestinationRectangle(const TKreaTvGfxRectangle& rectangle) throw () = 0;
};

#endif
