/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * KreaTvGfxTypes.h
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

/** \file */

#ifndef KREATVGFXTYPES_H
#define KREATVGFXTYPES_H

#include <stdint.h>
#include <boost/shared_ptr.hpp>
#include <boost/intrusive_ptr.hpp>

class IKreaTvGfx;
class IKreaTvGfxBlitter;
class IKreaTvGfxBlitterContext;
class IKreaTvGfxSimpleBlitterContext;
class IKreaTvGfxDetailedBlitterContext;
class IKreaTvGfxDisplay;
class IKreaTvGfxExtension;
class IKreaTvGfxPixmap;

class IIOEventLoop;


/** Pointer type for IKreaTvGfx objects. Used for automatic reference
    counting.
*/
typedef boost::shared_ptr<IKreaTvGfx> TKreaTvGfxPointer;


/** Pointer type for IKreaTvGfxDisplay objects. Used for automatic
    reference counting.
 */
typedef boost::shared_ptr<IKreaTvGfxDisplay> TKreaTvGfxDisplayPointer;


/** Pointer type for IKreaTvGfxPixmap objects. Used for automatic
    reference counting.
 */
typedef boost::intrusive_ptr<IKreaTvGfxPixmap> TKreaTvGfxPixmapPointer;


/** Pointer type for IKreaTvGfxBlitter objects. Used for automatic
    reference counting.
 */
typedef boost::shared_ptr<IKreaTvGfxBlitter> TKreaTvGfxBlitterPointer;


/** Pointer type for IKreaTvGfxBlitterContext objects. Used for
    automatic reference counting.
 */
typedef boost::shared_ptr<IKreaTvGfxBlitterContext>
TKreaTvGfxBlitterContextPointer;

/** Pointer type for IKreaTvGfxPorterDuffContext objects. Used for
    automatic reference counting.
 */
typedef boost::shared_ptr<IKreaTvGfxSimpleBlitterContext>
TKreaTvGfxSimpleBlitterContextPointer;

/** Pointer type for IKreaTvGfxDetailedBlitterContext objects. Used for
    automatic reference counting.
 */
typedef boost::shared_ptr<IKreaTvGfxDetailedBlitterContext>
TKreaTvGfxDetailedBlitterContextPointer;

/** Pointer type for IKreaTvGfxExtension objects. Used for
    automatic reference counting.
 */
typedef boost::shared_ptr<IKreaTvGfxExtension> TKreaTvGfxExtensionPointer;


/** \enum TKreaTvGfxDisplayType

    Display type identifiers.

    \see IKreaTvGfxDisplay interface for more information.
*/
enum TKreaTvGfxDisplayType {
  /** Identifier for the display for SD resolution outputs,
      e.g. SCART, S-Video and Composite. */
  KGFX_DISPLAY_TYPE_SD,

  /** Identifier for the display for HD resolution outputs, e.g HDMI
      and Component. */
  KGFX_DISPLAY_TYPE_HD
};


/** \enum TKreaTvGfxResult

    Possible return values.
*/
enum TKreaTvGfxResult
{
  /** The operation completed successfully. */
  KGFX_RESULT_OK,

  /** One or several arguments are invalid. */
  KGFX_RESULT_INVALID_ARGUMENT,

  /** The requested operation cannot be performed in the current state. */
  KGFX_RESULT_INVALID_PRECONDITION,

  /** An operation was attempted on an object that does not have the
      necessary capabilities. */
  KGFX_RESULT_OPERATION_NOT_SUPPORTED,

  /** The operation cannot be performed due to lack of system resources. */
  KGFX_RESULT_NOT_ENOUGH_RESOURCES,

  /** The pixmap is used in a blit and address cannot be acquire for
      read/write access. */
  KGFX_RESULT_BUSY_BLITTING,

  /** The display or blitter was not able to scale the
      pixmap. Different chipsets have different scaling
      constraints. */
  KGFX_RESULT_SCALING_ERROR,

  /** The operation requires vertical scaling, when it is not possible with flicker filtering enabled.*/
  KGFX_RESULT_SCALING_FLICKER_FILTER_ERROR,

  /** The display is disabled. */
  KGFX_RESULT_DISPLAY_DISABLED
};


/** \enum TKreaTvGfxAspectRatio

    Enumeration of possible aspect ratios. This is the aspect ratio of
    a video frame, rather than the aspect ratio of individual
    pixels.

    Note that SDTV is sometimes encoded with 704 horizontal pixels and
    sometimes with 720 pixels. However in both cases the pixel aspect
    ratio will be exactly the same. This means that the frame aspect
    ratio will vary slightly between 4 : 3 (704) and 4.1 : 3 (720) and
    this must be taken into account for a perfect result.

    TODO: Add cinema 2.21 : 1 aspect ratio?
*/
enum TKreaTvGfxAspectRatio
{
  /** 4:3 aspect ratio. */
  KGFX_ASPECT_RATIO_4_3,

  /** Widescreen 16:9 aspect ratio. */
  KGFX_ASPECT_RATIO_16_9
};


/** \enum TKreaTvGfxActiveArea

    Enumeration of possible active area aspect ratios. The active area
    is the part of a video frame that contains the actual
    image. E. g. letterboxing adds black bars to the top and bottom of
    an 16:9 image when it is scaled to fit within a 4:3 frame. In that
    case the frame aspect ratio would be 4:3 while the active area
    would retain the original 16:9 aspect ratio.

    For safe modes the image will extend to the borders of the
    frame but only a smaller part of the image is considered important
    and must be kept when changing the frame aspect ratio.

    TODO: Add all values defined by ETSI AFD standard?
*/
enum TKreaTvGfxActiveArea
{
  /** The image has a 4:3 aspect ratio. This value is allowed for both
      4:3 and 16:9 frames. */
  KGFX_ACTIVE_AREA_4_3,

  /** It is safe to keep only the center 4:3 portion of the image when
      converting from 16:9 to 4:3. */
  KGFX_ACTIVE_AREA_4_3_SAFE,

  /** The image has a 14:9 aspect ratio. This value is allowed for both
      4:3 and 16:9 frames. */
  KGFX_ACTIVE_AREA_14_9,

  /** It is safe to keep only the center 14:9 portion of the image when
      converting from 16:9 to 4:3 or from 4:3 to 16:9. */
  KGFX_ACTIVE_AREA_14_9_SAFE,

  /** The image has a 16:9 aspect ratio. This value is allowed for
      both 4:3 and 16:9 frames. Note that if the frame has 4:3 aspect
      ratio applications must make sure that OSD graphics is not
      placed in the top and bottom black bars since widescreen TV sets
      may scale the frame and keep only the center 16:9 portion. */
  KGFX_ACTIVE_AREA_16_9
};


/** \enum TKreaTvGfxPixelFormat

    Enumeration of possible pixel formats. All pixel formats should be
    supported by all chipsets.
*/
enum TKreaTvGfxPixelFormat
{
  /** 8-bit color lookup table */
  KGFX_PIXEL_FORMAT_CLUT8,

  /** RGB without alpha. When converting to ARGB format, the alpha
      component is set to 255 (opaque). */
  KGFX_PIXEL_FORMAT_RGB565,

  /** RGB with alpha. For the alpha component 0 is transparent and 255
      is opaque. */
  KGFX_PIXEL_FORMAT_ARGB8888
};

/** \enum TKreaTvGfxVideoSourceType

    Enumeration of possible video source types.
*/
enum TKreaTvGfxVideoSourceType {
  /** Video stream from the main video decoder. */
  KGFX_VIDEO_SOURCE_TYPE_VIDEO,

  /** Video stream from the picture-in-picture video decoder. */
  KGFX_VIDEO_SOURCE_TYPE_PIP,

  /** Video stream from the analog video input (ITU BT.656).
      \todo Do we need to support this now?
   */
  KGFX_VIDEO_SOURCE_TYPE_ANALOG_INPUT
};

/** Blend operations used in detailed context*/
enum TKreaTvGfxBlendOperation {
  /** Source + Destination */
  KGFX_BLEND_OPERATION_SOURCE_PLUS_DESTINATION,
  /** Source - Destination */
  KGFX_BLEND_OPERATION_SOURCE_MINUS_DESTINATION,
  /** Destination - Source */
  KGFX_BLEND_OPERATION_DESTINATION_MINUS_SOURCE
};

/** Blend factor used in detailed context*/
enum TKreaTvGfxBlendFactor {
  /** Constant factor equal to RGBA=(1.0, 1.0, 1.0, 1.0) */
  KGFX_BLEND_FACTOR_ONE,
  /** Constant factor equal to RGBA=(0.0, 0.0, 0.0, 0.0) */
  KGFX_BLEND_FACTOR_ZERO,
  /** Factor equal to RGBA=(Rsource, Gsource, Bsource, Asource) */
  KGFX_BLEND_FACTOR_SOURCE_COLOR,
  /** Factor equal to
      RGBA=(1.0 - Rsource, 1.0 - Gsource, 1.0 - Bsource, 1.0 - Asource) */
  KGFX_BLEND_FACTOR_INVERSE_SOURCE_COLOR,
  /** Factor equal to RGBA=(Rdest, Gdest, Bdest, Adest) */
  KGFX_BLEND_FACTOR_DESTINATION_COLOR,
  /** Factor equal to
      RGBA=(1.0 - Rdest, 1.0 - Gdest, 1.0 - Bdest, 1.0 - Adest) */
  KGFX_BLEND_FACTOR_INVERSE_DESTINATION_COLOR,
  /** Factor equal to RGBA=(Asource, Asource, Asource, Asource) */
  KGFX_BLEND_FACTOR_SOURCE_ALPHA,
  /** Factor equal to
      RGBA=(1.0 - Asource, 1.0 - Asource, 1.0 - Asource, 1.0 - Asource) */
  KGFX_BLEND_FACTOR_INVERSE_SOURCE_ALPHA,
  /** Factor equal to RGBA=(ConstantR, ConstantG, ConstantB, ConstantA). */
  KGFX_BLEND_FACTOR_CONSTANT,
  /** Factor equal to RGBA=(1.0 - ConstantR, 1.0 - ConstantG,
      1.0 - ConstantB, 1.0 - ConstantA).*/
  KGFX_BLEND_FACTOR_INVERSE_CONSTANT
};


/** \struct TKreaTvGfxSupportedBlending

    Data structure with a blitter blending operation/factors combination.
*/
struct TKreaTvGfxSupportedBlending
{
  /** The blend operation of the combination. */
  TKreaTvGfxBlendOperation BlendOperation;
  /** The source blend factor of the combination. */
  TKreaTvGfxBlendFactor SourceBlendFactor;
  /** The destination blend factor of the combination. */
  TKreaTvGfxBlendFactor DestinationBlendFactor;
};


/** \struct TKreaTvGfxVideoMode

    Data structure with output video mode information.
*/
struct TKreaTvGfxVideoMode
{
  /** True if display is enabled. */
  bool EnabledFlag;

  /** Number of active pixels in the horizontal direction of the display. */
  uint32_t Width;

  /** Number of active pixels in the vertical direction of the display. */
  uint32_t Height;

  /** Number of times the display is refreshed per second, rounded to
      the neares integer. For interlaced modes the refresh rate is
      equal to the field rate rather than the frame rate. */
  uint32_t RefreshRate;

  /** Flag indicating whether each refresh will update a single field
      or the entire frame. Traditional SD video modes are
      interlaced. */
  bool Interlaced;
};


/** \struct TKreaTvGfxWidescreenSignaling

    Data structure with widescreen signaling (WSS) parameters.
*/
struct TKreaTvGfxWidescreenSignaling
{
  /** Aspect ratio of the video frame. */
  TKreaTvGfxAspectRatio AspectRatio;

  /** Aspect ratio of the active area. */
  TKreaTvGfxActiveArea ActiveArea;
};



/** \struct TKreaTvGfxRectangle

    Data structure representing a rectangle.

    \todo Replace this with a sophisticated class instead.
*/
struct TKreaTvGfxRectangle
{
  /** Create rectangle with all attributes set to zero. */
  TKreaTvGfxRectangle()
    : X(0), Y(0), Width(0), Height(0)
  { /* Empty */ }

  /** Create rectangle with given dimensions. */
  TKreaTvGfxRectangle(int32_t x, int32_t y,
                      uint32_t width, uint32_t height)
    : X(x), Y(y), Width(width), Height(height)
  { /* Empty */ }

  /** Horizontal position of left edge. */
  int32_t X;

  /** Vertical position of top edge. */
  int32_t Y;

  /** Horizontal size. */
  uint32_t Width;

  /** Vertical size. */
  uint32_t Height;
};

#endif
