/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * IKreaTvInput.h
 *
 * Entry point to the KreaTV input library.
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

/** \file */

#ifndef IKREATVINPUT_H
#define IKREATVINPUT_H

#include "KreaTvInputTypes.h"
#include <boost/shared_ptr.hpp>

class IKreaTvInputObserver;
class IIOEventLoop;

/** \class IKreaTvInput IKreaTvInput.h "kreatvinput/IKreaTvInput.h"

    This is the main interface of the KreaTV input library. An
    instance of this interface can be created using the global function
    CreateKreaTvInput(). Only one instance can be created per application.
*/
class IKreaTvInput
{
public:
  /** Do not call the destructor directly. Always use the reference
      counting pointer type.

      \see TKreaTvInputPointer
  */
  virtual ~IKreaTvInput() { /* Empty */ }


  /** Get the current state of a key.

      \param[in] key    Key which state should be polled.

      \param[out] state True if active (down), false otherwise.

      \retval KINPUT_RESULT_OK
                        On success.
      \retval KINPUT_RESULT_INVALID_ARGUMENT
                        If the key is unknown.
      \retval KINPUT_RESULT_INTERNAL_ERROR
                        If there is some internal error
  */
  virtual TKreaTvInputResult GetKeyState(TKreaTvInputKey key,
                                         bool& state) throw () = 0;


  /** Get all currently active modifiers.

      \param[out] modifiers
                        Bitmask with currently active modifiers.

      \retval KINPUT_RESULT_OK
                        On success.
      \retval KINPUT_RESULT_INTERNAL_ERROR
                        If there is some internal error
  */
  virtual TKreaTvInputResult GetModifiers(uint32_t& modifiers) throw () = 0;

  /** Lookup the key symbol for a given key and modifier state. The
      value returned depends on the currently selected keyboard
      layout.

      \param key        Key for which a lookup should be performed.

      \param modifiers  The modifier state.

      \param keysym     Key symbol. The same values as in X11 and GTK+
                        are used.

      \retval           KINPUT_RESULT_OK on success.
  */
  virtual TKreaTvInputResult LookupKeysym(TKreaTvInputKey key,
                                          uint32_t modifiers,
                                          uint32_t& keysym) throw () = 0;
};

/** Pointer type for IKreaTvInput objects. Used for automatic
    reference counting. */
typedef boost::shared_ptr<IKreaTvInput> TKreaTvInputPointer;


/** Global entry point.

    \param[in] eventLoop 
                        Event loop that the implementation will use to
                        monitor file descriptors and create timers.
    \param[in] observer 
                        An application object implementing the
                        IKreaTvInputObserver interface. NULL can be
                        used if no callbacks are desired.

    \param[out] kreaTvInput
                        The entry point for KreaTV input. If this
                        function is called more than once, the same
                        underlying object will be returned.
*/
void CreateKreaTvInput(IIOEventLoop* eventLoop,
                       IKreaTvInputObserver* observer,
                       TKreaTvInputPointer& kreaTvInput) throw ();

#endif
