






#ifndef MPEG2CLOCK_H
#define MPEG2CLOCK_H

#include <stdint.h>

namespace Mpeg2Clock {

const uint32_t FACTOR_27MHZ_90KHZ = 300;
const uint32_t FACTOR_90KHZ_45KHZ = 2;
const uint64_t PTS_MAX = 1ULL << 33;
const uint64_t STC_MAX = PTS_MAX * FACTOR_27MHZ_90KHZ;

/**
 * Represents an MPEG2 clock running at 45 kHz.
 *
 * There is no operator uint32_t() to avoid the risk that the compiler does the
 * equivalent of e.g. T90KhzClock(clock45Khz.Get()) as that would result in an
 * incorrect 90 kHz clock.
 *
 * To convert to e.g. a 90 kHz clock, do T90KhzClock(clock45Khz). To convert it
 * to a 27 MHz clock, do T27MhzClock(T90KhzClock(clock45Khz)).
 */
class T45KhzClock
{
private:
  uint32_t Clock;

public:
  explicit T45KhzClock(uint32_t clock) : Clock(clock) {}

  uint32_t GetInMs() const { return Clock / 45; }
  static T45KhzClock FromMs(uint32_t ms) { return T45KhzClock(ms * 45); }

  
  uint32_t Get() const { return Clock; }
  void Set(uint32_t clock) { Clock = clock; }
};

/**
 * Represents an MPEG2 clock running at 90 kHz.
 *
 * See the documentation for T45KhzClock for the reason why there is no
 * operator uint64_t().
 */
class T90KhzClock
{
private:
  uint64_t Clock;

public:
  explicit T90KhzClock(uint64_t clock) : Clock(clock) {}
  T90KhzClock(uint32_t base, uint8_t bit33) { Set(base, bit33); }
  T90KhzClock(T45KhzClock clock45Khz) { Set(clock45Khz); }

  uint32_t GetInMs() const { return Clock / 90; }
  inline static T90KhzClock FromMs(uint32_t ms);

  
  uint64_t Get() const { return Clock; }
  void Set(uint64_t clock) { Clock = clock; }

  
  inline void Get(uint32_t& base, uint8_t& bit33) const;
  inline void Set(uint32_t base, uint8_t bit33);

  
  inline T45KhzClock Get45KhzClock() const;
  inline void Set(T45KhzClock clock45Khz);
};

inline T90KhzClock T90KhzClock::FromMs(uint32_t ms)
{
  return T90KhzClock(static_cast<uint64_t>(ms) * 90);
}

inline void T90KhzClock::Get(uint32_t& base, uint8_t& bit33) const
{
  base = static_cast<uint32_t>(Clock);
  bit33 = Clock & (1ULL << 32) ? 1 : 0;
}

inline void T90KhzClock::Set(uint32_t base, uint8_t bit33)
{
  Set((bit33 ? 1ULL << 32 : 0) + static_cast<uint64_t>(base));
}

inline T45KhzClock T90KhzClock::Get45KhzClock() const
{
  return T45KhzClock(Clock / FACTOR_90KHZ_45KHZ);
}

inline void T90KhzClock::Set(T45KhzClock clock45Khz)
{
  Set(static_cast<uint64_t>(clock45Khz.Get()) * FACTOR_90KHZ_45KHZ);
}

/**
 * Represents an MPEG2 clock running at 27 MHz.
 *
 * See the documentation for T45KhzClock for the reason why there is no
 * operator uint64_t().
 */
class T27MhzClock
{
private:
  uint64_t Clock;

public:
  explicit T27MhzClock(uint64_t clock) : Clock(clock) {}
  T27MhzClock(uint32_t base, uint8_t bit33, uint16_t extension)
  { Set(base, bit33, extension); }
  T27MhzClock(uint32_t high, uint32_t low) { Set(high, low); }
  T27MhzClock(T90KhzClock clock90Khz) { Set(clock90Khz); }

  uint32_t GetInMs() const { return Clock / 27000; }
  static inline T27MhzClock FromMs(uint32_t ms);

  
  uint64_t Get() const { return Clock; }
  void Set(uint64_t clock) { Clock = clock; }

  
  inline void Get(uint32_t& base, uint8_t& bit33, uint16_t& extension) const;
  inline void Set(uint32_t base, uint8_t bit33, uint16_t extension);

  
  
  inline void Get(uint32_t& high, uint32_t& low) const;
  inline void Set(uint32_t high, uint32_t low);

  
  inline T90KhzClock Get90KhzClock() const;
  inline void Set(T90KhzClock clock90Khz);
};

inline T27MhzClock T27MhzClock::FromMs(uint32_t ms)
{
  return T27MhzClock(static_cast<uint64_t>(ms) * 27000);
}

inline void T27MhzClock::Get(
  uint32_t& base, uint8_t& bit33, uint16_t& extension) const
{
  Get90KhzClock().Get(base, bit33);
  extension = Clock % FACTOR_27MHZ_90KHZ;
}

inline void T27MhzClock::Set(uint32_t base, uint8_t bit33, uint16_t extension)
{
  Set(T90KhzClock(base, bit33));
  Clock += extension;
}

inline void T27MhzClock::Get(uint32_t& high, uint32_t& low) const
{
  const T90KhzClock clock90 = Get90KhzClock();
  high = clock90.Get() >> 1;
  low = ((clock90.Get() & 0x1) << 9) | (Clock % FACTOR_27MHZ_90KHZ);
}

inline void T27MhzClock::Set(uint32_t high, uint32_t low)
{
  const uint8_t bit33 = (high & 0x80000000) ? 1 : 0;
  const uint32_t base = (high << 1) | ((low & 0x200) >> 9);
  Set(base, bit33, low & 0x1ff);
}

inline T90KhzClock T27MhzClock::Get90KhzClock() const
{
  return T90KhzClock(Clock / FACTOR_27MHZ_90KHZ);
}

inline void T27MhzClock::Set(T90KhzClock clock90Khz)
{
  Set(clock90Khz.Get() * FACTOR_27MHZ_90KHZ);
}


template<typename T> struct TValueType;
template<> struct TValueType<T45KhzClock> { typedef uint32_t ValueType; };
template<> struct TValueType<T90KhzClock> { typedef uint64_t ValueType; };
template<> struct TValueType<T27MhzClock> { typedef uint64_t ValueType; };

/**
 * An MPEG2 clock that in addition to holding the clock value can indicate if
 * it is valid (i.e. set) or not.
 */
template<typename T>
class TClock
{
private:
  typedef typename TValueType<T>::ValueType TValueType;

  T Clock;
  bool Valid;

public:
  TClock() : Clock(0), Valid(false) {}
  bool IsValid() const { return Valid; }

  void Set(TValueType clock) { Clock.Set(clock); Valid = true; }
  void Set(T clock) { Clock = clock; Valid = true; }
  void Unset() { Valid = false; }

  TValueType Get() const { return Clock.Get(); }
  uint32_t GetInMs() const { return Clock.GetInMs(); }

  T GetClock() const { return Clock; }
  operator T() const { return GetClock(); }

  const T& operator=(const T& clock) { Set(clock); return clock; }
};





inline int64_t PtsDiff(T90KhzClock pts1, T90KhzClock pts2)
{
  int64_t diff = pts1.Get() - pts2.Get();
  const int64_t max = PTS_MAX;
  if (diff > max / 2) {
    diff = diff - max;
  }
  else if (diff < -max / 2) {
    diff = diff + max;
  }
  return diff;
}

inline int64_t StcDiff(T27MhzClock stc1, T27MhzClock stc2)
{
  int64_t diff = stc1.Get() - stc2.Get();
  const int64_t max = STC_MAX;
  if (diff > max / 2) {
    diff = diff - max;
  }
  else if (diff < -max / 2) {
    diff = diff + max;
  }
  return diff;
}

inline T27MhzClock StcAdd(T27MhzClock stc1, T27MhzClock stc2)
{
  return T27MhzClock((stc1.Get() + stc2.Get()) % STC_MAX);
}

inline T27MhzClock StcSubtract(T27MhzClock stc1, T27MhzClock stc2)
{
  if (stc1.Get() >= stc2.Get()) {
    return T27MhzClock(stc1.Get() - stc2.Get());
  }
  else {
    return T27MhzClock(STC_MAX - (stc2.Get() - stc1.Get()));
  }
}

inline int64_t ClockDiffMs(T27MhzClock stc1, T27MhzClock stc2)
{
  return StcDiff(stc1, stc2) / 27000;
}

} 

#endif
