/*
 *--------------------------------------------------------------------
 *
 * TAc3FrameHandle.h
 *
 * A class for accessing an ac3 synch frame.
 *
 * Each synchronization frame contains 6 coded
 * audio blocks (AB), each of which represent 256 
 * new audio samples.
 *
 * Copyright (c) 2004 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_LIBH222_TAC3FRAMEHANDLE_H
#define MEDIA_LIBH222_TAC3FRAMEHANDLE_H

#include <endian.h>
#include <stdint.h>

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif

struct TAc3FrameHeader
{
  unsigned syncword           :16;
  unsigned crc1               :16;

  unsigned frmsizecod         :6;
  unsigned fscod              :2;
  
  unsigned bsmod              :3;
  unsigned bsid               :5;
  
  unsigned                    :5;
  unsigned acmod              :3;
};

class TAc3FrameHandle
{
public:
  enum { HEADER_SIZE = 7 };

private:
  static const int SampleRates[4];
  static const int BitRates[19];
  static const int Channels[8];
  static const int FrameLengths[38][3];

  const TAc3FrameHeader* Header;

public:
  TAc3FrameHandle(const uint8_t* data) throw ();

  uint16_t GetSampleRate() const throw ();
  uint16_t GetNumberOfChannels() const throw ();  
  uint16_t GetBitRate() const throw ();  
  uint16_t GetFrameLength() const throw ();

  bool IsValid() const throw ();  
};


inline TAc3FrameHandle::TAc3FrameHandle(const uint8_t* data) throw ()
  : Header(reinterpret_cast<const TAc3FrameHeader*>(data))
{
}

inline uint16_t TAc3FrameHandle::GetSampleRate() const throw ()
{
  return SampleRates[Header->fscod];
}

inline uint16_t TAc3FrameHandle::GetNumberOfChannels() const throw ()
{
  return Channels[Header->acmod];
}

inline uint16_t TAc3FrameHandle::GetBitRate() const throw ()
{
  return BitRates[Header->frmsizecod >> 1];
}

inline uint16_t TAc3FrameHandle::GetFrameLength() const throw ()
{
  return FrameLengths[Header->frmsizecod][2 - Header->fscod] << 1;
}

inline bool TAc3FrameHandle::IsValid() const throw ()
{
  return ((Header->syncword == 0x770b) && Header->fscod != 3); 
}

#endif
