/*
 *--------------------------------------------------------------------
 *
 * TAdtsFrameHandle
 *
 * A class for accessing an ADTS frame.
 *
 * Copyright (c) 2005 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_LIBH222_TADTSFRAMEHANDLE_H
#define MEDIA_LIBH222_TADTSFRAMEHANDLE_H

#include <endian.h>
#include <stdint.h>
#include <cstdio>

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif



struct TAdtsFrameHeader
{
  unsigned syncword_high        :8;
                                
  unsigned protection_absent    :1;
  unsigned layer                :2;
  unsigned ID                   :1;
  unsigned syncword_low         :4;
                                
  unsigned channel_config_2     :1;
  unsigned private_bit          :1;
  unsigned sampling_frequency   :4;
  unsigned profile              :2;
                                
  unsigned frame_length_11_12   :2;
  unsigned copyright_id_start   :1;
  unsigned copyright_id_bit     :1;
  unsigned home                 :1;
  unsigned original_or_copy     :1;
  unsigned channel_config_0_1   :2;
                                
  unsigned frame_length_3_10    :8;

  unsigned buffer_fullness_6_10 :5;
  unsigned frame_length_0_2     :3;

  unsigned no_raw_blocks        :2;
  unsigned buffer_fullness_0_5  :6;
};

class TAdtsFrameHandle
{
public:
  enum { HEADER_SIZE = 7 };

protected:
  const TAdtsFrameHeader* Header;

public:
  TAdtsFrameHandle(const TAdtsFrameHeader* data) throw ();
  TAdtsFrameHandle(const uint8_t* data) throw ();

  
  uint16_t GetSyncWord() const throw ();
  uint8_t GetId() const throw ();
  uint8_t GetLayer() const throw ();
  uint8_t GetProtectionAbsentBit() const throw ();
  uint8_t GetProfile() const throw ();
  uint8_t GetSamplingFrequency() const throw ();
  uint8_t GetPrivateBit() const throw ();
  uint8_t GetChannelConfiguration() const throw ();
  uint8_t GetOriginalOrCopy() const throw ();
  uint8_t GetHome() const throw ();
  uint8_t GetCopyrightId() const throw ();
  uint8_t GetCopyrightStart() const throw ();
  uint16_t GetFrameLength() const throw ();
  uint16_t GetBufferFullness() const throw ();
  uint16_t GetNoRawBlocks() const throw ();

  uint32_t GetLookedUpSamplingFrequency() const throw ();
  bool IsVbr() const throw ();

  
  bool IsValid() const throw ();  
};

inline
TAdtsFrameHandle::TAdtsFrameHandle(const TAdtsFrameHeader* data) 
  throw ()
  : Header(data)
{
}

inline TAdtsFrameHandle::TAdtsFrameHandle(const uint8_t* data) throw ()
  : Header(reinterpret_cast<const TAdtsFrameHeader*>(data))
{
}

inline uint16_t TAdtsFrameHandle::GetSyncWord() const throw ()
{
  return (Header->syncword_high << 4 | Header->syncword_low);
}

inline uint8_t TAdtsFrameHandle::GetId() const throw ()
{
  return Header->ID;
}

inline uint8_t TAdtsFrameHandle::GetLayer() const throw ()
{
  return Header->layer;
}

inline uint8_t TAdtsFrameHandle::GetProtectionAbsentBit() const throw ()
{
  return Header->protection_absent;
}

inline uint8_t TAdtsFrameHandle::GetProfile() const throw ()
{
  return Header->profile;
}

inline uint8_t TAdtsFrameHandle::GetSamplingFrequency() const throw ()
{
  return Header->sampling_frequency;
}

inline uint8_t TAdtsFrameHandle::GetPrivateBit() const throw ()
{
  return Header->private_bit;
}

inline uint8_t TAdtsFrameHandle::GetChannelConfiguration() const throw ()
{
  return Header->channel_config_0_1 | (Header->channel_config_2 << 2);
}

inline uint8_t TAdtsFrameHandle::GetOriginalOrCopy() const throw ()
{
  return Header->original_or_copy;
}

inline uint8_t TAdtsFrameHandle::GetHome() const throw ()
{
  return Header->home;
}

inline uint8_t TAdtsFrameHandle::GetCopyrightId() const throw ()
{
  return Header->copyright_id_bit;
}

inline uint8_t TAdtsFrameHandle::GetCopyrightStart() const throw ()
{
  return Header->copyright_id_start;
}

inline uint16_t TAdtsFrameHandle::GetFrameLength() const throw ()
{
  return Header->frame_length_0_2
    | (Header->frame_length_3_10 << 3)
    | (Header->frame_length_11_12 << 11);
}

inline uint16_t TAdtsFrameHandle::GetBufferFullness() const throw ()
{
  return Header->buffer_fullness_0_5 | (Header->buffer_fullness_6_10 << 6);
}

inline uint16_t TAdtsFrameHandle::GetNoRawBlocks() const throw ()
{
  return Header->no_raw_blocks;
}

inline bool TAdtsFrameHandle::IsVbr() const throw ()
{
  return GetBufferFullness() == 0x7ff;
}

inline bool TAdtsFrameHandle::IsValid() const throw ()
{
  return Header != NULL
    && GetSyncWord() == 0xfff
    && GetLayer() == 0
    && GetId() == 1
    && GetFrameLength() >= HEADER_SIZE;
}

#endif
