/*
 *--------------------------------------------------------------------
 *
 * libH222/TMpeg2Util.h
 *
 * A generic class for MPEG2 utilities.
 *
 * Copyright (c) 2002 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_COMMON_TMPEG2UTIL_H
#define MEDIA_COMMON_TMPEG2UTIL_H

#include <stdint.h>

class TMpeg2Util
{
public:
  static uint16_t Make16(uint8_t high, uint8_t low) throw ();
  static uint32_t Make32(uint8_t highest, uint8_t high,
                         uint8_t low, uint8_t lowest) throw ();
  static uint64_t Make64(uint8_t high1, uint8_t high2,
                         uint8_t high3, uint8_t high4,
                         uint8_t low1, uint8_t low2,
                         uint8_t low3, uint8_t low4) throw ();
  static uint16_t Reorder16(uint16_t data) throw ();
  static uint32_t Reorder24(uint32_t data) throw ();
  static uint32_t Reorder32(uint32_t data) throw ();

  static void SetBitField(uint8_t* buffer, uint32_t bitOffset,
                          uint32_t value, uint32_t fieldSize) throw ();

  /**
   * Get a boolean value from a byte array
   * 
   * @param buffer the byte array
   * @param pos  I the bit to get
   * 
   * @return the value of the bit (true if bit is 1)
   */
  static bool GetBool(const uint8_t* const buffer, const uint16_t pos);
  
  /** 
   * Get data ffrom a byte array
   *
   * 
   * @param buffer the byte array
   * @param start  I Start bit
   * @param stop   I Stop bit
   * 
   * @return  The data requested
   */
  static uint8_t GetUint8(const uint8_t* const buffer, const uint16_t start, const uint16_t stop = 8);
  
  /** 
   * Get data from a byte array
   * 
   * @param buffer the byte array
   * @param start  I Start bit
   * @param stop   I Stop bit
   * 
   * @return  The data requested
   */
  static uint16_t GetUint16(const uint8_t* const buffer, const uint16_t start, const uint16_t stop = 16);
  
  /** 
   * Get data from a byte array
   *
   * @param buffer the byte array
   * @param start  I Start bit
   * @param stop   I Stop bit
   * 
   * @return  The data requested
   */
  static uint32_t GetUint32(const uint8_t* const buffer, const uint16_t start, const uint16_t stop = 32);

 protected:
  /** 
   * Get bits
   *
   * Get certain bits from a byte array
   * 
   * @param x         I  The array
   * @param p         I  Position in the array
   * @param n         I  Number of bits to get
   * 
   * @return 
   */
  static uint32_t GetBits(const uint8_t* const x,
			  const uint16_t p,
			  const uint16_t n);
};

inline uint16_t TMpeg2Util::Make16(uint8_t high, uint8_t low) throw ()
{
  return (high << 8) | low;
}

inline uint32_t TMpeg2Util::Make32(uint8_t highest, uint8_t high,
                                   uint8_t low, uint8_t lowest) throw ()
{
  return (highest << 24) | (high << 16) | (low << 8) | lowest;
}

inline uint64_t TMpeg2Util::Make64(uint8_t high1, uint8_t high2,
                                   uint8_t high3, uint8_t high4,
                                   uint8_t low1, uint8_t low2,
                                   uint8_t low3, uint8_t low4) throw ()
{
  uint64_t result = high1;
  result = (result << 8) | high2;
  result = (result << 8) | high3;
  result = (result << 8) | high4;
  result = (result << 8) | low1;
  result = (result << 8) | low2;
  result = (result << 8) | low3;
  result = (result << 8) | low4;
  return result;
}

inline uint16_t TMpeg2Util::Reorder16(uint16_t data) throw ()
{
  return (((data & 0x00ff) << 8) |
          ((data & 0xff00) >> 8));
}

inline uint32_t TMpeg2Util::Reorder24(uint32_t data) throw ()
{
  return (((data & 0x000000ff) << 16) |
          ((data & 0x0000ff00)) |
          ((data & 0x00ff0000) >> 16));
}

inline uint32_t TMpeg2Util::Reorder32(uint32_t data) throw ()
{
  return (((data & 0x000000ff) << 24) |
          ((data & 0x0000ff00) << 8) |
          ((data & 0x00ff0000) >> 8) |
          ((data & 0xff000000) >> 24));
}

#endif
