/*
 *--------------------------------------------------------------------
 *
 * TMpegAudioFrameHandle.h
 *
 * A class for accessing an mpeg audio frame.
 *
 * Copyright (c) 2004 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_LIBH222_TMPEGAUDIOFRAMEHANDLE_H
#define MEDIA_LIBH222_TMPEGAUDIOFRAMEHANDLE_H

#include <endian.h>
#include <stdint.h>
#include <cstdio>

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif



struct TMpegAudioFrameHeader
{
  unsigned syncword_high      :8;

  unsigned protection_bit     :1;
  unsigned layer              :2;
  unsigned ID                 :1;
  unsigned mpeg25             :1;
  unsigned syncword_low       :3;

  unsigned private_bit        :1;
  unsigned padding_bit        :1;
  unsigned sampling_frequency :2;
  unsigned bitrate_index      :4;

  unsigned emphasis           :2;
  unsigned original_or_copy   :1;
  unsigned copyright          :1;
  unsigned mode_extension     :2;
  unsigned mode               :2;
};

class TMpegAudioFrameHandle
{
public:
  enum { HEADER_SIZE = 4 };

protected:
  const TMpegAudioFrameHeader* Header;

public:
  TMpegAudioFrameHandle(const TMpegAudioFrameHeader* data) throw ();
  TMpegAudioFrameHandle(const uint8_t* data) throw ();

  
  uint16_t GetSynchWord() const throw ();
  uint8_t GetId() const throw ();
  uint8_t GetMpeg25() const throw ();
  uint8_t GetLayer() const throw ();
  uint8_t GetProtectionBit() const throw ();
  uint8_t GetBitrateIndex() const throw ();
  uint8_t GetSamplingFrequency() const throw ();
  uint8_t GetPaddingBit() const throw ();
  uint8_t GetPrivateBit() const throw ();
  uint8_t GetMode() const throw ();
  uint8_t GetModeExtension() const throw ();
  uint8_t GetCopyright() const throw ();
  uint8_t GetOriginalOrCopy() const throw ();
  uint8_t GetEmphasis() const throw ();
  
  uint16_t GetCalculatedFrameLength() const throw ();
  uint16_t GetLookedUpSampleRate() const throw ();
  uint16_t GetLookedUpBitRate() const throw ();  
  uint8_t GetLookedUpLayer() const throw ();

  
  bool IsValid() const throw ();  
};

inline
TMpegAudioFrameHandle::TMpegAudioFrameHandle(const TMpegAudioFrameHeader* data) 
  throw ()
  : Header(data)
{
}

inline TMpegAudioFrameHandle::TMpegAudioFrameHandle(const uint8_t* data) throw ()
  : Header(reinterpret_cast<const TMpegAudioFrameHeader*>(data))
{
}

inline uint16_t TMpegAudioFrameHandle::GetSynchWord() const throw ()
{
  return (Header->syncword_high << 3 | Header->syncword_low);
}

inline uint8_t TMpegAudioFrameHandle::GetId() const throw ()
{
  return Header->ID;
}

inline uint8_t TMpegAudioFrameHandle::GetMpeg25() const throw ()
{
  return Header->mpeg25;
}

inline uint8_t TMpegAudioFrameHandle::GetLayer() const throw ()
{
  return Header->layer;
}

inline uint8_t TMpegAudioFrameHandle::GetProtectionBit() const throw ()
{
  return Header->protection_bit;
}

inline uint8_t TMpegAudioFrameHandle::GetBitrateIndex() const throw ()
{
  return Header->bitrate_index;
}

inline uint8_t TMpegAudioFrameHandle::GetSamplingFrequency() const throw ()
{
  return Header->sampling_frequency;
}

inline uint8_t TMpegAudioFrameHandle::GetPaddingBit() const throw ()
{
  return Header->padding_bit;
}

inline uint8_t TMpegAudioFrameHandle::GetPrivateBit() const throw ()
{
  return Header->private_bit;
}

inline uint8_t TMpegAudioFrameHandle::GetMode() const throw ()
{
  return Header->mode;
}

inline uint8_t TMpegAudioFrameHandle::GetModeExtension() const throw ()
{
  return Header->mode_extension;
}

inline uint8_t TMpegAudioFrameHandle::GetCopyright() const throw ()
{
  return Header->copyright;
}

inline uint8_t TMpegAudioFrameHandle::GetOriginalOrCopy() const throw ()
{
  return Header->original_or_copy;
}

inline uint8_t TMpegAudioFrameHandle::GetEmphasis() const throw ()
{
  return Header->emphasis;
}

inline bool TMpegAudioFrameHandle::IsValid() const throw ()
{
  return Header != NULL
    && GetSynchWord() == 0x7ff
    && !(GetId() == 1 && GetMpeg25() == 0)
    && GetLayer() != 0
    && GetSamplingFrequency() != 3
    && GetBitrateIndex() != 15
    && GetBitrateIndex() != 0;
}

#endif
