/*
 *--------------------------------------------------------------------
 *
 * TMpegSequenceHeaderHandle.h
 *
 * A class for accessing an mpeg sequence header.
 *
 * Copyright (c) 2004 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_LIBH222_TMPEGSEQUENCEHEADERHANDLE_H
#define MEDIA_LIBH222_TMPEGSEQUENCEHEADERHANDLE_H

#include <endian.h>
#include <stdint.h>

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif

class TMpegSequenceHeaderHandle
{
protected:
  enum {
    SEQUENCE_HEADER_START_CODE = 0xb3,
    SEQUENCE_HEADER_EXTENSION_START_CODE = 0xb5,
    GROUP_START_CODE = 0xb8,
    SEQUENCE_HEADER_EXTENSION_START_CODE_ID = 1
  };

  
  struct TMpegSequenceHeader
  {
    unsigned sequence_header_code   : 32;
    unsigned horizontal_size_high   : 8;

    unsigned vertical_size_high     : 4;
    unsigned horizontal_size_low    : 4;

    unsigned vertical_size_low      : 8;

    unsigned frame_rate             : 4;
    unsigned aspect_ratio           : 4;

    unsigned bit_rate_high          : 8;

    unsigned bit_rate_middle        : 8;

    unsigned vbv_buffer_size_high   : 5;
    unsigned marker                 : 1;
    unsigned bit_rate_low           : 2;

    unsigned                        : 2;
    unsigned constrained_parameters : 1;
    unsigned vbv_buffer_size_low    : 5;
  };

  
  struct TMpegSequenceHeaderExtension
  {
    unsigned extension_start_code           : 32;

    unsigned profile_and_level_high         : 4;
    unsigned extension_start_code_id        : 4;

    unsigned horizontal_size_extension_high : 1;
    unsigned chroma_format                  : 2;
    unsigned progressive_sequence           : 1;
    unsigned profile_and_level_low          : 4;

    unsigned bit_rate_extension_high        : 5;
    unsigned vertical_size_extension        : 2;
    unsigned horizontal_size_extension_low  : 1;

    unsigned marker                         : 1;
    unsigned bit_rate_extension_low         : 7;

    unsigned vbv_buffer_size_extension      : 8;

    unsigned frame_rate_extension_d         : 5;
    unsigned frame_rate_extension_n         : 2;
    unsigned low_delay                      : 1;
  };

  const TMpegSequenceHeader* Header;
  const TMpegSequenceHeaderExtension* Extension;

  bool HasExtension;
  bool Valid;
  
  void CheckForExtension() throw ();

public:
  enum TAspectRatio {
    ASPECT_UNKNOWN,
    ASPECT_SQUARE,
    ASPECT_4_3,
    ASPECT_16_9,
    ASPECT_221_1
  };
  enum { MinimalDataSize = sizeof(TMpegSequenceHeader) +
         64 + 64 + sizeof(TMpegSequenceHeaderExtension) };

  TMpegSequenceHeaderHandle(const uint8_t* data) throw ();

  uint16_t GetVerticalSize() const throw();
  uint16_t GetHorizontalSize() const throw();
  uint32_t GetBitRate() const throw();
  float GetFrameRate() const throw();
  bool GetProgressiveSequence() const throw();
  uint8_t GetProfileAndLevel() const throw();
  TAspectRatio GetAspectRatio() const throw();

  bool IsValid() const throw();
};

inline TMpegSequenceHeaderHandle::
TMpegSequenceHeaderHandle(const uint8_t* data) throw ()
  : Header(reinterpret_cast<const TMpegSequenceHeader*>(data)),
    HasExtension(false)
{
  CheckForExtension();
}

inline uint16_t TMpegSequenceHeaderHandle::GetVerticalSize() const throw()
{
  return (Header->vertical_size_high << 8) | Header->vertical_size_low
          | (HasExtension ? (Extension->vertical_size_extension << 12) : 0);
}

inline uint16_t TMpegSequenceHeaderHandle::GetHorizontalSize() const throw()
{
  return (Header->horizontal_size_high << 4) | Header->horizontal_size_low
          | (HasExtension ? ((Extension->horizontal_size_extension_low << 12)
                             | (Extension->horizontal_size_extension_high << 13)) : 0);
}

inline uint32_t TMpegSequenceHeaderHandle::GetBitRate() const throw()
{
  return 400 * ((Header->bit_rate_high << 10) | (Header->bit_rate_middle << 2)
                | (Header->bit_rate_low)
                | (HasExtension ? ((Extension->bit_rate_extension_low << 18)
                                   | (Extension->bit_rate_extension_high << 25)) : 0));
}

inline bool TMpegSequenceHeaderHandle::GetProgressiveSequence() const throw()
{
  return HasExtension ? (Extension->progressive_sequence != 0) : false;
}

inline uint8_t TMpegSequenceHeaderHandle::GetProfileAndLevel() const throw()
{
  return HasExtension ? (Extension->profile_and_level_high << 4 |
                         Extension->profile_and_level_low) : 0;
}

#endif
