/*
 *--------------------------------------------------------------------
 *
 * TPesPacketHandle.h
 *
 * A class for accessing basic PES packet structure. The class uses
 * the bridge pattern (also called the handle pattern) described
 * in Design Patterns, Gamme et. al., p151.
 * 
 * Copyright (c) 2003 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_COMMON_TPESPACKETHANDLE_H
#define MEDIA_COMMON_TPESPACKETHANDLE_H

#include "TMpeg2Util.h"
#include <endian.h>
#include <stdint.h>
#include <cstdio>

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif




struct TPesPacketHeader
{
  unsigned start_code_prefix          :24;
  unsigned stream_id                  :8;
  unsigned packet_length              :16;

  unsigned original_or_copy           :1;
  unsigned copyright                  :1;
  unsigned data_alignment_indicator   :1;
  unsigned PES_priority               :1;
  unsigned PES_scrambling_control     :2;
  unsigned padding_1_0                :2;

  unsigned PES_extension_flag         :1;
  unsigned PES_CRC_flag               :1;
  unsigned additional_copy_info_flag  :1;
  unsigned DSM_trick_mode_flag        :1;
  unsigned ES_rate                    :1;
  unsigned ESCR_flag                  :1;
  unsigned PTS_DTS_flags              :2;

  unsigned PES_header_data_length     :8;
};

struct TPesHeaderPtsDts
{
  unsigned marker_bit_1               :1;
  unsigned xTS_30_32                  :3;
  unsigned padding                    :4;

  unsigned xTS_22_29                  :8;

  unsigned marker_bit_2               :1;
  unsigned xTS_15_21                  :7;

  unsigned xTS_7_14                   :8;

  unsigned marker_bit_3               :1;
  unsigned xTS_0_6                    :7;
};

struct TPesHeaderDsmTrickModeFast
{
  unsigned trick_mode_control         :3;
  unsigned field_id                   :2;
  unsigned intra_slice_refresh        :1;
  unsigned frequency_truncation       :2;
};

struct TPesHeaderDsmTrickModeSlow
{
  unsigned trick_mode_control         :3;
  unsigned rep_cntrl                  :5;
};

struct TPesHeaderDsmTrickModeFreeze
{
  unsigned trick_mode_control         :3;
  unsigned field_id                   :2;
  unsigned rep_cntrl                  :3;
};

union TPesHeaderDsmTrickMode
{
  TPesHeaderDsmTrickModeFast Fast;
  TPesHeaderDsmTrickModeSlow Slow;
  TPesHeaderDsmTrickModeFreeze Freeze;
};




class TPesPacketHandle
{
public:
  enum { HEADER_SIZE = 9 };
  enum { START_CODE = 0x000001 };

  enum TStreamId
  {
    STREAM_UNKNOWN           = 0x00,

    STREAM_PROGRAM_MAP       = 0xbc,
    STREAM_PRIVATE_1         = 0xbd,
    STREAM_PADDING           = 0xbe,
    STREAM_PRIVATE_2         = 0xbf,

    STREAM_AUDIO             = 0xc0,

    STREAM_VIDEO             = 0xe0,

    STREAM_ECM               = 0xf0,
    STREAM_EMM               = 0xf1,
    STREAM_DSMCC             = 0xf2,
    STREAM_ISO13522          = 0xf3,
    STREAM_H222_TYPE_A       = 0xf4,
    STREAM_H222_TYPE_B       = 0xf5,
    STREAM_H222_TYPE_C       = 0xf6,
    STREAM_H222_TYPE_D       = 0xf7,
    STREAM_H222_TYPE_E       = 0xf8,
    STREAM_ANCILLARY         = 0xf9,
    STREAM_RESERVED_A        = 0xfa,
    STREAM_RESERVED_B        = 0xfb,
    STREAM_RESERVED_C        = 0xfc,
    STREAM_RESERVED_D        = 0xfd,
    STREAM_RESERVED_E        = 0xfe,
    STREAM_PROGRAM_DIRECTORY = 0xff
  };

protected:
  const TPesPacketHeader* Header;
  uint8_t MpegVersion;
  uint8_t Mpeg1HeaderLength;
  uint8_t Mpeg1PtsOffset;
  uint8_t Mpeg1PtsDtsFlags;
  
  void CheckMpegVersion() throw();
  
public:
  TPesPacketHandle(const TPesPacketHeader* packetData) throw ();
  TPesPacketHandle(const uint8_t* packetData) throw ();

  uint32_t GetStartCodePrefix() const throw ();
  uint8_t GetStreamId() const throw ();
  uint16_t GetPacketLength() const throw ();
  uint16_t GetTotalPacketLength() const throw ();
  uint8_t GetPesFixedBits() const throw ();
  uint8_t GetPesScramblingControl() const throw ();
  uint8_t GetPesPriority() const throw ();
  uint8_t GetDataAlignmentIndicator() const throw ();
  uint8_t GetCopyright() const throw ();
  uint8_t GetOriginalOrCopy() const throw ();
  uint8_t GetPtsDtsFlags() const throw ();
  uint8_t GetEscrFlag() const throw ();
  uint8_t GetEsRate() const throw ();
  uint8_t GetDsmTrickModeFlag() const throw ();
  uint8_t GetAdditionalCopyInfoFlag() const throw ();
  uint8_t GetPesCrcFlag() const throw ();
  uint8_t GetPesExtensionFlag() const throw ();
  uint8_t GetPesHeaderDataLength() const throw ();

  bool IsValid() const throw ();
  TStreamId DecodeStreamId() const throw ();
  uint32_t GetHeaderLength() const throw ();

  
  bool GetPts(uint64_t& pts) const throw ();
  bool GetDts(uint64_t& dts) const throw ();
  bool GetEscr(uint64_t& escr, uint16_t& extension) const throw ();
  bool GetEsRate(uint32_t& esRate) const throw ();

  uint32_t Write(uint8_t* buffer, uint32_t size) const throw ();
};





inline
TPesPacketHandle::TPesPacketHandle(const TPesPacketHeader* packetData) throw ()
  : Header(packetData)
{
  CheckMpegVersion();
}

inline TPesPacketHandle::TPesPacketHandle(const uint8_t* packetData) throw ()
  : Header(reinterpret_cast<const TPesPacketHeader*>(packetData))
{
  CheckMpegVersion();
}

inline uint32_t TPesPacketHandle::GetStartCodePrefix() const throw ()
{
  return TMpeg2Util::Reorder24(Header->start_code_prefix);
}

inline uint8_t TPesPacketHandle::GetStreamId() const throw ()
{
  return Header->stream_id;
}

inline uint16_t TPesPacketHandle::GetPacketLength() const throw ()
{
  return TMpeg2Util::Reorder16(Header->packet_length);
}

inline uint16_t TPesPacketHandle::GetTotalPacketLength() const throw ()
{
  return GetPacketLength() + 6;
}

inline uint8_t TPesPacketHandle::GetPesFixedBits() const throw ()
{
  return Header->padding_1_0;
}

inline uint8_t TPesPacketHandle::GetPesScramblingControl() const throw ()
{
  return Header->PES_scrambling_control;
}

inline uint8_t TPesPacketHandle::GetPesPriority() const throw ()
{
  return Header->PES_priority;
}

inline uint8_t TPesPacketHandle::GetDataAlignmentIndicator() const throw ()
{
  return Header->data_alignment_indicator;
}

inline uint8_t TPesPacketHandle::GetCopyright() const throw ()
{
  return Header->copyright;
}

inline uint8_t TPesPacketHandle::GetOriginalOrCopy() const throw ()
{
  return Header->original_or_copy;
}

inline uint8_t TPesPacketHandle::GetPtsDtsFlags() const throw ()
{
  return Header->PTS_DTS_flags;
}

inline uint8_t TPesPacketHandle::GetEscrFlag() const throw ()
{
  return Header->ESCR_flag;
}

inline uint8_t TPesPacketHandle::GetEsRate() const throw ()
{
  return Header->ES_rate;
}

inline uint8_t TPesPacketHandle::GetDsmTrickModeFlag() const throw ()
{
  return Header->DSM_trick_mode_flag;
}

inline uint8_t TPesPacketHandle::GetAdditionalCopyInfoFlag() const throw ()
{
  return Header->additional_copy_info_flag;
}

inline uint8_t TPesPacketHandle::GetPesCrcFlag() const throw ()
{
  return Header->PES_CRC_flag;
}

inline uint8_t TPesPacketHandle::GetPesExtensionFlag() const throw ()
{
  return Header->PES_extension_flag;
}

inline uint8_t TPesPacketHandle::GetPesHeaderDataLength() const throw ()
{
  return Header->PES_header_data_length;
}

inline bool TPesPacketHandle::IsValid() const throw ()
{
  return (Header != NULL && GetStartCodePrefix() == START_CODE);
}

#endif
