/*
 *--------------------------------------------------------------------
 *
 * libH222/TPsPsmHandle.h
 *
 * A class for accessing program stream map (PSM) structures used in
 * MPEG2 program streams. The class uses the bridge pattern (also
 * called the handle pattern) described in Design Patterns, Gamme
 * et. al., p151.
 *
 * Copyright (c) 2002 Kreatel Communications AB. All rights reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_COMMON_TPSPSMHANDLE_H
#define MEDIA_COMMON_TPSPSMHANDLE_H

#include "media/libH222/TMpeg2DescriptorHandle.h"
#include "media/libH222/TMpeg2Util.h"

#include <endian.h>

#ifndef NDEBUG
#include <iostream>
#endif

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif



/* All bits in a byte is defined with the lowest bit first. That means
   that the bits are upsidedown compared to the ISO/IEC 13818-1. */

struct TMpeg2PsPsm
{
  unsigned prefix_1                     :8;

  unsigned prefix_2                     :8;

  unsigned prefix_3                     :8;

  unsigned map_stream_id                :8;

  unsigned psm_length_high              :8;

  unsigned psm_length_low               :8;


  unsigned psm_version                  :5;  
  unsigned                              :2;  
  unsigned current_next_indicator       :1;

  unsigned marker_bit                   :1;
  unsigned                              :7;  
 
  unsigned program_stream_info_length_high :8;

  unsigned program_stream_info_length_low  :8;
};

struct TMpeg2PsPsmStreamInfo
{
  unsigned stream_type                  :8;

  unsigned elementary_stream_id         :8;

  unsigned ES_info_length_high          :8;
  unsigned ES_info_length_low           :8;
};

#define SIZEOF_TMPEG2PSPSM           10  
#define SIZEOF_TMPEG2PSPSMSTREAMINFO  4  


class TPsPsmHandle 
{
public:
  class TStreamHandle
  {
    friend class TPsPsmHandle;

  private:
    const TMpeg2PsPsmStreamInfo* DataPtr;

    explicit TStreamHandle(const uint8_t* dataPtr) throw ();
    const uint8_t* GetDataPtr() const throw ();

  public:
    TStreamHandle() throw ();

    bool IsValid() const throw ();
    uint32_t GetSize() const throw ();
    bool CheckSanity() const throw ();

    uint8_t GetStreamType() const throw ();
    uint8_t GetEsId() const throw ();
    uint16_t GetEsInfoLength() const throw ();

    TMpeg2DescriptorHandle GetFirstDescriptor() const throw ();
    TMpeg2DescriptorHandle GetNextDescriptor
    (TMpeg2DescriptorHandle previous) const throw ();
  };

protected:
  const TMpeg2PsPsm* Psm;

  const uint8_t* GetStreamInfoBegin() const throw ();
  const uint8_t* GetStreamInfoEnd() const throw ();

public:
  TPsPsmHandle(const uint8_t* buffer) throw ();

  bool CheckSanity() const throw ();

  uint8_t GetMapStreamId() const throw ();
  uint8_t GetVersionNumber() const throw ();
  bool GetCurrentNextIndicator() const throw ();
  uint16_t GetProgramStreamInfoLength() const throw ();
  uint16_t GetProgramStreamMapLength() const throw ();
  uint32_t GetCrc32() const throw ();
  bool CheckCrc32() const throw ();

  TMpeg2DescriptorHandle GetFirstDescriptor() const throw ();
  TMpeg2DescriptorHandle GetNextDescriptor(TMpeg2DescriptorHandle previous)
    const throw ();

  TStreamHandle GetFirstStream() const throw ();
  TStreamHandle GetNextStream(TStreamHandle previous) const throw ();

#ifndef NDEBUG
  void PrintInfo(std::ostream& s) const throw ();
#endif
};


inline TPsPsmHandle::TPsPsmHandle(const uint8_t* buffer) throw ()
  : Psm(reinterpret_cast
        <const TMpeg2PsPsm*>(buffer))
{
  
}

inline const uint8_t* TPsPsmHandle::GetStreamInfoBegin() const throw ()
{
  return reinterpret_cast<const uint8_t*>(Psm)
    + SIZEOF_TMPEG2PSPSM
    + GetProgramStreamInfoLength()
    + 2; 
}

inline const uint8_t* TPsPsmHandle::GetStreamInfoEnd() const throw ()
{
  return reinterpret_cast<const uint8_t*> (Psm)
    + GetProgramStreamMapLength()
    + 6 
        
    - sizeof(uint32_t);  
}

inline uint8_t TPsPsmHandle::GetMapStreamId() const throw ()
{
  return Psm->map_stream_id;
}


inline uint8_t TPsPsmHandle::GetVersionNumber() const throw ()
{
  return Psm->psm_version;
}

inline bool TPsPsmHandle::GetCurrentNextIndicator() const throw ()
{
  return Psm->current_next_indicator;
}


inline uint16_t TPsPsmHandle::GetProgramStreamInfoLength() const throw ()
{
  return TMpeg2Util::Make16(Psm->program_stream_info_length_high,
                            Psm->program_stream_info_length_low);
}

inline uint16_t TPsPsmHandle::GetProgramStreamMapLength() const throw ()
{
  return TMpeg2Util::Make16(Psm->psm_length_high,
                            Psm->psm_length_low);
}

inline uint32_t TPsPsmHandle::GetCrc32() const throw ()
{
  const uint8_t* sum = reinterpret_cast<const uint8_t*>(GetStreamInfoEnd());
     
  return TMpeg2Util::Make32(sum[0], sum[1], sum[2], sum[3]);
}


inline TPsPsmHandle::TStreamHandle::TStreamHandle(const uint8_t* dataPtr)
  throw ()
  : DataPtr(reinterpret_cast<const TMpeg2PsPsmStreamInfo*>(dataPtr))
{
  
}

inline TPsPsmHandle::TStreamHandle::TStreamHandle() throw ()
  : DataPtr(NULL)
{
  
}

inline const uint8_t* TPsPsmHandle::TStreamHandle::GetDataPtr() const throw ()
{
  return reinterpret_cast<const uint8_t*>(DataPtr);
}

inline uint32_t TPsPsmHandle::TStreamHandle::GetSize() const throw ()
{
  return SIZEOF_TMPEG2PSPSMSTREAMINFO + GetEsInfoLength();
}

inline bool TPsPsmHandle::TStreamHandle::IsValid() const throw ()
{
  return DataPtr != NULL;
}

inline uint8_t TPsPsmHandle::TStreamHandle::GetStreamType() const throw ()
{
  return DataPtr->stream_type;
}

inline uint8_t TPsPsmHandle::TStreamHandle::GetEsId() const throw ()
{
  return DataPtr->elementary_stream_id;
}

inline uint16_t TPsPsmHandle::TStreamHandle::GetEsInfoLength() const throw ()
{
  return TMpeg2Util::Make16(DataPtr->ES_info_length_high,
                            DataPtr->ES_info_length_low);
}


inline TMpeg2DescriptorHandle
TPsPsmHandle::TStreamHandle::GetFirstDescriptor() const throw ()
{
  if (GetEsInfoLength() > 0) {
    return GetDataPtr() + SIZEOF_TMPEG2PSPSMSTREAMINFO;
  }
  else {
    return NULL;
  }
}

inline TMpeg2DescriptorHandle
TPsPsmHandle::TStreamHandle::GetNextDescriptor(TMpeg2DescriptorHandle previous)
  const throw ()
{
  return previous.GetNext(GetDataPtr() + SIZEOF_TMPEG2PSPSMSTREAMINFO,
                          GetEsInfoLength());
}

#endif
