/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *--------------------------------------------------------------------
 *
 * TSwSectionFilter.h
 *
 * This sw section filter copies the behaviour of the hw section filter
 *
 * index:       This is the position in the section data where the
 *              filter should start to compare. If index == 0 the section
 *              length will be skipped. Any other index will be used directly.
 *
 * filterData:  The data to which the section data should be compared with.
 *              Both the include and exclude tests should pass in order
 *              for the whole filter to match.
 *
 * includeMask: The inclusion mask.
 *              If the (section data & mask == filter data & mask),
 *              the include mask filter matched. If set to NULL all the
 *              include mask bytes internally will be set to 0xff
 *              (all inclusion).
 *
 * excludeMask: The exclusion mask.
 *              If the (section data & mask != filter data & mask),
 *              the exclude mask filter matched. If set to NULL all the
 *              exclude mask bytes internally will be set to 0x00
 *              (no exclusion). 
 *
 * length:      The length is the actual filter length.
 *
 * WIDE_CHECK can be set to make the filter comparison as wide as possible
 * for each check (up to 32bit wide).
 *
 * Copyright (c) 2008 Motorola Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TSWSECTIONFILTER_H
#define TSWSECTIONFILTER_H

#include "stdint.h"

#define WIDE_CHECK

class TSwSectionFilter
{
private:
  uint32_t Length;
  uint32_t Index;
  const uint8_t* FilterData;
  const uint8_t* IncludeMask;
  const uint8_t* ExcludeMask;

  TSwSectionFilter() throw() { /* Empty */ };

  template<class Type>
  bool IsMatched(const Type sectionData,
                 const Type filterData,
                 const Type includeMask,
                 const Type excludeMask) throw ();
public:
  TSwSectionFilter(const TSwSectionFilter& other) throw ();
  TSwSectionFilter(uint32_t index, const uint8_t* filterData,
                   const uint8_t* includeMask,
                   const uint8_t* excludeMask,
                   uint32_t length) throw();
  ~TSwSectionFilter() throw();
  bool IsFilterMatched(const uint8_t* sectionData) throw();

  uint32_t GetFilterLength() const throw ()
  {
    return Length;
  };
  uint32_t GetIndex() const throw ()
  {
    return Index;
  }
  const uint8_t* GetFilterData() const throw ()
  {
    return FilterData;
  }
    const uint8_t* GetIncludeMask() const throw ()
  {
    return IncludeMask;
  }
    const uint8_t* GetExcludeMask() const throw ()
  {
    return ExcludeMask;
  }
};

#endif
