/*
 *--------------------------------------------------------------------
 *
 * libH222/TTsAdaptationFieldHandle.h
 *
 * A class for accessing the adaptation field structure used as
 * extensions to MPEG2 transport streams packet headers. The class
 * uses the bridge pattern (also called the handle pattern) described
 * in Design Patterns, Gamma et. al., p151.
 *
 * Copyright (c) 2004 Kreatel Communications AB. All Rights Reserved.
 * Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
 *
 * This program is confidential and proprietary to Motorola Mobility, Inc and
 * may not be copied, reproduced, disclosed to others, published or used, in
 * whole or in part, without the expressed prior written permission of Motorola
 * Mobility, Inc.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_COMMON_TTSADAPTATIONFIELDHANDLE_H
#define MEDIA_COMMON_TTSADAPTATIONFIELDHANDLE_H

#include "media/libH222/TMpeg2Util.h"
#include "media/libH222/TMpeg2SystemClock.h"

#include <endian.h>
#include <cstdio>
#include <cassert>

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif



struct TMpeg2TsAdaptationField
{
  unsigned adaptation_field_extension_flag	:1;
  unsigned transport_private_data_flag		:1;
  unsigned splicing_point_flag			:1;
  unsigned OPCR_flag				:1;
  unsigned PCR_flag				:1;
  unsigned elementary_stream_priority_flag	:1;
  unsigned random_access_flag			:1;
  unsigned discontinuity_indicator		:1;
};

struct TMpeg2TsAdaptationFieldExtension
{
  unsigned adaptation_field_extension_length	:8;

  unsigned reserved				:5;
  unsigned seamless_flag			:1;
  unsigned piecewise_rate_flag			:1;
  unsigned ltw_flag				:1;
};


class TTsAdaptationFieldHandle
{
public:
  
  enum { MAX_LENGTH = 183 };

protected:
  enum TFieldIndex {
    PCR_INDEX = 0,
    OPCR_INDEX = 1,
    SPLICING_POINT_INDEX = 2,
    TRANSPORT_PRIVATE_DATA_INDEX = 3,
    LEGAL_TIME_LIMIT_INDEX = 4,
    PIECEWISE_RATE_INDEX = 5,
    SEAMLESS_SPLICE_INDEX = 6,
    FIELD_END = 7
  };

  enum { OFFSET_TABLE_LENGTH = FIELD_END + 1 };

  const TMpeg2TsAdaptationField* Field;
  uint32_t OffsetTable[OFFSET_TABLE_LENGTH];

  void CreateOffsetTable() throw ();
  uint8_t GetTransportPrivateDataLength() const throw ();

public:
  TTsAdaptationFieldHandle(const TMpeg2TsAdaptationField* field) throw ();
  TTsAdaptationFieldHandle(const uint8_t* fieldData) throw ();

  uint32_t GetRequiredSize() const throw ();

  bool GetPcr(TMpeg2SystemClock& pcr) const throw ();
  bool GetOriginalPcr(TMpeg2SystemClock& opcr) const throw ();
  bool GetSplicingPoint(uint8_t& spliceCountdown) const throw ();
  bool GetTransportPrivateData(uint32_t& length, uint8_t* buffer) const
    throw ();
  bool GetLegalTimeWindow() const throw ();
  bool GetPiecewiseRate() const throw ();
  bool GetSeamlessSplice(TMpeg2SystemClock& dtsNextAu) const throw ();
  bool GetDiscontinuityIndicator() const throw ();
  bool GetRandomAccessFlag() const throw ();
  bool GetElementaryStreamPriorityFlag() const throw ();
};


inline uint8_t TTsAdaptationFieldHandle::GetTransportPrivateDataLength() const
  throw ()
{
  
  assert(OffsetTable[TRANSPORT_PRIVATE_DATA_INDEX] != 0);
  const uint8_t* data = reinterpret_cast<const uint8_t*>(Field);
  return data[OffsetTable[TRANSPORT_PRIVATE_DATA_INDEX]];
}

inline TTsAdaptationFieldHandle::
TTsAdaptationFieldHandle(const TMpeg2TsAdaptationField* field) throw ()
  : Field(field)
{
  CreateOffsetTable();
}

inline TTsAdaptationFieldHandle::
TTsAdaptationFieldHandle(const uint8_t* fieldData) throw ()
  : Field(reinterpret_cast<const TMpeg2TsAdaptationField*>(fieldData))
{
  CreateOffsetTable();
}

inline uint32_t TTsAdaptationFieldHandle::GetRequiredSize() const throw ()
{
  return OffsetTable[FIELD_END];
}

inline bool TTsAdaptationFieldHandle::GetPcr(TMpeg2SystemClock& pcr) const
  throw ()
{
  if (OffsetTable[PCR_INDEX] != 0) {
    const uint8_t* data = reinterpret_cast<const uint8_t*>(Field);
    pcr.ReadTimeStamp_33_e9(data + OffsetTable[PCR_INDEX]);
    return true;
  }
  else {
    return false;
  }
}

inline bool TTsAdaptationFieldHandle::GetOriginalPcr(TMpeg2SystemClock& opcr)
  const throw ()
{
  if (OffsetTable[OPCR_INDEX] != 0) {
    const uint8_t* data = reinterpret_cast<const uint8_t*>(Field);
    opcr.ReadTimeStamp_33_e9(data + OffsetTable[OPCR_INDEX]);
    return true;
  }
  else {
    return false;
  }
}

inline bool TTsAdaptationFieldHandle::GetSplicingPoint(uint8_t& spliceCountdown)
  const throw ()
{
  if (OffsetTable[SPLICING_POINT_INDEX] != 0) {
    const uint8_t* data = reinterpret_cast<const uint8_t*>(Field);
    spliceCountdown = data[OffsetTable[SPLICING_POINT_INDEX]];
    return true;
  }
  else {
    return false;
  }
}

inline bool TTsAdaptationFieldHandle::GetLegalTimeWindow() const throw ()
{
  
  return false;
}

inline bool TTsAdaptationFieldHandle::GetPiecewiseRate() const throw ()
{
  
  return false;
}

inline
bool TTsAdaptationFieldHandle::GetSeamlessSplice(TMpeg2SystemClock& dtsNextAu)
  const throw ()
{
  if (OffsetTable[SEAMLESS_SPLICE_INDEX] != 0) {
    const uint8_t* data = reinterpret_cast<const uint8_t*>(Field);
    dtsNextAu.ReadTimeStamp_3_15_15(data + OffsetTable[SEAMLESS_SPLICE_INDEX]);
    return true;
  }
  else {
    return false;
  }
}

inline
bool TTsAdaptationFieldHandle::GetDiscontinuityIndicator() const throw ()
{
  return Field->discontinuity_indicator == 1;
}

inline
bool TTsAdaptationFieldHandle::GetRandomAccessFlag() const throw ()
{
  return Field->random_access_flag == 1;
}

inline
bool TTsAdaptationFieldHandle::GetElementaryStreamPriorityFlag() const throw ()
{
  return Field->elementary_stream_priority_flag == 1;
}


#endif
