/*
 *--------------------------------------------------------------------
 *
 * libH222/TTsPacketHandle.h
 *
 * A class for accessing the basic packet structure used in MPEG2
 * transport streams. The class uses the bridge pattern (also called the
 * handle pattern) described in Design Patterns, Gamma et. al., p151.
 *
 * Copyright (c) 2002 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_COMMON_TTSPACKETHANDLE_H
#define MEDIA_COMMON_TTSPACKETHANDLE_H

#include "media/libH222/TMpeg2Util.h"

#include <endian.h>
#include <cstdio>

#ifndef NDEBUG
#include <iostream>
#endif

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif



struct TMpeg2TsPacketHeader
{
  unsigned sync_byte			:8;

  unsigned PID_high			:5;
  unsigned transport_priority		:1;
  unsigned payload_unit_start_indicator	:1;
  unsigned transport_error_indicator	:1;
  
  unsigned PID_low			:8;
  
  unsigned continuity_counter		:4;
  unsigned adaptation_field_control	:2;
  unsigned transport_scrambling_control	:2;
};


class TTsPacketHandle
{
public:
  enum { PACKET_SIZE = 188 };
  enum { HEADER_SIZE = 4 };
  enum { SYNC_BYTE   = 0x47 };

protected:
  const TMpeg2TsPacketHeader* Header;

public:
  TTsPacketHandle(const TMpeg2TsPacketHeader* packetData) throw ();
  TTsPacketHandle(const uint8_t* packetData) throw ();

  bool IsValid() const throw ();
  uint8_t GetSyncByte() const throw ();
  bool GetTransportErrorIndicator() const throw ();
  bool GetPayloadUnitStartIndicator() const throw ();
  bool GetTransportPriority() const throw ();
  uint16_t GetPid() const throw ();
  uint8_t GetTransportScramblingControl() const throw ();
  uint8_t GetAdaptationFieldControl() const throw ();
  uint8_t GetContinuityCounter() const throw ();

  bool HasAdaptationField() const throw ();
  bool HasPayload() const throw ();
  uint32_t GetAdaptationFieldLength() const throw ();
  uint32_t GetPayloadLength() const throw ();
  const uint8_t* GetAdaptationFieldData() const throw ();

  uint32_t GetSectionOffset() const throw ();
  const uint8_t* GetPacketBegin() const throw ();
  const uint8_t* GetSectionStart() const throw ();
  const uint8_t* GetPayloadBegin() const throw ();
  const uint8_t* GetPayloadEnd() const throw ();
  uint8_t GetPointerFieldValue() const throw ();

#ifndef NDEBUG
  void PrintInfo(std::ostream& s) const throw ();
  void PrintHex(std::ostream& s) const throw ();
#endif
};


inline TTsPacketHandle::TTsPacketHandle(const TMpeg2TsPacketHeader* packetData)
  throw ()
  : Header(packetData)
{
  
}

inline TTsPacketHandle::TTsPacketHandle(const uint8_t* packetData) throw ()
  : Header(reinterpret_cast<const TMpeg2TsPacketHeader*>(packetData))
{
  
}

inline uint8_t TTsPacketHandle::GetSyncByte() const throw ()
{
  return Header->sync_byte;
}

inline bool TTsPacketHandle::GetTransportErrorIndicator() const throw ()
{
  return Header->transport_error_indicator;
}

inline bool TTsPacketHandle::GetPayloadUnitStartIndicator() const throw ()
{
  return Header->payload_unit_start_indicator;
}

inline bool TTsPacketHandle::GetTransportPriority() const throw ()
{
  return Header->transport_priority;
}

inline uint16_t TTsPacketHandle::GetPid() const throw ()
{
  return TMpeg2Util::Make16(Header->PID_high, Header->PID_low);
}

inline uint8_t TTsPacketHandle::GetTransportScramblingControl() const throw ()
{
  return Header->transport_scrambling_control;
}

inline uint8_t TTsPacketHandle::GetAdaptationFieldControl() const throw ()
{
  return Header->adaptation_field_control;
}

inline uint8_t TTsPacketHandle::GetContinuityCounter() const throw ()
{
  return Header->continuity_counter;
}

inline bool TTsPacketHandle::HasAdaptationField() const throw ()
{
  return (GetAdaptationFieldControl() & 2) != 0;
}

inline bool TTsPacketHandle::HasPayload() const throw ()
{
  return (GetAdaptationFieldControl() & 1) != 0;
}

inline uint32_t TTsPacketHandle::GetAdaptationFieldLength() const throw ()
{
  if (HasAdaptationField()) {
    
    const uint8_t* data = reinterpret_cast<const uint8_t*>(Header);
    return data[HEADER_SIZE] + 1; 
  }
  else {
    return 0;
  }
}

inline uint32_t TTsPacketHandle::GetPayloadLength() const throw ()
{
  if (HasPayload()) {
    return PACKET_SIZE - HEADER_SIZE - GetAdaptationFieldLength();
  }
  else {
    return 0;
  }
}

inline const uint8_t* TTsPacketHandle::GetAdaptationFieldData() const throw ()
{
  
  
  if (GetAdaptationFieldLength() > 1) {
    const uint8_t* data = reinterpret_cast<const uint8_t*>(Header);
    return data + HEADER_SIZE + 1;
  }
  else {
    return NULL;
  }
}

inline const uint8_t* TTsPacketHandle::GetPacketBegin() const throw ()
{
  return reinterpret_cast<const uint8_t*>(Header);
}

inline const uint8_t* TTsPacketHandle::GetPayloadBegin() const throw ()
{
  const uint8_t* data = reinterpret_cast<const uint8_t*>(Header);
  return data + HEADER_SIZE + GetAdaptationFieldLength();
}

inline const uint8_t* TTsPacketHandle::GetPayloadEnd() const throw ()
{
  const uint8_t* data = reinterpret_cast<const uint8_t*>(Header);
  return data + PACKET_SIZE;
}

inline uint8_t TTsPacketHandle::GetPointerFieldValue() const throw ()
{
  return GetPayloadBegin()[0];
}

#endif
