/*
 *--------------------------------------------------------------------
 *
 * libH222/TTsPatHandle.h
 *
 * A class for accessing program allocation table (PAT) structures used
 * in MPEG2 transport streams.
 *
 * The class uses the bridge pattern (also called the handle pattern)
 * described in Design Patterns, Gamma et. al., p151.
 *
 * Copyright (c) 2002 Kreatel Communications AB. All rights reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_COMMON_TTSPATHANDLE_H
#define MEDIA_COMMON_TTSPATHANDLE_H

#include "media/libH222/TTsSectionHandle.h"
#include "media/libH222/TMpeg2DescriptorHandle.h"

#ifndef NDEBUG
#include <iostream>
#endif

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif



/* All bits in a byte is defined with the lowest bit first. That means
   that the bits are upside down compared to the ISO/IEC 13818-1. */

struct TMpeg2TsPatProgramInfo
{
  unsigned program_number_high          :8;

  unsigned program_number_low           :8;

  unsigned PID_high                     :5;
  unsigned                              :3;

  unsigned PID_low                      :8;
};


class TTsPatHandle : public TTsSectionHandle
{
public:
  class TProgramHandle
  {
    friend class TTsPatHandle;

  public:
    enum { HEADER_SIZE = 4 };
    enum { MAX_SECTION_LENGTH = 1024 };  

  private:
    const TMpeg2TsPatProgramInfo* DataPtr;

    explicit TProgramHandle(const uint8_t* dataPtr) throw ();
    const uint8_t* GetDataPtr() const throw ();

  public:
    TProgramHandle() throw ();

    bool IsValid() const throw ();
    uint32_t GetSize() const throw ();
    bool CheckSanity() const throw ();

    uint16_t GetProgramNumber() const throw ();
    uint16_t GetPid() const throw ();
  };

protected:
  const uint8_t* GetProgramInfoBegin() const throw ();
  const uint8_t* GetProgramInfoEnd() const throw ();

public:
  TTsPatHandle(const uint8_t* buffer) throw ();

  bool CheckSanity() const throw ();

  uint16_t GetTransportStreamId() const throw ();

  TProgramHandle GetFirstProgram() const throw ();
  TProgramHandle GetNextProgram(TProgramHandle previous) const throw ();
  uint32_t GetProgramCount() const throw ();

#ifndef NDEBUG
  void PrintInfo(std::ostream& s) const throw ();
#endif
};


inline TTsPatHandle::TTsPatHandle(const uint8_t* buffer) throw ()
  : TTsSectionHandle(buffer)
{
  
}

inline const uint8_t* TTsPatHandle::GetProgramInfoBegin() const throw ()
{
  return GetSectionDataBegin();
}

inline const uint8_t* TTsPatHandle::GetProgramInfoEnd() const throw ()
{
  return GetSectionDataEnd();
}

inline uint16_t TTsPatHandle::GetTransportStreamId() const throw ()
{
  return GetTableIdExtension();
}

inline uint32_t TTsPatHandle::GetProgramCount() const throw ()
{
  return (GetProgramInfoEnd() - GetProgramInfoBegin())
    / TTsPatHandle::TProgramHandle::HEADER_SIZE;
}

inline
TTsPatHandle::TProgramHandle::TProgramHandle(const uint8_t* dataPtr) throw ()
  : DataPtr(reinterpret_cast<const TMpeg2TsPatProgramInfo*>(dataPtr))
{
  
}

inline TTsPatHandle::TProgramHandle::TProgramHandle() throw ()
  : DataPtr(NULL)
{
  
}

inline const uint8_t* TTsPatHandle::TProgramHandle::GetDataPtr() const throw ()
{
  return reinterpret_cast<const uint8_t*>(DataPtr);
}

inline uint32_t TTsPatHandle::TProgramHandle::GetSize() const throw ()
{
  return TTsPatHandle::TProgramHandle::HEADER_SIZE;
}

inline bool TTsPatHandle::TProgramHandle::IsValid() const throw ()
{
  return DataPtr != NULL;
}

inline uint16_t TTsPatHandle::TProgramHandle::GetProgramNumber() const throw ()
{
  return TMpeg2Util::Make16(DataPtr->program_number_high,
                            DataPtr->program_number_low);
}

inline uint16_t TTsPatHandle::TProgramHandle::GetPid() const throw ()
{
  return TMpeg2Util::Make16(DataPtr->PID_high, DataPtr->PID_low);
}

#endif
