/*
 *--------------------------------------------------------------------
 *
 * libH222/TTsPmtHandle.h
 *
 * A class for accessing program map table (PMT) structures used in MPEG2
 * transport streams. The class uses the bridge pattern (also called the
 * handle pattern) described in Design Patterns, Gamme et. al., p151.
 *
 * Copyright (c) 2002 Kreatel Communications AB. All rights reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_COMMON_TTSPMTHANDLE_H
#define MEDIA_COMMON_TTSPMTHANDLE_H

#include "media/libH222/TTsSectionHandle.h"
#include "media/libH222/TMpeg2DescriptorHandle.h"

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif



/* All bits in a byte is defined with the lowest bit first. That means
   that the bits are upsidedown compared to the ISO/IEC 13818-1. */

struct TMpeg2TsPmtSection
{
  unsigned PCR_PID_high                 :5;
  unsigned                              :3;  

  unsigned PCR_PID_low                  :8;
  
  unsigned program_info_length_high     :4;
  unsigned                              :4;  
  
  unsigned program_info_length_low      :8;
};

struct TMpeg2TsPmtStreamInfo
{
  unsigned stream_type                  :8;

  unsigned elementary_PID_high          :5;
  unsigned                              :3;  

  unsigned elementary_PID_low           :8;

  unsigned ES_info_length_high          :4;
  unsigned                              :4;  

  unsigned ES_info_length_low           :8;
};


class TTsPmtHandle : public TTsSectionHandle
{
public:
  enum { HEADER_SIZE = 4 };
  enum { MAX_SECTION_LENGTH = 1024 };  

  enum { TS_TYPE_11172_VIDEO = 0x01, 
         TS_TYPE_13818_VIDEO = 0x02,
         TS_TYPE_11172_AUDIO = 0x03,
         TS_TYPE_13818_AUDIO = 0x04,
         TS_TYPE_GALAXIE = 0x05,
         TS_TYPE_PRIVATE = 0x06,
         TS_TYPE_13818_6_TYPE_A = 0x0a,
         TS_TYPE_13818_6_TYPE_B = 0x0b,
         TS_TYPE_13818_6_TYPE_C = 0x0c,
         TS_TYPE_13818_6_TYPE_D = 0x0d,
         TS_TYPE_13818_7_AUDIO = 0x0f,
         TS_TYPE_14496_2_VIDEO = 0x10,
         TS_TYPE_14496_3_AUDIO = 0x11,
         TS_TYPE_14496_10_VIDEO = 0x1b,
         TS_TYPE_USER_PRIVATE_0 = 0x80,
         TS_TYPE_ATSC_AC3 = 0x81,
         TS_TYPE_ATSC_EAC3_87 = 0x87 };
  
  class TStreamHandle
  {
    friend class TTsPmtHandle;

  public:
    enum { HEADER_SIZE = 5 };  

  private:
    const TMpeg2TsPmtStreamInfo* DataPtr;

    explicit TStreamHandle(const uint8_t* dataPtr) throw ();
    const uint8_t* GetDataPtr() const throw ();

  public:
    TStreamHandle() throw ();

    bool IsValid() const throw ();
    uint32_t GetSize() const throw ();
    bool CheckSanity() const throw ();

    uint8_t GetStreamType() const throw ();
    uint16_t GetElementaryPid() const throw ();
    uint16_t GetEsInfoLength() const throw ();

    TMpeg2DescriptorHandle GetFirstDescriptor() const throw ();
    TMpeg2DescriptorHandle GetNextDescriptor(TMpeg2DescriptorHandle previous)
      const throw ();
  };

protected:
  const TMpeg2TsPmtSection* GetSection() const throw ();
  const uint8_t* GetStreamInfoBegin() const throw ();
  const uint8_t* GetStreamInfoEnd() const throw ();

public:
  TTsPmtHandle(const uint8_t* buffer) throw ();

  bool CheckSanity() const throw ();

  uint16_t GetProgramNumber() const throw ();
  uint16_t GetPcrPid() const throw ();
  uint16_t GetProgramInfoLength() const throw ();

  TMpeg2DescriptorHandle GetFirstDescriptor() const throw ();
  TMpeg2DescriptorHandle GetNextDescriptor(TMpeg2DescriptorHandle previous)
    const throw ();

  TStreamHandle GetFirstStream() const throw ();
  TStreamHandle GetNextStream(TStreamHandle previous) const throw ();

#ifndef NDEBUG
  void PrintInfo(std::ostream& s) const throw ();
#endif
};


inline TTsPmtHandle::TTsPmtHandle(const uint8_t* buffer) throw ()
  : TTsSectionHandle(buffer)
{
  
}

inline const TMpeg2TsPmtSection* TTsPmtHandle::GetSection() const throw ()
{
  return reinterpret_cast<const TMpeg2TsPmtSection*>(GetSectionDataBegin());
}

inline const uint8_t* TTsPmtHandle::GetStreamInfoBegin() const throw ()
{
  return GetSectionDataBegin()
    + HEADER_SIZE
    + GetProgramInfoLength();
}

inline const uint8_t* TTsPmtHandle::GetStreamInfoEnd() const throw ()
{
  return GetSectionDataEnd();
}

inline uint16_t TTsPmtHandle::GetProgramNumber() const throw ()
{
  return GetTableIdExtension();
}

inline uint16_t TTsPmtHandle::GetPcrPid() const throw ()
{
  return TMpeg2Util::Make16(GetSection()->PCR_PID_high,
                            GetSection()->PCR_PID_low);
}

inline uint16_t TTsPmtHandle::GetProgramInfoLength() const throw ()
{
  return TMpeg2Util::Make16(GetSection()->program_info_length_high,
                            GetSection()->program_info_length_low);
}

inline TTsPmtHandle::TStreamHandle::TStreamHandle(const uint8_t* dataPtr)
  throw ()
  : DataPtr(reinterpret_cast<const TMpeg2TsPmtStreamInfo*>(dataPtr))
{
  
}

inline TTsPmtHandle::TStreamHandle::TStreamHandle() throw ()
  : DataPtr(NULL)
{
  
}

inline const uint8_t* TTsPmtHandle::TStreamHandle::GetDataPtr() const throw ()
{
  return reinterpret_cast<const uint8_t*>(DataPtr);
}

inline uint32_t TTsPmtHandle::TStreamHandle::GetSize() const throw ()
{
  return TTsPmtHandle::TStreamHandle::HEADER_SIZE + GetEsInfoLength();
}

inline bool TTsPmtHandle::TStreamHandle::IsValid() const throw ()
{
  return DataPtr != NULL;
}

inline uint8_t TTsPmtHandle::TStreamHandle::GetStreamType() const throw ()
{
  return DataPtr->stream_type;
}

inline uint16_t TTsPmtHandle::TStreamHandle::GetElementaryPid() const throw ()
{
  return TMpeg2Util::Make16(DataPtr->elementary_PID_high,
                            DataPtr->elementary_PID_low);
}

inline uint16_t TTsPmtHandle::TStreamHandle::GetEsInfoLength() const throw ()
{
  return TMpeg2Util::Make16(DataPtr->ES_info_length_high,
                            DataPtr->ES_info_length_low);
}

inline TMpeg2DescriptorHandle
TTsPmtHandle::TStreamHandle::GetFirstDescriptor() const throw ()
{
  if (GetEsInfoLength() > 0) {
    return GetDataPtr() + HEADER_SIZE;
  }
  else {
    return NULL;
  }
}

inline TMpeg2DescriptorHandle
TTsPmtHandle::TStreamHandle::GetNextDescriptor(TMpeg2DescriptorHandle previous)
  const throw ()
{
  return previous.GetNext(GetDataPtr() + HEADER_SIZE, GetEsInfoLength());
}

#endif
