/*
 *--------------------------------------------------------------------
 *
 * libH222/TTsPsiBuffer.h
 *
 * A class for program specific information (PSI) data used in MPEG2
 * transport streams.
 *
 * Copyright (c) 2002 Kreatel Communications AB. All rights reserved.
 *
 *--------------------------------------------------------------------
 */

/*
 * This class can be used in two ways, the simple way which may skip
 * some sections and a slightly more complex way which should handle
 * all possible ways of packing sections.
 *
 * The first way will generate at most one section per append call:
 *
 * foreach tspacket:
 *   if PsiBuffer.Append(tspacket):
 *     doSomethingWithSection(PsiBuffer.GetData())
 *
 * The second way can generate many sections on a single append call:
 *
 * foreach tspacket:
 *   if PsiBuffer.Append(tspacket):
 *     do
 *       doSomethingWithSection(PsiBuffer.GetData())
 *     while (PsiBuffer.GetNext())
 *
 * The second way should be used in new code but you may encounter the
 * first way in old code as it has been the only way for a long time.
 * 
 */

#ifndef MEDIA_COMMON_TTSPSIBUFFER_H
#define MEDIA_COMMON_TTSPSIBUFFER_H

#include "media/libH222/TTsPacketHandle.h"




class TTsPsiBuffer
{
public:
  
  
  enum { MAX_SIZE = 4096 };

protected:
  bool CheckCrcFlag;

  uint8_t Buffer[MAX_SIZE];
  uint32_t BufferPosition;
  uint32_t Size;
  uint32_t SectionSize;
  uint16_t Pid;
  bool ValidFlag;

  uint8_t TmpBuffer[TTsPacketHandle::PACKET_SIZE];
  uint16_t TmpPid;
  uint32_t TmpBufferSize;
  
  bool CheckBuffer() throw ();
  bool Append(const uint8_t* data, uint32_t size) throw ();
  bool ReValidate(const uint8_t* data, bool copyData) throw();
  void StorePayload(const TTsPacketHandle& packet) throw ();
  
public:
  TTsPsiBuffer(bool checkCrcFlag = true) throw ();
  TTsPsiBuffer(const uint8_t* data, bool checkCrcFlag = true) throw ();

  bool IsValid() const throw ();
  const uint8_t* GetData() const throw ();
  uint32_t GetSize() const throw ();
  uint32_t GetPid() const throw ();
  void SetPid(uint32_t pid) throw ();

  bool GetNext() throw ();
  
  bool Append(const TTsPacketHandle& packet) throw ();
  void Clear() throw ();
  bool SetData(const uint8_t* data) throw ();
  bool ReValidate() throw ();
};


inline TTsPsiBuffer::TTsPsiBuffer(bool checkCrcFlag) throw ()
  : CheckCrcFlag(checkCrcFlag),
    BufferPosition(0),
    Size(0),
    Pid(0),
    ValidFlag(false),
    TmpPid(0),
    TmpBufferSize(0)
{
  
}

inline bool TTsPsiBuffer::IsValid() const throw ()
{
  return ValidFlag;
}

inline const uint8_t* TTsPsiBuffer::GetData() const throw ()
{
  return Buffer + BufferPosition;
}

inline uint32_t TTsPsiBuffer::GetSize() const throw ()
{
  return SectionSize;
}

inline uint32_t TTsPsiBuffer::GetPid() const throw ()
{
  return Pid;
}

inline void TTsPsiBuffer::SetPid(uint32_t pid) throw ()
{
  Pid = pid;
}

inline void TTsPsiBuffer::Clear() throw ()
{
  Size = 0;
  SectionSize = 0;
  Pid = 0;
  ValidFlag = false;
}

#endif
