/*
 *--------------------------------------------------------------------
 *
 * libH222/TTsPsiHandle.h
 *
 * A base class for program specific information (PSI) structures used
 * in MPEG2 transport streams.
 *
 * The class uses the bridge pattern (also called the handle pattern)
 * described in Design Patterns, Gamma et. al., p151.
 *
 * Copyright (c) 2002 Kreatel Communications AB. All rights reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_COMMON_TTSPSIHANDLE_H
#define MEDIA_COMMON_TTSPSIHANDLE_H

#include "media/libH222/TMpeg2Util.h"

#include <endian.h>

#ifndef NDEBUG
#include <iostream>
#endif

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif



/* All bits in a byte is defined with the lowest bit first. That means
   that the bits are upsidedown compared to the ISO/IEC 13818-1. */

struct TMpeg2TsPsiHeader
{
  unsigned table_id                     :8;
  
  unsigned section_length_high          :4;
  unsigned                              :2;  
  unsigned private_indicator            :1;
  unsigned section_syntax_indicator     :1;
  
  unsigned section_length_low           :8;
};


class TTsPsiHandle
{
public:
  enum { HEADER_SIZE = 3 };

protected:
  const TMpeg2TsPsiHeader* Header;

public:
  TTsPsiHandle(const uint8_t* buffer) throw ();

  uint8_t GetTableId() const throw ();
  bool GetSectionSyntaxIndicator() const throw ();
  bool GetPrivateIndicator() const throw ();
  uint32_t GetSectionLength() const throw ();
  uint32_t GetCrc32() const throw ();

  bool CheckCrc() const throw ();
  uint32_t Write(uint8_t* buffer, uint32_t size) const throw ();

  uint32_t GetDataLength() const throw ();
  const uint8_t* GetDataBegin() const throw ();
  const uint8_t* GetDataEnd() const throw ();

#ifndef NDEBUG
  void PrintInfo(std::ostream& s) const throw ();
#endif
};


inline TTsPsiHandle::TTsPsiHandle(const uint8_t* buffer) throw ()
  : Header(reinterpret_cast<const TMpeg2TsPsiHeader*>(buffer))
{
  
}

inline uint8_t TTsPsiHandle::GetTableId() const throw ()
{
  return Header->table_id;
}

inline bool TTsPsiHandle::GetSectionSyntaxIndicator() const throw ()
{
  return Header->section_syntax_indicator;
}

inline bool TTsPsiHandle::GetPrivateIndicator() const throw ()
{
  return Header->private_indicator;
}

inline uint32_t TTsPsiHandle::GetSectionLength() const throw ()
{
  return TMpeg2Util::Make16(Header->section_length_high,
                            Header->section_length_low);
}

inline uint32_t TTsPsiHandle::GetCrc32() const throw ()
{
  
  const uint8_t* sum = reinterpret_cast<const uint8_t*>(Header) +
    TTsPsiHandle::HEADER_SIZE + GetSectionLength() - sizeof(uint32_t);
  return TMpeg2Util::Make32(sum[0], sum[1], sum[2], sum[3]);
}

inline uint32_t TTsPsiHandle::GetDataLength() const throw ()
{
  return TTsPsiHandle::HEADER_SIZE + GetSectionLength();
}

inline const uint8_t* TTsPsiHandle::GetDataBegin() const throw ()
{
  const uint8_t* data = reinterpret_cast<const uint8_t*>(Header);
  return data;
}

inline const uint8_t* TTsPsiHandle::GetDataEnd() const throw ()
{
  const uint8_t* data = reinterpret_cast<const uint8_t*>(Header);
  return data + GetDataLength();
}

#endif
