/*
 *--------------------------------------------------------------------
 *
 * libH222/TTsSectionHandle.h
 *
 * A base class for section structures with section syntax bit set
 * used in MPEG2 transport streams.
 *
 * The class uses the bridge pattern (also called the handle pattern)
 * described in Design Patterns, Gamma et. al., p151.
 *
 * Copyright (c) 2005 Kreatel Communications AB. All rights reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TTSSECTIONHANDLE_H
#define TTSSECTIONHANDLE_H

#include "media/libH222/TMpeg2Util.h"
#include "media/libH222/TTsPsiHandle.h"

#include <endian.h>

#ifndef NDEBUG
#include <iostream>
#endif

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif



/* All bits in a byte is defined with the lowest bit first. That means
   that the bits are upsidedown compared to the ISO/IEC 13818-1. */

struct TTsSectionHeader
{
  unsigned table_id_extension_high      : 8;

  unsigned table_id_extension_low       : 8;

  unsigned current_next_indicator       : 1;
  unsigned version_number               : 5;
  unsigned                              : 2;  

  unsigned section_number               : 8;

  unsigned last_section_number          : 8;  
};


class TTsSectionHandle : public TTsPsiHandle
{
public:
  enum { HEADER_SIZE = 5 };

protected:
  const TTsSectionHeader* GetSection() const throw ();

public:
  TTsSectionHandle(const uint8_t* buffer) throw ();

  uint16_t GetTableIdExtension() const throw ();
  uint8_t GetVersionNumber() const throw ();
  bool GetCurrentNextIndicator() const throw ();
  uint8_t GetSectionNumber() const throw ();
  uint8_t GetLastSectionNumber() const throw ();

  uint32_t GetSectionDataLength() const throw ();
  const uint8_t* GetSectionDataBegin() const throw ();
  const uint8_t* GetSectionDataEnd() const throw ();

#ifndef NDEBUG
  void PrintInfo(std::ostream& s) const throw ();
#endif
};


inline TTsSectionHandle::TTsSectionHandle(const uint8_t* buffer) throw ()
  : TTsPsiHandle(buffer)
{
  
}

inline const TTsSectionHeader* TTsSectionHandle::GetSection() const throw ()
{
  return reinterpret_cast<const TTsSectionHeader*>(GetDataBegin() +
                                                   TTsPsiHandle::HEADER_SIZE);
}

inline uint16_t TTsSectionHandle::GetTableIdExtension() const throw ()
{
  return TMpeg2Util::Make16(GetSection()->table_id_extension_high,
                            GetSection()->table_id_extension_low);
}

inline uint8_t TTsSectionHandle::GetVersionNumber() const throw ()
{
  return GetSection()->version_number;
}

inline bool TTsSectionHandle::GetCurrentNextIndicator() const throw ()
{
  return GetSection()->current_next_indicator;
}

inline uint8_t TTsSectionHandle::GetSectionNumber() const throw ()
{
  return GetSection()->section_number;
}

inline uint8_t TTsSectionHandle::GetLastSectionNumber() const throw ()
{
  return GetSection()->last_section_number;
}

inline uint32_t TTsSectionHandle::GetSectionDataLength() const throw ()
{
  return GetSectionLength() - HEADER_SIZE - 4;
}

inline const uint8_t* TTsSectionHandle::GetSectionDataBegin() const throw ()
{
  return GetDataBegin() + TTsPsiHandle::HEADER_SIZE + HEADER_SIZE;
}

inline const uint8_t* TTsSectionHandle::GetSectionDataEnd() const throw ()
{
  return GetSectionDataBegin() + GetSectionDataLength();
}

#endif
