/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *--------------------------------------------------------------------
 *
 * TPositionCounter.h
 *
 * Position counter.
 *
 * Copyright (c) 2003 Kreatel Communications AB. All rights reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TPOSITIONCOUNTER_H
#define TPOSITIONCOUNTER_H

#include "debug/TTracker.h"
#include "sys/times.h"

#include <cstddef>
#include <deque>
#include <stdint.h>


/** The TPositionCounter class uses the tick counter to track an
    active position value with 1000 Hz accuracy. The class supports
    position changes at arbitrary pace, both forward and backward.

    Pace is 1000 at normal pace. Position unit is ms.
*/

class ITimesWrapper {
public:
  virtual clock_t Times() = 0;

  ITimesWrapper() {};
  virtual ~ITimesWrapper() {};
};


class TPositionCounter
{
private:
  class TTimesWrapper : public ITimesWrapper
  {
    clock_t Times() {return ::times(NULL);}
  };
  
  friend class TPositionCounterTests;

  uint32_t TicksPerSec;
  int Pace;
  bool ErrorCorrected;
  int CalculatedPace;
  int BasePosition;
  clock_t BaseTime;
  int TraceTimer;
  std::deque<int> CalculatedPaces;
  bool UsesInternalTimesWrapper;
  ITimesWrapper* TimesWrapper;
  bool RunningFlag;
  
  int64_t GetOffset(clock_t currentTime) const;
  int64_t GetErrorCorrectedOffset(clock_t currentTime) const;
public:
  TPositionCounter(ITimesWrapper* timesWrapper = NULL);
  ~TPositionCounter();

  void SetPosition(int pos);
  void SetPace(int pace);
  void SetPaceAndPosition(uint32_t position, int32_t pace);

  int GetPosition();
  int GetPace() const;

  void Reset() throw ();
  void Run() throw ();
  void Halt() throw ();
};

#define SET_POSITION_FREQ 3000
#include <cstdio>
inline int64_t TPositionCounter::GetOffset(clock_t currentTime) const
{
  int64_t duration =
    (static_cast<int64_t>(currentTime) - static_cast<int64_t>(BaseTime)) *
    static_cast<int64_t>(Pace) / static_cast<int64_t>(TicksPerSec);
  
  return duration;
}

inline int64_t TPositionCounter::GetErrorCorrectedOffset(clock_t currentTime) const
{
#ifdef DEBUG
  TTracker::GetProcessTracker()->Debug("(%lli - %lli) * %lli / %lli\n", 
                                       static_cast<int64_t>(currentTime),
                                       static_cast<int64_t>(BaseTime),
                                       static_cast<int64_t>(CalculatedPace),
                                       static_cast<int64_t>(TicksPerSec));
#endif

  int64_t duration =
    (static_cast<int64_t>(currentTime) - static_cast<int64_t>(BaseTime)) *
    static_cast<int64_t>(CalculatedPace) / static_cast<int64_t>(TicksPerSec);
  
  return duration;
}

#endif
