/*
 *--------------------------------------------------------------------
 *
 * IBlackboard.h
 *
 * The interface to the pipeline blackboard.
 *
 * Copyright (c) 2003 Motorola Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef IBLACKBOARD_H
#define IBLACKBOARD_H

#include <string>
#include <vector>

#ifdef PIPELINE_NAMESPACE
namespace Pipeline {
#endif


/** \class IBlackboard IBlackboard.h "media/streamer/IBlackboard.h"

    This is the interface to the pipeline blackboard. The blackboard
    publishes key/value pairs with information that can be retrieved
    by elements, by the Streamer core and by observers external to the
    Streamer process.

    Values can be added to the blackboard by the streamer core or by
    elements using this interface to set a value. But the most common
    way that values end up on the blackboard is when metadata is
    published using the PublishMetadata function in the IPump
    interface. The metadata will then be added to the blackboard when
    the stream is consumed "beyond" the position of the metadata.

    \see IPump::PublishMetadata
 */

class IBlackboard
{
public:
  virtual ~IBlackboard() {}

  typedef std::vector<std::string> TParameterList;
  typedef std::vector<std::string> TValueList;

  /** This function is used to subscribe to changes on parameters. The
      element must implement the IBlackboardObserver interface and
      will be notified on OnParametersChanged when a value changes
      and on OnParametersRemoved when the parameter is removed from
      the blackboard.

      When the subscription is made there will be an immediate call to
      OnParametersChanged if there is a value available.

      \param name    The name of the subscribed parameter. 
      \see IBlackboardObserver
  */
  virtual void SubscribeParameter(const std::string& name) throw () = 0;  

  /** This function removes a subscription and the observer will no
      longer be notified on changes.

      \param name    The name of the unsubscribed parameter.
  */
  virtual void UnsubscribeParameter(const std::string& name) throw () = 0;

  /** This function is used to subscribe to several parameters at
      once. Like SubscribeParameter the observer will be notified at
      once about any parameters available on the blackboard.

      \param names   A list of names of parameters subscribe.
      \see SubscribeParameter
      \see IBlackboardObserver
   */ 
  virtual void SubscribeParameters(const TParameterList& names) throw () = 0;
  
  /** This function is used to unsubscribe several parameters at once.

      \param names   A list of names of parameters to unsubscribe.
      \see UnsubscribeParameter
   */
  virtual void UnsubscribeParameters(const TParameterList& names) throw () = 0;



  /** This function is used to get the value of a parameter. On
      success the function returns true but if the parameter does not
      exist the function returns false.

      \param name       The name of the parameter to get the value for.
      \retval value     The returned value of the parameter.
      \return           True if the parameter existed in the blackboard.
   */
  virtual bool GetParameter(const std::string& name,
                            std::string& value) throw () = 0;

  /** This function is used to set the value of a parameter. All
      observers will be notified if the value actually changed
      otherwise nothing happens.

      \param name    The name of the parameter to set.
      \param value   The value set to the parameter.
   */
  virtual void SetParameter(const std::string& name,
                            const std::string& value) throw () = 0;

  /** This function is used to set several parameters at once. All
      observers of a parameter will be notified if the value actually
      changed. Observe that if an observer is observing several
      parameters that are changing there will be only one call to
      OnParametersChanged. This is useful for parameters with tight
      coupling, e.g. position in the stream and pace of the stream.

      \param names   The names of the parameters to set.
      \param values  The corresponding values of the parameters.
      \see IBlackboardObserver
   */ 
  virtual void SetParameters(const TParameterList& names,
                             const TValueList& values) throw () = 0;

  /** This function is used to remove a parameter if it exists. All
      observers of a parameter will be notified if a value actually is
      removed.
      
      \param name    The name of the parameter to remove.
      \see IBlackboardObserver
  */
  virtual void UnsetParameter(const std::string& name) throw () = 0;
};




/** \class IBlackboardObserver IBlackboard.h "media/streamer/IBlackboard.h"

    This is the callback interface for the pipeline blackboard.
    Elements receive callbacks on this interface when subscribed
    parameters change in the blackboard.
*/

class IBlackboardObserver
{
public:
  virtual ~IBlackboardObserver() {}

  /** This function is called to notify the observer that one or
      several observed parameters have changed at once, e.g. by a call
      to SetParameters or metadata at the same position in the stream
      published to the blackboard.
      
      The values of the parameters are not provided, they must be
      retrieved using the GetParameter function in the IBlackboard
      interface.

      \param params   The names of the observed parameters that have changed.
      \see IBlackboard::GetParameter
   */
  virtual void OnParametersChanged(const IBlackboard::TParameterList& params)
    throw () = 0;

  /** This function is called to notify the observer that one or
      several observed parameters have been removed, e.g. by a call
      to UnsetParameters.

      \param params   The names of the observed parameters that have been
                      removed.
   */
  virtual void OnParametersRemoved(const IBlackboard::TParameterList& params)
    throw () = 0;
};


#ifdef PIPELINE_NAMESPACE
}
#endif

#endif
