/*
 *--------------------------------------------------------------------
 *
 * IDebug.h
 *
 * This is an interface supplied by the streamer and may be used by
 * elements to output information to the log.
 * 
 * Copyright (c) 2004 Motorola Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef PIPELINE_IDEBUG_H
#define PIPELINE_IDEBUG_H

#include <cstdarg>

#ifdef PIPELINE_NAMESPACE
namespace Pipeline {
#endif

#ifdef __GNUC__
# define IDEBUG_CHECK_FORMAT(x, y) __attribute__ ((format (printf, x, y)))
#else
# define IDEBUG_CHECK_FORMAT(x, y)
#endif
#ifdef __GNUC__
# define IDEBUG_DEPRECATED() __attribute__ ((deprecated))
#else
# define IDEBUG_DEPRECATED()
#endif

/** \class IDebug IDebug.h "media/streamer/IDebug.h"

    This interface is implemented by the Streamer core and may be used
    by elements to print information to the log.
*/

class IDebug
{
public:
  virtual ~IDebug() {}

  enum TLevel {
    DEBUG_FATAL = 1,
    DEBUG_ERROR = 2,
    DEBUG_WARNING = 3,
    DEBUG_NOTE = 4,
    DEBUG_DEBUG = 5
  };

  /** This function enables or disables verbose debug output for the
      element in the Streamer core.

      \param enableFlag   Enables call log if true, disables otherwise.
   */
  virtual void EnableVerboseOutput(bool enableFlag) throw () = 0;

  /** This function allows the elements to print a message to the log.
      
      \param level   The severity of the message.
      \param format  A printf style variable argument format string.
      \param ap A printf style optional parameters.
   */
  virtual void LogOutput(TLevel level,
                         const char* format,
                         va_list ap) throw () IDEBUG_CHECK_FORMAT(3, 0) = 0;

  /** This function allows the elements to print a message to the log.
      
      \param level   The severity of the message.
      \param format  A printf style variable argument format string.
   */
  // This function should be pure virtual too but there seems to be a
  // bug in gcc 3.3.3 when compiling for mips.
  void LogOutput(TLevel level,
                 const char* format...) throw () IDEBUG_CHECK_FORMAT(3, 4);
  
  /** This function is the same as calling LogOutput with
      DEBUG_DEBUG level. Provided for your convenience.

      \param format  A printf style variable argument format string.
   */
  // This function should be pure virtual too but there seems to be a
  // bug in gcc 3.3.3 when compiling for mips.
  void Debug(const char* format...) throw () IDEBUG_CHECK_FORMAT(2, 3);

  // Trace is deprecated use Debug instead.
  void Trace(const char* format...) throw () IDEBUG_DEPRECATED();

  /** This function is the same as calling LogOutput with
      DEBUG_NOTE level. Provided for your convenience.

      \param format  A printf style variable argument format string.
   */
  // This function should be pure virtual too but there seems to be a
  // bug in gcc 3.3.3 when compiling for mips.
  void Note(const char* format...) throw () IDEBUG_CHECK_FORMAT(2, 3);
  
  /** This function is the same as calling LogOutput with
      DEBUG_WARNING level. Provided for your convenience.

      \param format  A printf style variable argument format string.
   */
  // This function should be pure virtual too but there seems to be a
  // bug in gcc 3.3.3 when compiling for mips.
  void Warning(const char* format...) throw () IDEBUG_CHECK_FORMAT(2, 3);

  /** This function is the same as calling LogOutput with
      DEBUG_ERROR level. Provided for your convenience.

      \param format  A printf style variable argument format string.
   */
  // This function should be pure virtual too but there seems to be a
  // bug in gcc 3.3.3 when compiling for mips.
  void Error(const char* format...) throw () IDEBUG_CHECK_FORMAT(2, 3);

  /** This function is the same as calling LogOutput with
      DEBUG_FATAL level. Provided for your convenience.

      \param format  A printf style variable argument format string.
   */
  // This function should be pure virtual too but there seems to be a
  // bug in gcc 3.3.3 when compiling for mips.
  void Fatal(const char* format...) throw () IDEBUG_CHECK_FORMAT(2, 3);
};

#undef IDEBUG_CHECK_FORMAT

inline void IDebug::LogOutput(TLevel level, const char* format...) throw ()
{
  va_list ap;
  va_start(ap, format);
  LogOutput(level, format, ap);
  va_end(ap);
}

inline void IDebug::Debug(const char* format...) throw ()
{
  va_list ap;
  va_start(ap, format);
  LogOutput(DEBUG_DEBUG, format, ap);
  va_end(ap);
}

inline void IDebug::Trace(const char* format...) throw ()
{
  va_list ap;
  va_start(ap, format);
  LogOutput(DEBUG_DEBUG, format, ap);
  va_end(ap);
}

inline void IDebug::Note(const char* format...) throw ()
{
  va_list ap;
  va_start(ap, format);
  LogOutput(DEBUG_NOTE, format, ap);
  va_end(ap);
}

inline void IDebug::Warning(const char* format...) throw ()
{
  va_list ap;
  va_start(ap, format);
  LogOutput(DEBUG_WARNING, format, ap);
  va_end(ap);
}

inline void IDebug::Error(const char* format...) throw ()
{
  va_list ap;
  va_start(ap, format);
  LogOutput(DEBUG_ERROR, format, ap);
  va_end(ap);
}

inline void IDebug::Fatal(const char* format...) throw ()
{
  va_list ap;
  va_start(ap, format);
  LogOutput(DEBUG_FATAL, format, ap);
  va_end(ap);
}


#ifdef PIPELINE_NAMESPACE
}
#endif

#endif
