/*
 *--------------------------------------------------------------------
 *
 * IElementDescriptor.h
 *
 * Copyright (c) 2003 Motorola Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TELEMENTDESCRIPTOR_H
#define TELEMENTDESCRIPTOR_H

#include "media/streamer/IElement.h"

#include <string>


#define INPUT_NONE            "_In NONE"
#define OUTPUT_SAME_AS_INPUT  "_Out SAME"
#define OUTPUT_NONE           "_Out NONE"


/** \class IElementDescriptor IElementDescriptor.h \
    "media/streamer/IElementDescriptor.h"

    Every Streamer element must have an element descriptor, that
    implements this interface, to be recognized by the Streamer. The
    descriptor contains properties that is used to decide when and
    where the element should be used. The descriptor is also used to
    create the actual element.
 */
class IElementDescriptor
{
public:
  /** \class TMetadataInfo IElementDescriptor.h \
      "media/streamer/IElementDescriptor.h"
      
      This class describes information about a metadata parameter that
      is used, destroyed or produced by the element. Elements that are
      not used, destroyed or produced should not be in the list of
      metadata info. All other incoming metadata is assumed to pass
      right through intermediate elements and to be published by
      sinks.
   */
  class TMetadataInfo
  {
  public:
    /** Note! The INPUT_PRODUCED and OUTPUT_RELEASED combination is
        illegal.
     */
    enum TInput {
      /** This metadata is not an incoming metadata. It is produced
          inside the element itself. */
      INPUT_PRODUCED,

      /** This metadata must be available in the stream as input in to
          the element. */
      INPUT_REQUIRED,

      /** This metadata is optional but desired in the stream as input
          to the element. */
      INPUT_OPTIONAL
    };


    enum TOutput {
      /** The metadata is committed to the stream by the element. */
      OUTPUT_COMMITTED,

      /** The metadata is published and will be available on the
          blackboard when the stream is consumed. The metadata is no
          longer available in the stream. */
      OUTPUT_PUBLISHED,

      /** The metadata is released and will neither be available in
          the stream nor published on the blackboard. */
      OUTPUT_RELEASED
    };

    
    enum TType {
      /** The type should only be stated by the producer. */
      TYPE_NOT_PRODUCED,

      /** The value of the metadata is valid until the next value. */
      TYPE_CONTINUAL,

      /** The value of the metadata is valid only at the point in the
          stream where the metadata is produced. */
      TYPE_MOMENTARY
    };


    /** The name of parameter that the metadata information describes.
     */
    const char* ParameterName;

    /** Used to tell whether the parameter is expected as an input
        from the stream to the element or if the parameter is produced
        in the element. Parameters expected as input may be required
        or optional. If a metadata parameters is required as input the
        streamer makes sure an element producing the metadata is added
        in the element chain before this element.
     */
    TInput Input;

    /** Used to tell whether the parameter is committed to the stream,
        published or released. Observe that it is illegal to declare a
        parameter to be released if it is produced in the element.
     */
    TOutput Output;

    /** Used to tell what type a produced metadata is of. It is
        important that producers of metadata define whether the
        metadata is continual, i.e. describing a state that is valid
        until next metadata is created (e.g. playback mode), or
        momentary, i.e. describing an entity that is valid only at the
        position of the metadata (e.g. clock reference).
        
        If the metadata is not produced by the element, the type
        should be set to TYPE_NOT_PRODUCED.
     */
    TType Type;
  };


  /** \class TProperties IElementDescriptor.h \
      "media/streamer/IElementDescriptor.h"

      This class provides the element properties needed by the
      streamer to set up a correct element chain.
   */
  class TProperties
  {
  public:
    enum TType {
      /** The element is a source element which is a special case of
          root element. The source element is the element reading the
          stream, e.g. from the network, to the first pipeline. */
      ELEMENT_SOURCE,

      /** The element is a intermediary element, neither root nor leaf. */
      ELEMENT_INTERMEDIATE,

      /** The element is a sink element which is a special case of
          leaf element. Sink elements may not open any other
          pipeline. */
      ELEMENT_SINK,

      /** The element is a root element, used in the beginning of a
          pipeline. */
      ELEMENT_ROOT,

      /** The element is a leaf element, used at the end of a
          pipeline. A leaf element doesn't open any pads but might
          open another pipeline. */
      ELEMENT_LEAF
    };

    /** The name of the element. This name is used to identify the
        element in the log.
     */
    const char* Name;

    /** If two elements offer the same service the one with highest
        priority will be used. Priority 100-1000 is reserved for
        elements created by KreaTV, if elements are built to replace
        KreaTV elements use priority > 1000.  Elements with priority
        0 will not be used.
    */
    uint32_t Priority;

    /** A string in Format Expression Syntax describing which input
        formats the element supports. For source elements, the value
        must be INPUT_NONE.
    */
    const char* InputDataFormats;

    /** A string in Format Expression Syntax describing which output
        formats the element supports. For sink elements, the value
        must be OUTPUT_NONE. If the element always opens exactly one
        output pad with the same format as the input format, the value
        must be OUTPUT_SAME_AS_INPUT.

        For leaf elements, this is the input format(s) in the
        following pipelines opened by the element.
    */
    const char* OutputDataFormats;

    /** The element type (source, intermediate or sink).
     */
    TType Type;

    /** Information about all metadata used and produced by the element.
        \see TMetadataInfo
     */
    const IElementDescriptor::TMetadataInfo* const* MetadataInfo;

    /** If element B names element A as its predecessor, and element A
        opens an output pad that matches the input data formats of
        element B, element B will be inserted directly after element A
        in the pipeline. This mechanism is useful for controlling the
        extension of a pipeline or a single element with a high level
        of precision. If this kind of association is not desirable,
        this field must be NULL, which means that the placement of the
        element in the pipeline is determined by the regular data flow
        analysis.

        Chaining with a predecessor this way is only allowed for
        elements which output format is the same as the input format,
        and they do not consume any metadata.

        If two elements name the same element as their respective
        predecessor, the priority will be used to determine which one
        of the two elements will be inserted after the predecessor.
        The element with the lower priority will not be inserted in
        the pipeline.
    */
    const char* Predecessor;
  };


  virtual ~IElementDescriptor() throw () { }


  /** Gets the information structure describing the element.

      \param properties   The properties are assigned to this object.
   */
  virtual void GetProperties(TProperties* properties) throw () = 0;


  /** This function is used for source elements to see if the element
      is able to open the URI.
      
      \return             True if the source element can open the URI,
                          and false otherwise.
   */
  virtual bool MatchUri(const std::string& uri) throw () = 0;


  /** This function is used to know whether the element possesses
      certain traits that the Streamer core is interested in,
      e.g. whether the element can extract clock reference information
      from the stream.

      \return             True if the element possesses the trait, and
                          false otherwise.
   */
  virtual bool HasTrait(const std::string& trait) throw () = 0;


  /** This function is called to actually create an instance of the element.

      \return             Pointer to the created element.
   */
  virtual IElement* CreateElement() throw () = 0;
};

#endif
