/*
 *--------------------------------------------------------------------
 *
 * IEvent.h
 *
 * The interface to the element event mechanism.
 *
 * Copyright (c) 2004 Motorola Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef PIPELINE_IEVENT_H
#define PIPELINE_IEVENT_H

#include <stdint.h>
#include <event/IIOEventLoop.h>

#ifdef PIPELINE_NAMESPACE
namespace Pipeline {
#endif


/** \class IEvent IEvent.h "media/streamer/IEvent.h"

    This is the interface to the element event mechanism. An element
    acquires a pointer to this interface by using the
    IPump::QueryInterface() function. Elements using this interface
    must implement the IEventCallback interface.

    \see IPump::QueryInterface,
         IEventCallback
*/
class IEvent
{
public:
  virtual ~IEvent() {}

  // Matches TEvent in IIOEventLoop:
  /** These are combined to form the <em>eventMask</em> bitfield used
      when adding a file descriptor.
  */
  enum {
    /** Get event whe the file descriptor is ready for reading. */
    EVENT_READ = 1 << 0,

    /** Get event when the file descriptor is ready for writing. */
    EVENT_WRITE = 1 << 1
  };


  virtual IIOEventLoop* GetEventLoop() const throw () = 0;
  
  /** This function adds a file descriptor to the event engine. When
      the file descriptor is ready for reading and/or writing, the
      event engine will call the IEventCallback::OnEvent() function
      implemented by the element.

      \param fd          The file descriptor to add.

      \param eventMask   A mask specifying whether the element wants to
                         receive an event callback when the file
                         descriptor is ready for reading and/or
                         qwriting. If the value is EVENT_READ the
                         event engine will call the callback function
                         only when the file descriptor is ready for
                         reading. Likewise, if the value is
                         EVENT_WRITE the event engine will call the
                         callback function only when the file
                         descriptor is ready for writing. Specifying
                         EVENT_READ | EVENT_WRITE, callbacks will be
                         triggered in both cases.

      \see RemoveFileDescriptor(),
           IEventCallback::OnEvent()

      \see For information about file descriptors in general please
           refer to regular UNIX/Linux textbooks on the subject.
   */
  virtual void AddFileDescriptor(int fd, uint32_t eventMask) throw () = 0;


  /** Removes a file descriptor from the event engine. The file
      descriptor must have previously been added with a call to
      AddFileDescriptor().
      
      \param fd   The file descriptor to remove.

      \see AddFileDescriptor()
   */
  virtual void RemoveFileDescriptor(int fd) throw () = 0;


  /** Creates a periodic timer which will generate timeout events with
      a specified interval. When the timeout occurs the event engine
      will call the IEventCallback::OnTimeout() function provided by
      the element. The first timeout will occur <em>interval</em>
      milliseconds from the call to this function. The timer will
      continue to generate timeouts with the specified interval until
      the timer is removed.

      \param interval   The interval between two consecutive
                        timeouts in milliseconds.

      \return           A unique ID identifying the new timer.

      \see RemoveTimer(),
           IEventCallback::OnTimeout()
   */
  virtual int AddTimer(uint32_t interval) throw () = 0;


  /** Removes a timer so that no more timeouts will occur. The timer
      must have been previously added with a call to AddTimer(). A
      timer may be removed at any time.

      \param timerId    The ID of the timer to remove.

      \see AddTimer()
   */
  virtual void RemoveTimer(int timerId) throw () = 0;


  /** This function works in a similar fashion as AddFileDescriptor
      but instead of calling IEventCallback::OnEvent it will call the
      function provided.

      It is possible for several elements to call this function for
      the same file descriptor (with the same function and
      eventMask). In that case the function will still only be called
      once for each event but the event engine will not stop calling
      the function until all elements has called
      RemoveFileDescriptorFunction().
      
      \param fd          The file descriptor to add.

      \param eventMask   A mask specifying whether the element wants
                         to receive an event callback when the file
                         descriptor is ready for reading and/or
                         writing. If the value is EVENT_READ the event
                         engine will call the callback function only
                         when the file descriptor is ready for
                         reading. Likewise, if the value is
                         EVENT_WRITE the event engine will call the
                         callback function only when the file
                         descriptor is ready for writing. Specifying
                         EVENT_READ + EVENT_WRITE, callbacks will be
                         triggered in both cases.

      \param fun         The function to call when an event occurs.

      \see AddFileDescriptor(),
           RemoveFileDescriptorFunction(),           
  */
  virtual void AddFileDescriptorFunction
  (int fd, uint32_t eventMask, void (*fun)(int fd, uint32_t eventMask))
    throw () = 0;

  /** Removes the file descriptor from the event engine when all
      elements called this function. The file descriptor must have
      previously been added with a call to AddFileDescriptorFunction().
      
      \param fd   The file descriptor to remove.

      \see AddFileDescriptorFunction()      
  */
  virtual void RemoveFileDescriptorFunction(int fd) throw () = 0;
};



/** \class IEventCallback IEvent.h "media/streamer/IEvent.h"

    This is the callback interface for the element event mechanism.
    Elements receive callbacks on this interface when events occur.
*/

class IEventCallback
{
public:
  virtual ~IEventCallback() {}

  /** This function is called when there is an event on a registered
      file descriptor.
      
      \param descriptor   The file descriptor that is ready for reading
                          and/or writing.

      \param eventMask    Mask telling whether the descriptor is ready
                          for reading or writing.
  */
  virtual void OnEvent(int descriptor, uint32_t eventMask) throw () = 0;


  /** This function is called when a timeout has occured on a timer.

      \param timerId      The ID of the timer.
  */
  virtual void OnTimeout(int timerId) throw () = 0;
};


#ifdef PIPELINE_NAMESPACE
}
#endif

#endif
