// Copyright (c) 2009-2010 Motorola, Inc. All rights reserved.
// Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
//
// This program is confidential and proprietary to Motorola Mobility, Inc and
// may not be copied, reproduced, disclosed to others, published or used, in
// whole or in part, without the expressed prior written permission of Motorola
// Mobility, Inc.

#ifndef IHARDWARE_H
#define IHARDWARE_H

#include <cstddef>
#include <string>

class IDemuxer;
class IFrontend;
class IStreamInput;
class IDiskWriter;
class IDiskReader;
class IDvrAsset;
class IMediaDecoder;
class ITimeStamp;
class ISocket;
class IMediaTranscoder;
class IMediaSource;
class IVbi;
class ICa;
class ISocketSink;
class IInformation;

#ifdef PIPELINE_NAMESPACE
namespace Pipeline {
#endif

  /** \class IHardware IHardware.h "media/streamer/IHardware.h"

      This interface is used by streamer elements to gain access to
      hardware resources.
  */

class IHardware
{
public:
  enum TTargetPlatform {
    TARGET_UNKNOWN,
    TARGET_VIP_19xx,
    TARGET_VIP_20xx
  };

  virtual ~IHardware() throw () { /* Empty */ }

  /** This function returns the primary IDemuxer
   *  Returns a NULL pointer if there is no Demuxer resource available.
   */
  virtual IDemuxer* GetPrimaryDemuxer() throw () = 0;

  /** This function returns the secondary IDemuxer
   *  Returns a NULL pointer if there is no Demuxer resource available.
   */
  virtual IDemuxer* GetSecondaryDemuxer() throw () = 0;

  /** This function returns an IFrontend.
   *  Returns a NULL pointer if there is no Demuxer resource available.
   */
  virtual IFrontend* GetFrontend() throw () = 0;

  /** This function returns an IFrontend corresponding to out-of-band.
   *  Returns a NULL pointer if there is no Demuxer resource available.
   */
  virtual IFrontend* GetOobFrontend() throw () = 0;

  /** This function returns an IStreamInput.
   *  Returns a NULL pointer if there is no StreamInput resource available.
   */
  virtual IStreamInput* GetStreamInput() throw () = 0;

  /** This function returns a backup IStreamInput which used for a second payload sink.
   *  Returns a NULL pointer if there is no StreamInput resource available.
   */
  virtual IStreamInput* GetStreamInput2() throw () = 0;

  /** This function returns a backup IStreamInput which used for Vbi sink.
   *  Returns a NULL pointer if there is no StreamInput resource available.
   */
  virtual IStreamInput* GetVbiStreamInput() throw () = 0;

  /** This function returns a IDiskWriter.
   *  Returns a NULL pointer if there is no DiskWriter resource available.
   */
  virtual IDiskWriter* GetDiskWriter() throw () = 0;

  /** This function returns a IDiskReader.
   *  Returns a NULL pointer if there is no DiskReader resource available.
   */
  virtual IDiskReader* GetDiskReader() throw () = 0;

  /** This function returns a IDvrAsset.
   *  Returns a NULL pointer if the asset couldn't be opened.
   */
  virtual IDvrAsset* OpenDvrAsset(const char* assetPath) throw () = 0;

  /** This function returns an audio IMediaDecoder.
   *  Returns a NULL pointer if there is no MediaDecoder resource available.
   */
  virtual IMediaDecoder* GetAudioDecoder() throw () = 0;

  /** This function returns an audio IMediaDecoder for PCM audio.
   *  Returns a NULL pointer if there is no MediaDecoder resource available.
   */
  virtual IMediaDecoder* GetPcmAudioDecoder() throw () = 0;

  /** This function returns an audio IMediaDecoder for WM9 audio.
   *  Returns a NULL pointer if there is no MediaDecoder resource available.
   */
  virtual IMediaDecoder* GetWm9AudioDecoder() throw () = 0;

  /** This function returns a video IMediaDecoder.
   *  Returns a NULL pointer if there is no MediaDecoder resource available.
   */
  virtual IMediaDecoder* GetVideoDecoder() throw () = 0;

  /** This function returns a video IMediaDecoder for WM9 video.
   *  Returns a NULL pointer if there is no MediaDecoder resource available.
   */
  virtual IMediaDecoder* GetWm9VideoDecoder() throw () = 0;

  /** This function returns a video IMediaDecoder to use for
   *  picture in picture.
   *  Returns a NULL pointer if there is no MediaDecoder resource available.
   */
  virtual IMediaDecoder* GetPipVideoDecoder() throw () = 0;

  /** This function returns a ITimeStamp.
   *  Returns a NULL pointer if there is no TimeStamp resource available.
   */
  virtual ITimeStamp* GetTimeStamp() throw () = 0;

  /** This function returns a ISocket.
   *  Returns a NULL pointer if there is no Socket resource available.
   */
  virtual ISocket* GetSocket() throw () = 0;

  /** This function returns a IMediaTranscoder.
   *  Returns a NULL pointer if there is no MediaTranscoder resource available.
   */
  virtual IMediaTranscoder* GetMediaTranscoder() throw () = 0;

  /** This function returns a IMediaSource.
   *  Returns a NULL pointer if there is no MediaSource resource available.
   */
  virtual IMediaSource* GetMediaSource() throw () = 0;

  /** This function returns a ICa.
   *  Returns a NULL pointer if there is no Ca resource available.
   */
  virtual ICa* GetCa(const char* name = NULL) throw () = 0;

  /** This function returns a IVbi.
   *  Returns a NULL pointer if there is no Vbi resource available.
   */
  virtual IVbi* GetVbi() throw () = 0;

  /** This function returns a ISocketSink.
   *  Returns a NULL pointer if there is no SocketSink resource available.
   */
  virtual ISocketSink* GetSocketSink() throw () = 0;

  /** This function returns a IInformation.
   *  Returns a NULL pointer if there is no Information resource available.
   */
  virtual IInformation* GetInformation() throw () = 0;

  /** This function returns the target platform.
   */
  virtual TTargetPlatform GetTargetPlatform() throw () = 0;

};

#ifdef PIPELINE_NAMESPACE
}
#endif

#endif
