/*
 *--------------------------------------------------------------------
 *
 * IInputPad.h
 *
 * This is the interface to the input pad of an element.
 * 
 * Copyright (c) 2004 Motorola Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef PIPELINE_IINPUTPAD_H
#define PIPELINE_IINPUTPAD_H

#include "media/streamer/TSegmentList.h"

#ifdef PIPELINE_NAMESPACE
namespace Pipeline {
#endif


/** \class IInputPad IInputPad.h "media/streamer/IInputPad.h"

    This is the interface to the input pad of an element. All elements
    except source elements have exactly one input pad. All non-source
    elements must implement this interface. Source elements are
    must not implement this interface.
*/

class IInputPad
{
public:
  virtual ~IInputPad() throw () {}

  /** This function is called by the Streamer core when the element
      has received additional segments of data that are ready for
      processing by the element or when the element has requested to
      be processed. The task of the element is to process as much of
      the segments in the segment list as possible.  Segments are
      placed in the list in chronological order, i.e. the segment with
      the oldest stream data is placed at the beginning of the list,
      with segments holding increasingly younger data to follow. The
      element must process the segments from the beginning of the list
      and towards the end.

      When the element does not need the stream data it must either
      commit the data to one of its output pads, or release it from
      the pipeline. Stream data cannot be committed or released in
      larger chunks than one complete segment, but it is possible to
      commit or release a fraction of the data in a segment provided
      that the stream data is handled in strict chronological order.
      
      For example, suppose the element receives a segment of 300 bytes
      of stream data. The element may commit the first 100 bytes of a
      segment to one of its output pad and the next 100 bytes to
      another (or perhaps release it to remove it from further
      processing down the pipeline). If the element needs more data to
      decide what to do with the remaining 100 bytes, it should return
      from the Process() call (or postpone the remaining data if it is
      a framing element) . The next time its Process() function is
      called the first segment in the segment list will contain just
      those 100 bytes remaining from the previous call.
      
      \param segments    A list of data segments and attached metadata.
  */
  virtual void Process(const TSegmentList& segments) throw () = 0;


  /** This function is called by the Streamer core to flush the
      pipeline. The function is similar to Process(), but the element
      is required to commit or release all segments in the segment
      list.

      \param segments    A list of data segments and attached metadata.
  */
  virtual void Flush(const TSegmentList& segments) throw () = 0;
};

#endif
