/*
 *--------------------------------------------------------------------
 *
 * IOutputPad.h
 *
 * This is the interface to an elements output pad.
 *
 * Copyright (c) 2004 Motorola Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef PIPELINE_IOUTPUTPAD_H
#define PIPELINE_IOUTPUTPAD_H

#include <stdint.h>

#ifdef PIPELINE_NAMESPACE
namespace Pipeline {
#endif


class TMetadata;

/** \class IOutputPad IOutputPad.h "media/streamer/IOutputPad.h"

    This is the interface used by elements to commit metadata and data
    segments to an output pad connected to another element. Sink
    elements do not use output pads but rather release all segment
    and publish or release all metadata using the IPump interface.

    \see IPump
*/

class IOutputPad
{
public:
  virtual ~IOutputPad() {}

  /** This function is used to commit a metadata structure to the
      output pad. Each incoming metadata structure attached to a
      segment must be committed, destroyed or published.

      Metadata that is attached to segments being processed by
      elements are considered to arrive "before" the segment and
      should be committed/destroyed/published before the corresponding
      segment is committed.

      \param metadata      The metadata structure to commit.
  */
  virtual void CommitMetadata(const TMetadata* metadata) throw () = 0;

  /** This function is used to commit a copy of a metadata structure
      to the output pad.

      If an element wants to commit metadata to several output pads it
      should only use CommitMetadata on one pad and this function on
      all the others.
      
      \param metadata      The metadata structure to copy and commit.
  */
  virtual void CommitMetadataCopy(const TMetadata* metadata) throw () = 0;

  /** This function commits a data segment to the output pad.  

      Each element has to commit or release all parts of all the
      incoming segments that it has processed. No other parts than
      what is being processed may be committed or released.

      \param position      Pointer to the starting position of the segment.
      \param frameCount    The number of frames in the segment.
      \param frameSize     The size of each frame.
      \param focusOffset   Offset to the focus in each frame.
      \param focusSize     The size of the focus in each frame.
  */
  virtual void CommitSegment(uint8_t* position,
                             uint32_t frameCount,
                             uint32_t frameSize,
                             uint32_t focusOffset,
                             uint32_t focusSize) throw () = 0;
};


#ifdef PIPELINE_NAMESPACE
}
#endif

#endif
