/*
 *--------------------------------------------------------------------
 *
 * IPump.h
 *
 * The main interface to the pump.
 *
 * Copyright (c) 2004 Motorola Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef PIPELINE_IPUMP_H
#define PIPELINE_IPUMP_H

#include <stdint.h>

#ifdef PIPELINE_NAMESPACE
namespace Pipeline {
#endif


class TMetadata;
class IOutputPad;
class IPostponePad;
class IPipelineLink;


/** \class IPump IPump.h "media/streamer/IPump.h"

    This is the main interface to the pump available to all elements
    participating in a pipeline.
*/

class IPump
{
public:
  virtual ~IPump() {}

  /** This function returns a pointer to the named interface. The
      caller must cast the returned pointer to the right interface.
      The pump will return NULL if it does not implement the named
      interface and the function will return the same pointer if it is
      called multiple times for the same interface name.

      \param interfaceName   The name of the interface for which the
                             caller requests a pointer, e.g. the name
                             could be "IBlackboard" and the pump will
                             return the pointer to its blackboard
                             interface.
       \return               A pointer to the interface. 
  */
  virtual void* QueryInterface(const char* interfaceName) throw () = 0;
  

  /** This function returns information about the buffer space in
      which all stream data is stored while being pushed through the
      pipeline.

      \retval buffer    A pointer to the first byte of the buffer
                         space.
      \retval key       The shared memory key as used by the
                         <em>shmget()</em> syscall.
  */
  virtual void GetBufferInfo(const uint8_t*& buffer,
                             uint32_t& key) const throw () = 0;
  

  /** Opens an ouput pad to another element. Segments and metadata may
      be committed to the output pad and the next element will process
      it.

      \param dataFormat  The format of the outcoming data in the
                         segments.
      \return            A pointer to the output pad.
   */
  virtual IOutputPad* OpenPad(const char* dataFormat) throw () = 0;


  /** Opens the postpone pad which is used to postpone the processing
      of a segment.
      
      \return             A pointer to the postpone pad.
  */
  virtual IPostponePad* OpenPostponePad() throw () = 0;


  /** Closes a pad and destroys the elements in branch of the pipeline.

      \param pad          The pad to close.
   */
  virtual void ClosePad(IOutputPad* pad) throw () = 0;

  
  /** Creates a new metadata structure to commit on an output pad.

      \return             The created metadata structure.
   */
  virtual TMetadata* CreateMetadata() throw () = 0;


  /** This function is used to destroy a metadata structure.

      Each incoming (to an element) metadata structure attached to a
      segment must be committed on an output pad, destroyed or
      published.

      Metadata that is attached to segments being processed by
      elements are considered to arrive "before" the segment and
      should be committed/destroyed/published before the corresponding
      segment is committed.
      
      \param metadata     The metadata structure being destroyed.
   */
  virtual void DestroyMetadata(const TMetadata* metadata) throw () = 0;


  /** This function publishes a metadata structure on the blackboard
      when the stream has been consumed to the position of the
      metadata. The metadata
      structure is then destroyed.
      
      Each incoming (to an element) metadata structure attached to a
      segment must be committed on an output pad, destroyed or
      published.

      Metadata that is attached to segments being processed by
      elements are considered to arrive "before" the segment and
      should be committed/destroyed/published before the corresponding
      segment is committed.

      \param metadata     The metadata structure being published.
  */
  virtual void PublishMetadata(const TMetadata* metadata) throw () = 0;


  /** This function releases a segment.

      All parts of each incoming (to an element) segment must be
      released or committed on an output pad.

      The release operation is a variation of commit but the segment
      is not committed to a pad but rather to "void".

         \param position Pointer to the starting position of the segment.

       \param frameCount The number of frames in the segment.

        \param frameSize The size of each frame.
  */
  virtual void ReleaseSegment(uint8_t* position,
                              uint32_t frameCount,
                              uint32_t frameSize) throw () = 0;


  /** An element may call this function to request a new call to its
      IInputPad::Process() function. This is useful, e.g. when a sink
      element discovers it needs to push more data to its decoder in
      HAL. For optimal data throughput, the sink element should call
      this function as soon as possible after it has discovered this
      condition.

      \see IInputPad::Process
   */
  virtual void RequestProcess() throw () = 0;


  /** Leaf elements can open another pipeline to follow the pipeline
      which the leaf is a part of. Only metadata flows directly
      between the pipelines through the IPipelineLink so the
      synchronization with segments is lost.

      Opening pipelines is useful e.g. when using hardware (HAL) to
      parse transport streams. The unparsed data is then sent down to
      HAL by the leaf element in the first pipeline and the parsed
      data is obtained from HAL by the root element in the next
      pipeline.

      \param dataFormat  The format of the incoming data to the new pipeline.
      \return            A pointer to pipeline link.
   */
  virtual IPipelineLink* OpenPipeline(const char* dataFormat) throw () = 0;
};


#ifdef PIPELINE_NAMESPACE
}
#endif

#endif
