/*
 *--------------------------------------------------------------------
 *
 * ISink.h
 *
 * This is the interface to sink elements.
 *
 * Copyright (c) 2004 Motorola Inc. All Rights Reserved.
 * Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
 *
 * This program is confidential and proprietary to Motorola Mobility, Inc and
 * may not be copied, reproduced, disclosed to others, published or used, in
 * whole or in part, without the expressed prior written permission of Motorola
 * Mobility, Inc.
 *
 *--------------------------------------------------------------------
 */

#ifndef PIPELINE_ISINK_H
#define PIPELINE_ISINK_H

#include "media/streamer/TSegmentList.h"

#include <stdint.h>

#ifdef PIPELINE_NAMESPACE
namespace Pipeline {
#endif


/** \class ISink ISink.h "media/streamer/ISink.h"

    This is the interface that must be implemented by all sink
    elements participating in the pipeline.
*/

class ISink
{
public:
  virtual ~ISink() {}

  enum TType {
    SINK_AUDIO,
    SINK_VIDEO,
    SINK_PIPVIDEO,
    SINK_SOCKET,
    SINK_RECORD,
    SINK_TELETEXT,
    SINK_OTHER
  };

  /** This function is used to find out what type of sink this is.

      \return    The type of the sink.
   */
  virtual TType GetSinkType() throw () = 0;

  /** This function is called when the sink should become active,
      which means open up necessary resources and start processing.
  */
  virtual void Activate() throw () = 0;

  /** This function is called when the sink should become passive,
      which means close open resources and stop processing.
  */
  virtual void Deactivate() throw () = 0;

  /** This function is called to make the sink block all data from
      reaching hardware.
   */
  virtual void Block() throw () = 0;


  /** This function is called to make the sink unblock the hardware.
   */
  virtual void Unblock() throw () = 0;
};



/** \class ISinkCallback ISink.h "media/streamer/ISink.h"

    This is the callback interface used by sink elements to report
    state changes.
*/

class ISinkCallback
{
public:
  virtual ~ISinkCallback() {}

  /** The sink element must call this function when all stream buffers
      it knows about becomes empty. Under normal circumstances, the
      pump expects this to happen at end of stream, but it may also
      happen intermittently during normal processing depending on the
      buffering behavior of the underlying media decoder and other
      aspects of the system unknown to the sink element.
   */
  virtual void OnEmpty() throw () = 0;

  /** The sink element should call this function after OnEmpty when
      buffers are full enough to resume playing.
  */
  virtual void OnRunning() throw () = 0;

  /** The sink element should call this when there is an error specific to the
      sink.
  */
  virtual void OnError(const std::string& reason) = 0;
};


#ifdef PIPELINE_NAMESPACE
}
#endif

#endif
