/*
 *--------------------------------------------------------------------
 *
 * ISource.h
 *
 * This is the interface to source elements.
 *
 * Copyright (c) 2004 Motorola Inc. All Rights Reserved.
 * Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
 *
 * This program is confidential and proprietary to Motorola Mobility, Inc and
 * may not be copied, reproduced, disclosed to others, published or used, in
 * whole or in part, without the expressed prior written permission of Motorola
 * Mobility, Inc.
 *
 *--------------------------------------------------------------------
 */

#ifndef PIPELINE_ISOURCE_H
#define PIPELINE_ISOURCE_H

#include "media/streamer/IStreamerTypes.h"

#include <string>
#include <vector>
#include <stdint.h>

#ifdef PIPELINE_NAMESPACE
namespace Pipeline {
#endif


/** \class ISource ISource.h "media/streamer/ISource.h"

    This is the interface that must be implemented by all source
    elements participating in a pipeline.
*/

class ISource
{
public:
  virtual ~ISource() {}

  typedef std::vector< const char* > TFeatureList;

  /** This function is used by the streamer core to query the element
      for features it supports.
      
      \param feature    A null-terminated text string containing the name of
                        the feature.
      \return           True if the element supports the feature. If the
                        element does not recognize the feature name it must
                        return false.

   */
  virtual bool IsFeatureSupported(const char* feature) throw () = 0;


  /** Open a new session using the URI describing the media stream.
      When called, the element must allocate all resources necessary
      to be able to receive the stream from the source. This function
      is called in closed state only.
      
      \param uri        A null-terminated text string containing the URI
                        describing the location of the media stream.
  */
  virtual void Open(const char* uri) throw () = 0;


  /** Close the session. All resources allocated by the element when
      the session was opened must be freed. This function may be
      called in any state except the closed state.
   */
  virtual void Close() throw () = 0;
  

  /** Start/resume playback of the stream.

      \param pace       The playback pace in milliseconds per second.
                        Normal pace is thus 1000 milliseconds per second.
   */
  virtual void Play(int32_t pace) throw () = 0;


  /** Reposition the stream.

      \param position   The number of milliseconds from beginning of the
                        stream that the playback will be repositioned to.

      \param pace       The playback pace in milliseconds per second.
                        Normal pace is thus 1000 milliseconds per second.
   */
  virtual void PlayFromPosition(uint32_t position, int32_t pace) throw () = 0;
};



/** \class ISourceCallback ISource.h "media/streamer/ISource.h"

    This is the callback interface used by source elements to
    report state, position and pace changes.
*/

class ISourceCallback
{
public:
  virtual ~ISourceCallback() {}

  /** The source element should indicate state changes occuring from
      calls to Open/Play/FastForward/Rewind/Pause/Close by calling
      this function. It must be called before OnCommandResponse.

      A state change because of end-of-stream or beginning-of-stream
      should be indicated by changing state to paused.

      \param state      The new state.
      \param reason     The reason of state change.
      \see ISource
      \see OnCommandResponse
   */
  virtual void OnStateChanged(IStreamerTypes::TState state,
                              IStreamerTypes::TSourceStateReason reason =
                              IStreamerTypes::STATE_REASON_UNKNOWN)
    throw () = 0;

  /** All commands must end with a command response.

      \param success    True if the command was successful, otherwise false.
   */
  virtual void OnCommandResponse(bool success) throw () = 0;

  /** Current playback position and pace.

      \param position The current position in milliseconds.

      \param pace     The playback pace in milliseconds per second.
                      Normal pace is thus 1000 milliseconds per second.
   */
  virtual void OnPlaybackStatus(uint32_t position, int32_t pace) throw () = 0;

  /** Current duration.

      \param duration The current playback duration in milliseconds.
   */
  virtual void OnDurationChanged(uint32_t duration) throw () = 0;

  /** Used by the source element to indicate any discontinuity detected
      in the stream.
   */
  virtual void OnDiscontinuityDetected() throw () = 0;
  
  /** Indicate the supported features have been changed.
   */
  virtual void OnSupportedFeaturesChanged(
    const ISource::TFeatureList& features) throw () = 0;
};


#ifdef PIPELINE_NAMESPACE
}
#endif

#endif
