// Copyright (c) 2003 Motorola Inc. All Rights Reserved.
// Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
//
// This program is confidential and proprietary to Motorola Mobility, Inc and
// may not be copied, reproduced, disclosed to others, published or used, in
// whole or in part, without the expressed prior written permission of Motorola
// Mobility, Inc.

#ifndef ISTREAMERTYPES_H
#define ISTREAMERTYPES_H

#include <stdint.h>

#include <string>

// DO NOT ADD ANY TYPES HERE THAT ARE NOT NEEDED BY STREAMER
// ELEMENTS. THIS IS A PUBLIC INTERFACE!

/** \class IStreamerTypes IStreamerTypes.h "media/streamer/IStreamerTypes.h"

    This interface contains some datatypes used by the Streamer and the
    elements.
 */
class IStreamerTypes
{
public:
  virtual ~IStreamerTypes() { /* Empty. */ }

  /** Enumeration of various sources that can cause an OnError event
      to occur. Typically, this can be a streamer element or similar
      low-level component.
   */
  enum TErrorSource
  {
    /** SOURCE_UNKNOWN: Default error state.
    */
    SOURCE_UNKNOWN,
    /** SOURCE_BITBAND: Error originated in a Bitband source element.
    */
    SOURCE_BITBAND
  };

  /** Structure holding both the source of an error and the error code
      produced by that source. This structure is meant to be computer-readable.
   */
  struct TError
  {
    TErrorSource Source;
    int Details;
  };

  /** Enumeration of the different source element states.
   */
  enum TState
  {
    STATE_CLOSED,
    STATE_PLAYING,
    STATE_PAUSED,
    STATE_FASTFORWARDING,
    STATE_REWINDING,
    STATE_FAILED
  };

  /* Reasons of source state change.
   */
  enum TSourceStateReason
  {
    STATE_REASON_UNKNOWN,
    STATE_REASON_COMMAND,
    STATE_REASON_POSITION_START,
    STATE_REASON_POSITION_END,
    STATE_REASON_DATA_FLOW_SUSPENDED,
    STATE_REASON_DATA_FLOW_RESUMED,
    STATE_REASON_CONNECTION_FAILED
  };

  /** The recorder element states.
   */
  enum TRecorderState
  {
    RECORDER_STATE_CLOSED,
    //RECORDER_STATE_INITIALIZED,
    RECORDER_STATE_RECORDING,
    RECORDER_STATE_FAILED
  };

  enum TRecorderFailureReason
  {
    ERROR_PERMISSION_DENIED,
    ERROR_INVALID_DISKASSET,
    ERROR_EMPTY_DISKASSET,
    ERROR_NO_DATA_FILES,
    ERROR_UNSUPPORTED_DISKASSET_VERSION,
    ERROR_DISK_FULL,
    ERROR_UNIMPLEMENTED,
    ERROR_UNKNOWN,

    // Indicates broken filesystem or HW-failure.
    ERROR_LOWLEVEL_IO_FAILURE
  };

  /** Enumeration of different streamer playback modes. The playback mode
      is posted as PARAM_PLAYBACK_MODE on the blackboard by the streamer core.
   */
  enum TPlaybackMode
  {
    PLAYBACK_MODE_PLAYING,
    PLAYBACK_MODE_FASTFORWARDING,
    PLAYBACK_MODE_REWINDING
  };

  /** Enumeration of different copy protection modes. The mode can be committed
      in the protection metadata parameter by some element in the pipeline.
   */
  enum TProtectMode
  {
    PROTECT_NO_RESTRICTION,
    PROTECT_COPYING_PERMITTED,
    PROTECT_ONE_COPY_PERMITTED,
    PROTECT_COPYING_RESTRICTED
  };

  /** Enumeration of different aspect ratios. The aspect ration is part of the
      _DecoderVideoStreamInfo.&lt;pid&gt; and _SubtitleStreamInfo.&lt;pid&gt;
      metadata parameters that should be committed by some elements in the
      pipeline.
   */
  enum TAspectRatio
  {
    ASPECT_RATIO_UNKNOWN,
    ASPECT_RATIO_4_3,
    ASPECT_RATIO_16_9
  };

  /** Enumeration of different types of active format descriptors.
   */
  enum TActiveFormatDescriptor
  {
    ACTIVE_FORMAT_UNKNOWN,
    ACTIVE_FORMAT_16_9_TOP,
    ACTIVE_FORMAT_14_9_TOP,
    ACTIVE_FORMAT_MORE_THAN_16_9_CENTRE,
    ACTIVE_FORMAT_SAME_AS_CODED,
    ACTIVE_FORMAT_4_3_CENTRE,
    ACTIVE_FORMAT_16_9_CENTRE,
    ACTIVE_FORMAT_14_9_CENTRE,
    ACTIVE_FORMAT_4_3_SHOOT_AND_PROTECT_14_9_CENTRE,
    ACTIVE_FORMAT_16_9_SHOOT_AND_PROTECT_14_9_CENTRE,
    ACTIVE_FORMAT_16_9_SHOOT_AND_PROTECT_4_3_CENTRE
  };

  /** Enumeration of different types of subtitles. The subtitle type is part of
      the _SubtitleStreamInfo.&lt;pid&gt; metadata parameter that should be
      committed by some element in the pipeline.
   */
  enum TSubtitleType
  {
    SUBTITLE_UNKNOWN,
    SUBTITLE_NORMAL,
    SUBTITLE_HEARING_IMPAIRED
  };

  /** Enumeration of different formats of subtitles. The subtitle format is
      part of the _SubtitleStreamInfo.&lt;pid&gt; metadata parameter that
      should be committed by some element in the pipeline.
   */
  enum TSubtitleFormat
  {
    SUBTITLE_DVB,
    SUBTITLE_ARIB,
    SUBTITLE_TELETEXT,
    SUBTITLE_DVD,
    SUBTITLE_XSUB,
    SUBTITLE_SRT
  };

  enum TReceiveDmfMode
  {
    RECEIVE_DMF_MODE_AUTO,
    RECEIVE_DMF_MODE_NON_AUTO,
    RECEIVE_DMF_MODE_SELECT,
    RECEIVE_DMF_MODE_SPECIAL
  };

  enum TRecordDmfMode {
    RECORD_DMF_MODE_AUTO,
    RECORD_DMF_MODE_NON_AUTO,
    RECORD_DMF_MODE_SELECT
  };

  struct TVideoStreamInfo
  {
    std::string Encoding;
    std::string StillPicture;
    int32_t ComponentTag;
  };

  enum TAudioChannelFormat
  {
    AUDIO_CHANNEL_FORMAT_UNKNOWN,
    AUDIO_CHANNEL_FORMAT_SINGLE_MONO,
    AUDIO_CHANNEL_FORMAT_DUAL_MONO,
    AUDIO_CHANNEL_FORMAT_STEREO,
    AUDIO_CHANNEL_FORMAT_3_FRONT_1_BACK,
    AUDIO_CHANNEL_FORMAT_3_FRONT_2_BACK,
    AUDIO_CHANNEL_FORMAT_3_FRONT_2_BACK_LFE,
    AUDIO_CHANNEL_FORMAT_3_FRONT_2_SIDE_2_BACK_LFE
  };

  enum TAudioChannelMode
  {
    AUDIO_CHANNEL_MODE_NORMAL,
    AUDIO_CHANNEL_MODE_DUAL_MONO_LEFT_ONLY,
    AUDIO_CHANNEL_MODE_DUAL_MONO_RIGHT_ONLY,
    AUDIO_CHANNEL_MODE_DUAL_MONO_STEREO,
    AUDIO_CHANNEL_MODE_DUAL_MONO_MIXED
  };

  enum TAudioAudienceType
  {
    AUDIO_AUDIENCE_TYPE_UNKNOWN,
    AUDIO_AUDIENCE_TYPE_NORMAL,
    AUDIO_AUDIENCE_TYPE_VISUALLY_IMPAIRED,
    AUDIO_AUDIENCE_TYPE_HEARING_IMPAIRED
  };

  struct TAudioStreamInfo
  {
    std::string Encoding;
    int32_t SampleRate;
    int32_t BitsPerSample;
    bool BigEndian;
    TAudioChannelFormat ChannelFormat;
    TAudioChannelMode ChannelMode;
    std::string Language;
    std::string SubLanguage;
    TAudioAudienceType AudienceType;
    int32_t ComponentTag;
    bool IsSupported;
  };

  struct TSubtitleStreamInfo
  {
    std::string Language;
    std::string Compression;
    TSubtitleType Type;
    TSubtitleFormat Format;
    TAspectRatio AspectRatio;
    TReceiveDmfMode ReceiveDmfMode;
    TRecordDmfMode RecordDmfMode;
    int32_t PageId;
    int32_t AncillaryPageId;
    int32_t Pid;
    uint16_t Width;
    uint16_t Height;
  };

  struct TTeletextStreamInfo
  {
    std::string Language;
    int32_t Pid;
  };

  struct TDecoderVideoStreamInfo
  {
    int32_t ResolutionX;
    int32_t ResolutionY;
    TAspectRatio AspectRatio;
    TActiveFormatDescriptor ActiveFormatDescriptor;
    int32_t FrameRate;
  };

  struct TFrontendInfo
  {
    bool Locked;
    int32_t SignalNoiseRatio;
    int32_t BitErrorRate;
    int32_t UncorrectedBlocks;
    int32_t RFLevel;
    int32_t Quality;
  };

  /** Enumeration of different DRM errors. These error codes are used for the
      _DrmError parameter comitted by decryption elements.
   */
  enum TDrmError
  {
    DRM_ERROR_DECRYPTION_FAILED,   // Indicates that the Conditional Access
                                   // scrambling could not decrypt the stream.
    DRM_ERROR_NO_DECRYPTION_KEY,   // Decryption key not available.
    DRM_ERROR_CERTIFICATE_STORE    // Certificate store error.
  };
};

#endif
