//
// --------------------------------------------------------------------
//
// Some definitions of parameter names in the Blackboard.
//
// Copyright (c) 2003, 2010 Motorola, Inc. All rights reserved.
// Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
//
// This program is confidential and proprietary to Motorola Mobility, Inc and
// may not be copied, reproduced, disclosed to others, published or used, in
// whole or in part, without the expressed prior written permission of Motorola
// Mobility, Inc.
//
// --------------------------------------------------------------------
//

// IMPORTANT!
// This file is parsed by a ./extractParameters.py to create documentation

#ifndef PARAMTERNAMES_H
#define PARAMTERNAMES_H

/*! \file ParameterNames.h
    \brief Some definitions of parameter names in the Blackboard.
    
    Parameters that can be used to configure the streamer.
*/

/**
 * Prefix of parameters that shouldn't be altered directly from outside.
 */
#define CORE_ONLY_PREFIX "_"


// ----------------------------------------------------------------------------
// Parameters (used by TStreamer, elements etc).

/**
 * Position discontinuity. Increments when the position in
 * the stream is manually changed (e.g. by calls to PlayFromPosition).
 * It can be monitored by other services (e.g. subtitles, ca-systems)
 * that needs to know when a jump has been done.
 * The absolute value is not of interest, only changes.
 *
 * Values:    0 - 255
 * Set by:    Source elements.
 */
#define PARAM_POSITION_DISCONTINUITY "_PositionDiscontinuity"

/**
 * The current value of the STC associated with the stream.
 *
 * Values:  A positive integer (33 bit 90kHz counter).
 * Set by:    The program selector element.
 */
#define PARAM_STREAM_STC "_StreamStc"

/**
 * Tells the element which checks for discontinuities to ignore
 * errors. Used, for example, for "false" trick play.
 *
 * Values:   true, false
 * Set by:   PVR Source (and perhaps other source elements)
 */
#define PARAM_IGNORE_DISCONTINUITY "_IgnoreDiscontinuity"

/** 
 * Tells the element that there is discontinuity in data
 * 
 * Values:   true, false
 * Set by:   DLNA Source element(and perhaps other source element)
 */
#define PARAM_INJECT_DISCONTINUITY "_InjectDiscontinuity"

/** 
 * Tells the element that a new frame has been decoded and ready to display
 * 
 * Values:   true, false
 * Set by:   Media Decoder Controller element(and perhaps other element)
 */
#define PARAM_NEW_PICTURE_TO_BE_DISPLAYED "_NewPictureToBeDisplayed"

/**
 * Enables playback of only IFrames in HAL. Used for trick play.
 *
 * Values:   true, false, first_i
 * Set by:   PVR Source (and perhaps other source elements)
 */
#define PARAM_IFRAME_ONLY "_IFrameOnly"

/**
 * The URL to the stream.
 *
 * Values:    URL
 * Set by:    Source element.
 */
#define PARAM_STREAM_URL "_StreamUrl"

/**
 * The title of the stream.
 *
 * Values:    Title
 * Set by:    Asf, Shoutcast & Bitband elements.
 */
#define PARAM_STREAM_TITLE "_StreamTitle"

/**
 * The connection type to the stream.
 *
 * Values:    RTSP, HTTP, Multicast, ...
 * Set by:    Source element.
 */
#define PARAM_CONNECTION_TYPE "_ConnectionType"

/**
 * The stream format.
 *
 * Values:    MPEG1/System, MPEG2/Program, MPEG2/Transport, ...
 * Set by:    Framing element.
 */
#define PARAM_STREAM_FORMAT "_StreamFormat"

/**
 * The play time of the stream in milliseconds.
 *
 * Values:    0 - XXX ms
 * Set by:    Source element.
 */
#define PARAM_STREAM_PLAY_TIME "_StreamPlayTime"

/**
 * The valid paces in which the position can be updated during this stream.
 *
 * Values:    pace list seperated by comma
 * Set by: RTSP source element
 */
#define PARAM_STREAM_PACES "_StreamPaces"

/**
 * List of pids of video streams (in the current program).
 *
 * Values:    XML data (see TMetaStreamList)
 * Set by:    Video stream info decoder element.
 */
#define PARAM_VIDEO_STREAM_LIST "_VideoStreamList"

/**
 * List of pids of audio streams (in the current program).
 *
 * Values:    XML data (see TMetaStreamList)
 * Set by:    Audio stream info decoder element.
 */
#define PARAM_AUDIO_STREAM_LIST "_AudioStreamList"

/**
 * List of pids of subtitle streams (in the current program).
 *
 * Values:    XML data (see TMetaStreamList)
 * Set by:    Subtitle stream info decoder element.
 */
#define PARAM_SUBTITLE_STREAM_LIST "_SubtitleStreamList"

/**
 * List of pids of teletext streams (in the current program).
 *
 * Values:    XML data (see TMetaStreamList)
 * Set by:    Subtitle stream info decoder element.
 */
#define PARAM_TELETEXT_STREAM_LIST "_TeletextStreamList"

/**
 * Information about a video stream. The pid of the stream should be
 * added at the end of the parameter name.
 *
 * Values:    XML data (see TMetaVideoStreamInfo)
 * Set by:    Video stream info decoder element.
 */
#define PARAM_VIDEO_STREAM_INFO "_VideoStreamInfo."   // add pid

/**
 * Information about an audio stream. The pid of the stream should be
 * added at the end of the parameter name.
 *
 * Values:    XML data (see TMetaAudioStreamInfo)
 * Set by:    Audio stream info decoder element.
 */
#define PARAM_AUDIO_STREAM_INFO "_AudioStreamInfo."   // add pid

/**
 * Information about a subtitle stream. The pid of the stream should be
 * added at the end of the parameter name.
 *
 * Values:    sti
 * Set by:    Subtitle stream info decoder element.
 */
#define PARAM_SUBTITLE_STREAM_INFO "_SubtitleStreamInfo."// add pid

/**
 * Information about a teletext stream. The pid of the stream should be
 * added at the end of the parameter name.
 *
 * Values:    XML data (see TMetaTeletextStreamInfo)
 * Set by:    Subtitle stream info decoder element.
 */
#define PARAM_TELETEXT_STREAM_INFO "_TeletextStreamInfo."// add pid

/**
 * Socket name used by subtitle render to receive subtitles data.
 *
 * Values:    string
 * Set by:    Subtitle render
 * Read by:   Subtitle sink element
 */
#define PARAM_SUBTITLE_SOCKET_NAME "SubtitleSocketName"

/**
 * Decoder specific Information an audio stream.
 * The pid of the stream should be added at the end of the parameter name.
 *
 * Values:    see TMetaAudioDecoderInfo
 * Set by:    AudioController
 * Read by:   TS Audio Stream Info Decoder HW
 */
#define PARAM_AUDIO_DECODER_INFO "_AudioDecoderInfo"

/**
 * Additional information about a video stream. The pid of the stream
 * should be added at the end of the parameter name.
 *
 * Values:    XML data (see TMetaDecoderVideoStreamInfo)
 * Set by:    Video sink element or Video Source element(Avi element).
 */
#define PARAM_DECODER_VIDEO_STREAM_INFO "_DecoderVideoStreamInfo."

/**
 * The active video stream resolution is stored in this parameter. 
 * If unknown the parameter is empty. If known the parameter value
 * is specified as "w=\<width\>,h=\<height\>" where \<width\> and \<height\>
 * are unsigned integer values.
 *
 * Values:    XML data
 * Set by:    Video sink element or video controller element.
 */
#define PARAM_ACTIVE_VIDEO_STREAM_RESOLUTION "_ActiveVideoStreamResolution"

/**
 * The pid of the currently active video stream.
 *
 * Values:    0 - XXX
 * Set by:    The video selector element.
 */
#define PARAM_ACTIVE_VIDEO_STREAM "_ActiveVideoStream"

/**
 * The pid of the currently active audio stream.
 *
 * Values:    0 - XXX
 * Set by:    The audio selector element.
 */
#define PARAM_ACTIVE_AUDIO_STREAM "_ActiveAudioStream"

/**
 * The pid of the currently active subtitle stream.
 *
 * Values:    0 - XXX
 * Set by:    The subtitle selector element.
 */
#define PARAM_ACTIVE_SUBTITLE_STREAM "_ActiveSubtitleStream"

/**
 * The pid of the currently active teletext stream.
 *
 * Values:    0 - XXX
 * Set by:    The teletext vbi element.
 */
#define PARAM_ACTIVE_TELETEXT_STREAM "_ActiveTeletextStream"

/**
 * This parameter is used by the Session to request an active video stream 
 * the from video sink element.
 *
 * Values:    0 - XXX
 * Set by:    TStreamer
 */
#define PARAM_REQUEST_VIDEO_STREAM "_RequestVideoStream"

/**
 * This parameter is used by the Session to request an active audio stream 
 * from the audio sink element.
 *
 * Values:    0 - XXX
 * Set by:    TStreamer
 */
#define PARAM_REQUEST_AUDIO_STREAM "_RequestAudioStream"

/**
 * This parameter is used to request an active audio stream mode from audio
 * sink element.
 *
 * Values:    0 - XXX (see IStreamerType::TAudioChannelMode)
 * Set by:    TStreamer
 */
#define PARAM_REQUEST_AUDIO_CHANNEL_MODE "_RequestAudioChannelMode"

/**
 * This parameter is used by the Session to request an active subtitle stream 
 * from the subtitle sink element.
 *
 * Values:    0 - XXX
 * Set by:    TStreamer
 */
#define PARAM_REQUEST_SUBTITLE_STREAM "_RequestSubtitleStream"

/**
 * This parameter is used by the Session to request an active teletext stream 
 * from the teletext sink element.
 *
 * Values:    0 - XXX
 * Set by:    TStreamer
 */
#define PARAM_REQUEST_TELETEXT_STREAM "_RequestTeletextStream"

/**
 * The program number of the currently active program.
 *
 * Values:    0 - XXX
 * Set by:    The program selector element.
 */
#define PARAM_ACTIVE_PROGRAM "_ActiveProgram"

/**
 * This parameter holds the program map table and pid used for transport
 * streams.
 *
 * Values:    Binary encoded to base64 format by TMetaProgramSpecificInfo.
 * Set by:    The dvr source and program extractor elements.
 */
#define PARAM_INJECT_PMT "_InjectPmt"


/**
 * This parameter holds the program map table and pid used for transport
 * streams.
 *
 * Values:    Binary encoded to base64 format by TMetaProgramSpecificInfo.
 * Set by:    The dvr source and program extractor elements.
 */
#define PARAM_INJECT_PAT "_InjectPat"

/**
 * This parameter holds the ECM and pid.
 *
 * Values:    Binary encoded to base64 format by TMetaProgramSpecificInfo.
 * Set by:    The program extractor element and the dvr source element.
 */
#define PARAM_INJECT_ECM "_InjectEcm"

/**
 * This parameter is used to enable PMT monitoring in the
 * program selector element to be able to suspend a program
 * if the PMT disappears. Must be done according to ARIB specification
 * TR-B15.
 *
 * Values:    true, false
 * Set by:    The isdb source element
 */
#define PARAM_MONITOR_PMT "_MonitorPmt"

/**
 * Indicates whether the PMT has been found or not.
 *
 * Values:    true, false
 * Set by:    The program selector element.
 */
#define PARAM_PMT_FOUND "_PmtFound"

/**
 * This parameter is used by the Session to request an active program 
 * from the program selector element.
 *
 * Values:    0 - XXX
 * Set by:    TStreamer
 */
#define PARAM_REQUEST_PROGRAM "_RequestProgram"

/**
 * This parameter data is created by the TStreamer.
 *
 * Values:    Xmlbased relay method
 * Set by:    TTsDataRelayElement
 */
#define PARAM_DATA_RELAY_METHOD "_DataRelayMethod"

/**
 * This parameter is used to indicate end of stream to the source
 * element if it cannot realize it itself.
 *
 * Values:    end, beginning
 * Set by:    TTsDsmCcElement, Streamer Core
 */
#define PARAM_END_OF_STREAM "_EndOfStream"

/**
 * This parameter shows how many times undesired prebuffering has been
 * performed when processing a stream. This happens when the buffer
 * becomes empty and can be used to indicate streamer
 * disturbances visible to the user.
 *
 * Values:    0 - XXX
 * Set by:    Streamer Core
 */
#define PARAM_BUFFER_UNDERRUN_ERROR_COUNT "_BufferUnderrunErrorCount"

/**
 * This parameter shows how many times undesired prebuffering has been
 * performed when processing a stream. This happens when the buffer
 * becomes full and can be used to indicate streamer
 * disturbances visible to the user.
 *
 * Values:    0 - XXX
 * Set by:    Streamer Core
 */
#define PARAM_BUFFER_OVERRUN_ERROR_COUNT "_BufferOverrunErrorCount"

/**
 * This parameter shows how many times discontinuities has been
 * detected when processing a stream.  This can be used to indicate
 * streamer disturbances visible to the user.
 *
 * Values:    0 - XXX
 * Set by:    TTsContinuityElement
 */
#define PARAM_DISCONTINUITY_COUNT "_DiscontinuityCount"

/**
 * Priority list for automatic audio language selection
 *
 * Values:    comma separated list of language codes
 * Set by:    MediaPlayer
 */
#define PARAM_AUDIO_LANGUAGE_PRIO_LIST "AudioLanguagePriorityList"

/**
 * Priority list for automatic audio mode selection
 *
 * Values:    comma separated list of audio mode
 * Set by:    MediaPlayer
 */
#define PARAM_AUDIO_TYPE_PRIO_LIST "AudioTypePriorityList"

/**
 * Priority list for automatic audio encoding selection
 *
 * Values:    comma separated list of encodings
 * Set by:    MediaPlayer
 */
#define PARAM_AUDIO_ENCODING_PRIO_LIST "AudioEncodingPriorityList"

/**
 * Used for enable or disable automatic audio audio language selection
 *
 * Values:    Boolean ("true"/"false")
 * Set by:    MediaPlayer (reading from infobroker)
 */
#define PARAM_AUDIO_LANGUAGE_AUTO_SELECT "AudioLanguageAutomaticSelection"

/**
 * Preferred audio language selection which overrides
 * PARAM_AUDIO_ENCODING_PRIO_LIST
 *
 * Values:    language code
 * Set by:    Audio controller element
 */
#define PARAM_PREFERRED_AUDIO_LANGUAGE "PreferredAudioLanguage"

/**
 * Preferred audio language descriptor which overrides
 * default behavior of using MPEG-2 ISO-639 Language Descriptor.
 *
 * Values:    AC3/MPEG
 * Set by:    Streamer Core
 */
#define PARAM_PREFERRED_AUDIO_LANG_DESCRIPTOR "PreferredAudioLangDescriptor"

/**
 * Preferred audio encoding selection which overrides
 * PARAM_AUDIO_ENCODING_PRIO_LIST
 *
 * Values:    encoding
 * Set by:    Audio controller element
 */
#define PARAM_PREFERRED_AUDIO_ENCODING "PreferredAudioEncoding"

/**
 * Sound mode
 *
 * Values:    currently "SPDIF" or != "SPDIF"
 * Set by:    MediaPlayer
 */
#define PARAM_SOUND_MODE "SoundMode"

/**
 * Priority list for automatic subtitle language selection
 *
 * Values:    comma separated list of language codes
 * Set by:    MediaPlayer (reading from infobroker)
 */
#define PARAM_SUBTITLE_LANGUAGE_PRIO_LIST "SubtitleLanguagePriorityList"

/**
 * Used for enable or disable automatic subtitle selection
 *
 * Values:    Boolean ("true"/"false")
 * Set by:    MediaPlayer (reading from infobroker)
 */
#define PARAM_SUBTITLE_AUTO_SELECT "SubtitleAutomaticSelection"

/**
 * Priority list for automatic subtitle type selection
 *  
 * Values:    comma separated list of subtitle types (normal, hearing_impaired)
 * Set by:    MediaPlayer (reading from infobroker)
 */
#define PARAM_SUBTITLE_TYPE_PRIO_LIST "SubtitleTypePriorityList"

/**
 * Priority list for automatic subtitle mode selection
 *
 * Values:    comma separated list of subtitle modes (DVB, Teletext)
 *
 * Set by:    MediaPlayer (reading from infobroker)
 */
#define PARAM_SUBTITLE_MODE_PRIO_LIST "SubtitleModePriorityList"

/**
 * This parameter holds the prebuffer time.
 *
 * Values:    A positive integer in milliseconds.
 * Set by:    Streamer Core
 *            (The value can be adjusted from application
 *            by setting the "buftime" parameter)
 */
#define PARAM_PREBUFFER_TIME "_PrebufferTime"

/**
 * This is the playback mode of the stream. It can either be playing,
 * rewinding or fastforwarding. This parameter should only be read by
 * applications and it should be removed when a better interface is
 * in place.
 *
 *
 * Values:    0 - XXX
 * Set by:    Streamer core.
 */
#define PARAM_PLAYBACK_MODE "_PlaybackMode"

/**
 * DEPRECATED
 *
 * This parameter holds the current program map table used
 * for transport streams.
 *
 * Values:    Binary is encoded to Base64 format
 * Set by:    TTsProgramSelectorElement
 */
#define PARAM_PMT_BASE64_DATA "PMT_BASE64"

// ----------------------------------------------------------------------------
// Element parameters (used by elements only)


/**
 * The name of the server when known (e.g. "Kasenna XMP" or "nCube")
 *
 * Values:    Server name
 * Set by:    Source element.
 */
#define PARAM_SOURCE_SERVER "_SourceServer"

/**
 * Indicates that the source is clocked.
 *
 * Values:    true, false
 * Set by:    Streamer Core
 */
#define PARAM_SOURCE_CLOCKED "_SourceClocked"

/**
 * Indicates if closed captioning is enabled or not.
 *
 * Values:    true, false
 * Set by:    MediaPlayer
 */
#define PARAM_CLOSED_CAPTIONING "ClosedCaptioning"

/**
 * Indicates if Macrovision copy protection is enabled or not.
 *
 * Values:    true, false
 * Set by:    MediaPlayer
 */
#define PARAM_MACROVISION "Macrovision"

/**
 * Indicates if CGMSA copy protection is enabled or not.
 *
 * Values:    true, false
 * Set by:    MediaPlayer
 */
#define PARAM_CGMSA "Cgmsa"

/**
 * The value of analog copy protection level
 *
 * Values:  Integer
 * Set by:  LatensDaemon
 */
#define PARAM_ANALOG_COPY_PROTECTION "AnalogCopyProtection"

/**
 * Indicates if APS copy protection for encoding is enabled or not.
 * 
 * Values:  String
 * Set by:    Analog Source Element
 */
#define PARAM_ENCODING_APS "EncodingAps"

/**
 * Indicates if DivX DRM authorization is enabled or not.
 *
 * Values:  true, false
 * Set by:  Avi File Source, PS-HW Elements
 */
#define PARAM_DRM_AUTHORIZED_FLAG "DrmAuthorizedFlag"

/**
 * Indicates if DivX DRM rental is enabled or not.
 *
 * Values:  true, false
 * Set by:  Avi File Source Element
 */
#define PARAM_DRM_RENTAL_FLAG "DrmRentalFlag"

/**
 * The number of playbacks allowed for the DivX file.
 *
 * Values:  Integer
 * Set by:  Avi File Source Element
 */
#define PARAM_DRM_USE_LIMIT "DrmUseLimit"

/**
 * The number of time the DivX file has been played back.
 *
 * Values:  Integer
 * Set by:  Avi File Source Element
 */
#define PARAM_DRM_USE_COUNT "DrmUseCount"

/**
 * Indicates if DivX DRM file is played back or not.
 *
 * Values:  true, false
 * Set by:  Avi File Source Element
 */
#define PARAM_DIVX_DRM_FILE "DivxDrmFile"

/**
 * Indicates the most recently found CCI marker token in dvr playback. Cci
 * marker tokens indicate when to set output control.
 *
 * Values:  String
 * Set by:    Source Element
 */
#define PARAM_CURRENT_CCI_MARKER_TOKEN "CurrentCciMarkerToken"

/**
 * Indicates the contentId of the currently played dvr asset. This is used
 * to verify IPRM licenses
 *
 * Values:  String
 * Set by:    Source Element
 */
#define PARAM_CURRENT_CONTENT_ID "_CurrentContentId"

/**
 * Indicates if component output should be enabled or disabled.
 *
 * Values:    true, false
 * Set by:    MediaPlayer
 */
#define PARAM_COMPONENT_OUTPUT "ComponentOutput"

/**
 * Controls if video should be blacked out
 * or freezed when paused.
 *
 * Values:   true, false
 * Default:  false
 * Set by:   Application
 */
#define PARAM_PAUSE_BLACK "PauseBlack"

/**
 * Controls if video should be blacked out or freezed when closed.
 *
 * Values:   true, false
 * Default:  true
 * Set by:   Application
 */
#define PARAM_CLOSE_BLACK "CloseBlack"

/**
 * Indicates decoding play speed.
 *
 * Values:   0 - XXX
 * Set by:   Source elements.
 */
#define PARAM_PLAY_SPEED "_PlaySpeed"

/**
 * Used by sourece elements to initiate the Decoders flush.
 * mainly intended for use during state transsitions
 *
 * Values: String.
 * Set by: Source elements.
 */
#define PARAM_FLUSH_DECODER "_FlushDecoder"

/**
 * Force the streamer to send an OnEmpty message. Instead of 
 * waiting for hal to notice that the end is reached.
 *
 * Values:   true, false
 * Set by:   PVR source element (playing MPEG4 content)
 */
#define PARAM_FORCE_ON_STREAM_EMPTY "_ForceEmptyNotification"

/**
 * Contains the currently used proxy address by the system.
 *
 * Values:   IP address
 */
#define PARAM_PROXY_LIST "ProxyList"

/**
 * Contains track info extracted from galaxie audio
 *
 * Values:   Track info
 * Set by:   galaxie element
 */
#define PARAM_GALAXIE_TRACK_DATA "_GalaxieAudioTrackData"

/**
 * Contains the IGMP setup time in milliseconds.
 *
 * Values:    0 - XXX ms
 * Set by:    Multicast Source element.
 */
#define PARAM_IGMP_SETUP_TIME "_IgmpSetupTime"

/**
 * This parameter contains the file descriptor of the receive socket.
 *
 * Values:    0 - XXX.
 * Set by:    Source Element
 */
#define PARAM_RECEIVE_SOCKET "_FdReceiveSocket"

/**
 * This parameter contains the decoder type for the sink element.
 *
 * Values:    A string.
 * Set by:    Source Element
 */
#define PARAM_VIDEO_DECODER_TYPE "_VideoDecoderType"

/**
 * This parameter contains the video bitrate.
 *
 * Values:    A string.
 * Set by:    Application
 */
#define PARAM_VIDEO_BIT_RATE "VideoBitRate"

/**
 * This parameter contains the video width.
 *
 * Values:    A string.
 * Set by:    Application
 */
#define PARAM_VIDEO_RESOLUTION_X "VideoResolutionX"

/**
 * This parameter contains the video height.
 *
 * Values:    A string.
 * Set by:    Application
 */
#define PARAM_VIDEO_RESOLUTION_Y "VideoResolutionY"

/**
 * This parameter contains the video frame rate.
 *
 * Values:    A string.
 * Set by:    Application
 */
#define PARAM_VIDEO_FRAME_RATE "VideoFrameRate"

/**
 * This parameter contains the audio codec type.
 *
 * Values:    A string.
 * Set by:    Application
 */
#define PARAM_AUDIO_CODEC_TYPE "AudioCodecType"

/**
 * This parameter contains the audio sampling frequency.
 *
 * Values:    A string.
 * Set by:    Application
 */
#define PARAM_AUDIO_SAMPLING_FREQ "AudioSamplingFreq"

/**
 * This parameter contains Recording type.
 * Recording type may be AUDIO_ONLY or VIDEO_ONLY or AUDIO_VIDEO
 *
 * Values:    A string.
 * Set by:    Application
 */
#define PARAM_RECORDING_TYPE "RecordingType"

/**
 * This parameter is used to indicate end of stream/sequence thru
 * the streamer chain to the halserver that it should decode available
 * payload without waiting for any more.
 *
 * Values:    String  ("end")
 * Set by:    Source elements that have a finite end
 * or interruptions in the stream.
 */
#define PARAM_END_OF_SEQUENCE "_EndOfSequence"

/**
 * Indicates if Macrovision for encoding is enabled or not.
 *
 * Values:    Integer
 * Set by:    Analog Source Element
 */
#define PARAM_ENCODING_MACROVISION "EncodingMacrovision"

/**
 * Indicates if copy protection for encoding is enabled or not.
 *
 * Values:    Integer
 * Set by:    Analog Source Element
 */
#define PARAM_ENCODING_CGMSA "EncodingCgmsa"

/**
 * The video decoder saturation time.
 *
 * Values:    0 - XXX.
 * Set by:    Source element.
 */
#define PARAM_DECODER_SATURATION_TIME "HalBufferSaturationTime"

/**
 * The Transcoder input parameters.
 * PARAM_INPUT_STREAM_TYPE
 * PARAM_INPUT_VIDEO_PID
 * PARAM_INPUT_AUDIO_PID
 * PARAM_INPUT_PCR_PID
 * PARAM_INPUT_VIDEO_FORMAT
 * PARAM_INPUT_AUDIO_FORMAT
 *
 * Set by:    DLNA service.
 */
#define PARAM_TRANSCODER_CONFIG "TranscoderConfig"
#define PARAM_INPUT_STREAM_TYPE "InputStreamType"
#define PARAM_INPUT_VIDEO_PID "InputVideoPid"
#define PARAM_INPUT_AUDIO_PID "InputAudioPid"
#define PARAM_INPUT_PCR_PID "InputPcrPid"
#define PARAM_INPUT_VIDEO_FORMAT "InputVideoFormat"
#define PARAM_INPUT_AUDIO_FORMAT "InputAudioFormat"

/**
 * The Transcoder output parameters.
 * PARAM_OUTPUT_STREAM_TYPE
 * PARAM_OUTPUT_VIDEO_PID
 * PARAM_OUTPUT_AUDIO_PID
 * PARAM_OUTPUT_PCR_PID
 * PARAM_OUTPUT_VIDEO_FORMAT
 * PARAM_OUTPUT_AUDIO_FORMAT
 *
 * Set by:    DLNA service.
 */
#define PARAM_OUTPUT_STREAM_TYPE "OutputStreamType"
#define PARAM_OUTPUT_VIDEO_PID "OutputVideoPid"
#define PARAM_OUTPUT_AUDIO_PID "OutputAudioPid"
#define PARAM_OUTPUT_PCR_PID "OutputPcrPid"
#define PARAM_OUTPUT_VIDEO_FORMAT "OutputVideoFormat"
#define PARAM_OUTPUT_AUDIO_FORMAT "OutputAudioFormat"

/**
 * The AVI Parameter among File Source, AVI and Payload Sink Elements.
 *
 * Values:    A string.
 * Set by:    AVI Element.
 */
#define PARAM_AVI_CURRENT_IFRAME                   "_AviCurrentIFrame"
#define PARAM_AVI_AUDIO_PRELOAD_TIME               "_AviAudioPreloadTime."
#define PARAM_AVI_VIDEO_FRAME_TIME                 "_AviVideoFrameTime"
#define PARAM_AVI_SUBTITLE_TYPE_CODE               "_AviSubtitleTypeCode"
#define PARAM_AVI_SUBTITLE_XSUB_PLUS               "_AviSubtitleXSubPlus"
#define PARAM_AVI_IFRAME_INDEX_TABLE               "_AviIFrameIndexTable"
#define PARAM_AVI_AUDIO_DW_SCALE                   "_AviAudioDwScale."
#define PARAM_AVI_AUDIO_DW_RATE                    "_AviAudioDwRate."
#define PARAM_AVI_AUDIO_BLOCK_ALIGN                "_AviAudioBlockAlign."
#define PARAM_AVI_INDEXING_STATUS                  "_AviIndexingStatus"
#define PARAM_AVI_LAST_IFRAME_PTS                  "_AviLastFramePts"

#define PARAM_BLANK_VIDEO "BlankVideo"
#define PARAM_APP_BLANK_VIDEO "AppBlankVideo"

/**
 * The Copyright identifier got from PMT copyright descriptor. 
 *
 * Values:     A string
 * Set by:     The program selector element.
 */
#define PARAM_COPYRIGHT_ID    "_CopyrightId"
/**
 * The Copyright additional info got from PMT copyright descriptor.
 *
 * Values:     Boolean
 * Set by:     The program selector element.
 */
#define PARAM_DRM_COPYRIGHT_ASSERTED "_DrmCopyrightAsserted"
#define PARAM_DRM_COPYING_RESTRICTED "_DrmCopyingRestricted"
#define PARAM_DRM_DVR_RESTRICTED "_DrmDvrRestricted"

/**
* The ES Parameter, used by Open Internet container formats related elements,
* such as Mp4, Matroska, AVI and etc.
* Values:      A string.
* Set by:      MP4/Matroska/AVI Elements.
*/
#define PAYLOADPARAM_PTS                           "_PTS"
#define PAYLOADPARAM_AVC_SPS                       "_SPS"
#define PAYLOADPARAM_AVC_PPS                       "_PPS"
#define PAYLOADPARAM_AVC_NAL_SIZEHEADER_LENGTH     "_NAL_SIZEHEADER_LENGTH"
#define PAYLOADPARAM_ADTS_HEADER                   "_ADTS_HEADER"
#define PAYLOADPARAM_DECODER_EMPTY                 "_DecoderEmpty" 
#define PAYLOADPARAM_TYPE                          "_PayloadType"
#define PAYLOADPARAM_FIRST_SANE                    "_FirstSane"
#define PAYLOADPARAM_CODEC_DATA                    "_CodecData."
#define PAYLOADPARAM_DEFAULT_DURATION              "_DefaultDuration."
#define PAYLOADPARAM_DURATION                      "_PayloadDuration"
#define PAYLOADPARAM_STRIPPED_DATA                 "_PayloadStrippedData."
#define SUBTITLE_CODEC_DATA                        "_SubtitleCodecData."

/**
 * This parameter is used to change how much data (ms) to buffer before 
 * starting to play a non H.264 stream.If the value is set too low there 
 * is a risk that the buffers might be emptied because of network lag 
 * (this will cause the service to once again buffer up data again). If 
 * the value is set too high, on the other hand, the buffer might be filled 
 * too much giving a similar effect. Also the higher the value is set the 
 * longer it takes to start playing a stream, e.g. when switching TV channels.
 * 
 * Values:    0 - 3000 ms
 * Default:   500 ms
 * Set by:    Streamer config, URI
 */
#define PARAM_BUFTIME "buftime"

/**
 * The parameter will configures how the streamer starts a stream.
 * When the parameter is set to adaptive, the STB will show video in 
 * "slow motion" until A/V sync is correct. This can be useful when playing 
 * h264 streams with large A/V diff.
 * When the parameter is set to normal, the STB will display a black image 
 * until audio and video are in sync.
 * NOTE!
 * VIP1500/1700 only support fast or normal mode.
 * VIP1200 always use normal mode.
 *
 * Values:    fast     - Show a picture as fast as posible.
 * Values:    normal   - Don't show anything until A/V sync.
 * Values:    adaptive - Show video in slow motion until A/V sync is correct.
 * Default:   normal
 * Set by:    Streamer config, URI
 */
#define PARAM_STARTMODE "startmode"

/**
 * The errorrecovery parameter determines how the STB will try to 
 * handle corrupt streams, e.g. in the case of packet loss. 
 * high means the STB will try to hide stream errors as much as possible. This may 
 * result in that instead of showing an area on screen with errors, an entire 
 * frame may be repeated (small picture freeze).
 * NOTE!
 * This is not available for VIP1200.
 *
 * Values:    high, medium, low
 * Default:   high
 * Set by:    Streamer config, URI
 */
#define PARAM_ERROR_RECOVERY "errorrecovery"

/**
 * Specifies the size (kB) of the media buffer to use. For very high bitrate streams 
 * the default value may not be enough. If less buffering is required, this can be 
 * lowered to free up memory for other tasks. 
 *
 * Values:    512 - 4096 kB
 * Default:   2048 kB
 * Set by:    Streamer config, URI
 */
#define PARAM_BUFSIZE "bufsize"

/**
 * Channel for Closed Captions.
 * NOTE!
 * Only available for VIP1200. 
 *
 * Values:    1 - 8
 * Default:   1
 * Set by:    Streamer config, URI
 */
#define PARAM_CLOSED_CAPTION_CHANNEL "closedCaptionChannel"

/**
 * Set the Closed Captioning data mode. Valid values are 608 for EIA-608 captions 
 * (VIP1200/VIP1900), 708 for EIA-708 captions (VIP1200) and disable to disable 
 * Closed Captions.
 *
 * Values:    608, 708, disable
 * Default:   disable
 * Set by:    Streamer config, URI
 */
#define PARAM_CLOSED_CAPTION_MODE "closedCaptionMode"

/**
 * When this parameter is true Closed Captions will be rendered on the 
 * video by the STB. When this parameter is false Closed Captions will 
 * be output as VBI data in the video signal. 
 * This parameter only has an effect when Closed Captions are enabled by closedCaptionMode.
 * NOTE!
 * Only available for VIP1200.
 *
 * Values:    true, false
 * Default:   false
 * Set by:    Streamer config, URI
 */
#define PARAM_CLOSED_CAPTION_OVERLAY "closedCaptionOverlay"

/**
 * In some cases it is desirable to restrict the range of playback for 
 * an asset. This parameter determines the start time in seconds into 
 * the asset. While this parameter is in effect, it is not possible to 
 * go to a position before the defined time. The parameter is primarily 
 * intended for use together with the duration parameter.
 * NOTE! Only implemented for RTSP type servers
 *
 * Values:    0 - XXX s
 * Set by:    Streamer config, URI
 */
#define PARAM_STARTTIME  "starttime"

/**
 * This parameter determines if the server should avoid playing the 
 * entire asset, and selects how much of the asset should be played 
 * before reporting end of stream.
 * NOTE! Only implemented for RTSP type servers
 *
 * Values:    0 - XXX s
 * Set by:    Streamer config, URI
 */
#define PARAM_DURATION   "duration"

/**
 * When this parameter is true, data received before the new PAT/PMT when 
 * changing to a new channel is considered valid. On average this gives 
 * slightly faster zap times, but can also result in image/video disturbances. 
 *
 * Set by:    Streamer config, URI
 * Values:    true, false
 * Default:   true
 */
#define PARAM_SMART_START  "SmartStart"

/**
 * This is used to control if the statistics element will precede the CA element.
 *
 * Values:    true, false
 * Default:   false
 * Set by:    Streamer config, URI
 */
#define PARAM_STATISTICS_BEFORE_CA "StatisticsBeforeCa"

/**
 * If this parameter is set together with an igmp:// URI the incoming data is 
 * assumed to be RTP and the RTP header is discarded before the data is processed. 
 * This can be used to increase performance in the case where the RTP format is 
 * used on the network but the RTP functionality is not needed once the data reaches 
 * the STB.
 *
 * Values:     true, false
 * Default:    false
 * Set by:     URI
 */
#define PARAM_DISCARD_RTP_HEADERS "discardRtpHeaders"

/**
 * This parameter is used for communication with a Bitband VOD server. 
 * It represents the client ID in the RTSP setup message. 
 *
 * Values:     A string
 * Set by:     URI or by using SetParameter()
 */
#define PARAM_BITBAND_STBID "STBID"

/**
 * The videodecoder can either be in a ready state or awaiting configuration.
 *
 * Values: ready, stopped
 * Set by: TVideoDecoderControllerElement 
 */
#define PARAM_VIDEO_DECODER_STATE "VideoDecoderState"

/**
 * The audiodecoder can either be in a ready state or awaiting configuration.
 *
 * Values: ready, stopped
 * Set by: TAudioDecoderControllerElement
 */
#define PARAM_AUDIO_DECODER_STATE "AudioDecoderState"

/**
 * When this parameter is set to true, the RTSP play message will be issued
 * with the range "npt=now-". This is only valid when playing a stream from
 * a RTSP server that supports this feature.
 *
 * Values:     true, false
 * Default:    false
 * Set by:     URI
 */
#define PARAM_RTSP_NOW "RtspNow"

/**
 * This parameter is used to inform streamer elements if video controller ready
 * or not.
 *
 * Set by:    Video controller element
 * Values:    true, false
 * Default:   false
 */
#define PARAM_VIDEO_CONTROLLER_READY "_VideoControllerReady"

/**
 * This parameter holds the current program map table used
 * for transport streams.
 *
 * Values: Binary data
 * Set by: TTsProgramSelectorElement
 */
#define PARAM_PMT_DATA "PMT"

/**
 * This parameter holds the source buffer requirement.
 *
 * Values: 0 - XXX bytes
 * Set by: Streamer Core
 *
 */
#define PARAM_BUFFER_REQUIREMENT "_BufferRequirement"

/**
 * This parameter is set to true when the CA system has set the
 * descrambler keys, used by ProgramExtractorElement to control data
 * postponement
 *
 * Values: The CRC32 of the ECM triggering the key change.
 * Set by: The CA system extension.
 */
#define PARAM_DESCRAMBLER_KEYS_SET "DescramblerKeysSet"

/**
 * Used for inform streamer element current player is in PiP mode or not.
 *
 * Values:    Boolean ("true"/"false")
 * Set by:    TVideoControllerElement
 */
#define PARAM_PiP_MODE "PipMode"

/**
 * Used to inform streamer element(s) if the streamer is in atsc mode
 * and hence to conditionally recognize DC-II as a valid video type.
 *
 * Values: atsc (or null)
 * Set by: Streamer Config
 */
#define PARAM_STREAMER_MODE "StreamerMode"

/**
 *  Contains an optional field used to convey session entitlement
 *  to the client of an RTSP OpenStream session that can be used
 *  by conditional access elements (EMM). Entitlements will not exceed
 *  1024 characters and are encoded as hexadecimal strings.
 */
#define PARAM_OPENSTREAM_ENTITLEMENT "_OpenstreamEntitlement"

/*
 * The Analog input signal present state
 * Values: lost, present
 *
 * Set by: TAnalogDvrSourceElement
 */

#define PARAM_ANALOG_INPUT_SIGNAL_STATE "_AnalogInputSignalState"

/**
 * Used to inform the cable card service about the ParserBand number
 * being used for the current session
 */
#define PARAM_DEMUXER_INSTANCE "_DemuxerInstance"

/**
 * Controls if the Program Cache should be used.
 *
 * Values:    true, false
 * Default:   false
 * Set by:    Typically a source element that want cached PMT:s (like dvb)
 */
#define PARAM_USE_PROGRAM_CACHE "UseProgramCache"

/**
 * Contains destination URL for content distribution.
 *
 * Values:   protocol://ip-address
 * Set by:   Streamer core
 */
#define PARAM_DISTRIBUTION_DESTINATION_URL "_DistributorDestinationUrl"

#endif
