/*
 *--------------------------------------------------------------------
 *
 * TMetadata.h
 *
 * Copyright (c) 2010 Motorola Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TMETADATA_H
#define TMETADATA_H

#include <string>
#include <stdint.h>
#include <cstdio>

/** \class TMetadata TMetadata.h "media/streamer/TMetadata.h"

    Metadata is parameters attached to segments in the stream being
    processed by the Streamer.
 */
class TMetadata
{
protected:
  TMetadata* Previous;
  TMetadata* Next;
  std::string Key;
  std::string Value;

  // Not allowed to create metadata except by calling IPump.
  TMetadata() throw ();  

public:
  virtual ~TMetadata() throw ();

  /** This function returns the key, i.e. the name of the parameter.

      \return   The key (name of the parameter).
   */
  virtual const std::string& GetKey() const throw ();

  /** This function returns the value of the parameter.

      \return   The value of the parameter.
   */
  virtual const std::string& GetValue() const throw ();

  /** This function returns the previous metadata in a list.

      \return   Pointer to the previous metadata.
  */
  virtual const TMetadata* GetPrevious() const throw ();

  /** This function returns the next metadata in a list.

      \return   Pointer to the next metadata.
  */
  virtual const TMetadata* GetNext() const throw ();


  /** This function sets the key of the metadata.
      
      \param key    The key of the metadata.
  */
  virtual void SetKey(const std::string& key) throw ();

  /** This function sets a string value.
      
      \param value  The value of the metadata.
  */
  virtual void SetValue(const std::string& value) throw ();

  /** This function sets an unsigned integer value.
      
      \param value  The value of the metadata.
  */
  virtual void SetValue(uint32_t value) throw ();

  /** This function sets a signed integer value.
      
      \param value  The value of the metadata.
  */
  virtual void SetValue(int value) throw ();

  /** This function sets the key and a string value.
      
      \param key    The key of the metadata.
      \param value  The value of the metadata.
  */
  virtual void Set(const std::string& key, const std::string& value) throw ();

  /** This function sets the key and an unsigned integer value.
      
      \param key    The key of the metadata.
      \param value  The value of the metadata.
  */
  virtual void Set(const std::string& key, uint32_t value) throw ();

  /** This function sets the key and a signed integer value.
      
      \param key    The key of the metadata.
      \param value  The value of the metadata.
  */
  virtual void Set(const std::string& key, int value) throw ();


  /** This function checks if the keys are equal of two metadata.
      
      \param other  The other metadata.
  */
  virtual bool AreKeysEqual(const TMetadata& other) const throw ();

  /** This function checks if the key of the metadata is equal to a string.
      
      \param key    The key to check.
  */
  virtual bool AreKeysEqual(const std::string& key) const throw ();
};


// ----------------------------------------------------------------------------
// Inlined functions

inline const std::string& TMetadata::GetKey() const throw ()
{
  return Key;
}

inline const std::string& TMetadata::GetValue() const throw ()
{
  return Value;
}

inline const TMetadata* TMetadata::GetPrevious() const throw ()
{
  return Previous;
}

inline const TMetadata* TMetadata::GetNext() const throw ()
{
  return Next;
}

inline void TMetadata::SetKey(const std::string& key) throw ()
{
  Key = key;
}

inline void TMetadata::SetValue(const std::string& value) throw ()
{
  Value = value;
}

inline void TMetadata::SetValue(uint32_t value) throw ()
{
  char v[16];
  ::sprintf(v, "%u", value);
  SetValue(v);
}

inline void TMetadata::SetValue(int value) throw ()
{
  char v[16];
  ::sprintf(v, "%d", value);
  SetValue(v);
}

inline void TMetadata::Set(const std::string& key,
                           const std::string& value) throw ()
{
  SetKey(key);
  SetValue(value);
}

inline void TMetadata::Set(const std::string& key,
                           uint32_t value) throw ()
{
  SetKey(key);
  SetValue(value);
}

inline void TMetadata::Set(const std::string& key,
                           int value) throw ()
{
  SetKey(key);
  SetValue(value);
}

inline bool TMetadata::AreKeysEqual(const TMetadata& other) const throw ()
{
  return Key == other.Key;
}

inline bool TMetadata::AreKeysEqual(const std::string& key) const throw ()
{
  return Key == key;
}

#endif
