/*
 *--------------------------------------------------------------------
 *
 * TMetadataWrapper.h
 *
 * This class is used by elements to simplify the processing of
 * incoming metadata.
 *
 * Copyright (c) 2004 Motorola Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TMETADATAWRAPPER_H
#define TMETADATAWRAPPER_H

#include "TMetadata.h"
#include "IPump.h"
#include "IOutputPad.h"
#include "IPipelineLink.h"

class TSegment;

class TMetadataWrapper
{
private:
  // These pointers will be changed when metadata is committed or destroyed. 
  const TMetadata* First;
  const TMetadata* Last;

public:
  explicit TMetadataWrapper(const TSegment* segment) throw ();
  ~TMetadataWrapper() throw ();

  // GetValue will return the "latest" value (further to the end of
  // the list) if there are several values. Be aware of this when
  // handling metadata, perhaps you should handle all values!
  bool GetValue(const std::string& key, std::string& value) const throw ();
  bool IsEmpty() const throw ();

  const TMetadata* GetFirst() const throw ();
  // Commit a copy of the first metadata. Don't move pointer.
  void CommitCopyFirst(IOutputPad* pad) throw ();
  // Commit the first metadata and move the pointer to the next metadata.
  void CommitFirst(IOutputPad* pad) throw ();
  void CommitFirst(IPipelineLink* link) throw ();
  // Publish the first metadata and move the pointer to the next metadata.
  void PublishFirst(IPump* pump) throw ();
  // Destroy the first metadata move the pointer to the next metadata.
  void DestroyFirst(IPump* pump) throw ();

  // Commit all the metadata that is left in wrapper.
  void CommitCopyAll(IOutputPad* pad) throw ();
  void CommitAll(IOutputPad* pad) throw ();
  void CommitAll(IPipelineLink* link) throw ();
  void PublishAll(IPump* pump) throw ();
};


// ----------------------------------------------------------------------------
// Inlined functions

inline bool TMetadataWrapper::IsEmpty() const throw ()
{
  return First == NULL;
}

inline const TMetadata* TMetadataWrapper::GetFirst() const throw ()
{
  return First;
}

inline void TMetadataWrapper::CommitCopyFirst(IOutputPad* pad) throw ()
{
  pad->CommitMetadataCopy(First);
}

inline void TMetadataWrapper::CommitFirst(IOutputPad* pad) throw ()
{
  pad->CommitMetadata(First);
  First = First->GetNext();
  if (First == NULL) {
    Last = NULL;
  }
}

inline void TMetadataWrapper::CommitFirst(IPipelineLink* link) throw ()
{
  link->CommitMetadata(First);
  First = First->GetNext();
  if (First == NULL) {
    Last = NULL;
  }
}

inline void TMetadataWrapper::PublishFirst(IPump* pump) throw ()
{
  pump->PublishMetadata(First);
  First = First->GetNext();
  if (First == NULL) {
    Last = NULL;
  }
}

inline void TMetadataWrapper::DestroyFirst(IPump* pump) throw ()
{
  pump->DestroyMetadata(First);
  First = First->GetNext();
  if (First == NULL) {
    Last = NULL;
  }
}

inline void TMetadataWrapper::CommitCopyAll(IOutputPad* pad) throw ()
{
  const TMetadata* m = First;
  while (m != NULL) {
    pad->CommitMetadataCopy(m);
    m = m->GetNext();
  }
}

inline void TMetadataWrapper::CommitAll(IOutputPad* pad) throw ()
{
  while (First != NULL) {
    CommitFirst(pad);
  }
}

inline void TMetadataWrapper::CommitAll(IPipelineLink* link) throw ()
{
  while (First != NULL) {
    CommitFirst(link);
  }
}

inline void TMetadataWrapper::PublishAll(IPump* pump) throw ()
{
  while (First != NULL) {
    PublishFirst(pump);
  }
}

#endif
