/*
 *--------------------------------------------------------------------
 *
 * TSegment.h
 *
 * Copyright (c) 2003 Motorola Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TSEGMENT_H
#define TSEGMENT_H

#include "TMetadata.h"
#include <stdint.h>


/** \class TSegment TSegment.h "media/streamer/TSegment.h"
    
    A segment encapsulates a section of the stream buffer. The segment
    may be divided into several equally large frames with a focus
    indicating the relevant part of each frame.
 */
class TSegment
{
protected:
  TSegment* Next;       // The next segment in the list.
  TSegment* Previous;   // The previous segment in the list.

  uint8_t* Position;    // The position of the segment.
  uint32_t FrameCount;  // The number of frames in the segment.
  uint32_t FrameSize;   // The length of each interval.
  uint32_t FocusOffset; // The offset of the data in each frame.
  uint32_t FocusSize;   // The length of each data element in each frame.

  TMetadata* FirstMetadata;
  TMetadata* LastMetadata;

  // Not allowed to create segments in elements.
  TSegment() throw ();

public:
  /** This function returns a pointer to the encapsulated buffer
      position.

      \return    The buffer position of the segment.
   */
  uint8_t* GetPosition() const throw ();

  /** This function returns the number of frames in the segment.

      \return    The number of frames.
  */
  uint32_t GetFrameCount() const throw ();

  /** This function returns the size of each frame in the segment.

      \retval    Returns the size of each frame.
  */
  uint32_t GetFrameSize() const throw ();

  /** This function returns the offset of the focus in each frame.

      \return    The offset of the focus.
  */
  uint32_t GetFocusOffset() const throw ();

  /** This function returns the size of the focus slice in each frame.

      \return    The size of the focus.
   */
  uint32_t GetFocusSize() const throw ();

  /** This function returns the size of the entire segment.

      \return    The size of the segment.
  */
  uint32_t GetSize() const throw ();

  /** This function returns the total size of the data in focus in all frames.

      \return    The size of the data in focus.
   */
  uint32_t GetDataSize() const throw ();


  /** This function is used to check if the segment is empty (has size 0).

      \return    True if the segment is empty.
  */
  bool IsEmpty() const throw ();

  
  /** This function returns a pointer to the buffer position of the
      focus of a single frame.

      \param frameNumber   The index of the frame.
      \return              A pointer to the focus of the frame.
   */
  uint8_t* GetFocus(uint32_t frameNumber) const throw ();

  /** This function copies all the data in focus into the buffer.

      \param buf           The buffer to copy the data to.
      \param size          Size of the buffer.
      \return              The number of bytes copied.
  */
  uint32_t CopyData(uint8_t* buf, uint32_t size) const throw ();


  /** This function returns the previous segment in a segment list or
      NULL if this is the first segment.

      \return     A pointer to the previous segment.
   */
  const TSegment* GetPrevious() const throw ();

  /** This function returns the next segment in a segment list or NULL
      if this is the last segment.

      \return     A pointer to the next segment.
   */
  const TSegment* GetNext() const throw ();

  /** This function returns the first metadata attached to the segment
      or NULL if no metadata is attached.

      \return     A pointer to the first metadata attached.
   */

  const TMetadata* GetFirstMetadata() const throw ();

  /** This function returns the last metadata attached to the segment
      or NULL if no metadata is attached.

      \return     A pointer to the last metadata attached.
   */
  const TMetadata* GetLastMetadata() const throw ();
};



// ----------------------------------------------------------------------------
// Inlined functions

inline uint8_t* TSegment::GetPosition() const throw ()
{
  return Position;
}

inline uint32_t TSegment::GetFrameSize() const throw ()
{
  return FrameSize;
}

inline uint32_t TSegment::GetFocusOffset() const throw ()
{
  return FocusOffset;
}

inline uint32_t TSegment::GetFrameCount() const throw ()
{
  return FrameCount;
}

inline uint32_t TSegment::GetFocusSize() const throw ()
{
  return FocusSize;
}

inline uint32_t TSegment::GetSize() const throw ()
{
  return FrameSize * FrameCount;
}

inline bool TSegment::IsEmpty() const throw ()
{
  return FrameCount == 0;
}

inline uint32_t TSegment::GetDataSize() const throw ()
{
  return FocusSize * FrameCount;
}

inline const TSegment* TSegment::GetPrevious() const throw ()
{
  return Previous;
}

inline const TSegment* TSegment::GetNext() const throw ()
{
  return Next;
}

inline const TMetadata* TSegment::GetFirstMetadata() const throw ()
{
  return FirstMetadata;
}

inline const TMetadata* TSegment::GetLastMetadata() const throw ()
{
  return LastMetadata;
}

#endif
