/*
 *--------------------------------------------------------------------
 *
 * TSegmentList.h
 *
 * Copyright (c) 2003 Motorola Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TSEGMENTLIST_H
#define TSEGMENTLIST_H

#include "TSegment.h"
#include <cassert>


/** \class TSegmentList TSegmentList.h "media/streamer/TSegmentList.h"

    This class encapsulates a list of segments. It is used as input to
    Streamer elements to provide it with all the segments in its input
    queue.
 */
class TSegmentList
{
public:
  /** \class TConstIterator TSegmentList.h \
      "media/streamer/TSegmentList.h"

      The iterator is used to traverse the list of segments.
   */
  class TConstIterator
  {
    friend class TSegmentList;
  protected:
    const TSegment* Segment;

    explicit TConstIterator(const TSegment* segment) throw ();

  public:
    TConstIterator() throw ();

    /** The prefix increment-operator moves the iterator to the next
        segment in the list. Make sure to check if the iterator is at
        the end before using the segment.

        \return   A reference to itself.
     */
    const TConstIterator& operator++() throw ();

    /** Type conversion operator returning a pointer to the segment
        the iterator points to.

        \return   The referenced segment.
     */
    operator const TSegment*() const throw ();

    /** Dereferencing operator to allow easy access to the segment the
        iterator points to.
        
        \return   Returns the referenced segment.
     */
    const TSegment* operator->() const throw ();

    /** Checks if the iterator is at the end of the segment list.

        \return   True if the iterator passed the last segment.
    */
    bool AtEnd() const throw ();
  };

protected:
  TSegment* Start;
  TSegment* End;

  TSegmentList() throw ();
  // Trick to allow sub-classes to create iterators even though they
  // aren't friends.
  static TConstIterator CreateConstIterator(const TSegment* segment) throw ();

public:

  /** Checks if the list is empty.

      \return   True if the list is empty.
  */
  bool IsEmpty() const throw ();

  /** This function returns an iterator pointing to the first segment
      in the list.

      \return   An iterator to the first segment.
   */
  TConstIterator Begin() const throw ();

  /** Gets the size of all data in focus in all the segments.

      \return   The size of the data in focus.
   */
  uint32_t GetDataSize() const throw ();

  /** Gets the size of all data in focus in the segments from the
      iterator to the end.
      
      \return   The size of the data in focus.
  */
  uint32_t GetDataSize(const TConstIterator& iter) const throw ();
};


// ----------------------------------------------------------------------------
// Inlined functions

// Const Iterator

inline
TSegmentList::TConstIterator::TConstIterator(const TSegment* segment) throw ()
  : Segment(segment)
{
  // empty
}

inline TSegmentList::TConstIterator::TConstIterator() throw ()
  : Segment(NULL)
{
  // empty
}

inline const TSegmentList::TConstIterator&
TSegmentList::TConstIterator::operator++() throw ()
{
  assert(Segment != NULL);
  Segment = Segment->GetNext();
  return *this;
}

inline TSegmentList::TConstIterator::operator const TSegment*() const throw ()
{
  return Segment;
}

inline const TSegment* TSegmentList::TConstIterator::operator->() const
  throw ()
{
  return Segment;
}

inline bool TSegmentList::TConstIterator::AtEnd() const throw ()
{
  return Segment == NULL;
}


// List

inline TSegmentList::TSegmentList() throw ()
  : Start(NULL),
    End(NULL)
{
  // empty
}

inline TSegmentList::TConstIterator
TSegmentList::CreateConstIterator(const TSegment* segment) throw ()
{
  return TConstIterator(segment);
}

inline bool TSegmentList::IsEmpty() const throw ()
{
  return Start == NULL;
}

inline TSegmentList::TConstIterator TSegmentList::Begin() const throw ()
{
  return TConstIterator(Start);
}

#endif
