/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *--------------------------------------------------------------------
 *
 * platform/module/interface/ILoadableModule.h
 *
 * Generic C interface for loadable modules.
 *
 * Copyright (c) 2003 Motorola Inc. All rights reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef PLATFORM_MODULE_INTERFACE_ILOADABLEMODULE_H
#define PLATFORM_MODULE_INTERFACE_ILOADABLEMODULE_H

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif


struct IModuleClassFactoryStruct;
typedef struct IModuleClassFactoryStruct IModuleClassFactory;

/** \struct IModuleClassFactoryStruct ILoadableModule.h \
    "platform/ILoadableModule.h"

    Every class in the loadable module must have a class factory
    capable of creating and destroying instances of that specific
    class. This structure holds all functions associated with one
    class factory.
 */
struct IModuleClassFactoryStruct
{
  /** This function is used for getting the name of the class. The
      class name is primarily used for problem diagnostics, but the
      application may also define other uses.

                 \return The name of the class.
   */
  const char* (*GetClassName)();


  /** This function is used to get the version of the class. The
      number should be increased each time a new version of the class
      is written. How this number is used must be defined by the
      application.
      
                 \return The version of the class.
   */
  uint32_t (*GetClassVersion)();


  /** This function returns the name of the interface implemented by
      the class. How this name is used must be defined by the
      application.
      
                 \return The name of the interface implemented by the
                         class.
   */
  const char* (*GetInterfaceName)();


  /** This function returns the language binding descriptor string,
      e.g. "C++" for C++ classes. How this string is used must be
      defined by the application.

                 \return The language binding.
   */
  const char* (*GetBinding)();


  /** This function creates an object of the class. The returned
      object is a void pointer that should be casted to the interface
      provided by the GetInterfaceName() function. Casting details
      must be defined by the application.

                 \return A new object of the class. If the module
                         failed to create the object the returned
                         pointer is NULL.
  */
  void* (*CreateObject)();


  /** All objects created by the class factory must also be destroyed
      by the class factory. Each created object must be destroyed
      exactly once. This way the module gets full control over its
      memory management.

           \param object The object to destroy. This pointer must have
                         been previously returned from the
                         CreateObject() function.
  */
  void (*DestroyObject)(void* object);
};


struct ILoadableModuleStruct;
typedef struct ILoadableModuleStruct ILoadableModule;

/** \struct ILoadableModuleStruct ILoadableModule.h \
    "platform/ILoadableModule.h"

    This is the main interface to the loadable module. For a shared
    library to implement this interface, the library is required to
    export a C function with the following prototype:

    \code ILoadableModule* GetModule(void) \endcode

    The returned pointer is then used to enquire additional
    information about the classes exported by the module.
 */
struct ILoadableModuleStruct
{
  /** The module loader uses this function to get the name of the
      module. The module name is primarily used for problem
      diagnostics.

                 \return The name of the module.
   */
  const char* (*GetModuleName)();


  /** The module loader uses this function to get the number of
      classes provided by the module.

                 \return The number of classes in the module.
   */
  uint32_t (*GetClassCount)();


  /** The module loader uses this function to get access to a class
      factory provided by the module. Each class provided by the
      module is identified by an integer index in the range from zero
      to the number of classes (as provided by the GetClassCount()
      function) minus one.
      
            \param index The index of the class.

                 \return A pointer to a factory structure which
                         contains functions for creating and
                         destroying objects of the class.
   */
  const IModuleClassFactory* (*GetFactory)(uint32_t index);


  /** The module loader calls this function when the module has loaded
      successfully. No other function in the ILoadableModule interface
      is called before this function is called. The loader will not
      call the function if the function pointer is NULL.
  */
  void (*OnLoad)();


  /** The module loader calls this function just before the module is
      unloaded. No other function in the ILoadableModule interface or
      any other interface derived thereof will be called after this
      function has been called. The loader will not call the function
      if the function pointer is NULL.
  */
  void (*OnUnload)();
};


/** This is the type of the function that must be implemented by all
    modules. It should return a pointer to a static module object.
 */
typedef ILoadableModule* (*TGetModuleFunction)();


#ifdef __cplusplus
}
#endif

#endif  // PLATFORM_MODULE_INTERFACE_ILOADABLEMODULE_H
