/*
 *---------------------------------------------------------------------------
 *
 * SharedMemory.h
 *
 * Copyright (c) 2004 Kreatel Communications AB. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef SHAREDMEMORY_H
#define SHAREDMEMORY_H

#include <sys/shm.h>
#include <unistd.h>
#include <stdint.h>
#include <cerrno>

template<typename T>
class TSharedMemory
{
private:
  key_t Key;
  key_t KeySeed;
  int Id;
  void* Memory;

  bool IsValid() const throw ();
  T* AttachMemory(int flags) throw ();

public:
  TSharedMemory() throw ();
  ~TSharedMemory() throw ();
  TSharedMemory(const TSharedMemory<T>& other) throw ();
  TSharedMemory<T>& operator=(const TSharedMemory<T>& other) throw ();

  bool CreateSegment(uint32_t size = 1,
                     int permissions = SHM_R | SHM_W | SHM_R >> 3) throw ();
  bool CreateSegmentWithKey(key_t key,
                            uint32_t size = 1,
                            int permissions = SHM_R | SHM_W | SHM_R >> 3) throw ();
  key_t GetKey() const throw ();
  uint32_t GetSize() const throw ();
  uint32_t GetRef() const throw ();
  bool FindSegment(key_t key) throw ();  
  T* AttachMemory() throw ();
  T* AttachMemoryReadOnly() throw ();
  void DetachMemory() throw ();
  bool Chmod(unsigned short mode) throw ();
};

template<typename T>
T* TSharedMemory<T>::AttachMemory(int flags) throw ()
{
  if (Id == -1) {
    return NULL;
  }

  if (Memory == NULL) {
    if ((Memory = ::shmat(Id, NULL, flags)) == (void*)-1) {
      Id = -1;
      Memory = NULL;
    }
  }

  return static_cast<T*>(Memory);
}

template<typename T>
TSharedMemory<T>::TSharedMemory() throw ()
  : Id(-1), Memory(NULL)
{
  KeySeed = getpid() * 0xBAD;
}

template<typename T>
TSharedMemory<T>&
TSharedMemory<T>::operator=(const TSharedMemory<T>& other) throw ()
{
  if (&other != this) {
    Key = other.Key;
    Id = other.Id;
    Memory = NULL;
  }
  return *this;
}

template<typename T>
TSharedMemory<T>::TSharedMemory(const TSharedMemory<T>& other) throw ()
{
  Key = other.Key;
  Id = other.Id;
  Memory = NULL;
}

template<typename T>
TSharedMemory<T>::~TSharedMemory() throw ()
{
  DetachMemory();
}

template<typename T>
bool TSharedMemory<T>::CreateSegment(uint32_t size, int permissions) throw ()
{
  DetachMemory();
  while ((Id = ::shmget(KeySeed, size * sizeof(T),
                        IPC_CREAT | IPC_EXCL | permissions)) == -1
         && errno == EEXIST) {
    KeySeed++;
  }
  Key = KeySeed++;
  return (Id != -1);
}

template<typename T>
bool TSharedMemory<T>::CreateSegmentWithKey(key_t key,
                                            uint32_t size,
                                            int permissions) throw ()
{
  DetachMemory();
  if ((Id = ::shmget(key, size * sizeof(T),
                     IPC_CREAT | IPC_EXCL | permissions)) != -1) 
  {
    Key = key;
  }

  return (Id != -1);
}

template<typename T>
key_t TSharedMemory<T>::GetKey() const throw ()
{
  return Key;
}

template<typename T>
bool TSharedMemory<T>::FindSegment(key_t key) throw ()
{
  if (IsValid() && key == Key) {
    return true;
  }
  DetachMemory();
  Id = ::shmget(key, 0, 0);
  Key = key;
  return (Id != -1);
}

template<typename T>
bool TSharedMemory<T>::IsValid() const throw ()
{
  if (Id == -1) {
    return 0;
  }
  shmid_ds ds;
  return (::shmctl(Id, IPC_STAT, &ds) != -1);
}

template<typename T>
uint32_t TSharedMemory<T>::GetSize() const throw ()
{
  if (Id == -1) {
    return 0;
  }
  shmid_ds ds;
  if (::shmctl(Id, IPC_STAT, &ds) == -1) {
    return 0;
  }
  return ds.shm_segsz;
}

template<typename T>
uint32_t TSharedMemory<T>::GetRef() const throw ()
{
  if (Id == -1) {
    return 0;
  }
  shmid_ds ds;
  if (::shmctl(Id, IPC_STAT, &ds) == -1) {
    return 0;
  }
  return ds.shm_nattch;
}

template<typename T>
T* TSharedMemory<T>::AttachMemory() throw ()
{
  return AttachMemory(0);
}

template<typename T>
T* TSharedMemory<T>::AttachMemoryReadOnly() throw ()
{
  return AttachMemory(SHM_RDONLY);
}

template<typename T>
void TSharedMemory<T>::DetachMemory() throw ()
{
  if (Memory != NULL) {
    ::shmdt(Memory);
    if (GetRef() == 0) {
      
      
      
      
      
      ::shmctl(Id, IPC_RMID, NULL);
    }
    Memory = NULL;
  }
}

template<typename T>
bool TSharedMemory<T>::Chmod(unsigned short mode) throw ()
{
  if (Id == -1) {
    return true;
  }
  shmid_ds ds;
  if (::shmctl(Id, IPC_STAT, &ds) == -1) {
    return false;
  }
  ds.shm_perm.mode = mode;
  if (::shmctl(Id, IPC_SET, &ds) == -1) {
    return false;
  }
  return true;
}

#endif
