









#ifndef UTILS_TIPCOBSERVERMANAGER_H
#define UTILS_TIPCOBSERVERMANAGER_H

#include "base/NonCopyable.h"
#include "debug/TTracker.h"

#include "boost/foreach.hpp"

#include <memory.h>
#include <cassert>
#include <vector>
#include <list>
#include <map>
#include <algorithm>


/** Generic observer implementation.
    TIpcObserverManager is a template class to inherit or aggregate. It
    provides functionality add and remove observers, and means to Notify
    the observers by calling their member functions. The template parameter
    type should have pointer semantics, i.e. it can be a C++pointer or a
    smart pointer to the observer type. Examples:
        TIpcObserverManager<IMyInterface*> mySubject;
        TIpcObserverManager<boost::shared_ptr<IMyInterface> > mySubject;
    Limitations:
    TIpcObserverManager does not yet handle adding and removal of subscribers
    during a notification.
 */
template <class TObserver>
class TIpcObserverManager
{
private:
  class TObserverInfo
  {
  public:
    int ApplicationId;
    std::string Address;
    TObserver Observer;
    std::list<std::string> Filters;

    TObserverInfo(int applicationId, std::string address, TObserver observer)
      : ApplicationId(applicationId),
        Address(address),
        Observer(observer),
        Filters()
    { /* Empty */ };

    bool DoesFilterExist(const std::string& name)
    {
      for (std::list<std::string>::iterator it = Filters.begin();
           it != Filters.end(); ++it) {
        if (*it == name) {
          return true;
        }
      }
      return false;
    }

    bool EraseFilter(const std::string& name)
    {
      for (std::list<std::string>::iterator it = Filters.begin();
           it != Filters.end(); ++it) {
        if (*it == name) {
          Filters.erase(it);
          return true;
        }
      }
      return false;
    }
  };

  
  typedef std::vector<TObserverInfo> TObservers;

  
  typedef std::multimap<std::string, TObserverInfo*> TFilters;
  typedef typename TFilters::iterator TFilterIterator;

  
  TObservers Observers;
  TFilters Filters;

  NON_COPYABLE(TIpcObserverManager);

  TObserverInfo* GetObserverInfo(const std::string& address)
  {
    BOOST_FOREACH(TObserverInfo& info, Observers) {
      if (info.Address == address) {
        return &info;
      }
    }
    return NULL;
  }

public:
  /** Constructor. Is empty but needed since we have other
      [private] constructors.
  */
  TIpcObserverManager()
  {
    
  }

  /** Get a list of all filter names that have been added. */
  void GetFilterNames(std::list<std::string>& names)
  {
    names.clear();
    for (TFilterIterator it = Filters.begin(); it != Filters.end(); ++it) {
      if (names.empty() || names.back() != it->first) {
        names.push_back(it->first);
      }
    }
  }

  /** Notify function for observer methods with no parameters.
      \param p A predicate function that decides which observers to call.
               The predicate takes an IObserverInfo object as argument and
	       returns a bool.
      \param f A pointer to the observer member function to call.
  */
  template <typename Predicate, typename F>
  void NotifyIf(Predicate p, F f)
  {
    typename TObservers::iterator i;
    TObservers observersCopy(Observers); 
    for (i = observersCopy.begin(); i != observersCopy.end(); ++i) {
      if (p(i->ApplicationId, i->Address)) {
	try {
	  (*i->Observer.*f)();
	}
	catch (std::exception const& e) {
	  TTracker::GetProcessTracker()->Error("%s Unexpected exception caught: %s", 
						 __PRETTY_FUNCTION__, 
						 e.what());
	}
	catch (...) {
	  TTracker::GetProcessTracker()->Error("%s Unknown exception caught",
						 __PRETTY_FUNCTION__);
	}
      }
    }
  }

  /** Notify function for observer methods with one parameter.
      \param p A predicate function that decides which observers to call.
               The predicate takes an IObserverInfo object as argument and
	       returns a bool.
      \param f A pointer to the observer member function to call.
      \param p1 A parameter to send to the observer function.
  */
  template <typename Predicate, typename F, typename P1>
  void NotifyIf(Predicate p, F f, P1 p1)
  {
    typename TObservers::iterator i;
    for (i = Observers.begin(); i != Observers.end(); ++i) {
      if (p(i->ApplicationId, i->Address, p1)) {
	try {
	  (*i->Observer.*f)(p1);
	}
	catch (std::exception const& e) {
	  TTracker::GetProcessTracker()->Error("%s Unexpected exception caught: %s", 
					       __PRETTY_FUNCTION__, 
					       e.what());
	}
	catch (...) {
	  TTracker::GetProcessTracker()->Error("%s Unknown exception caught",
					       __PRETTY_FUNCTION__);
	}
      }
    }
  }

  /** Notify function for observer methods with two parameters.
      \param p A predicate function that decides which observers to call.
               The predicate takes an IObserverInfo object as argument and
	       returns a bool.
      \param f A pointer to the observer member function to call.
      \param p1 The first parameter to send to the observer function.
      \param p2 The second parameter to send to the observer function.
  */
  template <typename Predicate, typename F, typename P1, typename P2>
  void NotifyIf(Predicate p, F f, P1 p1,  P2 p2)
  {
    typename TObservers::iterator i;
    for (i = Observers.begin(); i != Observers.end(); ++i) {
      if (p(i->ApplicationId, i->Address, p1, p2)) {
	try {
	  (*i->Observer.*f)(p1, p2);
	}
	catch (std::exception const& e) {
	  TTracker::GetProcessTracker()->Error("%s Unexpected exception caught: %s", 	
					       __PRETTY_FUNCTION__, 
					       e.what());
	}
	catch (...) {
	  TTracker::GetProcessTracker()->Error("%s Unknown exception caught",
					       __PRETTY_FUNCTION__);
	}
      }
    }
  }

  /** Notify function for observer methods with three parameters.
      \param p A predicate function that decides which observers to call.
               The predicate takes an IObserverInfo object as argument and
	       returns a bool.
      \param f A pointer to the observer member function to call.
      \param p1 The first parameter to send to the observer function.
      \param p2 The second parameter to send to the observer function.
      \param p3 The third parameter to send to the observer function.
  */
  template <typename Predicate, typename F, typename P1, typename P2, typename P3>
  void NotifyIf(Predicate p, F f, P1 p1,  P2 p2,  P3 p3)
  {
    typename TObservers::iterator i;
    for (i = Observers.begin(); i != Observers.end(); ++i) {
      if (p(i->ApplicationId, i->Address, p1, p2, p3)) {
	try {
	  (*i->Observer.*f)(p1, p2, p3);
	}
	catch (std::exception const& e) {
	  TTracker::GetProcessTracker()->Error("%s Unexpected exception caught: %s", 
					       __PRETTY_FUNCTION__, 
					       e.what());
	}
	catch (...) {
	  TTracker::GetProcessTracker()->Error("%s Unknown exception caught",
					       __PRETTY_FUNCTION__);
	}
      }
    }
  }

  /** Notify function for observer methods with three parameters.
      \param p A predicate function that decides which observers to call.
               The predicate takes an IObserverInfo object as argument and
	       returns a bool.
      \param f A pointer to the observer member function to call.
      \param p1 The first parameter to send to the observer function.
      \param p2 The second parameter to send to the observer function.
      \param p3 The second parameter to send to the observer function.
      \param p4 The fourth parameter to send to the observer function.
  */
  template <typename Predicate, typename F, typename P1, typename P2, typename P3, typename P4>
  void NotifyIf(Predicate p, F f, P1 p1,  P2 p2,  P3 p3,  P4 p4)
  {
    typename TObservers::iterator i;
    for (i = Observers.begin(); i != Observers.end(); ++i) {
      if (p(i->ApplicationId, i->Address, p1, p2, p3, p4)) {
	try {
	  (*i->Observer.*f)(p1, p2, p3, p4);
	}
	catch (std::exception const& e) {
	  TTracker::GetProcessTracker()->Error("%s Unexpected exception caught: %s", 
					       __PRETTY_FUNCTION__, 
					       e.what());
	}
	catch (...) {
	  TTracker::GetProcessTracker()->Error("%s Unknown exception caught",
					       __PRETTY_FUNCTION__);
	}
      }
    }
  }
  
  /** Notify function for observer methods with no parameters.
      \param f A pointer to the observer member function to call.
  */
  template <typename F>
  void Notify(F f)
  {
    typename TObservers::iterator i;
    TObservers observersCopy(Observers); 
    for (i = observersCopy.begin(); i != observersCopy.end(); ++i) {
      try {
        (*i->Observer.*f)();
      }
      catch (std::exception const& e) {
	TTracker::GetProcessTracker()->Error("%s Unexpected exception caught: %s", 
					     __PRETTY_FUNCTION__, 
					     e.what());
      }
      catch (...) {
	TTracker::GetProcessTracker()->Error("%s Unknown exception caught",
					     __PRETTY_FUNCTION__);
      }
    }
  }

  /** Filtered Notify for observer methods with no parameters.
      \param filter Notifies only observers with this filter set. An empty
             string is regarded as a call to Notify() (without filter).
      \param f A pointer to the observer member function to call.
  */
  template <typename F>
  void NotifyWithFilter(const std::string& filter, F f)
  {
    if (filter.empty()) {
      Notify(f);
      return;
    }

    std::pair<TFilterIterator, TFilterIterator> range =
      Filters.equal_range(filter);
    for (; range.first != range.second; ++range.first) {
      try {
        (*range.first->second->Observer.*f)();
      }
      catch (const std::exception& e) {
        TTracker::GetProcessTracker()->Error(
          "%s Unexpected exception caught: %s", __func__, e.what());
      }
      catch (...) {
        TTracker::GetProcessTracker()->Error(
          "%s: Unknown exception caught", __func__);
      }
    }
  }

  /** Notify function function for observer methods with one parameter.
      \param f A pointer to the observer member function to call.
      \param p1 A parameter to send to the observer function.
  */
  template <typename F, typename P1>
  void Notify(F f, P1 p1)
  {
    typename TObservers::iterator i;
    for (i = Observers.begin(); i != Observers.end(); ++i) {
      try {
        (*i->Observer.*f)(p1);
      }
      catch (std::exception const& e) {
	TTracker::GetProcessTracker()->Error("%s Unexpected exception caught: %s", 
					     __PRETTY_FUNCTION__, 
					     e.what());
      }
      catch (...) {
	TTracker::GetProcessTracker()->Error("%s Unknown exception caught",
					     __PRETTY_FUNCTION__);
      }
    }
  }

  /** Filtered Notify function for observer methods with one parameter.
      \param filter Notifies only observers with this filter set. An empty
             string is regarded as a call to Notify() (without filter).
      \param f A pointer to the observer member function to call.
      \param p1 A parameter to send to the observer function.
  */
  template <typename F, typename P1>
  void NotifyWithFilter(const std::string& filter, F f, P1 p1)
  {
    if (filter.empty()) {
      Notify(f, p1);
      return;
    }

    std::pair<TFilterIterator, TFilterIterator> range =
      Filters.equal_range(filter);
    for (; range.first != range.second; ++range.first) {
      try {
        (*range.first->second->Observer.*f)(p1);
      }
      catch (const std::exception& e) {
        TTracker::GetProcessTracker()->Error(
          "%s Unexpected exception caught: %s", __func__, e.what());
      }
      catch (...) {
        TTracker::GetProcessTracker()->Error(
          "%s: Unknown exception caught", __func__);
      }
    }
  }

  /** Notify function for observer methods with two parameters.
      \param f A pointer to the observer member function to call.
      \param p1 The first parameter to send to the observer function.
      \param p2 The second parameter to send to the observer function.
  */
  template <typename F, typename P1, typename P2>
  void Notify(F f, P1 p1, P2 p2)
  {
    typename TObservers::iterator i;
    for (i = Observers.begin(); i != Observers.end(); ++i) {
      try {
        (*i->Observer.*f)(p1, p2);
      }
      catch (std::exception const& e) {
	TTracker::GetProcessTracker()->Error("%s Unexpected exception caught: %s", 	
					     __PRETTY_FUNCTION__, 
					     e.what());
      }
      catch (...) {
	TTracker::GetProcessTracker()->Error("%s Unknown exception caught",
					     __PRETTY_FUNCTION__);
      }
    }
  }

  /** Filtered Notify function for observer methods with two parameters.
      \param filter Notifies only observers with this filter set. An empty
             string is regarded as a call to Notify() (without filter).
      \param f A pointer to the observer member function to call.
      \param p1 The first parameter to send to the observer function.
      \param p2 The second parameter to send to the observer function.
  */
  template <typename F, typename P1, typename P2>
  void NotifyWithFilter(const std::string& filter, F f, P1 p1, P2 p2)
  {
    if (filter.empty()) {
      Notify(f, p1, p2);
      return;
    }

    std::pair<TFilterIterator, TFilterIterator> range =
      Filters.equal_range(filter);
    for (; range.first != range.second; ++range.first) {
      try {
        (*range.first->second->Observer.*f)(p1, p2);
      }
      catch (const std::exception& e) {
        TTracker::GetProcessTracker()->Error(
          "%s Unexpected exception caught: %s", __func__, e.what());
      }
      catch (...) {
        TTracker::GetProcessTracker()->Error(
          "%s: Unknown exception caught", __func__);
      }
    }
  }

  /** Notify function for observer methods with three parameters.
      \param f A pointer to the observer member function to call.
      \param p1 The first parameter to send to the observer function.
      \param p2 The second parameter to send to the observer function.
      \param p3 The third parameter to send to the observer function.
  */
  template <typename F, typename P1, typename P2, typename P3>
  void Notify(F f, P1 p1, P2 p2, P3 p3)
  {
    typename TObservers::iterator i;
    for (i = Observers.begin(); i != Observers.end(); ++i) {
      try {
        (*i->Observer.*f)(p1, p2, p3);
      }
      catch (std::exception const& e) {
	TTracker::GetProcessTracker()->Error("%s Unexpected exception caught: %s", 
					     __PRETTY_FUNCTION__, 
					     e.what());
      }
      catch (...) {
	TTracker::GetProcessTracker()->Error("%s Unknown exception caught",
					     __PRETTY_FUNCTION__);
      }
    }
  }

  /** Filtered Notify function for observer methods with three parameters.
      \param filter Notifies only observers with this filter set. An empty
             string is regarded as a call to Notify() (without filter).
      \param f A pointer to the observer member function to call.
      \param p1 The first parameter to send to the observer function.
      \param p2 The second parameter to send to the observer function.
      \param p3 The third parameter to send to the observer function.
  */
  template <typename F, typename P1, typename P2, typename P3>
  void NotifyWithFilter(const std::string& filter, F f, P1 p1, P2 p2, P3 p3)
  {
    if (filter.empty()) {
      Notify(f, p1, p2, p3);
      return;
    }

    std::pair<TFilterIterator, TFilterIterator> range =
      Filters.equal_range(filter);
    for (; range.first != range.second; ++range.first) {
      try {
        (*range.first->second->Observer.*f)(p1, p2, p3);
      }
      catch (const std::exception& e) {
        TTracker::GetProcessTracker()->Error(
          "%s Unexpected exception caught: %s", __func__, e.what());
      }
      catch (...) {
        TTracker::GetProcessTracker()->Error(
          "%s: Unknown exception caught", __func__);
      }
    }
  }

  /** Notify function for observer methods with three parameters.
      \param f A pointer to the observer member function to call.
      \param p1 The first parameter to send to the observer function.
      \param p2 The second parameter to send to the observer function.
      \param p3 The second parameter to send to the observer function.
      \param p4 The fourth parameter to send to the observer function.
  */
  template <typename F, typename P1, typename P2, typename P3, typename P4>
  void Notify(F f, P1 p1, P2 p2, P3 p3, P4 p4)
  {
    typename TObservers::iterator i;
    for (i = Observers.begin(); i != Observers.end(); ++i) {
      try {
        (*i->Observer.*f)(p1, p2, p3, p4);
      }
      catch (std::exception const& e) {
	TTracker::GetProcessTracker()->Error("%s Unexpected exception caught: %s", 
					     __PRETTY_FUNCTION__, 
					     e.what());
      }
      catch (...) {
	TTracker::GetProcessTracker()->Error("%s Unknown exception caught",
					     __PRETTY_FUNCTION__);
      }
    }
  }

  /** Filtered Notify function for observer methods with three parameters.
      \param filter Notifies only observers with this filter set. An empty
             string is regarded as a call to Notify() (without filter).
      \param f A pointer to the observer member function to call.
      \param p1 The first parameter to send to the observer function.
      \param p2 The second parameter to send to the observer function.
      \param p3 The second parameter to send to the observer function.
      \param p4 The fourth parameter to send to the observer function.
  */
  template <typename F, typename P1, typename P2, typename P3, typename P4>
  void NotifyWithFilter(
    const std::string& filter, F f, P1 p1, P2 p2, P3 p3, P4 p4)
  {
    if (filter.empty()) {
      Notify(f, p1, p2, p3, p4);
      return;
    }

    std::pair<TFilterIterator, TFilterIterator> range =
      Filters.equal_range(filter);
    for (; range.first != range.second; ++range.first) {
      try {
        (*range.first->second->Observer.*f)(p1, p2, p3, p4);
      }
      catch (const std::exception& e) {
        TTracker::GetProcessTracker()->Error(
          "%s Unexpected exception caught: %s", __func__, e.what());
      }
      catch (...) {
        TTracker::GetProcessTracker()->Error(
          "%s: Unknown exception caught", __func__);
      }
    }
  }

public:
  /** Adds a filter to all observers. For backward compatibility only -
      use AddFilter instead.
      \param name The filter to be added.
      \return true if the filter wasnt used by any observers before.
  */
  bool SubscribeParameter(const std::string& name)
  {
    bool result = false;
    for (typename TObservers::iterator it = Observers.begin();
         it != Observers.end(); ++it) {
      if (AddFilter(it->Address, name)) {
        result = true;
      }
    }
    return result;
  }

  /** Removes a filter from all observers. For backward compatibility only -
      use RemoveFilter instead.
      \param name The filter to be removed.
      \return true if the filter was used by an observer.
  */
  bool UnsubscribeParameter(const std::string& name)
  {
    bool result = false;
    TFilterIterator lowerBound = Filters.lower_bound(name);
    TFilterIterator upperBound = Filters.upper_bound(name);

    for (TFilterIterator it = lowerBound; it != upperBound; ++it) {
      if(it->second->EraseFilter(name)) {
        result = true;
      }
    }
    Filters.erase(lowerBound, upperBound);
    return result;
  }

  /** Adds a filter to an observer.
      \param address The IPC address to the observer.
      \param name The filter to be added.
      \return true if the filter wasn't used by any observers before.
  */
  bool AddFilter(const std::string& address,
                 const std::string& name)
  {
    TObserverInfo* observerInfo = GetObserverInfo(address);
    if (observerInfo == NULL || observerInfo->DoesFilterExist(name)) {
      return false;
    }
    observerInfo->Filters.push_back(name);

    TFilterIterator it = Filters.find(name);

    if (it == Filters.end()) {
      Filters.insert(make_pair(name, observerInfo));
      return true;
    }
    else {
      Filters.insert(it, make_pair(name, observerInfo));
      return false;
    }
  }

  /** Removes a filter from an observer.
      \param address The IPC address to the observer.
      \param name The filter to be removed.
      \return true if the last observer using that filter was removed.
  */
  bool RemoveFilter(const std::string& address,
                    const std::string& name)
  {
    TObserverInfo* observerInfo = GetObserverInfo(address);
    if (observerInfo == NULL || !observerInfo->DoesFilterExist(name)) {
      return false;
    }
    for (TFilterIterator it = Filters.lower_bound(name);
         it != Filters.upper_bound(name); ++it) {
      if (it->second == observerInfo) {
        Filters.erase(it);
        observerInfo->EraseFilter(name);
        if (Filters.find(name) == Filters.end()) {
          return true;
        }
        return false;
      }
    }
    return false;
  }

  /** Adds an observer.
      \param applicationId Id of the application owning the observer.
      \param address The IPC address to the observer.
      \param observer The observer object to add. Not that this object is only
             referenced, i.e. TIpcObserverManager does not assume ownership and
             does not explicitly delete the observer in the destructor. Caller
             is responsible for removing the observer from the subject
             before it is deleted.
      \return false If the the observer address is already present.
  */
  bool AddObserver(int applicationId,
                   const std::string& address,
                   TObserver observer)
  {
    for (typename TObservers::const_iterator it = Observers.begin();
         it != Observers.end(); ++it) {
      if (it->Address == address) {
        return false;
      }
    }

    TObserverInfo info(applicationId, address, observer);
    Observers.push_back(info);
    return true;
  }

  /** Removes an observer.
      \param observerAddress The observer object to remove.
      \return false if the the observer is not present.
  */
  bool RemoveObserver(const std::string& address)
  {
    typename TObservers::iterator i;
    for (i = Observers.begin(); i != Observers.end(); ++i) {
      if (i->Address == address) {
        Observers.erase(i);
        return true;
      }
    }
    return false;
  }

  /** Returns whether an observer exists or not
      \param address the address to the observer
      \return true if the observer exists
  */
  bool DoesObserverExist(const std::string& address)
  {
    return (GetObserverInfo(address) != NULL ? true : false);
  }

  /** Removes all observers belonging to a specific application.
      \param applicationId The application whose observers are to be
                           removed.
  */
  void RemoveAllObservers(int applicationId)
  {
    typename TObservers::iterator i = Observers.begin();
    while (i != Observers.end()) {
      if (i->ApplicationId == applicationId) {
        i = Observers.erase(i);
      }
      else {
        ++i;
      }
    }
  }

  /** Removes all observers
  */
  void RemoveAllObservers()
  {
    Observers.clear();
  }
};

#endif
