









#ifndef UTILS_TLOCKEDVAR_H
#define UTILS_TLOCKEDVAR_H

#include "base/NonCopyable.h"

#include <stdint.h>
#include <cassert>
#include <cstring>
#include <map>
#include <sys/mman.h>



#ifdef DEBUG
#include "debug/TTracker.h"
#define DEBUGTRACE(x...) TTracker::GetProcessTracker()->Debug(x)
#else
#define DEBUGTRACE(x...)
#endif

/** Help function to check if the locked memory is clean */
bool CheckMemoryClean();


/**
 * Base class for TLockedVar
 * It contains the static reference counters,
 * allocated memory address and the size
 */
class TLockedVarBase
{
protected:
  static std::map<int, int> ReferenceCount;
  static std::map<void *, unsigned int> AllocatedMemory;
  static const int PAGE_SIZE;

  /** Lock memory, the actual locked memory will be page-aligned */
  static void Lock(void* address, size_t size);

  /** Unlock the previous locked memory.
   *  But if the address range contains a page that other variables resident,
   *  the page will remain locked until the last variable is unlocked.
   */
  static void Unlock(void* address);

public:
  friend bool CheckMemoryClean();
};

/**
 * Template for array
 * T is a the array type, and the array can't be assigned
 */
template <typename T, unsigned int N = 0>
class TLockedVar : public TLockedVarBase
{
private:
  T* Var;
  NON_COPYABLE(TLockedVar);

public:
  TLockedVar() : Var(NULL)
  {
    Var = new T[N];
    DEBUGTRACE("new TLockedVar: %p\n", Var);
    Lock(Var, sizeof(T)*N);
  }

  ~TLockedVar()
  {
    DEBUGTRACE("delete TLockedVar: %p\n", Var);
    Unlock(Var);
    delete [] Var;
  }

  T operator [] (const int index) const
  {
    return Var[index];
  }

  T& operator [] (const int index)
  {
    return Var[index];
  }

  T* GetVar(void) const
  {
    return Var;
  }

  T* operator() () const
  {
    return Var;
  }

  T* operator * () const
  {
    return Var;
  }
};

/**
 * Template class for struct if N is 0
 * T can be a struct or any type that can be assigned or copied
 */
template < typename T >
class TLockedVar <T, 0> : public TLockedVarBase
{
private:
  T* Var;

public:

  void init()
  {
    Var = new T;
    DEBUGTRACE("new TLockedVar: %p\n", Var);
    Lock(Var, sizeof(T));
  }

  TLockedVar()
  {
    init();
  }

  
  TLockedVar (const TLockedVar& other)
  {
    init();
    *Var = other();
  }

  ~TLockedVar() {
    DEBUGTRACE("delete TLockedVar: %p\n", Var);
    Unlock(Var);
    delete Var;
  }

  
  TLockedVar& operator = (const TLockedVar& other)
  {
    *Var = other();
    return *this;
  }

  T& GetVar(void) const
  {
    return *Var;
  }

  T& operator() () const
  {
    return *Var;
  }

  T& operator * () const
  {
    return *Var;
  }
};

#endif

