









#ifndef UTILS_TOBSERVERMANAGER_H
#define UTILS_TOBSERVERMANAGER_H

#include "base/NonCopyable.h"
#include "debug/TTracker.h"

#include <memory.h>
#include <cassert>
#include <vector>
#include <algorithm>

/** Generic observer implementation.
    TObserverManager is a template class to inherit or aggregate. It provides
    functionality add and remove observers, and means to Notify the 
    observers by calling their member functions. The template parameter
    type should have pointer semantics, i.e. it can be a C++pointer or a
    smart pointer to the observer type. Examples:
        TObserverManager<IMyInterface*> mySubject;
	TObserverManager<boost::shared_ptr<IMyInterface> > mySubject;
    Limitations: 
    TObserverManager does not yet handle adding and removal of subscribers
    during a notification.
 */
template <class TObserver>
class TObserverManager
{
private:
  /** Observer collection type
  */
  typedef std::vector<TObserver> TObservers;
  /** All registered observers
   */
  TObservers Observers;

  NON_COPYABLE(TObserverManager);

public:
  /** Constructor. Is empty but needed since we have other 
      [private] constructors.
  */
  TObserverManager()
  {
  }
  
  /** Notify function for observer methods with no parameters.
      \param f A pointer to the observer member function to call.
  */
  template <typename F>
  void Notify(F f)
  {
    typename TObservers::iterator i;
    
    TObservers observerCopy(Observers);
    for (i = observerCopy.begin(); i != observerCopy.end(); ++i) {
      try {
	(**i.*f)();
      }
      catch (std::exception const& e) {
	TTracker::GetProcessTracker()->Error("%s Unexpected exception caught: %s", 
					     __PRETTY_FUNCTION__, 
					     e.what());
      }
      catch (...) {
	TTracker::GetProcessTracker()->Error("%s Unknown exception caught",
					     __PRETTY_FUNCTION__);
      }
    }
  }

  /** Notify function for observer methods with one parameter.
      \param f A pointer to the observer member function to call.
      \param p1 A parameter to send to the observer function.
  */
  template <typename F, typename P1>
  void Notify(F f, P1 p1)
  {
    typename TObservers::iterator i;
    
    TObservers observerCopy(Observers);
    for (i = observerCopy.begin(); i != observerCopy.end(); ++i) {
      try {
	(**i.*f)(p1);
      }
      catch (std::exception const& e) {
	TTracker::GetProcessTracker()->Error("%s Unexpected exception caught: %s", 
					     __PRETTY_FUNCTION__, 
					     e.what());
      }
      catch (...) {
	TTracker::GetProcessTracker()->Error("%s Unknown exception caught",
					     __PRETTY_FUNCTION__);
      }
    }
  }

  /** Notify function for observer methods with two parameters.
      \param f A pointer to the observer member function to call.
      \param p1 The first parameter to send to the observer function.
      \param p2 The second parameter to send to the observer function.
  */
  template <typename F, typename P1, typename P2>
  void Notify(F f, P1 p1,  P2 p2)
  {
    typename TObservers::iterator i;
    
    TObservers observerCopy(Observers);
    for (i = observerCopy.begin(); i != observerCopy.end(); ++i) {
      try {
	(**i.*f)(p1, p2);
      }
      catch (std::exception const& e) {
	TTracker::GetProcessTracker()->Error("%s Unexpected exception caught: %s", 	
					     __PRETTY_FUNCTION__, 
					     e.what());
      }
      catch (...) {
	TTracker::GetProcessTracker()->Error("%s Unknown exception caught",
					     __PRETTY_FUNCTION__);
      }
    }
  }

  /** Notify function for observer methods with three parameters.
      \param f A pointer to the observer member function to call.
      \param p1 The first parameter to send to the observer function.
      \param p2 The second parameter to send to the observer function.
      \param p3 The third parameter to send to the observer function.
  */
  template <typename F, typename P1, typename P2, typename P3>
  void Notify(F f, P1 p1,  P2 p2,  P3 p3)
  {
    typename TObservers::iterator i;
    
    TObservers observerCopy(Observers);
    for (i = observerCopy.begin(); i != observerCopy.end(); ++i) {
      try {
	(**i.*f)(p1, p2, p3);
      }
      catch (std::exception const& e) {
	TTracker::GetProcessTracker()->Error("%s Unexpected exception caught: %s", 
					     __PRETTY_FUNCTION__, 
					     e.what());
      }
      catch (...) {
	TTracker::GetProcessTracker()->Error("%s Unknown exception caught",
					     __PRETTY_FUNCTION__);
      }
    }
  }

  /** Notify function for observer methods with three parameters.
      \param f A pointer to the observer member function to call.
      \param p1 The first parameter to send to the observer function.
      \param p2 The second parameter to send to the observer function.
      \param p3 The second parameter to send to the observer function.
      \param p4 The fourth parameter to send to the observer function.
  */
  template <typename F, typename P1, typename P2, typename P3, typename P4>
  void Notify(F f, P1 p1,  P2 p2,  P3 p3,  P4 p4)
  {
    typename TObservers::iterator i;
    
    TObservers observerCopy(Observers);
    for (i = observerCopy.begin(); i != observerCopy.end(); ++i) {
      try {
	(**i.*f)(p1, p2, p3, p4);
      }
      catch (std::exception const& e) {
	TTracker::GetProcessTracker()->Error("%s Unexpected exception caught: %s", 
					     __PRETTY_FUNCTION__, 
					     e.what());
      }
      catch (...) {
	TTracker::GetProcessTracker()->Error("%s Unknown exception caught",
					     __PRETTY_FUNCTION__);
      }
    }
  }

public:
  /** Adds an observer.
      \param observer The observer object to add. Not that this object is 
                      only referenced, i.e. TObserverManager does not assume
		      ownership and does not explicitly delete the observer
		      in the destructor. Caller is responsible for removing
		      the observer from the subject before it is deleted. 
      \return false If the the observer is already present.
  */
  bool AddObserver(TObserver observer)
  {
    typename TObservers::iterator i = std::find(Observers.begin(), 
						Observers.end(), 
						observer);
    if (i != Observers.end()) {
      return false;
    }
    Observers.push_back(observer);
    return true;
  }

  /** Removes an observer.
      \param observer The observer object to remove.
      \return false If the the observer is not present.
  */
  bool RemoveObserver(TObserver observer)
  {
    typename TObservers::iterator i = std::find(Observers.begin(), 
						Observers.end(), 
						observer);
    if (i == Observers.end()) {
      return false;
    }
    Observers.erase(i);
    return true;
  }

  /** Removes all observers.
  */
  void RemoveAllObservers()
  {
    Observers.clear();
 }
};

#endif
