/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * TXmlElement.h
 *
 *   - A class representing an XML element. It has either a (potentially empty)
 *     text or a set of children.
 *
 * Copyright (c) 2002 Kreatel Communications AB. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef XML_TXMLELEMENT_H
#define XML_TXMLELEMENT_H

#include "base/TException.h"

#include <map>
#include <string>
#include <vector>

/**
 * Usage of this class
 *
 * A TXmlElement object represents a branch of the XML element tree.
 * It may either have a text attribute, which is an XML element in the
 * following format
 *   <elementName>text</elementName>
 * or it may consist of a number of other elements:
 *   <elementName>
 *     <elementName2>text2</elementName2>
 *     <elementName3>
 *       <elementName4>...
 *     ...
 *   </elementName>
 *
 * If the element consists of other elements, the usual usage is to
 * loop over those elements. However, elements with text can be retrieved
 * directly, e.g.
 *   <elementName1>
 *      <elementName2>text2</elementName2>
 *      <elementName3>text3</elementName3>
 *   </elementName>
 *
 * can be used with
 * element1.GetStringValue("elementName2", destStr);
 */

class TXmlElement;

std::ostream& operator<<(std::ostream& outStream, const TXmlElement& element);

class TXmlElement
{
  friend class TGeneralXmlParser;

public:
  typedef std::map<std::string, std::string> TAttributeMap;
  typedef std::vector<TXmlElement> TElementVector;
  bool operator==(const TXmlElement& other) const;

protected:
  std::string Name;
  std::string Text;
  TAttributeMap Attributes;
  TElementVector Elements;

public:
  TXmlElement(const char* name);
  TXmlElement(const char* name, const std::string& text);
  TXmlElement(const char* name, const char** attributes);
  TXmlElement(const std::string& name, const TAttributeMap& attributes);
  ~TXmlElement();

  
  bool IsTextElement() const throw();

  
  const std::string& GetName() const throw ();

  
  
  const std::string& GetText() const throw (TException);

  
  
  const TAttributeMap& GetAttributes() const throw (TException);
  const TAttributeMap& GetAttributesAllowEmpty() const throw ();

  
  
  std::string GetAttribute(const std::string& name) const
    throw (TException);

  
  
  
  const TElementVector& GetElements() const throw (TException);
  const TElementVector& GetElementsAllowEmpty() const throw ();

  
  
  TElementVector GetElementsAllowEmpty(const std::string& name) const throw ();

  
  
  
  
  bool GetStringValue(const std::string& name, std::string& value) const
    throw (TException);

  bool HasChild(const std::string& name) const;

  
  
  const TXmlElement& GetChild(const std::string& name) const throw (TException);

  std::string Serialize(int indentStepSize = 2, int indentTotalSize = 0) const;

  void AppendChild(const TXmlElement& child);
};

inline const std::string& TXmlElement::GetName() const throw ()
{
  return Name;
}

inline const TXmlElement::TAttributeMap& TXmlElement::GetAttributesAllowEmpty()
  const throw ()
{
  return Attributes;
}

inline const TXmlElement::TElementVector& TXmlElement::GetElementsAllowEmpty()
  const throw ()
{
  return Elements;
}

#endif 
