#!/usr/bin/perl
#############################################################################
#
# logprocplot
#
# Copyright (c) 2008 Motorola, Inc. All Rights Reserved.
#
#############################################################################

use strict;
use warnings FATAL => 'all';

use Cwd;

my $VERSION = "3.3.0 (20100830)";

# Bit fields
my $ENABLE_CPU = 0x1;
my $ENABLE_MEM = 0x2;
my $ENABLE_SYS_CPU = 0x4;
my $ENABLE_SYS_FREE = 0x8;
my $ENABLE_SYS_AVG1 = 0x10;
my $ENABLE_SYS_AVG5 = 0x20;
my $ENABLE_SYS_AVG15 = 0x40;

my %mem_components;
my %cpu_components;
my $plot = 0;
my $outfile = "";
my $outfile_format = "png";
my $logfile = "";
my $relative_time = 0;
my $output_base_dir = "";
my $ignore = "";
my $max_y = "";
my $memory_consumers = 0;
my $mem_limit = 0;

sub calculate_crc;
sub cmdline_error;
sub create_gnuplot;
sub parse_cmdline;
sub parse_log;
sub print_syntax;

#
# Main program
#
if (@ARGV < 1) {
    print_syntax();
    exit 0;
}

parse_cmdline(@ARGV);

if ($logfile eq "") {
    cmdline_error("no logfile given");
    exit 1;
}

if (! -e $logfile) {
    cmdline_error("logfile '$logfile' does not exist");
    exit 1;
}

if ($output_base_dir eq "") {
    $output_base_dir = "$logfile.logprocdata/";
}

if ($plot && $outfile) {
    cmdline_error("both -p and -f given");
    exit 1;
}

if (! -d $output_base_dir) {
    mkdir $output_base_dir;
}

my $crc = calculate_crc($logfile, $ignore);
my $crcfile = $output_base_dir.$crc."_".(stat($logfile))[7].".crc";

if (-e $crcfile) {
    print "File already parsed (CRC $crc), not parsing\n";
}
else {
    `rm -f $output_base_dir/*`;
    parse_log($logfile, $output_base_dir);
    `touch $crcfile`;
}

create_gnuplot($output_base_dir);

#
# Sub routines
#

sub print_syntax
{
    print <<END;
Usage: $0 [OPTIONS] <LOGFILE>
Parses output from logprocclient and creates memory and/or CPU graphs.

Available options:
  -h,  --help                This help text
  -C,  --system-cpu          Show total system CPU usage
  -F,  --system-free         Show total system free memory
  -M MIN, --memory-consumers Show components that consumes memory
                              MIN value is minimum difference between
                              first and last memory value.
                              (MIN default = 0)                           
  -L1, --system-avg1         Show system load average for the last minute
  -L2, --system-avg5         Show system load average for the 5 minutes
  -L3, --system-avg15        Show system load average for the 15 minutes
  -m COMPONENT               Show memory usage for COMPONENT
  -c COMPONENT               Show CPU usage for COMPONENT
  -o DIR                     Store component data files in DIR
                              (defaults to <LOGFILE>.logprocdata)
  -p,  --gnuplot             Show graph in gnuplot
  -f FILE                    Store $outfile_format file with graph in FILE
  -t FORMAT                  Use FORMAT when saving graph in file (default $outfile_format)
  -r,  --relative            Use relative time scale instead of absolute time
  -i COMPONENT               Ignore COMPONENT when parsing
  -y MAX                     Set Y axis to show data up to MAX

Options -m, -c and -i may be given several times.

COMPONENTs are matched on prefix. E.g., if '-c streamer-1' is given as
COMPONENT, then all streamers with pid starting with '1' will be
shown (or ignored).

Only one of -p and -f options may be given.
If neither -p nor -f option is given, the log file is just parsed.

logprocplot $VERSION (c) Motorola, Inc. 2008
END
}

sub parse_cmdline
{
    my $arg = shift;

    while ($arg) {
        for ($arg) {
            if (/^-h|--help$/) {
                print_syntax;
                exit 0;
            }
            elsif (/^-C|--system-cpu$/) {
                add_cpu_component("System", $ENABLE_SYS_CPU);
            }
            elsif (/^-F|--system-free$/) {
                add_mem_component("System", $ENABLE_SYS_FREE);
            }
            elsif (/^-L1|--system-avg1$/) {
                add_cpu_component("System", $ENABLE_SYS_AVG1);
            }
            elsif (/^-L2|--system-avg5$/) {
                add_cpu_component("System", $ENABLE_SYS_AVG5);
            }
            elsif (/^-L3|--system-avg15$/) {
                add_cpu_component("System", $ENABLE_SYS_AVG15);
            }
            elsif (/^-m$/) {
                add_mem_component(shift, $ENABLE_MEM);
            }
            elsif (/^-c$/) {
                add_cpu_component(shift, $ENABLE_CPU);
            }
            elsif (/^-o$/) {
		$output_base_dir = shift;
		$output_base_dir = $output_base_dir . "/";
            }
            elsif (/^-p|--gnuplot$/) {
                $plot = 1;
            }
            elsif (/^-f$/) {
                $outfile = shift;
            }
	    elsif (/^-t$/) {
		$outfile_format = shift;
	    }
            elsif (/^-r|--relative$/) {
                $relative_time = 1;
            }
	    elsif (/^-i$/) {
		if ($ignore eq "") {
		    $ignore = shift;
		}
		else {
		    $ignore = "$ignore|" . shift;
		}
	    }
	    elsif (/^-y$/) {
		$max_y = shift;
	    }
	    elsif (/^-M|--memory-consumers$/) {
		if ($_[0] =~ /^[0-9]+$/) {
		    $mem_limit = shift;
		}
		$memory_consumers = 1;
	    }
            elsif (/^-/) {
		cmdline_error("unknown option '$arg'");
                exit 1;
            }
            else {
		$logfile = $_;
		return;
            }
        }
        $arg = shift;
    }
}

sub cmdline_error
{
    my $error = shift;
    print STDERR "Error: $error\n";
    print STDERR "Try '$0 --help' for more information\n";
}

sub add_cpu_component
{
    my $component = shift;
    my $select = shift;

    my $val = $cpu_components{$component} || 0;
    $cpu_components{$component} = $val | $select;
}

sub add_mem_component
{
    my $component = shift;
    my $select = shift;

    my $val = $mem_components{$component} || 0;
    $mem_components{$component} = $val | $select;
}

sub sanitize_filename
{
    my $filename = shift;
    $filename =~ s/\//\#/g;
    return $filename;
}

sub dump_data
{
    my $outdir = shift;
    my $components = shift;

    foreach (keys %{$components}) {
	my $filename = sanitize_filename($_);
	$filename = $outdir . $filename . ".data";

	local *FILE;
	open FILE, ">> $filename" or die "Could not open $filename";
	print FILE $components->{$_};
	close FILE;
    }

    %{$components} = ()
}

sub parse_log
{
    my $infile = shift;
    my $outdir = shift;

    my %component_data;
    my $counter = 0;

    open(INDATA, $infile);
    while (<INDATA>) {
        my @words = split(/ /, $_);

        my $component;
        if (!$words[6] || $words[2] eq 'System') {
            $component = 'System';
        }
        else {
            $component = "$words[2]-$words[6]";
        }

	if ($ignore && $component =~ /^($ignore)/) {
	    next;
	}

	$component_data{$component} .= $_;

	# Dump data to disk every 10000 lines
	if (++$counter == 10000) {
	    dump_data($outdir, \%component_data);
	    $counter = 0;
	}
    }

    # Dump the last data to disk
    dump_data($outdir, \%component_data);

    close(INDATA);
}

sub calculate_crc
{
    my $infile = shift;
    my $ignore = shift;

    my $file_crc = `dd count=20 if='$logfile' 2>/dev/null | cksum | cut -d' ' -f1`;
    my $ignore_crc = `echo '$ignore' | cksum | cut -d' ' -f1`;

    my $crc = "${file_crc}${ignore_crc}";
    $crc =~ s/\s+//g;
    return $crc;
}

sub create_gnuplot
{
    my $outdir = shift;
    my $val;
    my $component;
    my $xcol = 1;

    my $enable_multiplot = 0;
    if (keys(%mem_components) > 0 && keys(%cpu_components) > 0) {
        $enable_multiplot = 1;
    }

    if ($relative_time) {
        $xcol = 2;
    }

    open(OUTFILE, ">".$outdir."/gnuplot_data.txt");
    print OUTFILE "# auto-generated gnuplot file\n";
    if ($outfile ne "") {
        print OUTFILE "set terminal $outfile_format\n";
        print OUTFILE "set output \"$outfile\"\n";
    }
    if ($relative_time) {
        print OUTFILE "set timefmt \"%H:%M:%S\"\n";
    }
    else {
        print OUTFILE "set timefmt \"%Y/%m/%d:%H:%M:%S\"\n";
    }
    print OUTFILE "set bmargin 3\n";
    print OUTFILE "set xdata time\n";
    print OUTFILE "set autoscale x\n";
    print OUTFILE "set autoscale y\n";
    print OUTFILE "set grid\n";

    if ($enable_multiplot) {
        print OUTFILE "set multiplot\n";
        print OUTFILE "set origin 0,0\n";
        print OUTFILE "set size 1, 0.5\n";
    }

    if ($max_y) {
	print OUTFILE "set yrange [* : $max_y]\n";
    }

    if (keys(%cpu_components) > 0) {
        print OUTFILE "set ylabel \"%\"\n";
        my $plotstr = "";

        foreach (keys %cpu_components) {
            $val = $cpu_components{$_};

            foreach (get_component_datafile($_, $outdir)) {
                if ($val & $ENABLE_CPU) {
                    $plotstr = add_graph($_, "$_ CPU", $outdir,
					 $xcol, 5, $plotstr);
                }
                if ($val & $ENABLE_SYS_CPU) {
                    $plotstr = add_graph($_, "System CPU", $outdir,
					 $xcol, 5, $plotstr);
                }
                if ($val & $ENABLE_SYS_AVG1) {
                    $plotstr = add_graph($_, "Load avg 1", $outdir,
					 $xcol, 6, $plotstr);
                }
                if ($val & $ENABLE_SYS_AVG5) {
                    $plotstr = add_graph($_, "Load avg 5", $outdir,
					 $xcol, 7, $plotstr);
                }
                if ($val & $ENABLE_SYS_AVG15) {
                    $plotstr = add_graph($_, "Load avg 15", $outdir,
					 $xcol, 8, $plotstr);
                }
            }

        }
        print OUTFILE $plotstr."\n";
    }

    if ($enable_multiplot) {
        print OUTFILE "set origin 0, 0.5\n";
        print OUTFILE "set size 1, 0.5\n";
    }

    if (keys(%mem_components) > 0) {
        print OUTFILE "set ylabel \"Kb\"\n";
        my $plotstr = "";

        foreach (keys %mem_components) {
            $val = $mem_components{$_};

            foreach (&get_component_datafile($_, $outdir)) {
                if ($val & $ENABLE_MEM) {
                    $plotstr = add_graph($_, $_. " mem", $outdir,
					 $xcol, 4, $plotstr);
                }
                if ($val & $ENABLE_SYS_FREE) {
                    $plotstr = add_graph($_, "System free", $outdir,
					 $xcol, 4, $plotstr);
                }
            }
        }
        print OUTFILE $plotstr."\n";
    }

    if ($memory_consumers) {
        print OUTFILE "set ylabel \"Kb\"\n";
        my $plotstr = "";
	foreach (&get_memory_consumers($outdir)) {
	    if (exists $mem_components{$_}) {
		$plotstr = add_graph($_, $_. " System free", $outdir,
				     $xcol, 4, $plotstr);
	    }
	    else {
		if ($_ ne "System") {
		    $plotstr = add_graph($_, $_. " mem", $outdir,
					 $xcol, 4, $plotstr);
		}
	    }
	}
        print OUTFILE $plotstr."\n";	
    }
    if ($enable_multiplot) {
        print OUTFILE "set nomultiplot\n";
    }

    if ($plot) {
        print OUTFILE "pause -1\n";
    }
    close(OUTFILE);

    system("gnuplot $outdir"."gnuplot_data.txt");
}

sub get_component_datafile
{
    my $component = sanitize_filename(shift);
    my $outdir = shift;
    my @result;

    if (-e "$outdir/$component.data") {
        push @result, "$component";
    }
    else {
        my $old_dir = getcwd;

        chdir $outdir;
        foreach (glob "$component*.data") {
            push @result, substr($_, 0, length($_)-5);
        }
        chdir "$old_dir";
    }
    return @result;
}

sub get_memory_consumers
{
    my $outdir = shift;
    my $olddir = getcwd;
    my @result;
    my $count = 0;

    chdir $outdir;
    foreach (glob "*.data") {
	open(INFILE, $_) || die("Could not open file!");
	my @lines = <INFILE>;
	my @firstLine = split(/ /, $lines[0]);
	my @lastLine = split(/ /, $lines[$#lines]);
	
	if ($lastLine[3] > ($firstLine[3]+$mem_limit) ) {
	    $_ =~ s/\.data//;
	    push @result, $_;
	    $count++;
	}
	close(INFILE);
    }
    print "Number of processes in graph: $count\n";
    chdir $olddir;    
    return @result;
}

sub add_graph
{
    my $component_file = sanitize_filename(shift);
    my $title = shift;
    my $outdir = shift;
    my $xcol = shift;
    my $ycol = shift;
    my $plotstr = shift;

    my $delim = ", ";
    if ($plotstr eq "") {
        $delim = "plot ";
    }

    return $plotstr.$delim."\"".$outdir.$component_file.".data\" using ".$xcol.
        ":".$ycol." title \"".$title."\" with lines";
}
